# package BuildForge::API::Project
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Adaptor
package BuildForge::API::Adaptor;

use BuildForge::Connection::Encoded;
use BuildForge::NameMap;
use BuildForge::API::Interface;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);

# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self = $class->SUPER::new(@_);
	
	$self->{'type'} = "Source";
	$self->{'name'} = "";
	$self->{'template'} = "Blank";
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'_public'}->{'xml'} = "";
	
	while(my($Var,$Val) = each %{$Params}) {
		$self->{$Var} = $Val;
	}
	$self->Sync() if($self->{'name'} ne "");
	return $self;
}

sub STORE {
	my($self,$key,$val) = @_;
	if(exists($self->{'_public'}->{$key})) {
		if($key eq "type") {
			if($val eq "S") {
				$val = "Source";
			}
			elsif($val eq "T") {
				$val = "Test";
			}
			elsif($val eq "D") {
				$val = "Defect";
			}
		}
	}
	$self->SUPER::STORE($key,$val);
}

sub Sync {
	my($self) = @_;
	if(!$self->{'name'}) {
		$self->{'_error'} = 1201;
		return $self;
	}
	my $Request = $self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_link=$self->{'name'} ORDER BY bf_lid";
	my $Results = $self->ProcessForArray({
		Command => "APIQueryRecords",
		Table => "bf_interfaces",
		Fields => "*",
		Clause => "WHERE bf_link='$self->{'name'}' ORDER BY bf_lid",
	},"bf_interfaces");
	$self->{'xml'} = "";
	foreach my $Result (@{$Results}) {
		$self->{'name'} = $Result->{'name'};
		$self->{'type'} = $Result->{'type'};
		$self->{'template'} = $Result->{'template'};
		chomp($Result->{'bf_interface'});
		$self->{'xml'}.=$Result->{'bf_interface'}."\n";
	}
	return $self->GetLastError();
}

sub Update {
	my ($self) = @_;
	$self->{'_error'} = 1201 unless($self->{'name'});
	$self->{'_error'} = 1202 unless($self->{'xml'});
	$self->{'_error'} = 1203 unless($self->{'type'});
	return $self->{'_error'} if($self->{'_error'});
	
	my $Request = $self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("EditInterface");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "type") {
			$RequestContent->{$Param}=($self->{'type'} eq "Test" ? "T" : ($self->{'type'} eq "Defect" ? "D" : "S"));
		}
		elsif($Param eq "template") {
			$RequestContent->{$Param} = "";
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "EditInterface";

	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_interfaces")});
	$self->{'_info'}=$QueryResults->{'_info'};
	return $self->GetLastError() if($self->{'_error'});
	return $self->Sync();
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 1201 unless($self->{'name'});
	$self->{'_error'} = 1204 unless($self->{'xml'} || $self->{'template'});
	$self->{'_error'} = 1203 unless($self->{'type'});
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddInterface");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "type") {
			$RequestContent->{$Param}=($self->{'type'} eq "Test" ? "T" : ($self->{'type'} eq "Defect" ? "D" : "S"));
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddInterface";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_interfaces");
	return $self->GetLastError() if($self->{'_error'});
	if($self->{'xml'}) {
		return $self->Update();
	}
	return $self->Sync();
}


__END__

=head1 I<NAME>

BuildForge::API::Adaptor

=head1 I<SYNOPSIS>

	my $Adaptor = new BuildForge::API::Adaptor();
	$Adaptor->{'template'} = "ClearQuest";
	$Adaptor->{'name'} = "My ClearQuest Adaptor";
	$Adaptor->{'type'} = "Defect";
	$Adaptor->Create();
	
	my $Adaptor = new BuildForge::API::Adaptor({name=>"My ClearQuest Adaptor"});
	print "Existing XML: $Adaptor->{'xml'}\n";
	$Adaptor->{'xml'} = $NewXML;
	$Adaptor->Update();
	print "New XML: $Adaptor->{'xml'}\n";


=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been 
instantiated. It is used to create new BuildForge Adaptors, and to access 
existing adaptor information.

=head1 I<METHODS>

=head2 new({AdaptorParamsHashRef})

Create an object to access existing adaptor information or to add a new adaptor
to the BuildForge console.

=over 8

=item I<name>

The name for the adaptor

=item I<type>

The adaptor type (Source, Test, or Defect)

=item I<template>

The template to use for populating a new adaptor

=item I<xml>

The xml for a new or existing adaptor

=back

=head2 Create()

Create a new Adaptor inside BuildForge. Requires Adaptor 'name', and 'type' 
to be set, as well as either the 'xml' or 'template' values. 

=head2 Update()

Updates the adaptor inside BuildForge. Requires the Adaptor 'name', 'type' and
'xml' fields to be set. This can update either or both of 'type' and 'xml'.

=head2 GetLastError()

Returns	the last error code encountered by the object.

=head2 GetLastMessage()

Returns an last message string encountered by the object.

=head2 Sync()

Syncs the object with information from the console database. Requires the
Adaptor 'name' to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
