# package BuildForge::API::Collector
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Collector
package BuildForge::API::Collector;

use BuildForge::Connection::Encoded;
use BuildForge::API::Environ;
use BuildForge::API::Interface;
use BuildForge::NameMap;
use BuildForge::API::Access;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	
	# Defaults:
	$self->{'name'} = "";
	$self->{'_public'}->{'properties'} = [];
	$self->{'BuiltIns'} = ["MEM_PAGESIZE","MEM_TOTAL","MEM_FREE","MEM_LOAD","NUM_CPU","WIN_SERVICEPACK","OS_SYSNAME","OS_RELEASE","OS_VERSION","CPU_LOAD1","CPU_LOAD5","CPU_LOAD15"];
	while(my($var,$val) = each %{$Params}) {
		$self->{$var} = $val;
	}
	return $self;
}

sub Sync {
	my($self) = @_;
	
	$self->{'_error'} = 2001 unless($self->{'name'});
	return undef if($self->{'_error'});
	
	my $WhereClause = "WHERE bf_coll_id='$self->{'name'}' AND bf_prop!=''";
	my $Results = $self->{'_connection'}->ProcessNewRequest({
			Command => "APIQueryRecords",
			Table => "bf_collector",
			Fields => "*",
			Clause => $WhereClause,
			Limit => 0,
		});
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_collector")});
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $ResHash = {};
	$self->{'properties'} = [];
	foreach my $ValueSet (@{$QueryResults}) {
		push @{$self->{'properties'}},
				{'property'=>$ValueSet->{'property'},
				 'value' => $ValueSet->{'value'},
				 'regex' => $ValueSet->{'regex'},
				 'isuser' => $ValueSet->{'userproperty'},
				};
	}
	return $ResHash;
}

sub BuiltInProperties {
	my($self) = @_;
	return $self->{'BuiltIns'};
}

sub AddProperty {
	my($self,$Property,$NoSync) = @_;
	$self->{'_error'} = 2002 unless($self->{'name'});
	$self->{'_error'} = 2003 unless($Property->{'property'});
	return undef if($self->{'_error'});
	$Property->{'value'} = "" unless($Property->{'value'});
	$Property->{'regex'} = "" unless($Property->{'regex'});
	$Property->{'userproperty'} = "" unless($Property->{'userproperty'});
	$self->Sync() unless($NoSync);
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddCollectorProperty");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "name") {
			$RequestContent->{$Param} = $self->{'name'};
			next;
		}
		$RequestContent->{$Param} = $Property->{$Var};
	}
	$RequestContent->{'Command'} = "AddCollectorProperty";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_collector");
	$self->Sync() unless($self->{'_error'} || $NoSync);
	return $self->GetLastError();
}	

sub DeleteProperty {
	my($self,$Property) = @_;
	$self->{'_error'} = 2002 unless($self->{'name'});
	$self->{'_error'} = 2003 unless($Property->{'property'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteCollectorProperty");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "name") {
			$RequestContent->{$Param} = $self->{'name'};
			next;
		}
		$RequestContent->{$Param} = $Property->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteCollectorProperty";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_collector");
	$self->Sync() unless($self->{'_error'});
	return $self->GetLastError();
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 2004 unless($self->{'name'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("CreateCollector");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "CreateCollector";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_collector");
	foreach my $Property (@{$self->{'properties'}}) {
		$self->AddProperty($Property,1);
	}
	$self->Sync() unless($self->{'_error'});
	return $self->GetLastError();
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 2004 unless($self->{'name'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteCollector");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteCollector";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_collector");
	$self->Sync() unless($self->{'_error'});
	return $self->GetLastError();
}

# Always Return Nicely
1;


__END__

=head1 I<NAME>

BuildForge::API::Collector

=head1 I<SYNOPSIS>

	my $Collector = new BuildForge::API::Collector({'name'=>"Yours",'properties'=>[{'property'=>"MEM_TOTAL"}]});
	$Collector->Sync();
	$Collector->AddProperty({
		'property'=>"WIN_SERVICEPACK"
	});
	$Collector->DeleteProperty({
		'property'=>"WIN_SERVICEPACK"
	});
		

=head1 I<DESCRIPTION>

Allows you to create and modify collectors for use with servers.

=head1 I<METHODS>

=head2 new({CollectorParamsHashRef})

Creates the collector object to create and modify collector data for use with
the BuildForge console.

=over 8

=item I<name>

The name for this collector.

=item I<properties>

A set of properties to use for this collector.

=back

=head2 Sync()

Syncs the object with the current console data for the collector.

=head2 BuiltInProperties()

Returns a list reference of Built-in auto-populated properties that are 
available for use in collectors.

=head2 AddProperty(PropertyParamsHashref)

Adds a property to the collector.

=over 8

=item I<property>

The name of this property

=item I<value>

The value to assign to this property

=item I<userproperty>

If this is a property that has to have a command run ('Y').

=item I<regex>

The regular expression to use to parse the output of the value command to 
populate the actual value in the corresponding servers manifest.

=back

=head2 DeleteProperty(PropertyParamsHashref)

Deletes a property from this collector

=over 8

=item I<property>

The name of the property to delete

=back

=head2 Create()

Creates a new collector for use by servers.

=head2 Delete()

Deletes the collector from the buildforge console. Note that no servers should
be using the collector prior to calling the delete.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut

