# package BuildForge::API::Schedule
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other intellectual property
# rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and BuildForge, Inc.

# BuildForge::API::Schedule
package BuildForge::API::Schedule;

use BuildForge::Connection::Encoded;
use BuildForge::API::Environ;
use BuildForge::API::Class;
use BuildForge::API::Access;
use BuildForge::NameMap;
use BuildForge::API::Interface;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self = $class->SUPER::new(@_);

	$self->{'schedule_id'} = 0;
	$self->{'description'} = "";
	$self->{'project_id'} = 0;
	$self->{'owner_id'} = 0;
	$self->{'environment_id'} = 0;
	$self->{'custom_environment'} = "";
	$self->{'active'} = 1;
	$self->{'minute'} = "0";
	$self->{'hour'} = "0";
	$self->{'day_of_month'} = "*";
	$self->{'month'} = "*";
	$self->{'day_of_week'} = "*";
	$self->{'server'} = "";
	$self->{'class'} = "";

	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	$self->Sync() if($Params->{'schedule_id'} > 0);
	return $self;
}

sub Sync {
	my($self) = @_;
	if(!$self->{'schedule_id'} && !$self->{'description'}) {
		$self->{'_error'} = 850;
		$self->{'info'} = "Can't sync schedule without an id or description";
		return $self;
	}
	my $Clause = "WHERE bf_cid=$self->{'schedule_id'}";
	if(!$self->{'schedule_id'}) {
		$Clause = "WHERE bf_description='$self->{'description'}'";
	}

	my $QueryResults = $self->ProcessForHash({
			Command => "APIQueryRecords",
			Table => "bf_cron",
			Fields => "*",
			Clause => $Clause,
			Limit => 1,
		},"bf_cron");
	
	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 855;
		$self->{'_info'} = "Did not find any matching schedule entries.";
		return $self->{'_error'};
	}	
	return $self;
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 853 unless($self->{'schedule_id'} || $self->{'description'});
	return $self->{'_error'} if($self->{'_error'});
	$self->Sync();
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteSchedule");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteSchedule";
	$self->ProcessForHash($RequestContent,"bf_cron");
	return $self->GetLastError();
}

sub Reset{
	my($self) = @_;
	$self->{'_error'} = 854 unless($self->{'schedule_id'} || $self->{'description'});
	return $self->{'_error'} if($self->{'_error'});
	$self->Sync();
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("ResetSchedule");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "EditSchedule";
	$self->ProcessForHash($RequestContent,"bf_cron");
	return $self->GetLastError();
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 851 unless($self->{'description'} ne "");
	$self->{'_error'} = 852 unless($self->{'project_id'} > 0);
	
	return undef if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddSchedule");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "owner_id") {
			if ($self->{$Var} < 1) {
				delete $RequestContent->{$Param};
			} else {
				$RequestContent->{$Param}=$self->{$Var};
			}
		}
		elsif($Var eq "project_id") {
			# Need to get the project server...
			my $TmpProject = new BuildForge::API::Project({project_id=>$self->{'project_id'}});
			$RequestContent->{$Param} = "$self->{'project_id'}:$TmpProject->{'selector'}";
		}
		else {
			$RequestContent->{$Param}=$self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddSchedule";
	my $Results = $self->{'_connection'}->ProcessNewRequest($RequestContent);
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_cron")});
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	map {$self->{$_} = $QueryResults->{$_} unless($_=~/^_/)} keys(%{$QueryResults});
	$self->Sync() unless($self->{'_error'});
	return $self;
}


# Always Return Nicely
1;

__END__

=head1 I<NAME>

BuildForge::API::Schedule

=head1 I<SYNOPSIS>

	my $Sched = new BuildForge::API::Schedule();
	$Sched->{'description'} = "API Scheduled Job";
	$Sched->{'project_id'} = $Project->{'project_id'};
	$Sched->Create();

=head1 I<DESCRIPTION>

    This class is available to user after a BuildForge::API object has been
instantiated. It is used to add, and delete schedules in the BuildForge 
engine, also allowing a schedule to be reset, and reevaluated by the engine.

=head1 I<METHODS>

=head2 new({ScheduleParamsHashRef})

Create an object to access, create or delete schedules in the BuildForge
engine.

=over 8

=item I<description>

The description for this schedule.

=item I<project_id>

The id of the project to run.

=item I<owner_id>

The user id of the schedule owner

=item I<environment_id>

The id of the environment to use for these builds

=item I<custom_environment>

The modified env to use for these builds.

=item I<active>

If the schedule should be active or not (1/0)

=item I<minute>

The minute cron for this schedule 

=item I<hour>

The hour cron for this schedule

=item I<day_of_month>

The day of month cron for this schedule

=item I<month>

The month cron for this schedule

=item I<day_of_week>

The day of week cron for this schedule

=item I<server>

The server to use for these builds

=item I<class>

(3.7 only) The class to run these builds as

=back

=head2 Create()

Creates a new Build Schedule, requires the object variables 'description' and 
'project_id' to be set.

=head2 Delete()

Deletes the Build Schedule, requires the object variable 'description' or
'schedule_id' to be set.

=head2 Reset()

Recalculates the next run-time for the Schedule. Requires the object variable
'description' or 'schedule_id' to be set.

=head2 Sync()

Syncs the Schedule object with information from the database. Requires the
object variable 'description' or 'schedule_id' to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
