# package BuildForge::API::Selector
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Selector
package BuildForge::API::Selector;

use BuildForge::Connection::Encoded;
use BuildForge::API::Environ;
use BuildForge::API::Interface;
use BuildForge::NameMap;
use BuildForge::API::Access;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	
	# Defaults:
	$self->{'name'} = "";
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'_public'}->{'properties'} = [];
	while(my($var,$val) = each %{$Params}) {
		$self->{$var} = $val;
	}
	return $self;
}

sub Sync {
	my($self) = @_;
	
	$self->{'_error'} = 2101 unless($self->{'name'});
	return undef if($self->{'_error'});
	
	my $WhereClause = "WHERE bf_selector_id='$self->{'name'}' AND bf_prop!=''";
	my $Results = $self->{'_connection'}->ProcessNewRequest({
			Command => "APIQueryRecords",
			Table => "bf_selector",
			Fields => "*",
			Clause => $WhereClause,
			Limit => 0,
		});
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_selector")});
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $ResHash = {};
	$self->{'properties'} = [];
	foreach my $ValueSet (@{$QueryResults}) {
		push @{$self->{'properties'}},
				{'property'=>$ValueSet->{'property'},
				 'value' => $ValueSet->{'value'},
				 'regex' => $ValueSet->{'operator'},
				 'isuser' => $ValueSet->{'required'},
				};
	}
	return $ResHash;
}

sub AddProperty {
	my($self,$Property,$NoSync) = @_;
	$self->{'_error'} = 2102 unless($self->{'name'});
	$self->{'_error'} = 2103 unless($Property->{'property'});
	$self->{'_error'} = 2104 unless($Property->{'value'});
	return undef if($self->{'_error'});
	$Property->{'operator'} = "==" unless($Property->{'operator'});
	$Property->{'required'} = "Y" unless($Property->{'required'});
	$self->Sync() unless($NoSync);
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddSelectorProperty");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "name") {
			$RequestContent->{$Param} = $self->{'name'};
			next;
		}
		$RequestContent->{$Param} = $Property->{$Var};
	}
	$RequestContent->{'Command'} = "AddSelectorProperty";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_selector");
	$self->Sync() unless($self->{'_error'} || $NoSync);
	return $self->GetLastError();
}	

sub DeleteProperty {
	my($self,$Property) = @_;
	$self->{'_error'} = 2102 unless($self->{'name'});
	$self->{'_error'} = 2103 unless($Property->{'property'});
	return undef if($self->{'_error'});
	$Property->{'operator'} = "" unless($Property->{'operator'});
	$Property->{'required'} = "" unless($Property->{'required'});
	$Property->{'value'} = "" unless($Property->{'value'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteSelectorProperty");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "name") {
			$RequestContent->{$Param} = $self->{'name'};
			next;
		}
		$RequestContent->{$Param} = $Property->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteSelectorProperty";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_selector");
	$self->Sync() unless($self->{'_error'});
	return $self->GetLastError();
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 2004 unless($self->{'name'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("CreateSelector");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "CreateSelector";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_selector");
	foreach my $Property (@{$self->{'properties'}}) {
		$self->AddProperty($Property,1);
	}
	$self->Sync() unless($self->{'_error'});
	return $self->GetLastError();
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 2004 unless($self->{'name'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteSelector");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteSelector";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_selector");
	$self->Sync() unless($self->{'_error'});
	return $self->GetLastError();
}

# Always Return Nicely
1;


__END__

=head1 I<NAME>

BuildForge::API::Selector

=head1 I<SYNOPSIS>

	my $Selector = new BuildForge::API::Selector({'name'=>"MySelector");
	$Selector->Create();

=head1 I<DESCRIPTION>

    This class allows you to create and modify selectors for use with projects
and steps to select servers to use with them.

=head1 I<METHODS>

=head2 new(SelectorParamsHashref)

Creates the selector object to allow creation and modification of selectors.

=over 8

=item I<name>

The name of the selector.

=item I<properties>

The properties to associate with this selector. Each property should be a hashref 
as described in the AddProperty method below.

=back

=head2 Sync()

Syncs this selectors properties with the data currently in the BuildForge console.

=head2 AddProperty(PropertyParamsHashref)

Add a property to this selector

=over 8

=item I<property>

The name of the property

=item I<value>

The value for the named property

=item I<operator>

Conditions to apply to the property, valid conditions are '>','<','=','>=','<='.

=item I<required>

If this condition must be met to select a server, then set this value to 'Y'.

=back

=head2 DeleteProperty(PropertyParamsHashref)

Deletes a property from this selector. The property hashref can contain as many
or few of the following properties as desired.

=over 8

=item I<property>

The name of the property

=item I<value>

The value for the named property

=item I<operator>

Conditions to apply to the property, valid conditions are '>','<','=','>=','<='.

=item I<required>

If this condition must be met to select a server, then set this value to 'Y'.

=back

=head2 Create()

Creates the Selector for use with projects and steps.

=head2 Delete()

Deletes the property from the console. Note that this will fail if the 
Selector is in use by any project or step.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
