# package BuildForge::Connection::Encoded
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::Connection::Encoded
# This package is used for:
# Connecting to the BF API.
package BuildForge::Connection::Encoded;

use LWP::UserAgent;
use HTTP::Request;

use strict;

# Make this a singleton for now...
{
	my $object_exists=0;
	my $object_ref=undef;
	sub exists {$object_exists}
	sub create_static {$object_exists++}
	sub destroy_static {$object_exists--}
	sub get_static {$object_ref}
	sub set_static {$object_ref=$_[1]}
}

sub DESTROY {
	my ($self) = @_;
	my $class = ref($self);
	if($class->exists()) {
		$class->set_static(undef);
		$class->destroy_static();
	}
}

# Instantiation
sub new {
	my ($class,$Params) = @_;
	if($ENV{'BF_API_DEBUG'}) {
		my($Pack,$File,$Line) = caller();
		print "Connection Reference Requested from $Pack($Line:$File)\n";
	}
	
	# Singleton object, only want one of these bad boys
	if($class->exists()) {
		return $class->get_static();
	}

	
	my $self={
		'UserAgent' => LWP::UserAgent->new(),
		'_error' => 0,
		'_info' => "",
		'default_request_headers'=>{
			'Host'=>$Params->{'host'},
			'Connection'=>"Keep-Alive",
			'Accept-Language'=>"en-us",
			'Accept'=>"*/*",
		},
		'cached_user'=>'',
		'cached_pass'=>'',
		'cached_domain'=>'',
		'token'=>0,
		'cached_host'=>"",
		'cached_base'=>"/api.php",
		'cached_port'=>80,
	};
	
	bless $self, $class || ref($class);
	
	unless(defined($self->{'UserAgent'})) {
		$self->{'_error'} = 720;
		return $self;
	}
	
	$self->{'UserAgent'}->agent('BuildForge/1.0 (compatible; BFAPI 1.0)');
	$self->{'UserAgent'}->protocols_allowed(['http']);
	
	# Authenticate us with our given user/pass
	$self->{'_error'} = 721 unless($self->Authenticate($Params));

	$class->create_static();
	$class->set_static($self);
	
	return $self;
}

sub TearDown {
	my ($self) = @_;
	my $class = ref($self);
	if($class->exists()) {
		$class->set_static(undef);
		$class->destroy_static();
	}
}

sub LogOut {
	my($self) = @_; 
	my $Request = $self->NewDefaultRequest();
	$self->AddContent({Request=>$Request,Name=>"Command",Value=>"APILogOut"});
	my $Results = $self->ProcessRequest($Request);
	my $ResultsHash = $self->ParseResultsAsHash({Results=>$Results});
	$self->TearDown();
	return $ResultsHash->{'_token'};
}

sub NewDefaultRequest {
	my ($self) = @_;
	# Construct our url
	my $URL = "http://".$self->{'cached_host'}.":".$self->{'cached_port'}.$self->{'cached_base'};
	# We're doing a POST request.
	my $Request = HTTP::Request->new(POST=>$URL);
	# Set up our standard header set. 
	while(my ($header,$value) = each(%{$self->{'default_request_headers'}})) {
		$Request->header($header=>$value);
	}
	$Request->method("POST");
	$Request->content_type("application/x-www-form-urlencoded");
	# Add our existing token info
	$self->AddContent({'Request'=>$Request,'Name'=>"token",'Value'=>$self->{'token'}});
	return $Request;
}

sub AddContent {
	my ($self,$Params) = @_;
	my $CurContent = $Params->{'Request'}->content();
	# Add/set the key and encoded value to the request.
	if($CurContent ne "") {
		$Params->{Request}->content($CurContent."&$Params->{'Name'}=".$self->EncodeValue($Params->{'Value'}));
	}
	else {
		$Params->{Request}->content("$Params->{'Name'}=".$self->EncodeValue($Params->{'Value'}));
	}
}

# This is just another way to set the content, given the request directly, allows setting a hash of info rather than single key/value pair.
sub AddRequestContent {
	my ($self,$Request,$Content) = @_;
	my $CurContent = $Request->content();
	if($CurContent ne "") {
	}
	while(my ($key,$value) = each(%{$Content})) {
		if($CurContent) {
			$CurContent.="&";
		}
		if(ref($value) eq "ARRAY") {
			# This allows for passing in arrays, such as x=y&x=z so that x will become a php array of (y,z).
			$CurContent.="$key=".join("&$key=",map {$self->EncodeValue($_)} @{$value});
		}
		else {
			$CurContent.="$key=".$self->EncodeValue($value);
		}
	}
	$Request->content($CurContent);
}		
		
sub ProcessRequest {
	my($self,$Request,$Try) = @_;
	$Try = 0 unless $Try;
	$self->{'_error'} = 0;
	$self->{'_info'} = "";
	# Allow 9 tries before we fail out.
	if($Try++ > 9) {
		$self->{'_error'}=730;
		$self->{'_info'}="Failed to connect to API after ".($Try-1)." attempts...";
		return "";
	}
	if($ENV{'BF_API_DEBUG'}) {
		while(my($K,$V) = each %{$Request}) {
			print "REQUEST:[$K]=[";
			my @RVals = split(/\&/,$V);
			foreach my $RVal (@RVals) {
				my ($RT,$RV) = split(/=/,$RVal,2);
				print "$RT";
				if(defined($RV)) {
					print "=";
					if($K eq "_content") {
						print $self->DecodeValue($RV);
					}
					else {
						print $RV;
					}
					print "\&" unless($RVal eq $RVals[$#RVals]);
				}
			}
			print "]\n";
		}
	}
	# Send the request to the server.
	my $Response = $self->{'UserAgent'}->request($Request);
	# Check for a valid token, error and info...
	foreach my $Line (split(/\n/,$Response->content)) {
		$Line=~s/\s//g;
		next unless($Line);
		my ($Attr,$Val) = split(/=/,$Line);
		$Val=$self->DecodeValue($Val);
		print "RESPONSE[$Attr=$Val]\n" if($ENV{'BF_API_DEBUG'});
		
		if($Attr eq "_token") {
			$self->{'token'} = $Val;
			$self->{'token'} =~ s/\0*$//;
		}
		if($Attr eq "_error") {
			$self->{'_error'} = 731;
			$self->{'_info'} = $Val;
			return "";
		}
		if($Attr eq "_info") {
			$self->{'_info'} = $Val;
		}
	}
	# If we don't have a valid token then add our credentials to the request and try again (only add on second attempt).
	unless($self->{'token'}) {
		if($Try == 1) {
			$self->AddContent({Request=>$Request,Name=>"login",Value=>$self->{'cached_user'}});
			$self->AddContent({Request=>$Request,Name=>"password",Value=>$self->{'cached_pass'}});
			$self->AddContent({Request=>$Request,Name=>"domain",Value=>$self->{'cached_domain'}})
				if length $self->{'cached_domain'};
		}
		# $Response = $self->{'UserAgent'}->request($Request);
		return $self->ProcessRequest($Request,$Try);
	}
	return $Response->content;
}

sub ProcessNewRequest {
	my($self,$RequestHash) = @_;
	# Set up the reqeust
	my $Request = $self->NewDefaultRequest();
	# Add the content
	$self->AddRequestContent($Request,$RequestHash);
	# Process it
	return $self->ProcessRequest($Request);
}

sub Authenticate {
	my($self,$Params) = @_;
	$self->{'cached_user'}=$Params->{'user'};
	$self->{'cached_pass'}=$Params->{'pass'};
	$self->{'cached_domain'}=$Params->{'domain'};
	$self->{'cached_host'}=$Params->{'host'};
	$self->{'cached_port'}=$Params->{'port'};
	$self->{'cached_base'}=$Params->{'base'};
	my $Request = $self->NewDefaultRequest();
	$self->AddContent({Request=>$Request,Name=>"login",Value=>$Params->{'user'}});
	$self->AddContent({Request=>$Request,Name=>"password",Value=>$Params->{'pass'}});
	$self->AddContent({Request=>$Request,Name=>"domain",Value=>$Params->{'domain'}})
		if length $Params->{'domain'};
	my $Results = $self->ProcessRequest($Request);
	# Return our token...
	my $ResultsHash = $self->ParseResultsAsHash({Results=>$Results});
	return $ResultsHash->{'_token'};
}

sub ParseResultsAsHash {
	my($self,$Params) = @_;
	my $ResHash = {};
	foreach my $Line (split(/\n/,$Params->{'Results'})) {
		$Line=~s/\s//g;
		next unless($Line);
		my ($Attr,$Val) = split(/=/,$Line);
		$Val=$self->DecodeValue($Val);
		# See if we have a name translation for this key.
		$Attr = $Params->{'NameMap'}->{$Attr} || $Attr;
		# $Attr =~ s/^bf_//;
		$ResHash->{$Attr}=$Val;
	}
	return $ResHash;
}

sub ParseResultsAsArrayHash {
	my($self,$Params) = @_;
	my $ResArray = [];
	my $ResHash = undef;
	foreach my $Line (split(/\n/,$Params->{'Results'})) {
		$Line=~s/\s//g;
		next unless($Line);
		my ($Attr,$Val) = split(/=/,$Line);
		# don't care about the _token, its used elsewhere, unless we have the ResHash, then something is weird, since the token should come first.
		next if($Attr eq "_token" && !defined($ResHash));
		next if($Attr eq "_version" && !defined($ResHash));
		$Val =$self->DecodeValue($Val);
		# Each record in an array set is seperated by a _record id
		if($Attr eq "_record") {
			push @{$ResArray},$ResHash if(defined($ResHash));
			my %New = ();
			$ResHash = \%New;
			next;
		}
		# See if we have a name translation for this key.
		$Attr = $Params->{'NameMap'}->{$Attr} || $Attr;
		# $Attr =~ s/^bf_//;
		$ResHash->{$Attr}=$Val;
	}	
	push @{$ResArray},$ResHash if(defined($ResHash));
	return $ResArray;
}

sub GetLastError {
	my ($self) = @_;
	return $self->{'_error'};
}

sub GetLastMessage {
	my ($self) = @_;
	return $self->{'_info'};
}

sub EncodeValue {
	my($self,$value) = @_;
	return unpack "H*",$value;
}

sub DecodeValue {
	my($self,$value) = @_;
	$value = pack "H*",$value;
	$value =~ s/\0*$//;
	return $value;
}

# Always Return Nicely
1;
