#!/usr/bin/perl

#
# Copyright ©2003–2006, BuildForge, Inc.  All rights reserved.
# BuildForge, Inc. owns the copyright and other intellectual property
# rights in this software.  Duplication or use of the Software is not
# permitted except as expressly provided in a written agreement between
# your company and BuildForge, Inc.
#

$|++;

use strict;
use lib '../engine';
use lib '../../engine';

my $PHP = undef;
my $ERRORS = 0;
my $db_provider = '';

#================================================================
# CHECK SEQUENCE
#================================================================

print "\nExamining your system ...\n\n";

print "\nChecking perl settings ...\n\n";
perl_version([ '>= 5.8.0', '< 5.9.0' ]);
perl_module('Config::IniFiles');
perl_module('Data::Dumper');
perl_module('File::Basename');
perl_module('File::Path');
perl_module('File::Temp');
perl_module('Getopt::Std');
perl_module('HTML::Template', [ '>= 2.8' ]);
perl_module('IO::Select');
perl_module('IO::Socket');
perl_module('IO::File');
perl_module('IPC::Open2');
perl_module('LWP::Simple');
perl_module('MIME::Lite');
perl_module('POSIX');
perl_module('Set::Crontab');
perl_module('Sys::Hostname');
perl_module('Text::Balanced');
perl_module('Time::Local');
perl_module('Time::Piece');
perl_module('XML::Writer');
perl_module('XML::Writer::String');
perl_module('XML::XPath', [ '>= 1.13' ]);

print "\nChecking database settings ...\n\n";
get_db_provider();
perl_module('DBI', [ '>= 1.49' ]);
my $dbd_provider = &get_dbd_provider($db_provider);
perl_module("DBD::$dbd_provider");
perl_module("BuildForge::DB::$db_provider");

php_version([ '>= 5.1.2' ]);
php_checklist();

die "\nFOUND $ERRORS ERROR", ($ERRORS == 1 ? '' : 'S'), "\n\n" if $ERRORS;
print "\nNo errors found.\n\n";
exit 0;

#================================================================
# UTILITY FUNCTIONS
#================================================================

sub version_split {
	my ($ver) = @_;
	$ver =~ y/_//d;
	$ver =~ /^\d[\d\.]*\d$/ and return split(/\./, $ver);
	return undef;
}

sub version_test {
	my ($ver1, $args) = @_;
	my ($oper, $ver2) = split(/ /, $args, 2);

	my @ver1 = version_split($ver1);
	return (@ver1) ? 1 : 0 if $oper =~ /def(ined)?/;
	return (!@ver1) ? 1 : 0 if $oper =~ /undef(ined)?/;

	my @ver2 = version_split($ver2);
	return undef unless (@ver1 && @ver2);

	my $cmp = 0;
	while (@ver1 || @ver2) {
		my $p1 = shift @ver1;
		my $p2 = shift @ver2;
		$cmp--, last if $p1 < $p2;
		$cmp++, last if $p1 > $p2;
	}

	return $cmp == 0 ? 1 : 0 if $oper eq '==';
	return $cmp != 0 ? 1 : 0 if $oper eq '!=';
	return $cmp <  0 ? 1 : 0 if $oper eq '<';
	return $cmp <= 0 ? 1 : 0 if $oper eq '<=';
	return $cmp >  0 ? 1 : 0 if $oper eq '>';
	return $cmp >= 0 ? 1 : 0 if $oper eq '>=';

	warn "Invalid oper given to check_version: '$oper'\n";
	return undef;
}

sub check {
	my ($data, $tests) = @_;
	$tests = [ 'defined' ] unless ref($tests) eq 'ARRAY';

	my @errors = ();
	my $fails = 0;
	while (@$tests) {
		my $arg = shift @$tests;
		my $optional = undef;
		my $flag = 'FAIL:';
		if (substr($arg,0,1) eq '?') {
			$optional = 1;
			$flag = 'WARN:';
			$arg = substr($arg, 1);
		}
		if( ! defined $data ) { 
			# undef $data means the module was missing
			$fails++ unless $optional;
		}

		next if version_test($data, $arg);
		push @errors, "$flag Expected $arg\n";
		$fails++ unless $optional;
		next;
	}

	foreach my $error (@errors) { print "\t$error\n"; }
	$ERRORS += $fails;
	return !$fails;
}


#================================================================
# UNIX FUNCTIONS
#================================================================
sub unix_user { return getpwnam($_[0]); }
sub unix_group { return getgrnam($_[0]); }


#================================================================
# PERL FUNCTIONS
#================================================================
sub perl_version {
	my ($checks) = @_;
	print "Checking Perl version ... ";
	my $ver = sprintf("%vd", $^V);
	print "$ver\n";
	return check($ver, $checks);
}

sub perl_module {
	my ($module, $checks) = @_;
	print "Checking Perl module $module ... ";
	eval "use $module;";
	if ($@) {
		print "MISSING\n";
		return check(undef, $checks);
	}
	my $ver = eval "\$${module}::VERSION";
	if ($@ || !defined($ver) || !length($ver)) {
		print "found\n";
		return check('', $checks);
	}
	print "$ver\n";
	return check($ver, $checks);
}

sub get_db_provider {
	if (open(CONF, 'buildforge.conf')) {
		while (<CONF>) {
			next unless /^\s*db_provider\s*(\S+)/;
			$db_provider = $1;
			last;
		}
		close CONF;
	}
	if (length $db_provider) {
	#	print "$db_provider\n";
	} else {
		$db_provider = 'db2';
		$ERRORS++;
	}
}

sub get_dbd_provider {
	my ($provider) = @_;
	my $dbd_list = { 'db2' => 'DB2', 'mysql' => 'mysql', 'oracle' => 'Oracle', 'sybase' => 'ODBC', 'sqlserver' => 'ODBC' };

	return $dbd_list->{lc($provider)} || $provider;
}


#================================================================
# PHP FUNCTIONS
#================================================================

sub php_version {
	my ($checks) =@_;

	print "Checking PHP version ... ";
	$PHP = undef;
	my $ver = undef;

PHP_VERSION:
	foreach my $php ('php', 'php5') {
		if (open (PHP, "$php --version 2>&1 |")) {
			$ver = '';
			$PHP = $php;
			while (my $line = <PHP>) {
				if ($line =~ /^PHP (\d+\.\d+\.\d+)/) {
					$ver = $1;
					1 while $line = <PHP>;
				}
			}
			close PHP;
		}
	}

	# Not there, so I hope this one is ...
	if (!defined($ver) && open (PHP, 'php --version 2>&1 |')) {
		$ver = '';
		$PHP = 'php';
		while (my $line = <PHP>) {
			if ($line =~ /^PHP (\d+\.\d+\.\d+)/) {
				$ver = $1;
				last;
			}
		}
		close PHP;
	}

	if (!defined($ver)) {
		print "MISSING\n";
	} elsif (!length($ver)) {
		print "UNKNOWN VERSION\n";
	} else {
		print "$ver\n";
	}
	return check($ver, $checks);
}

sub php_checklist {
	print "Launching PHP checks ... ";
	if (!open (PHP, "$PHP -q checklist.php $db_provider |")) {
		print "FAILED\n";
		$ERRORS++;
		return undef;
	}
	print "launched\n";

	while (my $line = <PHP>) {
		if ($line =~ /^FAIL (\d+)$/) {
			$ERRORS += $1;
			next;
		}
		print $line;
	}

	close PHP;
}








