#!/usr/bin/cqperl
#
# Copyright (c)2003-2006, BuildForge, Inc. All rights reserved.
# BuildForge, Inc. owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and BuildForge, Inc.
#
#
# bfcqbuild.pl
#
# Description:
#
# This application is an extension of the ClearQuest API, 
# it is used to create and modify Build records inside of 
# ClearQuest.
#
#
# Created by: rfuller
# Modification Date: 11:01 AM 12/12/2005
#
# Usage:
# cqperl bfcqbuild.pl <update_state>
#
# The following Parameters are set via the environment.
#
# CQ_DBNAME
#	The ClearQuest Database Instance Name
# CQ_DBSET
#	The Database set or connection string (default "")
# CQ_USER
#	The ClearQuest User
# CQ_PASSWORD
#	Corresponding user password (default "")
# CQ_RELEASE_NAME
#	The Release Name as identified by ClearQuest
# CQ_RECORD_ID
#	The CQ Build Record ID (if updating).
# TAG
#	The Build Tag for the build.
# BUILD_URL
#	A URL pointing to relevant build information
# BUILD_LOG_TEXT
#	An extract of interesting build log information
# HIDE_PASSWORD
#	Do not log the password
# BFCQDEBUG
#	Print out debug info

# Import Libraries
use CQPerlExt;
use strict;

# We don't want CQ warnings reported in our output stream.
open (STDERR,"> nul");

# Scope of our application
{
	my $ReqState = shift || "Submit";

	my $CQSession;
	my $CurState = "";
	eval { $CQSession = ValidateInfo($ReqState); };
	Error("Could not connect to ClearQuest - Invalid Initialization Data",$@) if($@);
	
	# See if this is a new submission, or a resubmit.
	
	# Get the current state if we have one...

	eval {$CurState = RecordExists($CQSession); };
	Debug("Curstate Is: $CurState");
	
	if($ReqState eq "Submit" && !$CurState) {
		$ReqState = CheckSubmit($CurState);
		Debug("Reqstate Now: $ReqState");
	}
	else {
		$ReqState = ValidateNextState($CurState,$ReqState);
		Debug("ReqState Now: $ReqState");
	}
	
	
	if($ReqState eq "CantChange") {
		Error("Cannot change build record.","The build record is in a Retired state.");
	}

	# Do our action.
	if($ReqState eq "Submit") {
		#Check for CQ_Release_Name for backwards compatibility
		if (defined($ENV{'CQ_RELEASE_NAME'})) {
			eval {CheckReleaseRecord($CQSession);};
			Error("Could not create Release Record in ClearQuest",$@) if($@);
		}
		eval {SubmitBuild($CQSession);};
		Error("Could not submit build record to ClearQuest",$@) if($@);
	}
	else {
		eval {UpdateBuild($CQSession,$ReqState);};
		Error("Could not update build record in ClearQuest",$@) if($@);
	}
	
	# Drop our session, don't care if we error here, as we're leaving.
	eval { CQSession::Unbuild($CQSession); };
	
	exit 0;
} # Scope

# ValidateNextState
#
# Make sure we're trying to set a valid state.
sub ValidateNextState {
	my($CurState,$ReqState) = @_;
	my @ValidStates = ("Submit","Resubmit","Retire","Complete","Failure");
	my $Valid = 0;
	foreach my $State (@ValidStates) {
		if($ReqState =~ /^$State$/i) {
			$Valid = 1;
		}
	}
	Error("Invalid state requested","State [$ReqState] is not recognized as a valid state for a ClearQuest Build record. Valid states are [".join("],[",@ValidStates)."].") unless($Valid);
	@ValidStates = ();
	$Valid = 0;
	if($CurState =~ /Failed/i) {
		@ValidStates = ("Resubmit","Retire","Submit");
		if($ReqState =~ /Submit/i) {
			$ReqState = "Resubmit";
		}
	}
	elsif($CurState =~ /Completed/i) {
		@ValidStates = ("Retire");
	}
	elsif($CurState =~ /Retired/i) {
		@ValidStates = ();
	}
	elsif($CurState =~ /Submitted/i) {
		@ValidStates = ("Complete","Failure","Submit","NoChange");
		if($ReqState =~ /Submit/i) {
			$ReqState = "NoChange";
		}
	}
	foreach my $State (@ValidStates) {
		if($ReqState =~ /$State/i) {
			$Valid = 1;
		}
	}
	Error("Invalid state requested","Current state is [$CurState], requested next state as [$ReqState], valid nextstate(s) are [".join("],[",@ValidStates)."].") unless($Valid);
	
	return $ReqState;
} # ValidateNextState

# SubmitBuild
#
# Create a new CQ Build Record.
sub SubmitBuild {
	my($CQSession) = @_;
	my $BuildRec = $CQSession->BuildEntity("btbuild");
	$BuildRec->SetFieldValue("start_datetime",genTimeStamp());
	$BuildRec->SetFieldValue("build_system_id",$ENV{'TAG'});
	if (defined($ENV{'CQ_RELEASE_NAME'})) { 
		$BuildRec->SetFieldValue("releasename",$ENV{'CQ_RELEASE_NAME'});
	} 
	$BuildRec->SetFieldValue("build_system_url",$ENV{'BUILD_URL'});
	$BuildRec->SetFieldValue("buildlog",$ENV{'BUILD_LOG_TEXT'});
	my $RecordId = $BuildRec->GetDisplayName();
	
	# Validate and Commit.
	$BuildRec->Validate();
	$BuildRec->Commit();
	Log("Build Record Created:\n$RecordId");
} # SubmitBuild

sub CheckReleaseRecord {
	my($CQSession) = @_;
	my $ReleaseRec = undef;
	eval {$ReleaseRec = $CQSession->GetEntity("dtrelease",$ENV{'CQ_RELEASE_NAME'});};
	return if(ref($ReleaseRec) eq "CQEntity");
	# Need to create the release record...
	my $CreateReleaseRec = $CQSession->BuildEntity("dtrelease");
	$CreateReleaseRec->SetFieldValue("release_name",$ENV{'CQ_RELEASE_NAME'});
	$CreateReleaseRec->Validate();
	my $CommitResult = $CreateReleaseRec->Commit();
	if($CommitResult eq "") {
		Log("Release Record Created:\n$ENV{'CQ_RELEASE_NAME'}");
	}
	else {
		Error("Could not create Release Record.",$CommitResult);
	}
}
	
	
# UpdateBuild
#
# Update an already existing Build Record.
sub UpdateBuild {
	my($CQSession,$Action) = @_;
	my $BuildRec = $CQSession->GetEntity("btbuild",$ENV{'CQ_RECORD_ID'});
	$CQSession->EditEntity($BuildRec,"modify");
	$BuildRec->SetFieldValue("build_system_url",$ENV{'BUILD_URL'}) if(exists($ENV{'BUILD_URL'}));
	$BuildRec->SetFieldValue("buildlog",$ENV{'BUILD_LOG_TEXT'}) if(exists($ENV{'BUILD_LOG_TEXT'}));
	if($Action =~ /(NoChange|Resubmit)/i) {
		$BuildRec->SetFieldValue("start_datetime",genTimeStamp());
	}
	elsif($Action !~ /Retire/i) {
		$BuildRec->SetFieldValue("end_datetime",genTimeStamp());
	}
	$BuildRec->Validate();
	$BuildRec->Commit();
	
	if($Action=~/(Resubmit|Failure|Complete|Retire)/i) {
		$CQSession->EditEntity($BuildRec,$Action);
		$BuildRec->Validate();
		$BuildRec->Commit();
	}
	Log("Build Record Updated:\n$ENV{'CQ_RECORD_ID'}");
} # UpdateBuild

# RecordExists
# 
# Check to see if our record exists and if this should be resubmitted, or other.
sub RecordExists {
	my ($CQSession) = @_;
	my $RecId = $ENV{'CQ_RECORD_ID'};
	my $Entity = undef;
	if($RecId) {
		$Entity = $CQSession->GetEntity("btbuild",$RecId);
	}
	my $StateRec = $Entity->GetFieldValue("State");
	return $StateRec->GetValue;
} # RecordExists

# CheckSubmit
#
# Check to see if a Submit can happen or if its a resubmit...
sub CheckSubmit {
	my($State) = @_;
	if($State =~ /Retired/i) {
		return "CantChange";
	}
	elsif($State =~ /(Completed|Failed)/i) {
		return "Resubmit";
	}
	elsif($State =~ /Submitted/i) {
		return "NoChange";
	}
	return "Submit";
} # CheckSubmit

# ValidateInfo
#
# Make sure we have the correct env for our request.
sub ValidateInfo {
	my ($ForState) = @_;
	my @RequiredInfo = ("CQ_DBNAME","CQ_USER");
	
	# Check our entry data...
	if($ForState =~/Submit/i) {
		push @RequiredInfo,("TAG","BUILD_URL","BUILD_LOG_TEXT");
	}
	elsif($ForState =~ /Failure/i) {
		push @RequiredInfo,("CQ_RECORD_ID","BUILD_LOG_TEXT");
	}
	elsif($ForState =~ /Complete/i) {
		push @RequiredInfo,("CQ_RECORD_ID","BUILD_LOG_TEXT");
	}
	elsif($ForState =~ /Retire/i) {
		push @RequiredInfo,("CQ_RECORD_ID");
	}
	foreach my $Required (@RequiredInfo) {
		unless($ENV{$Required}) {
			die "The required environment parameter [$Required] for operation [$ForState] was not set";
		}
	}
	
	# Create a connection to CQ.
	my $CQSession = CQPerlExt::CQSession_Build();
	my $CQ_Password = $ENV{'CQ_PASSWORD'} || "";	
	my $DB_Set = $ENV{'CQ_DBSET'} || "";
	Debug("Connecting To ClearQuest using the following Credentials:");
	Debug("USER:$ENV{'CQ_USER'}") if($ENV{'CQ_USER'});
	Debug("PASSWORD:$CQ_Password") if($CQ_Password && !$ENV{'HIDE_PASSWORD'});
	Debug("DBNAME:$ENV{'CQ_DBNAME'}") if($ENV{'CQ_DBNAME'});
	Debug("DBSET:$DB_Set") if($DB_Set);
	$CQSession->UserLogon($ENV{'CQ_USER'},$CQ_Password,$ENV{'CQ_DBNAME'},$DB_Set);
	return $CQSession;
} # ValidateInfo

# genTimeStamp
# 
# Generate a CQ-friendly timestamp
sub genTimeStamp {
	my @Time=localtime();
	my $Time = sprintf("%d/%d/%d %d:%02d:%02d %s",($Time[4]+1),$Time[3],($Time[5]+1900),($Time[2] > 12 ? ($Time[2]-12) : ($Time[2] == 0 ? 12 : $Time[2])),$Time[1],$Time[0],($Time[2] > 11 ? "P" : "A"));
	return $Time;
} # genTimeStamp

# Log
#
# Generic Logging
sub Log {
	my($String) = @_;
	print "LOG: $String\n";
} # Log

# Debug
#
# Generic Debugging
sub Debug {
	my($String) = @_;
	print "DEBUG: $String\n" if($ENV{'BFCQDEBUG'});
} # Debug 
	
# Error
#
# Application Exit/Failing
sub Error {
	my ($ErrShort,$ErrLong) = @_;
	print "ERROR: $ErrLong\n";
	print "$ErrShort\n";
	exit 1;
} # Error

# END
