#!/usr/bin/cqperl
#
# Copyright (c)2003-2006, BuildForge, Inc. All rights reserved.
# BuildForge, Inc. owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and BuildForge, Inc.
#
# bfcqresolve.pl
#
# Description:
#
# This application is an extension of the ClearQuest API,
# it is used to resolve clearquest defects.
#
#
# Created by: rfuller
# Modification Date: 10:09 AM 12/14/2005
#
# Usage:
# cqperl bfcqresolve.pl <DefectId> <resolution> <note>
#
# The following parameters should be set via the environment:
#
# CQ_DBNAME
#	The ClearQuest Database Instance Name - (Taken from the Defect if not set).
# CQ_DBSET
#	The Database set or connection string (default "")
# CQ_USER
#	The ClearQuest User
# CQ_PASSWORD
#	Corresponding user password (default "")
# HIDE_PASSWORD
#	Do not log the password
# BFCQDEBUG
#	Print out debug info

# Import Libraries
use CQPerlExt;
use strict;
use Encode;
use Encode::Guess qw(euc-jp shiftjis 7bit-jis);

# We don't want CQ warnings or errors reported in our output stream.
open(STDERR,"> nul");

# Scope of our application
{
	my $DefectId = shift;
	my $Resolution = shift;
	my $Note = shift || "$Resolution on ".scalar(localtime);
	my $Processed = 0;
	
	# Basic Validation.
	unless($DefectId && $Resolution) {
		Error("usage: cqperl bfcqresolve.pl <defect_id> <resolution> <note>","DefectId, and Resolution are required. We were passed Defect [$DefectId], Resolution [$Resolution].");
	}
	
	if(!exists($ENV{'CQ_DBNAME'})) {
		$DefectId=~/^(.*?)[0-9]*$/;
		$ENV{'CQ_DBNAME'}=$1;
	}
	
	my $CQSession;
	eval { $CQSession = ValidateInfo(); };
	Error("Could not connect to ClearQuest - Invalid Initialization Data",$@) if($@);
	
	eval { $Processed = ResolveDefect({'Session'=>$CQSession,'Defect'=>$DefectId,'Resolution'=>$Resolution,'Note'=>$Note}); };
	Error("Error Encountered while resolving defect.",$@) if($@);
	
	if($Processed) {
		my $DefectEnt = $CQSession->GetEntity("defect",$DefectId);
		my $EntState = $DefectEnt->GetFieldValue("state")->GetValue();
		$Resolution = $EntState if(length($EntState));
		print "Defect $DefectId moved to state [$Resolution]\n";
	}
	
	# Drop our session, don't care if we error here, as we're leaving.
	eval { CQSession::Unbuild($CQSession); };
	exit 0;	
} # Scope

sub ResolveDefect {
	my($Params) = @_;
	
	# Set the resolution...
	my $DefectEnt = undef;
	eval {$DefectEnt = $Params->{'Session'}->GetEntity("defect",$Params->{'Defect'});};
	if(ref($DefectEnt) eq "") {
		print "$Params->{'Defect'} does not reference a defect.\n";
		return 0;
	}
	my $EntState = $DefectEnt->GetFieldValue("state");
	
	$Params->{'Session'}->EditEntity($DefectEnt,"modify");
	$DefectEnt->SetFieldValue("Resolution",$Params->{'Resolution'});
	$DefectEnt->Validate();
	$DefectEnt->Commit();
	
	# Check our state transition, and resolve it if possible...
	Debug("Current State is ".$EntState->GetValue());

	my $EntDefObj = $Params->{'Session'}->GetEntityDef($DefectEnt->GetEntityDefName());
	my $DesiredState = $EntDefObj->GetActionDestStateName("resolve");
	if($DesiredState ne $EntState->GetValue()) {
		Debug("Next Action Results in: $DesiredState");
		my $Transitions = $EntDefObj->DoesTransitionExist($EntState->GetValue(),$DesiredState);
		if(scalar(@{$Transitions}) < 1) {
			Error("No Transition to State $DesiredState","Defect [$Params->{'Defect'}] current state is [".$EntState->GetValue()."] and cannot be directly transitioned to state [$DesiredState].");
		}
		$Params->{'Session'}->EditEntity($DefectEnt,"resolve");
	}
	else {
		$Params->{'Session'}->EditEntity($DefectEnt,"modify");
	}
	my $CharsCheck = $Params->{'Session'}->ValidateStringInCQDataCodePage($Params->{'Note'});
	if(length($CharsCheck)) {
		Error($CharsCheck,"Invalid Characters in resolution text for [$Params->{'Defect'}]...");
	}
	$DefectEnt->SetFieldValue("note_entry",$Params->{'Note'});
	$DefectEnt->Validate();
	$DefectEnt->Commit();
	return 1;
}

# ValidateInfo
#
# Make sure we have the correct info to connect, then connect.
sub ValidateInfo {
	my @RequiredInfo = ("CQ_DBNAME","CQ_USER");
	foreach my $Required (@RequiredInfo) {
		unless($ENV{$Required}) {
			die "The required environment parameter [$Required] was not set";
		}
	}
	
	# Create a connection to CQ.
	my $CQSession = CQPerlExt::CQSession_Build();
	my $CQ_Password = $ENV{'CQ_PASSWORD'} || "";	
	my $DB_Set = $ENV{'CQ_DBSET'} || "";
	Debug("Connecting To ClearQuest using the following Credentials:");
	Debug("USER:$ENV{'CQ_USER'}") if($ENV{'CQ_USER'});
	Debug("PASSWORD:$CQ_Password") if($CQ_Password && !$ENV{'HIDE_PASSWORD'});
	Debug("DBNAME:$ENV{'CQ_DBNAME'}") if($ENV{'CQ_DBNAME'});
	Debug("DBSET:$DB_Set") if($DB_Set);
	$CQSession->UserLogon($ENV{'CQ_USER'},$CQ_Password,$ENV{'CQ_DBNAME'},$DB_Set);
	return $CQSession;
} # ValidateInfo


# Log
#
# Generic Logging
sub Log {
	my($String) = @_;
	print "LOG: $String\n";
} # Log

# Debug
#
# Generic Debugging
sub Debug {
	my($String) = @_;
	print "DEBUG: $String\n" if($ENV{'BFCQDEBUG'});
} # Debug 
	
# Error
#
# Application Exit/Failing
sub Error {
	my ($ErrShort,$ErrLong) = @_;
	print "ERROR: $ErrLong\n";
	print "$ErrShort\n";
	exit 1;
} # Error
