
package BuildForge::Services;
use strict;

use BuildForge::Ex ();
use BuildForge::Services::Constants ();
use BuildForge::Services::DBO ();
use BuildForge::Services::Connection ();

our $VERSION = '7.0.2.7.0.0003';

my $tags = {
	':Constants' => 'BuildForge::Services::Constants',
};

sub import {
	my $class = shift;
	@_ = keys %$tags if !@_ || grep ($_ eq ':all', @_);
	foreach my $tag (@_) {
		my $pkg = $tags->{$tag};
		next unless length $pkg;
		$pkg->export_to_level(1);
	}
}

1;

__END__

=pod

=head1 NAME

BuildForge::Services

=head1 SYNOPSIS

	use BuildForge::Services;

	# Creating connections
	$conn = new BuildForge::Services::Connection();
	$conn = new BuildForge::Services::Connection($hostname, $port);

	# Authentication
	$token = $conn->authUser($user, $pass);
	$token = $conn->authUser($user, $pass, $domain);
	$conn->authToken($token);

	# Ending a session
	$conn->logout;
	$conn->close;

	# Simple commands for checking status, controlling locale settings,
	# or obtaining message translations.  These do not require the user
	# to have authenticated.
	$pongValue = $conn->ping($pingValue);
	$locale = $conn->getLocale;
	$conn->setLocale($locale);
	$tzone = $conn->getTimeZone;
	$conn->setTimeZone($tzone);
	$text = $conn->translate($msg);
	$text = $conn->translate($msg, $locale, $tzone);

	# Low-level calls that the end user should not normally need to use
	$conn->request($cmd);
	$conn->writeEntry($key, $value);
	$response = $conn->call;

=head1 DESCRIPTION

The services layer API uses Perl to connect to the IBM Rational Build Forge
services layer server.  This is an application server, written in Java, that
provides a calling interface for clients in various programming languages to
integrate with the Build Forge system.

Security is enforced by the server based on the user's current authentication
status.  While the simple commands such as B<ping> and B<setLocale> are
available without authenticating, most of the functionality in the services
layer is only available to users that have properly authenticated using
B<authUser> or B<authToken>.

Once the user has authenticated with the services layer server, the user can
make many of the requests that would normally be made through the UI.  For
example, users can create new collectors, assign them to servers, start builds,
and retrieve build results, provided that the user belongs to the appropriate
access groups for these activities.  The details of how to do this are in the
documentation for the individual BuildForge::Services::DBO::I<ObjectType>
classes.

=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

