
package BuildForge::Services::Connection;

use strict;
use bytes;

use Data::Dumper;
use Encode qw(encode_utf8 decode_utf8);
use IO::Socket;
use IO::Socket::INET;
use Socket;
use UNIVERSAL qw(can isa);

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils qw(bf_encrypt);
use BuildForge::Services::DBO;

use constant DEFAULT_HOST  => 'localhost';
use constant DEFAULT_PORT  => 'bflckmgr(3966)';
use constant DEBUG_AUDIT   => $ENV{'BFDEBUG_SVC_AUDIT'};
use constant DEBUG         => $ENV{'BFDEBUG_SVC_PROTO'};

use constant UTF16_LEAD_OFFSET => 0xD800 - (0x10000 >> 10);
use constant UTF16_SURROGATE_OFFSET => 0x10000 - (0xD800 << 10) - 0xDC00;

# Pre-pack'd LEN constants...
use constant L1_ZERO     => chr(0);
use constant L2_ZERO     => pack('n', 0);

use constant BUFFER_SIZE => 8192;

# For debugging
my %COMMAND_MAP = ();
if (DEBUG) {
	no strict 'refs';
	my $pkg = 'BuildForge::Services::Constants';
	foreach my $key (keys %{$pkg . '::'}) {
		if ($key =~ /^COMMAND_/) {
			$COMMAND_MAP{ $pkg->can($key)->() } = $key;
		}
	}
	use strict;
}

# Convenience functions
sub _corrupted () { return BuildForge::Ex::ProtocolException->new('APIProtoResponseMalformed'); }
sub _debug (@) { print STDERR @_, "\n"; }
sub _eof () { return BuildForge::Ex::IOException->new('Unexpected EOF'); }

# Global data for default connection
{
	my $default_connection = undef;
	sub getDefault { return $default_connection; }
	sub setDefault { $default_connection = $_[1]; }
}

sub _reconstructServiceException ($$) {
	my ($self, $resp) = @_;
	my $className = $resp->{KEY_ERROR()};
	my $list = $resp->{KEY_INFO()};
	if (!defined($className)) {
		warn "_reconstructServiceException: \$className=undef\n";
		return undef;
	}
	if (!defined($list)) {
		warn "_reconstructServiceException: \$list=undef\n";
		return undef;
	}
	if (!isa($list, 'ARRAY')) {
		warn "_reconstructServiceException: \$list=", ref($list) || '(scalar)', "\n";
		return undef;
	}
	if (!@$list) {
		warn "_reconstructServiceException: \@\$list=@$list\n";
		return undef;
	}
	my $ret = eval {
		my $last = pop(@$list);
		my $msg = BuildForge::Services::DBO::Message->new($self)->fromArray($last);
		$className =~ s#^.*\.##;
		$className = 'BuildForge::Ex::' . $className;
		return $className->new($msg);
	};
	if ($@) {
		warn "_reconstructServiceException: $@\n";
		return undef;
	}
	return $ret;
}



#------------------------------------------------------------
# Constructors
#------------------------------------------------------------

sub new {
	my ($proto, $host, $port) = @_;
	my $class = ref($proto) || $proto;
	$host = DEFAULT_HOST unless defined($host) && length($host);
	$port = DEFAULT_PORT unless defined($port) && length($port);
	my $sock = new IO::Socket::INET->new(
			'PeerAddr' => $host,
			'PeerPort' => $port,
			'Proto' => 'tcp'
	) or die "IO::Socket::INET: new: $!\n";
	my $self = {
		'host' => $host,
		'port' => $port,
		'sock' => $sock,
		'rbuf' => '',
		'wbuf' => '',
		'framed' => 0,
		'invalidated' => undef,
		'as' => UID_SYSTEM,
		'haveFirstEntry' => 0,
	};
	BuildForge::Services::Connection->setDefault($self);
	return bless $self, $class;
}

sub invalidate {
	$_[0]->{'invalidated'} =
		BuildForge::Ex::IOException->new('Invalidate called', $_[1]);
}
sub check {
	my ($self) = @_;
	
	if (defined($_[0]->{'invalidated'})) {
		die BuildForge::Ex::IOException->new(
				'Referenced invalidated connection',
				$_[0]->{'invalidated'});
	}
	if (!$self->{'sock'}) {
		die BuildForge::Ex::IOException->new(
				'Referenced unopened connection');
	}
}
sub request {
	my ($self, $cmd) = @_;
	$self->check();
	_writeHeader($self, int($cmd), $self->{'as'});
	return $self;
}
sub as { $_[0]->{'as'} = int($_[1]); }
sub call {
	my ($self) = @_;
	my ($resp, $cmd);
	eval {
		_writeFooter($self);
		my $info = undef;
		($resp, $cmd) = $self->readResponse();
		if ($cmd == COMMAND_RESPONSE) {
			my $list = $resp->{'*info'};
			if (defined($list) && isa($list, 'ARRAY')) {
				$info = [];
				foreach my $msg (@$list) {
					push @$info, BuildForge::Services::DBO::Message->new($self)->fromArray($msg);
				}
				$self->{'info'} = $info;
				if (DEBUG_AUDIT) {
					foreach my $msg (@$info) {
						print STDERR "[AUDIT]  ", $self->translate($msg), "\n";
					}
				}
			}
			return $resp;
		}
		$cmd != COMMAND_ERROR and die _corrupted();
		my $se = _reconstructServiceException($self, $resp);
		!defined($se) and die _corrupted();
		$se->fillInStackTrace();
		die $se;
	};
	$@ or return $resp;
	isa($@, 'BuildForge::Ex::ServiceException') or die $@;
	isa($@, 'BuildForge::Ex::ProtocolException') and $self->invalidate($@);
	$self->{'info'} = [$@->getMessageObject()];
	die $@;
}


sub _writeHeader ($$$) {
	my ($self, $cmd, $as) = @_;
	my $sock = $self->{'sock'};
	my $wbuf = \$self->{'wbuf'};
	if (DEBUG) {
		_debug(sprintf("[PROTO] W  _writeHeader(cmd=%x%s)\n",
					$cmd,
					$as != UID_SYSTEM ? ", as=$as" : ''));
	}
	if ($self->{'framed'}) {
		$$wbuf .= "},\n{ ";
	} else {
		$$wbuf .= "[ { ";
		$self->{'framed'} = 1;
	}
	$self->{'haveFirstEntry'} = 0;
	$self->writeEntryString('*cmd', sprintf('%x', $cmd));
	$as != UID_SYSTEM and $self->writeEntryInteger('*as', $as);
}

sub _writeFooter ($) {
	my ($self) = @_;
	DEBUG and _debug("[PROTO] W  _writeFooter");
	$self->{'framed'} or die _corrupted();
	my $wbuf = \$self->{'wbuf'};
	$$wbuf .= "\n} ]\n";
	syswrite $self->{'sock'}, $$wbuf;
	$$wbuf = '';
	$self->{'framed'} = 0;
}

sub _writeBoolean ($$$) { ${ $_[1] } .= $_[2] ? 'true' : 'false'; }
sub _writeInteger ($$$) { _writeNumber($_[0], $_[1], $_[2]); }
sub _writeNumber ($$$) {
	my ($self, $wbuf, $number) = @_;
	if ($number != $number) {
		$$wbuf .= '"\u004EaN"';
	} elsif ($number == 9E999999) {
		$$wbuf .= '9E999999';
	} elsif ($number == -9E999999) {
		$$wbuf .= '-9E999999';
	} else {
		$$wbuf .= $number;
	}
}
sub _writeKey ($$$) {
	my ($self, $wbuf, $key) = @_;
	$$wbuf = $$wbuf . '"' . $key . '"';
}

my %Escapes = (
		"\010" => "\\b",
		"\011" => "\\t",
		"\012" => "\\n",
		"\014" => "\\f",
		"\015" => "\\r",
		"\""   => "\\\"",
		"\\"   => "\\\\",
		"/"    => "\\/",
);


sub _unicode ($) {
	my $c = ord($_[0]);
	$c <= 0xFFFF and return sprintf("\\u%04X", $c);
	return sprintf("\\u%04X\\u%04X",
			UTF16_LEAD_OFFSET + ($c >> 10),
			0xDC00 + ($c & 0x03FF));
}
sub _writeString ($$$) {
	my ($self, $wbuf, $s) = @_;
	if (!defined($s)) {
		$$wbuf .= 'null';
		return;
	}
	$s =~ s#([^\x20\x21\x23-\x2E\x30-\x5B\x5D-\x7E])#$Escapes{$1} || _unicode($1)#ge;
	$$wbuf = $$wbuf . '"' . $s . '"';
}
sub _writeEntryKey ($$$) {
	my ($self, $wbuf, $key) = @_;
	if ($self->{'haveFirstEntry'}) {
		$$wbuf .= ",\015\012 ";
	} else {
		$self->{'haveFirstEntry'} = 1;
	}
	$$wbuf = $$wbuf . '"' . $key . '": ';
}

sub writeEntryNull {
	my ($self, $key) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	$$wbuf .= 'null';
}

sub writeEntryBoolean {
	my ($self, $key, $boolean) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	$$wbuf .= $boolean ? 'true' : 'false';
}

sub writeEntryInteger {
	my ($self, $key, $int) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	$$wbuf .= int($int);
}

sub writeEntryNumber {
	my ($self, $key, $number) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	_writeNumber($self, $wbuf, $number);
}

sub writeEntryString {
	my ($self, $key, $string) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	_writeString($self, $wbuf, $string);
}

sub writeEntryArray {
	my ($self, $key, $array) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	_writeArray($self, $wbuf, $array);
}

sub writeEntryHash {
	my ($self, $key, $hash) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	_writeHash($self, $wbuf, $hash);
}

sub writeEntryDBO {
	my ($self, $key, $dbo) = @_;
	my $wbuf = \$self->{'wbuf'};
	_writeEntryKey($self, $wbuf, $key);
	_writeArray($self, $wbuf, defined($dbo) ? $dbo->toArray() : undef);
}


#------------------------------------------------------------
# Warning!  This is SLOW!  If only Perl had polymorphism...
# It is significantly faster, provided you know the type
# of your data, to call the appropriate writeEntryXYZ
# method instead.  That is, prefer
#   $conn->writeEntryString($key, $string);
# to
#   $conn->writeEntry($key, $string);
#------------------------------------------------------------
# $conn->writeEntry($key, $string);
#------------------------------------------------------------
sub writeEntry {
	my ($self, $key, $value) = @_;
	my $wbuf = \$self->{'wbuf'};

	_writeEntryKey($self, $wbuf, $key);
	_writeValue($self, $wbuf, $value);

	# maybe flush...
	if (length($$wbuf) >= BUFFER_SIZE) {
		syswrite $self->{'sock'}, $$wbuf;
		$$wbuf = '';
	}
}

#------------------------------------------------------------
# Warning!  This is SLOW!  If only Perl had polymorphism...
# It is significantly faster, provided you know the type
# of your data, to call the appropriate _writeXYZ
# function instead.  That is, prefer
#   _writeString($conn, $string);
# to
#   _writeValue($conn, $string);
#------------------------------------------------------------
# First arg is the _writeXXX function to invoke
# Second arg is true if the value should be dereferenced first
#------------------------------------------------------------
my %VALUE_MAP = (
	'' => [ \&_writeString, 0 ],
	'SCALAR' => [ \&_writeString, 1 ],
	'ARRAY' => [ \&_writeArray, 0 ],
	'HASH' => [ \&_writeHash, 0 ],
	'BuildForge::Types::Boolean' => [ \&_writeBoolean, 1 ],
	'BuildForge::Types::Integer' => [ \&_writeInteger, 1 ],
	'BuildForge::Types::Number' => [ \&_writeNumber, 1 ],
);

sub _writeValue ($$$) {
	my ($self, $wbuf, $value) = @_;
	my $entry;

	if (!defined($value)) {
		$$wbuf .= 'null';
	} else {
		defined(tied $value) and $value = tied $value;
		my $ref = ref $value;
		if (defined( $entry = $VALUE_MAP{$ref} )) {
			$entry->[0]->($self, $wbuf, $entry->[1] ? $$value : $value);
		} elsif ($ref =~ /^BuildForge::Services::DBO::/) {
			_writeArray($self, $wbuf, $value->toArray());
		} else {
			DEBUG and _debug('[PROTO] W  ', $ref, ' ???');
			$$wbuf .= 'null';
		}
	}

	# maybe flush...
	if (length($$wbuf) >= BUFFER_SIZE) {
		syswrite $self->{'sock'}, $$wbuf;
		$$wbuf = '';
	}
}

sub _writeArray ($$$) {
	my ($self, $wbuf, $array) = @_;
	if (!defined($array) || !isa($array, 'ARRAY')) {
		$$wbuf .= 'null';
	} elsif (@$array == 0) {
		$$wbuf .= '[]';
	} else {
		$$wbuf .= '[ ';
		_writeValue($self, $wbuf, $array->[0]);
		for (my $i=1; $i < @$array; $i++) {
			$$wbuf .= ",\015\012 ";
			_writeValue($self, $wbuf, $array->[$i]);

			# maybe flush...
			if (length($$wbuf) >= BUFFER_SIZE) {
				syswrite $self->{'sock'}, $$wbuf;
				$$wbuf = '';
			}
		}
		$$wbuf .= ' ]';
	}
}

sub _writeHash ($$$) {
	my ($self, $wbuf, $hash) = @_;
	if (!defined($hash) || !isa($hash, 'HASH')) {
		$$wbuf .= 'null';
	} else {
		$$wbuf .= '{ ';
		$self->{'haveFirstEntry'} = 0;
		foreach my $key (keys %$hash) {
			_writeEntryKey($self, $wbuf, $key);
			_writeValue($self, $wbuf, $hash->{$key});

			# maybe flush...
			if (length($$wbuf) >= BUFFER_SIZE) {
				syswrite $self->{'sock'}, $$wbuf;
				$$wbuf = '';
			}
		}
		$$wbuf .= ' }';
		$self->{'haveFirstEntry'} = 1;
	}
}

#------------------------------------------------------------
# DECODER
#------------------------------------------------------------

sub readResponse {
	my ($self) = @_;
	my $sock = $self->{'sock'};
	my $rbuf = \$self->{'rbuf'};

	my $map = undef;
	my $obj = _readValue($self, $sock, $rbuf);
	if (isa($obj, 'ARRAY') && @$obj) {
		$obj = $obj->[@$obj - 1];
		isa($obj, 'HASH') and $map = $obj;
	}
	!defined($map) and die _corrupted();

	my $cmd = $map->{'*cmd'};
	$cmd =~ /^[0-9a-fA-F]{1,8}$/ or die _corrupted();
	$cmd = hex($cmd);
	delete $map->{'*cmd'};

	exists($map->{'*as'}) && $map->{'*as'} != UID_SYSTEM and die _corrupted();
	return ($map, $cmd);
}

my %Dispatch = (
	'[' => \&_readArray,
	'{' => \&_readHash,
	'"' => \&_readString,
	'n' => \&_readNull,
	't' => \&_readTrue,
	'f' => \&_readFalse,
);
sub _findToken ($$$) {
	my ($self, $sock, $rbuf) = @_;
	$$rbuf =~ s/^\s+//;
	while (!length($$rbuf)) {
		$$rbuf = <$sock>;
		if (!defined($$rbuf)) {
			$$rbuf = '';
			die _eof();
		}
		$$rbuf =~ s/^\s+//;
	} 
	my $c = substr($$rbuf, 0, 1);
	my $code = ord($c);
	$code < 0x20 || $code > 0x7E and die _corrupted();
	substr($$rbuf, 0, 1) = '';
	return $c;
}
sub _readValue ($$$) {
	my ($self, $sock, $rbuf) = @_;
	my $c = _findToken($self, $sock, $rbuf);
	my $fn = $Dispatch{$c};
	defined($fn) and return $fn->($self, $sock, $rbuf);
	$c eq '-' || ($c ge '0' && $c le '9') or die _corrupted();
	return _readNumber($self, $sock, $rbuf, $c);
}
sub _readNull ($$$) {
	my ($self, $sock, $rbuf) = @_;
	length($$rbuf) >= 3 && substr($$rbuf, 0, 3) eq 'ull' or die _corrupted();
	substr($$rbuf, 0, 3) = '';
	return undef;
}
sub _readTrue ($$$) {
	my ($self, $sock, $rbuf) = @_;
	length($$rbuf) >= 3 && substr($$rbuf, 0, 3) eq 'rue' or die _corrupted();
	substr($$rbuf, 0, 3) = '';
	return 1;
}
sub _readFalse ($$$) {
	my ($self, $sock, $rbuf) = @_;
	length($$rbuf) >= 4 && substr($$rbuf, 0, 4) eq 'alse' or die _corrupted();
	substr($$rbuf, 0, 4) = '';
	return 0;
}



sub _readNumber ($$$$) {
	my ($self, $sock, $rbuf, $c) = @_;
	substr($$rbuf, 0, 0) = $c;	# yuck
	$$rbuf =~ /^(-?)([0-9]+)/ or die _corrupted();
	length($2) == 1 || substr($2, 0, 1) ne '0' or die _corrupted();
	my $sb = $1 . $2;
	substr($$rbuf, 0, length($sb)) = '';
	if (substr($$rbuf, 0, 1) eq '.') {
		$$rbuf =~ /^(\.[0-9]+)/ or die _corrupted();
		$sb .= $1;
		substr($$rbuf, 0, length($1)) = '';
	}
	if (substr($$rbuf, 0, 1) =~ /^[Ee]$/) {
		$$rbuf =~ /^(.[-+]?[0-9]+)/ or die _corrupted();
		$sb .= $1;
		substr($$rbuf, 0, length($1)) = '';
	}
	return 0 + $sb;
}

my %Unescapes = (
	'b'  => "\010",
	't'  => "\011",
	'n'  => "\012",
	'f'  => "\014",
	'r'  => "\015",
	"\"" => "\"",
	"\\" => "\\",
	"/"  => "/",
);

# Lazy implementation, for now...
sub _readKey ($$$) { return _readString($_[0], $_[1], $_[2]); }

sub _readString ($$$) {
	my ($self, $sock, $rbuf) = @_;
	my $escapedFirst = 0;
	my $sb = '';
	my $c;
	my $lead = 0;
	while (1) {
		# There are no multi-line strings in JSON, so the
		# whole thing should be in the buffer, already.
		my $len;
		while ( ($len = length ($$rbuf)) or die _corrupted() ) {
			$$rbuf =~ /^(.*?)([\\"])(.*)$/;
			$sb .= $1;
			if ($2 eq '"') {
				if ($lead) {
					$sb .= "\x{fffd}";
					$lead = 0;
				}
				$$rbuf = $3;
				return ($escapedFirst && $sb eq 'NaN') ? (0 + $sb) : $sb;
			}
			length $3 or die _corrupted();
			my $c = substr($3, 0, 1);
			$$rbuf = substr($3, 1);
			my $value = $Unescapes{$c};
			if (defined($value)) {
				if ($lead) {
					$sb .= "\x{fffd}";
					$lead = 0;
				}
				$sb .= $value;
				next;
			}
			$c eq 'u' or die _corrupted();
			length ($$rbuf) > 4 or die _corrupted();
			$c = hex(substr($$rbuf, 0, 4));
			substr($$rbuf, 0, 4) = '';
			if ($lead) {
				if ($c >= 0xDC00 && $c <= 0xDFFF) {
					$sb .= chr(($lead << 10) + $c + UTF16_SURROGATE_OFFSET);
				} else {
					$sb .= "\x{fffd}";
				}
				$lead = 0;
			} elsif ($c >= 0xDC00 && $c <= 0xDFFF) {
				$sb .= "\x{fffd}";
				next;
			}
			if ($c >= 0xD800 && $c <= 0xDBFF) {
				$lead = $c;
				next;
			}
			length ($sb) or $escapedFirst = 1;
			$sb .= chr($c);
		}
	}
}
sub _readArray ($$$) {
	my ($self, $sock, $rbuf) = @_;
	my $c = _findToken($self, $sock, $rbuf);
	$c eq ']' and return [];
	my @list = ();
	substr($$rbuf,0,0) = $c;	# yuck
	push @list, _readValue($self, $sock, $rbuf);
	while ( ($c = _findToken($self, $sock, $rbuf)) ne ']' ) {
		if ($c ne ',') {
			print "***$c***\n";
			die _corrupted();
		}
		push @list, _readValue($self, $sock, $rbuf);
	}
	return \@list;
}
sub _readHash ($$$) {
	my ($self, $sock, $rbuf) = @_;
	my %map = ();
	my $c = _findToken($self, $sock, $rbuf);
	$c eq '}' and return \%map;
	while (1) {
		$c ne '"' and die _corrupted();
		my $key = _readString($self, $sock, $rbuf);
		$c = _findToken($self, $sock, $rbuf);
		$c ne ':' and die _corrupted();
		$map{$key} = _readValue($self, $sock, $rbuf);
		$c = _findToken($self, $sock, $rbuf);
		$c eq '}' and last;
		$c ne ',' and die _corrupted();
		$c = _findToken($self, $sock, $rbuf);
	}
	return \%map;
}

#------------------------------------------------------------
# High-level client implementation
#------------------------------------------------------------

sub info { return $_[0]->{'_info'}; }

# $result = $conn->close;
# returns the same value as the perl close function would
sub close {
	my ($self) = @_;
	my $wbuf = \$self->{'wbuf'};
	#_abort($self, $wbuf);
	if (length $$wbuf) {
		syswrite $self->{'sock'}, $$wbuf;
		$$wbuf = '';
	}
	close $self->{'sock'};
	$self->{'sock'} = undef;
}

# $token = $conn->authUser($connection, $pass);
# $token = $conn->authUser($connection, $pass, $domain);
sub authUser {
	my ($self, $connection, $pass, $domain) = @_;
	DEBUG and _debug("[PROTO]  Authenticating user $connection");
	$self->request(COMMAND_LOGIN);
	defined($domain) && length($domain)
		and $self->writeEntryString(KEY_LOGIN_DOMAIN, $domain);
	$self->writeEntryString(KEY_LOGIN_USERNAME, $connection);
	$self->writeEntryString(KEY_LOGIN_PASSWORD, bf_encrypt($pass));
	my $resp = $self->call();
	my $token = $resp->{KEY_LOGIN_TOKEN()};
	!defined($token) || !length($token)
		and die BuildForge::Ex::APIException->missing(KEY_LOGIN_TOKEN());
	DEBUG and _debug("[PROTO]  Login token: $token");
	return $token;
}

# $conn->authToken($token);
# returns $conn for convenience
sub authToken {
	my ($self, $token) = @_;
	DEBUG and _debug("[PROTO]  Authenticating token $token");
	$self->request(COMMAND_LOGIN);
	$self->writeEntryString(KEY_LOGIN_TOKEN, $token);
	$self->call;
	return $self;
}

# $conn->logout;
sub logout {
	my ($self) = @_;
	DEBUG and _debug("[PROTO]  Logging out");
	$self->request(COMMAND_LOGOUT);
	$self->call;
	return undef;
}

# $hello = $conn->ping('Hello, world.');
sub ping {
	my ($conn, $value) = @_;
	$conn->request(COMMAND_PING);
	$conn->writeEntry(KEY_SIMPLE_PING, $value);
	my $resp = $conn->call;
	return $resp->{KEY_SIMPLE_PING()};
}

# $locale = $conn->getLocale;
sub getLocale {
	my ($conn) = @_;
	$conn->request(COMMAND_LOCALE);
	my $resp = $conn->call;
	return $resp->{KEY_SIMPLE_LOCALE()};
}

# $conn->setLocale('en_US');
sub setLocale {
	my ($conn, $locale) = @_;
	$conn->request(COMMAND_LOCALE);
	$conn->writeEntryString(KEY_SIMPLE_LOCALE, $locale);
	$conn->call;
}

# $tzone = $conn->getTimeZone;
sub getTimeZone {
	my ($conn) = @_;
	$conn->request(COMMAND_TIMEZONE);
	my $resp = $conn->call;
	return $resp->{KEY_SIMPLE_TIMEZONE()};
}

# $conn->setTimeZone('America/Chicago');
sub setTimeZone {
	my ($conn, $tzone) = @_;
	$conn->request(COMMAND_TIMEZONE);
	$conn->writeEntryString(KEY_SIMPLE_TIMEZONE, $tzone);
	$conn->call;
}

# $conn->translate($msg);
# $conn->translate($msg, 'en_US', 'America/Chicago');
sub translate {
	my ($conn, $msg, $locale, $tzone) = @_;
	defined($msg) or return 'Message[(undef)]';
	isa($msg, 'BuildForge::Services::DBO::Message') or return 'Message[' . ref($msg) . ']';
	$conn->request(COMMAND_TRANSLATE);
	$conn->writeEntryDBO(KEY_SIMPLE_MESSAGE, $msg);
	$conn->writeEntryString(KEY_SIMPLE_LOCALE, $locale) if defined($locale) && length($locale);
	$conn->writeEntryString(KEY_SIMPLE_TIMEZONE, $tzone) if defined($tzone) && length($tzone);
	my $resp = $conn->call;
	my $text = $resp->{KEY_SIMPLE_MESSAGE()};
	return defined($text) ? $text : $msg->toString();
}

1;



__END__

=pod

=head1 NAME

BuildForge::Services::Connection


=head1 SYNOPSIS

	use BuildForge::Services;

	# Connecting and authentication
	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($connection, $pass);
	$conn = $conn->authToken($token);

	# Connection functions
	$ping = $conn->ping("ping!");
	$translation = $conn->translate($message);
	
	# Getter and setter functions
	$locale = $conn->getLocale();
	$timezone = $conn->getTimeZone();
	
	$conn->setLocale('en_US');
	$conn->setTimeZone('PDT');

	# Disconnection
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Connection allows creation and manipulation of a connection to a BuildForge Service 
Layer server.  Through this connection, you can then access and manipulate database
objects.


=head1 METHODS


=head2 new BuildForge::Services::Connection(hostname, portnumber)

Opens a connection to the services layer server and returns a BuildForge::Services::Connection
object through which to access that connection.

=over 8

=item hostname

The hostname on which the server resides.  Defaults to 'localhost'.

=item portnumber

The port number of the services layer.  Defaults to 3966, the standard services layer
port.

=back


=head2 $connection->authUser(user, password, domain)

Authenticates the connection with the given user login.  No BuildForge::Services::DBO objects
will allow any activity without the connection having been authenticated against.
Returns a token which may be used to log in with in the future for single sign on purposes.
This token becomes invalid after this session is over - when the connection has been logged out
of with $connection->logout() or closed with $connection->close().
No BuildForge::Services::DBO objects
will allow any activity without the connection having been authenticated against.

=over 8

=item user

The user login with which to authenticate.

=item password

The password with which to authenticate.

=item domain

If the user is an LDAP user, the domain ID (from $ldap->getDomain()) with which to 
authenticate as previously set up in the BuildForge database - see 
BuildForge::Services::DBO::LDAP for details.  If the user isn't an LDAP user, leave this
unset.

=back


=head2 $connection->authToken(token)

Authenticates against the given login token if the login token is from a currently 
active session.  Returns the connection object.
No BuildForge::Services::DBO objects
will allow any activity without the connection having been authenticated against.

=over 8

=item token

A login token as returned by $connection->authUser().

=back


=head2 $connection->ping(pingString)

Pings the services layer server and returns the ping response object
which contains, among other things, the sent ping string.

=over 8

=item pingString

The ping string to send.

=back


=head2 $connection->translate(msg, locale, tzone)

Contacts the services layer server and has it translate the translation key of the given message,
then returns the translated message. 
then returns the translated string.

=over 8

=item msg

The BuildForge::Services::DBO::Message object to be translated.

=item locale

The locale with which to translate the string.  If this is unset, uses the connection's locale
which defaults to 'en_US'.

=item tzone

The time zone used to translate time-related strings.  If this is unset, uses the connection's 
timezone.

=back


=head2 $connection->getLocale()

Returns the connection's current locale ID.


=head2 $connection->getTimeZone()

Returns the connection's current timezone ID.


=head2 $connection->setLocale(locale)

Sets the connection's locale for translation purposes.

=over 8

=item locale

The locale ID to use.

=back


=head2 $connection->setTimeZone(tzone)

Sets the connection's timezone for date/time translation purposes.

=over 8

=item tzone

The timezone ID to use.

=back


=head2 $connection->logout()

Logs the user out of the connection.  The connection requires reauthentication before
BuildForge::Services::DBO objects will allow any actions.


=head2 $connection->close()

Closes the connection.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

