
package BuildForge::Services::DBO::AccessGroup;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(level name def ldapDN controlLevel);
my $prototype = {
	level        => 0,
	name         => undef,
	def          => 0,
	ldapDN       => undef,
	controlLevel => 0,
};
sub getTypeKey { 'AccessGroup' }
sub toArray { return [ @{$_[0]}{@keys} ]; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	return $self;
	$self->{'_ldapDN'} = $self->{'ldapDN'};
	$self->{'ldapDN'} = undef;
	Integer(\$self->{'level'});
	Boolean(\$self->{'def'});
	Integer(\$self->{'controlLevel'});
	return $self;
}

sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_ACCESS_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findDefault {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_ACCESS_FIND_DEF);
	my $list = $conn->call()->{+KEY_ACCESS_GROUP_LIST};
	$list =~ s/[\{\}]//g;
	my @groups = split(/, */, $list);
	return \@groups;
}

sub findById {
	my ($proto, $conn, $level) = @_;
	$proto->checkConnection($conn);
	_checkLevel($level);
	$conn->request(COMMAND_ACCESS_FIND_ID);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	my $dbo = $conn->call()->{+KEY_ACCESS_GROUP_DBO};
	return defined($dbo) ? $proto->new($conn)->fromArray($dbo) : undef;
}

sub findByName {
	my ($proto, $conn, $name) = @_;
	$proto->checkConnection($conn);
	!defined($name) || !length($name) and die $proto->invalid('Name');
	$conn->request(COMMAND_ACCESS_FIND_NAME);
	$conn->writeEntryString(KEY_ACCESS_GROUP_NAME, $name);
	my $dbo = $conn->call()->{+KEY_ACCESS_GROUP_DBO};
	return defined($dbo) ? $proto->new($conn)->fromArray($dbo) : undef;
}

sub getPermissionList { ENUM_VALUES(ENUM_PERMISSION); }

sub getGroupPermissions {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	$conn->request(COMMAND_ACCESS_LIST_PERM);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);

	my $list = $conn->call()->{+KEY_PERMISSION_LIST};
	!defined($list) || ref($list) ne 'ARRAY' and die $self->missing(KEY_PERMISSION_LIST);
	return $list;
}

sub checkPermission {
	my ($self, $perm) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkPerm($perm);
	$conn->request(COMMAND_ACCESS_CHECK_PERM);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_PERMISSION_NAME, $perm);
	return $conn->call()->{+KEY_PERMISSION_EXISTS};
}

sub grantPermission {
	my ($self, $perm) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkPerm($perm);
	$conn->request(COMMAND_ACCESS_ADD_PERM);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_PERMISSION_NAME, $perm);
	$conn->call();
}

sub revokePermission {
	my ($self, $perm) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkPerm($perm);
	$conn->request(COMMAND_ACCESS_DEL_PERM);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_PERMISSION_NAME, $perm);
	$conn->call();
}

sub getMemberUsers {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	$conn->request(COMMAND_ACCESS_GET_USERS);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	my $resp = $conn->call();

	my $list = $resp->{+KEY_USER_LIST};
	!defined($list) || ref($list) ne 'ARRAY' and die $self->missing(KEY_USER_LIST);
	my @users = map { BuildForge::Services::DBO::User->new($conn)->fromArray($_) } @$list;
	return \@users;
}

sub addMemberUser {
	my ($self, $userId) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkUserId($userId);
	$conn->request(COMMAND_ACCESS_ADD_USER);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_ACCESS_USER_ID, $userId);
	$conn->call();
}

sub deleteMemberUser {
	my ($self, $userId) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkUserId($userId);
	$conn->request(COMMAND_ACCESS_DEL_USER);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_ACCESS_USER_ID, $userId);
	$conn->call();
}

sub getParentGroups {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	$conn->request(COMMAND_ACCESS_GET_GROUPS);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	my $resp = $conn->call();

	my $list = $resp->{+KEY_ACCESS_GROUP_LIST};
	!defined($list) || ref($list) ne 'ARRAY' and die $self->missing(KEY_ACCESS_GROUP_LIST);
	my @groups = map { BuildForge::Services::DBO::AccessGroup->new($conn)->fromArray($_) } @$list;
	return \@groups;
}

sub getChildGroups {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	$conn->request(COMMAND_ACCESS_FIND_CHILDREN);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	my $resp = $conn->call();

	my $list = $resp->{+KEY_ACCESS_GROUP_LIST};
	!defined($list) || ref($list) ne 'ARRAY' and die $self->missing(KEY_ACCESS_GROUP_LIST);
	my @groups = map { BuildForge::Services::DBO::AccessGroup->new($conn)->fromArray($_) } @$list;
	return \@groups;
}

sub addParentGroup {
	my ($self, $groupId) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkLevel($groupId);
	$conn->request(COMMAND_ACCESS_ADD_GROUP);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_ACCESS_GID, $groupId);
	$conn->call();
}

sub addChildGroup {
	my ($self, $groupId) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkLevel($groupId);
	$conn->request(COMMAND_ACCESS_ADD_GROUP);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $groupId);
	$conn->writeEntryString(KEY_ACCESS_GID, $level);
	$conn->call();
}

sub deleteParentGroup {
	my ($self, $groupId) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkLevel($groupId);
	$conn->request(COMMAND_ACCESS_DEL_GROUP);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_ACCESS_GID, $groupId);
	$conn->call();
}

sub deleteChildGroup {
	my ($self, $groupId) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	_checkLevel($level);
	_checkLevel($groupId);
	$conn->request(COMMAND_ACCESS_DEL_GROUP);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $groupId);
	$conn->writeEntryString(KEY_ACCESS_GID, $level);
	$conn->call();
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $level = $self->getLevel();
	defined($level) && $level != 0 and die $self->live(__PACKAGE__);
	$conn->request(COMMAND_ACCESS_ADD);
	$conn->writeEntryDBO(KEY_ACCESS_GROUP_DBO, $self);
	my $resp = $conn->call();
	my $dbo = $resp->{KEY_ACCESS_GROUP_DBO()};
	return defined($dbo) ? $self->fromArray($dbo) : undef;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	_checkLevel($self->{'level'});
	$conn->request(COMMAND_ACCESS_UPD);
	$conn->writeEntryDBO(KEY_ACCESS_GROUP_DBO, $self);
	my $resp = $conn->call();
	my $dbo = $resp->{KEY_ACCESS_GROUP_DBO()};
	return defined($dbo) ? $self->fromArray($dbo) : undef;
}

sub delete {
	my ($self) = @_;
	$self->deleteById($self->{'_conn'}, $self->{'level'});
}

sub deleteById {
	my ($proto, $conn, $level) = @_;
	_checkLevel($level);
	$conn->request(COMMAND_ACCESS_DEL);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->call();
}

sub _checkLevel($) { !defined($_[0]) || $_[0] <= 0 and die __PACKAGE__->invalid('ID'); }
sub _checkUserId($) { !defined($_[0]) || $_[0] <= 0 and die BuildForge::Services::DBO::User->invalid('ID'); }
sub _checkPerm($) { defined(ENUM_PERMISSION($_[0])) or die __PACKAGE__->badEnum('Permission', defined($_[0]) ? $_[0] : '(undef)'); }

sub _getGroup($$) {
	my ($conn, $resp) = @_;
	my $group = _readDBO($resp);
	return defined($group) ? $group : undef;
}

sub _readDBO($) {
	my ($resp) = @_;
	my $dbo = $resp->{+KEY_ACCESS_GROUP_DBO};
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_ACCESS_GROUP_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @groups = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@groups;
}

# wrap is inherited

sub getControlLevel  { $_[0]->{'controlLevel'}; }
sub getDefault       { $_[0]->{'def'}; }
sub getLdapDN        { $_[0]->{'ldapDN'}; }
sub getLevel         { $_[0]->{'level'}; }
sub getName          { $_[0]->{'name'}; }

sub setControlLevel  { $_[0]->{'controlLevel'} = $_[1]; }
sub setDefault       { $_[0]->{'def'}          = $_[1]; }
sub setLdapDN        { $_[0]->{'ldapDN'}       = $_[0]->{'_ldapDN'} = $_[1]; }
sub setName          { $_[0]->{'name'}         = $_[1]; }

1;

__END__

=pod

=head1 NAME

BuildForge::Services::DBO::AccessGroup


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing access groups
	$allGroups = BuildForge::Services::DBO::AccessGroup->findAll($conn);
	$defaultGroups = BuildForge::Services::DBO::AccessGroup->findDefault($conn);
	$group1 = BuildForge::Services::DBO::AccessGroup->findById($conn, 6);
	$group2 = BuildForge::Services::DBO::AccessGroup->findByName($conn, 'MyAccessGroup');

	# Getter / setter functions
	$id = $group1->getLevel();
	$name = $group1->getName();
	$controlid = $group1->getControlLevel();
	$isdefault = $group1->getDefault();
	$ldapdn = $group1->getLdapDN();
	
	$group1->setName('New Group Name');
	$group1->setDefault(1);
	$group1->setLdapDN('*');
	$group1->setControlLevel(6);
	$group1->update();
	
	# Access group creation, updating, deletion
	$newgroup = new BuildForge::Services::DBO::AccessGroup($conn);
	$newgroup->setName('New Group');
	$newgroup->setControlLevel(4);
	$newgroup->create();

	$newgroup->setName('New Name For Existing Group');
	$newgroup->update();

	$newgroup->delete();
	BuildForge::Services::DBO::AccessGroup->deleteById($conn, $group2->getLevel());
	
	# Changing group permissions
	$permissionlist = BuildForge::Services::DBO::AccessGroup->getPermissionList();
	
	$groupPermissions = $group1->getGroupPermissions();
	$doesGroupHavePermission = $group1->checkPermission('permissionKey');
	$group1->grantPermission('AddProject');
	$group1->revokePermission('AddProject');
	
	# Group members
	$user = BuildForge::Services::DBO::User->findByLogin($conn, 'login');
	$group1->addMemberUser($user->getUserId());
	$members = $group1->getMemberUsers();
	$group1->deleteMemberUser($user->getUserId());
	
	# Child groups
	$group2 = BuildForge::Services::DBO::AccessGroup->findByName($conn, 'MyAccessGroup');
	$group1->addInheritedGroup($group2->getLevel());
	$childgroups = $group1->getInheritedGroups();
	$group1->deleteInheritedGroup($group2->getLevel());
	
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

AccessGroup is used to create and modify BuildForge access groups for use
in email notification, permission control, and object access rights.


=head1 METHODS

=head2 new BuildForge::Services::DBO::AccessGroup(connection, {AccessParamsHashRef})

Creates an AccessGroup instance.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item AccessParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item name

The name of the access group

=item def

Set this 1 to make this a default access group that a new user automatically joins, 
0 to make it non-default.

=item ldapDN

An LDAP filter for identifying which distinguished names are granted this access group 
by default. 

=item controlLevel

The level (access group id) of the access group that "controls" this access group. In 
order to change this group's properties, such as its name or ldapDN settings, the user 
must be a member of the control group. The default groups use the Security group (level 5) 
for this purpose.

=back

=back


=head2 BuildForge::Services::DBO::AccessGroup->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::AccessGroup objects, one
for each entry in the database that the current user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::AccessGroup->findDefault(connection)

Returns a reference to an array of access group IDs (from $accessGroup->getLevel())
for all default access groups.  A default access group is one that a newly created
user automatically belongs to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::AccessGroup->findById(connection, id)

Returns the BuildForge::Services::DBO::AccessGroup object that corresponds to the 
given ID, or undef if no such AccessGroup exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item id

The ID of the desired AccessGroup (as gotten from getLevel).

=back


=head2 BuildForge::Services::DBO::AccessGroup->findByName(connection, name)

Returns the BuildForge::Services::DBO::AccessGroup object that corresponds to the 
given name, or undef if no such AccessGroup exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item id

The name of the desired AccessGroup (as gotten from getName).

=back


=head2 $accessGroup->create()

Creates an entry for the access group in the database.


=head2 $accessGroup->update()

Updates the access group in the database to correspond to the current
state of the accessGroup object.


=head2 $accessGroup->delete()

Deletes the access group from the database.


=head2 BuildForge::Services::DBO::AccessGroup->deleteById(connection, accessGroupId)

Deletes the access group with the given ID from the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item accessGroupId

The ID of the access group to be deleted (as gotten from AccessGroup::getLevel()).

=back


=head2 BuildForge::Services::DBO::AccessGroup->getPermissionList()

Returns a reference to an array of all permission keys.


=head2 $accessgroup->getGroupPermissions()

Returns a reference to an array of the keys to all the permissions that the group has.


=head2 $accessgroup->checkPermission(permissionKey)

Returns 1 if the group has the specified permission and 0 if it doesn't.

=over 8

=item permissionKey

The key for the permission (from BuildForge::Services::DBO::AccessGroup->getPermissionList()).

=back


=head2 $accessGroup->grantPermission(permissionKey)

Grants the requested permission to the access group.

=over 8

=item permissionKey

The key for the given permission (from BuildForge::Services::DBO::AccessGroup->getPermissionList()).

=back


=head2 $accessGroup->revokePermission(permissionKey)

Removes the requested permission from the access group.

=over 8

=item permissionKey

The key for the given permission (from BuildForge::Services::DBO::AccessGroup->getPermissionList()).

=back


=head2 $accessGroup->getMemberUsers()

Returns a reference to an array of the BuildForge::Services::DBO::User objects
that correspond to the members of the access group.


=head2 $accessGroup->addMemberUser(userId)

Adds the user corresponding to the user ID to the access group.

=over 8

=item userId

The user ID of the desired user (from BuildForge::Services::DBO::User->getUserId()).

=back


=head2 $accessGroup->deleteMemberUser(userId)

Removes the user corresponding to the user ID from the access group.

=over 8

=item userId

The user ID of the desired user (from BuildForge::Services::DBO::User->getUserId()).

=back


=head2 $accessGroup->getParentGroups()

Returns a reference to an array of the BuildForge::Services::DBO::AccessGroup objects
that correspond to the parents of the access group.  A child access group shares all 
of the parent access group's permissions (and all of its parents, etc.).


=head2 $accessGroup->getChildGroups()

Returns a reference to an array of the BuildForge::Services::DBO::AccessGroup objects
that correspond to the children of the access group.  A child access group shares all 
of the parent access group's permissions (and all of its parents, etc.).


=head2 $accessGroup->addParentGroup(accessGroupId)

Makes the access group corresponding to the given ID into a parent of the 
current access group.  This group will have all the permissions of the parent
group(s).

=over 8

=item accessGroupId

The access group ID of the access group to be made a parent (from AccessGroup->getLevel()).

=back


=head2 $accessGroup->addChildGroup(accessGroupId)

Makes the access group corresponding to the given ID into a child of the 
current access group.  The child group will gain all this access group's permissions.

=over 8

=item accessGroupId

The access group ID of the access group to be made a child (from AccessGroup->getLevel()).

=back


=head2 $accessGroup->deleteParentGroup(accessGroupId)

Stops the access group corresponding to the given ID from being a parent of the 
current access group.

=over 8

=item accessGroupId

The access group ID of the access group to be removed from being a parent (from 
AccessGroup->getLevel()).

=back


=head2 $accessGroup->deleteChildGroup(accessGroupId)

Stops the access group corresponding to the given ID from being a child of the 
current access group.

=over 8

=item accessGroupId

The access group ID of the access group to be removed from being a child (from 
AccessGroup->getLevel()).

=back


=head2 $accessGroup->getLevel()

Returns the ID of the access group.


=head2 $accessGroup->getName()

Returns the name of the access group.


=head2 $accessGroup->getControlLevel()

Returns the ID of the access group that controls the given one.  In order to change this 
group's properties, such as its name or ldapDN settings, the user must be a member of 
the control group. The default groups use the Security group (level 5) for this purpose.


=head2 $accessGroup->getDefault()

Returns 1 if the access group is a default access group (that is, if new users belong
to the access group automatically), and 0 otherwise.


=head2 $accessGroup->getLdapDN()

Returns the LDAP filter for identifying which distinguished names are granted this access 
group by default. 


=head2 $accessGroup->setName(name)

Sets the name of the access group.  $accessGroup->update() must be run before changes are 
replicated in the database.

=over 8

=item name

The new name for the group.

=back


=head2 $accessGroup->setControlLevel(accessGroupId)

Sets the controlling access group.  In order to change this group's properties, such as 
its name or ldapDN settings, the user must be a member of the control group. The default 
groups use the Security group (level 5) for this purpose.  $accessGroup->update() must be 
run before changes are replicated in the database.

=over 8

=item accessGroupId

The ID of the controlling group (as gotten from AccessGroup->getLevel()).

=back


=head2 $accessGroup->setDefault(isDefault)

Sets whether or not this group is a default group.  If it is a default group, new users
get added to the group automatically when the users are created.  $accessGroup->update() 
must be run before changes are replicated in the database.

=over 8

=item isDefault

If this evaluates to true, becomes a default group.  If false, not a default group.

=back


=head2 $accessGroup->setLdapDN(ldapDN)

Sets the LDAP filter for identifying which distinguished names are granted this access 
group by default.   $accessGroup->update() must be run before changes are replicated 
in the database.

=over 8

=item ldapDN

The new LDAP distinguished name filter.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut


