
package BuildForge::Services::DBO::Adaptor;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(liveInterfaceId interfaceId type dataText template lineId level);
my $prototype = {
	'liveInterfaceId' => undef,
	'interfaceId' => undef,
	'type' => ENUM_INTERFACE_TYPE->SOURCE,
	'dataText' => '',
	'template' => '',
	'lineId' => 1,
	'level' => 0,
};
sub getTypeKey { 'Interface' }
sub fromArray { 
	my $ad = $_[0]->fromArrayImpl($_[1], \@keys); 
	return $ad;
}
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Enum(ENUM_INTERFACE_TYPE, \$self->{'type'});
	Integer(\$self->{'lineId'});
	Integer(\$self->{'level'});
	$self->{'interfaceId'} = $self->{'liveInterfaceId'};
	return $self;
}


sub findById {
	my ($proto, $conn, $interfaceId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_ADAPTOR_FIND_NAME);
	$conn->writeEntryString(KEY_INTERFACE_ID, $interfaceId);
	my $dbo = $conn->call()->{+KEY_INTERFACE_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_ADAPTOR_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub listTemplateIds {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_ADAPTOR_LIST_TEMPLATES);
	return $conn->call()->{+KEY_INTERFACE_TEMPLATE_ID_LIST};
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_ADAPTOR_ADD);
	$conn->writeEntryDBO(KEY_INTERFACE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_INTERFACE_DBO};
	defined($dbo) or die $self->missing(KEY_INTERFACE_DBO);
	$self->fromArray($dbo);
	return $self;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_ADAPTOR_UPD);
	$conn->writeEntryDBO(KEY_INTERFACE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_INTERFACE_DBO};
	defined($dbo) or die $self->missing(KEY_INTERFACE_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	$self->deleteById($self->{'_conn'}, $self->{'liveInterfaceId'});
	$self->{'interfaceId'} = $self->{'liveInterfaceId'};
	$self->{'liveInterfaceId'} = undef;
}

sub deleteById {
	my ($proto, $conn, $interfaceId) = @_;
	$proto->checkConnection($conn);
	!defined($interfaceId) || !length($interfaceId) and die $proto->invalid('ID');
	$conn->request(COMMAND_ADAPTOR_DEL);
	$conn->writeEntryString(KEY_INTERFACE_ID, $interfaceId);
	$conn->call();
}


# wrap is inherited

sub getDataText     { $_[0]->{'dataText'}; }
sub getInterfaceId  { defined($_[0]->{'interfaceId'}) ? $_[0]->{'interfaceId'} : $_[0]->{'liveInterfaceId'}; }
sub getLevel        { $_[0]->{'level'}; }
sub getTemplate     { $_[0]->{'template'}; }
sub getType         { $_[0]->{'type'}; }

sub setDataText     { $_[0]->{'dataText'}    = $_[1]; }
sub setInterfaceId  { $_[0]->{'interfaceId'} = $_[1]; }
sub setLevel        { $_[0]->{'level'}       = $_[1]; }
sub setTemplate     { $_[0]->{'template'}    = $_[1]; }
sub setType         { $_[0]->{'type'}        = $_[1]; }

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_INTERFACE_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @adaptors = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@adaptors;
}


# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Adaptor


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing adaptors
	$allAdaptors = BuildForge::Services::DBO::Adaptor->findAll($conn);
	$allTemplateNames = BuildForge::Services::DBO::Adaptor->listTemplateIds($conn);
	$oldadaptor = BuildForge::Services::DBO::Adaptor->findById($conn, 'AdaptorName');

	# Getter / setter functions
	$id = $oldadaptor->getInterfaceId();
	$adaptorlevel = $oldadaptor->getLevel();
	$templatename = $oldadaptor->getTemplate();
	$type = $oldadaptor->getType();
	$adaptorxml = $oldadaptor->getDataText();
	
	$oldadaptor->setInterfaceId('NewAdaptorName');
	$oldadaptor->setLevel(4);
	$oldadaptor->setTemplate('ClearQuest');
	$oldadaptor->setType('DEFECT');
	$oldadaptor->setDataText($newadaptorxml);
	$oldadaptor->update();
	
	# Adaptor creation, updating, deletion
	$newadaptor = new BuildForge::Services::DBO::Adaptor($conn);
	$newadaptor->setInterfaceId('New Adaptor');
	$newadaptor->setType('SOURCE');
	$newadaptor->setTemplate('ClearCase');
	$newadaptor->create();

	$newadaptor->setInterfaceId('New Name For Existing Adaptor');
	$newadaptor->update();

	$newadaptor->delete();
	BuildForge::Services::DBO::Adaptor->deleteById($conn, $oldadaptor->getInterfaceId());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Adaptor is used to create new BuildForge Adaptors and to access existing adaptor information.


=head1 METHODS


=head2 new BuildForge::Services::DBO::Adaptor(connection, {AdaptorParamsHashRef})

Returns an Adaptor instance.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item AdaptorParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item liveInterfaceId

The name/ID of the adaptor.

=item type

The type of the adaptor.  This may be 'SOURCE', 'DEFECT', 'PACKAGE', or 'TEST'.
If not set, the default is 'SOURCE'.

=item dataText

The actual adaptor XML.

=item level

The level (access group id) of the access group that this Adaptor belongs to.  You must be a
member of this access group to view, edit, or use this Adaptor.

=back

=back


=head2 BuildForge::Services::DBO::Adaptor->findById(connection, adaptorId)

Returns the Adaptor object that corresponds to the given adaptorId, or undef if
no such adaptor exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item adaptorId

The desired adaptor ID.

=back


=head2 BuildForge::Services::DBO::Adaptor->findAll(connection)

Returns a reference to an array of all Adaptors in the database that the user
has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 $adaptor->create()

Creates an entry for the adaptor in the database.


=head2 $adaptor->update()

Updates the adaptor in the database to correspond to the current
state of the Adaptor object.


=head2 $adaptor->delete()

Deletes the adaptor from the database.


=head2 BuildForge::Services::DBO::Adaptor->deleteById(connection, adaptorId)

Deletes the adaptor with the given ID from the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item adaptorId

The adaptor ID to delete (as given by Adaptor->getAdaptorId()).

=back


=head2 BuildForge::Services::DBO::Adaptor->listTemplateIds(connection)

Returns a reference to an array containing the names of all possible template
IDs.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 $adaptor->getInterfaceId()

Returns the adaptor ID / name.


=head2 $adaptor->getDataText()

Returns the adaptor XML.


=head2 $adaptor->getLevel()

Returns the adaptor's access group ID.


=head2 $adaptor->getTemplate()

Returns the adaptor's template name.


=head2 $adaptor->getType()

Returns the adaptor's type (which may be 'SOURCE', 'DEFECT', 'PACKAGE', or 'TEST').


=head2 $adaptor->setInterfaceId(adaptorId)

Sets the adaptor's ID / name.  $adaptor->update() must be run before changes are 
replicated in the database.

=over 8

=item adaptorId

The new name for the adaptor.

=back


=head2 $adaptor->setDataText(xml)

Sets the adaptor's XML.  $adaptor->update() must be run before changes are 
replicated in the database.

=over 8

=item xml

The new adaptor XML.

=back


=head2 $adaptor->setLevel(accessGroupId)

Sets the new access group to which the adaptor belongs.  The access group controls
who can view, edit, or use the adaptor.  $adaptor->update() must be run before changes are 
replicated in the database.

=over 8

=item accessGroupID

The ID of the new access group (as gotten from AccessGroup->getLevel()).

=back


=head2 $adaptor->setTemplate(templateName)

Sets the new template that this adaptor is listed as having.  This should come from
the list gotten from BuildForge::Services::DBO::Adaptor->listTemplateIds().
$adaptor->update() must be run before changes are replicated in the database.

=over 8

=item templateName

The name of the template to use.

=back


=head2 $adaptor->setType(type)

Sets the type of the adaptor.  $adaptor->update() must be run before changes are 
replicated in the database.

=over 8

=item type

The desired adaptor type.  The type may be 'SOURCE', 'DEFECT', 'PACKAGE', or 'TEST'.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut



