
package BuildForge::Services::DBO::AdaptorLink;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(scopeId projectId adaptorName active stamp
	status jobId name environmentGroupId);
my $prototype = {
	'scopeId' => 0,
	'projectId' => 0,
	'adaptorName' => '',
	'active' => ENUM_SCOPE_ACTIVE->YES,
	'stamp' => 0,
	'status' => 'Idle',
	'jobId' => 0,
	'name' => '',
	'environmentGroupId' => 0,
};
sub getTypeKey { 'Link' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'scopeId'});
	Integer(\$self->{'projectId'});
	Enum(ENUM_SCOPE_ACTIVE, \$self->{'active'});
	Integer(\$self->{'stamp'});
	Integer(\$self->{'jobId'});
	Integer(\$self->{'environmentGroupId'});
	return $self;
}

sub _getList ($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_SCOPE_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @scopes = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@scopes;
}

sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LINK_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findByProjectId {
	my ($proto, $conn, $projectId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LINK_FIND_PID);
	$conn->writeEntryString(KEY_PROJECT_ID, $projectId);
	return _getList($conn, $conn->call());
}

sub findByAdaptorName {
	my ($proto, $conn, $adaptorName) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LINK_FIND_ADAPTOR);
	$conn->writeEntryString(KEY_INTERFACE_ID, $adaptorName);
	return _getList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $scopeId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LINK_FIND_ID);
	$conn->writeEntryString(KEY_SCOPE_ID, $scopeId);
	my $dbo = $conn->call()->{+KEY_SCOPE_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByProjectIdAndAdaptorName {
	my ($proto, $conn, $projectId, $adaptorName) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LINK_FIND_PIDNAME);
	$conn->writeEntryString(KEY_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_INTERFACE_ID, $adaptorName);
	my $dbo = $conn->call()->{+KEY_SCOPE_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_LINK_CREATE);
	$conn->writeEntryDBO(KEY_SCOPE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_SCOPE_DBO};
	return defined($dbo) ? $self->fromArray($dbo) : undef;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_LINK_UPDATE);
	$conn->writeEntryDBO(KEY_SCOPE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_SCOPE_DBO};
	return defined($dbo) ? $self->fromArray($dbo) : undef;
}


sub delete {
	my ($self) = @_;
	$self->deleteById($self->{'_conn'}, $self->{'scopeId'});
}

sub deleteById {
	my ($proto, $conn, $scopeId) = @_;
	!defined($scopeId) || !$scopeId and die $proto->invalid('ID');
	$conn->request(COMMAND_LINK_DELETE);
	$conn->writeEntryString(KEY_SCOPE_ID, $scopeId);
	$conn->call();
}

sub deactivate { $_[0]->activate( ENUM_SCOPE_ACTIVE->NO ); }
sub debug { $_[0]->activate( ENUM_SCOPE_ACTIVE->DEBUG ); }
sub activate {
	my ($self, $active) = @_;
	$active = defined($active) ? ENUM_SCOPE_ACTIVE($active) : ENUM_SCOPE_ACTIVE->YES;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_LINK_ACTIVE);
	$conn->writeEntryString(KEY_SCOPE_ID, $self->{'scopeId'});
	$conn->writeEntryString(KEY_SCOPE_ACTIVE, $active);
	$conn->call();
	$self->{'active'} = $active;
}

sub getActive             { $_[0]->{'active'}; }
sub getAdaptorName        { $_[0]->{'adaptorName'}; }
sub getEnvironmentGroupId { $_[0]->{'environmentGroupId'}; }
sub getProjectId          { $_[0]->{'projectId'}; }
sub getScopeId            { $_[0]->{'scopeId'}; }

sub setActive             { $_[0]->{'active'}             = $_[1]; }
sub setAdaptorName        { $_[0]->{'adaptorName'}        = $_[1]; }
sub setEnvironmentGroupId { $_[0]->{'environmentGroupId'} = $_[1]; }
sub setProjectId          { $_[0]->{'projectId'}          = $_[1]; }

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::AdaptorLink


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing adaptor links
	$project = new BuildForge::Services::DBO::Project($conn);
	$project->setName('Test Project');
	$project->create();
	
	$allLinks = BuildForge::Services::DBO::AdaptorLink->findAll($conn);
	$allLinksWithProject = BuildForge::Services::DBO::AdaptorLink->findByProjectId($conn, $project->getProjectId());
	$allLinksWithAdaptor = BuildForge::Services::DBO::AdaptorLink->findByAdaptorName($conn);
	$link1 = BuildForge::Services::DBO::AdaptorLink->findById($conn, 1);
	$link2 = BuildForge::Services::DBO::AdaptorLink->findByProjectIdAndAdaptorName($conn, $project->getProjectId(), 'AdaptorName');

	# Getter / setter functions
	$id = $link1->getScopeId();
	$adaptorname = $link1->getAdaptorName();
	$projectid = $link1->getProjectId();
	$isactive = $link1->getActive();
	$environmentid = $link1->getEnvironmentGroupId();

	$link1->setAdaptorName('DifferentAdaptor');
	$link1->setProjectId($project->getProjectId());
	$link1->setActive('YES');
	$environment = BuildForge::Services::DBO::Environment->findByName($conn, 'MyEnvName');
	$link1->setEnvironmentGroupId($environment->getEnvGroupId());
	$link1->update();
	
	# Adaptor link creation, updating, deletion
	$newadaptorlink = new BuildForge::Services::DBO::AdaptorLink($conn);
	$newadaptorlink->setAdaptorName('MyAdaptor');
	$newadaptorlink->setProjectId($projectid);
	$newadaptorlink->setActive('DEBUG');
	$newadaptorlink->create();

	$newadaptorlink->setEnvironmentGroupId($environment->getEnvGroupId());
	$newadaptorlink->update();
	$newadaptorlink->deactivate();
	$newadaptorlink->debug();
	$newadaptorlink->activate();
	
	$newadaptorlink->delete();
	BuildForge::Services::DBO::AdaptorLink->deleteById($conn, $link1->getInterfaceId());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

AdaptorLink is used to create new BuildForge Adaptor Links and to access existing adaptor link
information.


=head1 METHODS


=head2 BuildForge::Services::DBO::AdaptorLink->new(connection, {AdaptorLinkHashRef})

Creates an AdaptorLink instance.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item AccessParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item projectId

The ID for the project that the adaptor should be linked to (from Project->getProjectId()).

=item adaptorName

The ID for the adaptor that the project should be linked to (from Adaptor->getInterfaceId()).

=item active

Set to 'YES' for an active link, 'NO' for an inactive link, or 'DEBUG' to put the 
adaptor link in debug mode.  The default, if not set, is 'YES'.

=item environmentGroupId

The ID for the environment the adaptor link should use (from Environment->getEnvGroupId()).
Set it to 0 (the default) to have no environment group used.

=back

=back


=head2 BuildForge::Services::DBO::AdaptorLink->findAll(connection)

Returns a reference to an array of AdaptorLink objects, one for each entry in the 
database that the current user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::AdaptorLink->findByProjectId(connection, projectId)

Returns a reference to an array of AdaptorLink objects, one for each entry in the 
database that the current user has access to and that connect to the given project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID that the desired adaptor links connect to (from Project->getProjectId()).

=back


=head2 BuildForge::Services::DBO::AdaptorLink->findByAdaptorName(connection, adaptorName)

Returns a reference to an array of AdaptorLink objects, one for each entry in the 
database that the current user has access to and that use the given adaptor.

=over 8

=item connection

A connected BuildForge::Services::Connection object.=item adaptorName

The adaptor ID that the desired adaptor links connect to (from Adaptor->getInterfaceId()).

=back


=head2 BuildForge::Services::DBO::AdaptorLink->findByProjectIdAndAdaptorName(connection, projectId, adaptorName)

Returns the AdaptorLink object that connects the given project to the given adaptor,
or undef if no such AdaptorLink exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID that the desired adaptor link connects to (from Project->getProjectId()).

=item adaptorName

The adaptor ID that the desired adaptor link connects to (from Adaptor->getInterfaceId()).

=back


=head2 BuildForge::Services::DBO::AdaptorLink->findById(connection, scopeId)

Returns the AdaptorLink object with the given ID, or undef if no such AdaptorLink exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item scopeId

The ID of the desired adaptor link (from AdaptorLink->getScopeId()).

=back


=head2 $adaptorlink->create()

Creates the adaptor link within the database.


=head2 $adaptorlink->update()

Updates the adaptor link within the database with the current values in the object.


=head2 $adaptorlink->delete()

Deletes the adaptor link from the database.


=head2 BuildForge::Services::DBO::AdaptorLink->deleteById(connection, scopeId)

Deletes the adaptor link from the database which has the given scopeId.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item scopeId

The ID of the adaptor link to delete (from AdaptorLink->getScopeId()).

=back


=head2 $adaptorlink->deactivate()

Deactivates the adaptor link in the database.  Unlike $adaptorlink->setActive('NO'), 
this actually performs the action without the need for an $adaptorlink->update() call.


=head2 $adaptorlink->debug()

Sets the adaptor link in the database to debug mode.  Unlike $adaptorlink->setActive('DEBUG'), 
this actually performs the action without the need for an $adaptorlink->update() call.


=head2 $adaptorlink->activate()

Activates the adaptor link in the database.  Unlike $adaptorlink->setActive('YES'), 
this actually performs the action without the need for an $adaptorlink->update() call.


=head2 $adaptorlink->getScopeId()

Returns the adaptor link ID.


=head2 $adaptorlink->getAdaptorName()

Returns the name / ID of the adaptor the adaptor link is linked to.


=head2 $adaptorlink->getProjectId()

Returns the ID of the project the adaptor link is linked to.


=head2 $adaptorlink->getActive()

Returns 'YES' if the adaptor is active, 'NO' if the adaptor is inactive, and 'DEBUG' if
the adaptor is in debug mode.


=head2 $adaptorlink->getEnvironmentGroupId()

Returns the ID of the environment group the adaptor link is linked to, or 0 if it isn't
linked to an environment group.


=head2 $adaptorlink->setAdaptorName(adaptorId)

Sets the adaptor the adaptor link is linked to.  $adaptorlink->update() must be run before 
changes are replicated in the database.

=over 8

=item adaptorId

The ID of the adaptor, gotten from $adaptor->getInterfaceId().

=back


=head2 $adaptorlink->setProjectId(projectId)

Sets the project that the adaptor link is linked to.  $adaptorlink->update() must be run 
before changes are replicated in the database.

=over 8

=item projectId

The ID of the project, from $project->getProjectId().

=back


=head2 $adaptorlink->setActive(state)

Sets the mode of the adaptor link - active, inactive, or debug.  $adaptorlink->update() 
must be run before changes are replicated in the database.

=over 8

=item state

May be one of 'YES' for active, 'NO' for inactive, or 'DEBUG' for debug mode.

=back


=head2 $adaptorlink->setEnvironmentGroupId(envGroupId)

Sets the environment group that the adaptor link is linked to.  $adaptorlink->update() 
must be run before changes are replicated in the database.

=over 8

=item envGroupId

The ID of the environment group to link to (from $environment->getEnvGroupId()), or 0
for no environment group.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

