
package BuildForge::Services::DBO::Build;

use strict;
use constant FLAG_FIND_ENV => 'E';
use constant FLAG_UNLOCK => 'U';

use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(
	buildId targetId startTime state process cancellation selectorId
	tag userId level result duration size stage fired jobId envText
	buildClass engineId geoId failChainId passChainId chainedBuildId);
my $prototype = {
	buildId             => 0,
	targetId            => 0,
	startTime           => 0,
	state               => ENUM_BUILD_STATE->WAITING,
	process             => ENUM_BUILD_PROCESS->BUILD,
	cancellation        => 0,
	resource            => '',
	tag                 => '',
	userId              => UID_SYSTEM,
	level               => 0,
	result              => ENUM_BUILD_RESULT->NONE,
	duration            => 0,
	size                => 0,
	stage               => '',
	fired               => 0,
	jobId               => 0,
	envText             => undef,
	buildClass          => '',
	engineId            => '',
	passChainId         => 0,
	failChainId         => 0,
	geoId				=> '',
	chainedBuildId      => 0,
};
sub getTypeKey { 'Build' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_envText'} = $self->{'envText'};
	$self->{'envText'} = undef;
	Integer(\$self->{'buildId'});
	Integer(\$self->{'targetId'});
	Integer(\$self->{'startTime'});
	Enum(ENUM_BUILD_STATE, \$self->{'state'});
	Enum(ENUM_BUILD_PROCESS, \$self->{'process'});
	Boolean(\$self->{'cancellation'});
	Integer(\$self->{'userId'});
	Integer(\$self->{'level'});
	Enum(ENUM_BUILD_RESULT, \$self->{'result'});
	Integer(\$self->{'duration'});
	Integer(\$self->{'size'});
	Integer(\$self->{'fired'});
	Integer(\$self->{'jobId'});
	Integer(\$self->{'passChainId'});
	Integer(\$self->{'failChainId'});
	Integer(\$self->{'chainedBuildId'});
	return $self;
}

sub findAll {
	my ($proto, $conn, $withEnvironment) = @_;
	$conn->request(COMMAND_BUILD_FIND_ALL);
	defined($withEnvironment) && $withEnvironment
		and $conn->writeEntryString(KEY_FLAGS, FLAG_FIND_ENV);
	return _getList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $buildId, $withEnvironment) = @_;
	$conn->request(COMMAND_BUILD_FIND_ID);
	$conn->writeEntryString(KEY_BUILD_ID, $buildId);
	defined($withEnvironment) && $withEnvironment
		and $conn->writeEntryString(KEY_FLAGS, FLAG_FIND_ENV);
	my $dbo = $conn->call()->{KEY_BUILD_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByProjectId {
	my ($proto, $conn, $projectId, $withEnvironment) = @_;
	$conn->request(COMMAND_BUILD_FIND_PID);
	$conn->writeEntryString(KEY_BUILD_PID, $projectId);
	defined($withEnvironment) && $withEnvironment
		and $conn->writeEntryString(KEY_FLAGS, FLAG_FIND_ENV);
	return _getList($conn, $conn->call());
}

sub findByState {
	my ($proto, $conn, $state, $withEnvironment) = @_;
	$conn->request(COMMAND_BUILD_FIND_STATE);
	$conn->writeEntryString(KEY_BUILD_STATE, $state);
	defined($withEnvironment) && $withEnvironment
		and $conn->writeEntryString(KEY_FLAGS, FLAG_FIND_ENV);
	return _getList($conn, $conn->call());
}

sub findWaiting {
	my ($proto, $conn, $withEnvironment) = @_;
	return $proto->findByState($conn, ENUM_BUILD_STATE->WAITING, $withEnvironment);
}
sub findRunning {
	my ($proto, $conn, $withEnvironment) = @_;
	return $proto->findByState($conn, ENUM_BUILD_STATE->RUNNING, $withEnvironment);
}
sub findCompleted {
	my ($proto, $conn, $withEnvironment) = @_;
	return $proto->findByState($conn, ENUM_BUILD_STATE->COMPLETED, $withEnvironment);
}
sub findLocked {
	my ($proto, $conn, $withEnvironment) = @_;
	return $proto->findByState($conn, ENUM_BUILD_STATE->LOCKED, $withEnvironment);
}
sub findArchived {
	my ($proto, $conn, $withEnvironment) = @_;
	return $proto->findByState($conn, ENUM_BUILD_STATE->ARCHIVED, $withEnvironment);
}

sub _checkLive($) { $_[0]->{'buildId'} <= 0 and die $_[0]->dead(__PACKAGE__); }

sub getResults {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	_checkLive($self);
	$self->{'process'} ne ENUM_BUILD_PROCESS->BUILD and return undef;
	$conn->request(COMMAND_RESULT_FIND_BID);
	return BuildForge::Services::DBO::Result->findByBuild($conn, $self->{'buildId'});
}

sub unlock {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $buildId = $self->{'buildId'};
	_checkLive($self);
	$conn->request(COMMAND_BUILD_LOCK);
	$conn->writeEntryString(KEY_BUILD_ID, $buildId);
	$conn->writeEntryString(KEY_FLAGS, FLAG_UNLOCK);
	$conn->call();
}

sub lock {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $buildId = $self->{'buildId'};
	_checkLive($self);
	$conn->request(COMMAND_BUILD_LOCK);
	$conn->writeEntryString(KEY_BUILD_ID, $buildId);
	$conn->call();
}

sub purge {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $buildId = $self->{'buildId'};
	_checkLive($self);
	$conn->request(COMMAND_BUILD_PURGE);
	$conn->writeEntry(KEY_BUILD_ID, $buildId);
	my $dbo = $conn->call()->{KEY_BUILD_DBO()};
	defined($dbo) or die $self->missing(KEY_BUILD_DBO);
	$self->{'buildId'} = 0;
	return __PACKAGE__->new($conn)->fromArray($dbo);
}

sub fire {
	my ($proto, $conn, $projectId, $selectorId, $buildClassName, $runLink) = @_;
	$conn->request(COMMAND_BUILD_FIRE);
	$conn->writeEntryString(KEY_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_SELECTOR_ID, $selectorId) if defined($selectorId);
	$conn->writeEntryString(KEY_CLASS_NAME, $buildClassName) if defined($buildClassName);
	$conn->writeEntryString(KEY_BUILD_RUNLINK, $runLink ? '1' : '0') if defined($runLink);
	my $dbo = $conn->call()->{KEY_BUILD_DBO()};
	defined($dbo) or die $proto->missing(KEY_BUILD_DBO);
	return __PACKAGE__->new($conn)->fromArray($dbo);
}

sub cancel {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $buildId = $self->{'buildId'};
	_checkLive($self);
	$conn->request(COMMAND_BUILD_CANCEL);
	$conn->writeEntry(KEY_BUILD_ID, $buildId);
	$conn->call();
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_BUILD_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @builds = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@builds;
}

sub getBuildClass    { $_[0]->{'buildClass'}; }
sub getBuildId       { $_[0]->{'buildId'}; }
sub getCancellation  { $_[0]->{'cancellation'}; }
sub getChainedBuildId { $_[0]->{'chainedBuildId'}; }
sub getDuration      { $_[0]->{'duration'}; }
sub getEngineId      { $_[0]->{'engineId'}; }
sub getEnvText       { $_[0]->{'_envText'}; }
sub getFired         { $_[0]->{'fired'}; }
sub getLevel         { $_[0]->{'level'}; }
sub getProcess       { $_[0]->{'process'}; }
sub getResult        { $_[0]->{'result'}; }
sub getSelectorId    { $_[0]->{'selectorId'}; }
sub getStage         { $_[0]->{'stage'}; }
sub getStartTime     { $_[0]->{'startTime'}; }
sub getState         { $_[0]->{'state'}; }
sub getTag           { $_[0]->{'tag'}; }
sub getTargetId      { $_[0]->{'targetId'}; }
sub getUserId        { $_[0]->{'userId'}; }
sub getPassChainID   { $_[0]->{'passChainId'}; }
sub getFailChainID   { $_[0]->{'failChainId'}; }
sub getGeoId         { $_[0]->{'geoId'}; }

# Limited modification through special methods...
# No generic use update or setters...

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Build


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing adaptor links
	$project = new BuildForge::Services::DBO::Project($conn);
	$project->setName('Test Project');
	$project->create();
	
	$allBuilds = BuildForge::Services::DBO::Build->findAll($conn);
	$projectBuilds = BuildForge::Services::DBO::Build->findByProjectId($conn, $project->getProjectId());
	$waitingBuilds = BuildForge::Services::DBO::Build->findByState($conn, 'WAITING');
	$waitingBuilds2 = BuildForge::Services::DBO::Build->findWaiting($conn);
	$runningBuilds = BuildForge::Services::DBO::Build->findRunning($conn);
	$completedBuilds = BuildForge::Services::DBO::Build->findCompleted($conn);
	$lockedBuilds = BuildForge::Services::DBO::Build->findLocked($conn);
	$archivedBuilds = BuildForge::Services::DBO::Build->findArchived($conn);
	$build = BuildForge::Services::DBO::Build->findById($conn, $buildid);

	# Getter functions
	$id = $build->getBuildId();
	$class = $build->getBuildClass();
	$wasCancelled = $build->getCancellation();
	$durationInSecs = $build->getDuration();
	$engineId = $build->getEngineId();
	$environmentBlock = $build->getEnvText();
	$firedTimestamp = $build->getFired();
	$accessGroupId = $build->getLevel();
	$typeOfBuild = $build->getProcess();
	$result = $build->getResult();
	$selectorId = $build->getSelectorId();
	$currentStage = $build->getStage();
	$startedTimestamp = $build->getStartTime();
	$currentState = $build->getState();
	$tag = $build->getTag();
	$projectOrBuildId = $build->getTargetId();
	$owningUserId = $build->getUserId();
	$geoId = $build->getGeoId();
	$passChainId = $build->getPassChainId();
	$failChainId = $build->getFailChainId();
	$chainedBuildId = $build->getChainedBuildId();
	$geoId = $build->getGeoId();
	$buildResults = $build->getResults();
	
	# Kicking off, modifying, and purging a build
	$newBuild = BuildForge::Services::DBO::Build->fire($conn, $project->getProjectId());
	$newBuild2 = BuildForge::Services::DBO::Build->fire($conn, $project->getProjectId(), $selectorId, 
			$class, 1);
	$newBuild->lock();
	$newBuild->unlock();
	$newBuild->cancel();
	$newBuild->purge();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

A Build object can be used to access the various build queues, to kick off new builds, and to perform 
actions on running and historic builds.


=head1 METHODS


=head2 BuildForge::Services::DBO::Build->findAll(connection, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findByProjectId(connection, projectId, withEnvironment)

Returns a reference to an array of all builds of the specified project in the database that the logged
in user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The ID of the project, from $project->getProjectId().

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findByState(connection, state, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to that are of the specified state.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item state

Which builds to return.  This may be one of 'WAITING', 'PENDING', 'RUNNING', 'COMPLETED',
'LOCKED', or 'ARCHIVED'.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findWaiting(connection, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to that have not yet started executing.  It's a wrapper for
BuildForge::Services::DBO::Build->findByState(connection, 'WAITING', withEnvironment).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findRunning(connection, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to that are currently running.  It's a wrapper for
BuildForge::Services::DBO::Build->findByState(connection, 'RUNNING', withEnvironment).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findCompleted(connection, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to that have finished running.  It's a wrapper for
BuildForge::Services::DBO::Build->findByState(connection, 'COMPLETED', withEnvironment).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findLocked(connection, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to that have been completed and locked.  It's a wrapper for
BuildForge::Services::DBO::Build->findByState(connection, 'LOCKED', withEnvironment).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findArchived(connection, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to that have been archived.  It's a wrapper for
BuildForge::Services::DBO::Build->findByState(connection, 'ARCHIVED', withEnvironment).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item withEnvironment

If this evaluates to true, the returned builds have the environments they were created
with included.  Otherwise, the environments are not included.

=back


=head2 BuildForge::Services::DBO::Build->findById(connection, buildId, withEnvironment)

Returns a reference to an array of all builds in the database that the logged
in user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The ID of the build to get (from $build->getBuildId()).

=item withEnvironment

If this evaluates to true, the returned build has the environment it was created
with included.  Otherwise, the environment is not included.

=back


=head2 $build->lock()

Locks the build.  Locked builds can't be deleted until they are unlocked.


=head2 $build->unlock()

Unlocks the build.


=head2 $build->purge()

Purges the build according to the settings of the build's class.


=head2 BuildForge::Services::DBO::Build->fire(connection, projectId, selectorId, buildClassName, runLink)

Starts a build of the specified project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The ID of the project to start off, from $project->getProjectId().

=item selectorId

The ID of the selector to use with the build, from $selector->getSelectorId().  If unset,
uses the selector of the specified project.

=item buildClassName

The ID of the class to use with the build, from $class->getName().  If unset, uses the
class of the specified project.

=item runLink

If this evaluates to true, if the project has an adaptor link associated with it,
the build will kick off with the appropriate adaptor link.  If this evaluates to false or
is unset, no such adaptor link will be run.

=back


=head2 $build->cancel()

Cancels a currently running build.


=head2 $build->getBuildId()

Returns the ID of the build.


=head2 $build->getBuildClass()

Returns the ID of the class of the build.


=head2 $build->getCancellation()

Returns the cancellation state of the build - 'N' if it isn't being cancelled / hasn't been
cancelled or 'Y' if it has been / is being cancelled.


=head2 $build->getDuration()

Returns the current duration of the build in seconds.


=head2 $build->getEngineId()

Returns the ID of the engine the build ran on.


=head2 $build->getEnvText()

Returns the text of the environment the build was started with.  Only returns a value if 
the build was gotten with its environment - for example, if 
BuildForge::Services::DBO::Build->findById($connection, $buildId, $withEnvironment) was called
with $withEnvironment set to 1.


=head2 $build->getFired()

Returns the timestamp for when the build was created.


=head2 $build->getGeoId()

For GDD-aware consoles, this returns the geo ID for the console on which the build has / will
run.  For non-GDD-aware consoles, this returns an empty string.

=head2 $build->getPassChainId()

Returns the identifier of the Project to be executed in the event this Build succeeds, or 0 for none.

=head2 $build->getFailChainId()

Returns the identifier of the Project to be executed in the event this Build fails, or 0 for none.


=head2 $build->getChainedBuildId()

Returns the identifier of the Build that was executed by this Build due to
pass or fail chaining, or 0 for none.


=head2 $build->getLevel()

Returns the access group ID for the build.


=head2 $build->getProcess()

Returns the type of build this is.  This will be one of:

=over 8

=item BUILD

A normal build.

=item PURGE

A build purge.

=item CLOBBER

A project clobber purge.

=back


=head2 $build->getResult()

Returns the build success.  This will be:

=over 8

=item NONE

If the build hasn't started running yet.

=item RUNNING

If the build is currently running.

=item PASSED

If the build passed.

=item WARNING_FILTER

If the build finished in Warning state because of a Warning filter match.

=item WARNING_FAILED_STEP

If the build finished in Warning state because of a step failing with Continue On Fail
set.

=item FAILED

If the build failed.

=back


=head2 $build->getSelectorId()

Returns the ID of the selector for the build.


=head2 $build->getStage()

Returns a description string describing what the build is currently doing.


=head2 $build->getStartTime()

Returns a timestamp for the time an engine started executing the build.


=head2 $build->getState()

Returns the state of the build.  This may be one of 'WAITING', 'PENDING', 'RUNNING', 'COMPLETED',
'LOCKED', or 'ARCHIVED'.


=head2 $build->getTag()

Returns the build tag string.


=head2 $build->getTargetId()

If this is a normal project build or a project clobber build, this will return the 
project ID.  If this is a build c


=head2 $build->getUserId()

Returns the user ID of the user who started this build.


=head2 $build->getResults()

Returns a reference to an array of BuildForge::Services::DBO::Result objects containing 
the individual step results records for this build.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

