
package BuildForge::Services::DBO::BuildClass;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(
	id name level maxBuilds purgeRule purgeDays purgeType
	purgeChainProjectId enterChainProjectId exitChainProjectId);
my $prototype = {
	id                  => undef,
	name                => undef,
	level               => 0,
	maxBuilds           => 0,
	purgeRule           => ENUM_BUILDCLASS_PURGERULE->ANY_BUILD,
	purgeDays           => 0,
	purgeType           => ENUM_BUILDCLASS_PURGETYPE->EVERYTHING,
	purgeChainProjectId => 0,
	enterChainProjectId => 0,
	exitChainProjectId  => 0,
};
sub getTypeKey { 'Class' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'level'});
	Integer(\$self->{'maxBuilds'});
	Enum(ENUM_BUILDCLASS_PURGERULE, \$self->{'purgeRule'});
	Integer(\$self->{'purgeDays'});
	Enum(ENUM_BUILDCLASS_PURGETYPE, \$self->{'purgeType'});
	Integer(\$self->{'purgeChainProjectId'});
	Integer(\$self->{'enterChainProjectId'});
	Integer(\$self->{'exitChainProjectId'});
	return $self;
}



sub findAll {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_CLASS_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findByName {
	my ($proto, $conn, $buildClassName) = @_;
	_checkName($buildClassName);
	$conn->request(COMMAND_CLASS_FIND_NAME);
	$conn->writeEntryString(KEY_CLASS_NAME, $buildClassName);
	my $dbo = $conn->call()->{KEY_CLASS_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_CLASS_ADD);
	$conn->writeEntryDBO(KEY_CLASS_DBO, $self);
	my $dbo = $conn->call()->{KEY_CLASS_DBO()};
	return defined($dbo) ? $self->fromArray($dbo) : undef;
}

sub delete {
	my ($self) = @_;
	my $id = $self->{'id'};
	!defined($id) || !length($id) and die $self->dead(__PACKAGE__);
	deleteByName($self, $self->{'_conn'}, $id);
}

sub deleteByName {
	my ($proto, $conn, $name) = @_;
	_checkName($name);
	$conn->request(COMMAND_CLASS_DEL);
	$conn->writeEntryString(KEY_CLASS_NAME, $name);
	$conn->call;
}

sub update {
	my ($self) = @_;
	my $conn = $self->getConnection;
	my $id = $self->{'id'};
	!defined($id) || !length($id) and die $self->dead(__PACKAGE__);
	$conn->request(COMMAND_CLASS_UPD);
	$conn->writeEntryDBO(KEY_CLASS_DBO, $self);
	my $dbo = $conn->call()->{KEY_CLASS_DBO()};
	return defined($dbo) ? $self->fromArray($dbo) : undef;
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_CLASS_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @apiList = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@apiList;
}

# wrap is inherited

sub _checkName($) { !defined($_[0]) || !length($_[0]) and die __PACKAGE__->invalid('Name'); }



sub getEnterChainProjectId { $_[0]->{'enterChainProjectId'}; }
sub getExitChainProjectId  { $_[0]->{'exitChainProjectId'}; }
sub getLevel               { $_[0]->{'level'}; }
sub getMaxBuilds           { $_[0]->{'maxBuilds'}; }
sub getName                { $_[0]->{'name'}; }
sub getPurgeChainProjectId { $_[0]->{'purgeChainProjectId'}; }
sub getPurgeDays           { $_[0]->{'purgeDays'}; }
sub getPurgeRule           { $_[0]->{'purgeRule'}; }
sub getPurgeType           { $_[0]->{'purgeType'}; }

sub setEnterChainProjectId { $_[0]->{'enterChainProjectId'} = $_[1]; }
sub setExitChainProjectId  { $_[0]->{'exitChainProjectId'} = $_[1]; }
sub setLevel               { $_[0]->{'level'} = $_[1]; }
sub setMaxBuilds           { $_[0]->{'maxBuilds'} = $_[1]; }
sub setName                { $_[0]->{'name'} = $_[1]; }
sub setPurgeChainProjectId { $_[0]->{'purgeChainProjectId'} = $_[1]; }
sub setPurgeDays           { $_[0]->{'purgeDays'} = $_[1]; }
sub setPurgeRule           { $_[0]->{'purgeRule'} = ENUM_BUILDCLASS_PURGERULE($_[1]); }
sub setPurgeType           { $_[0]->{'purgeType'} = ENUM_BUILDCLASS_PURGETYPE($_[1]); }

# toString is inherited


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::BuildClass


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing classes
	$allClasses = BuildForge::Services::DBO::BuildClass->findAll($conn);
	$oldclass = BuildForge::Services::DBO::BuildClass->findByName($conn, 'ClassName');

	# Getter / setter functions
	$id = $oldclass->getName();
	$accessGroupId = $oldclass->getLevel();
	$maxBuilds = $oldclass->getMaxBuilds();
	$daysUntilPurge = $oldclass->getPurgeDays();
	$whichBuildsToPurge = $oldclass->getPurgeRule();
	$whatDataToPurge = $oldclass->getPurgeType();
	$enterProjectId = $oldclass->getEnterChainProjectId();
	$exitProjectId = $oldclass->getExitChainProjectId();
	$purgeProjectId = $oldclass->getPurgeChainProjectId();
	
	$project = new BuildForge::Services::DBO::Project();
	$project->setName('ProjectName');
	$project->create();

	$oldclass->setName('newName');
	$oldclass->setLevel(6);
	$oldclass->setMaxBuilds(16);
	$oldclass->setPurgeDays(5);
	$oldclass->setPurgeRule('ANY_BUILD');
	$oldclass->setPurgeType('CONSOLE_DATA');
	$oldclass->setEnterChainProjectId($project->getProjectId());
	$oldclass->setExitChainProjectId($project->getProjectId());
	$oldclass->setPurgeChainProjectId($project->getProjectId());
	$oldclass->update();
	
	# Class creation, updating, deletion
	$newclass = new BuildForge::Services::DBO::BuildClass($conn);
	$newclass->setName('New Build Class');
	$newclass->setLevel(4);
	$newclass->create();

	$newclass->setLevel(2);
	$newclass->update();
	
	$newclass->delete();
	BuildForge::Services::DBO::BuildClass->deleteByName($conn, $oldclass->getName());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

BuildClass allows you to create, access, and delete BuildForge classes. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::BuildClass(connection, {ClassParamsHashRef})

Creates a new BuildClass instance.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item ClassParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item name

The name/ID of the build class.

=item level

The AccessGroup ID of the access group this class is in (from $accessGroup->getLevel()).
If a user is not in the access group, the user can't view, edit, or use this class.

=item maxBuilds

The maximum number of builds with this class of a given project that will be kept before
it is eligible for automatic purging.  A value of 0 causes no build to be purged due to 
number of builds of this class.

=item purgeDays

If a build of this class is more than this number of days old it becomes eligible for 
automatic purging.  A value of 0 means that no build is automatically purged due to its
age.

=item purgeRule

This sets the rule for what types of builds will be purged according to the maxBuilds and
purgeDays directives.  This can be any of:

=over 8

=item ANY_BUILD

Any build will be deleted.

=item KEEP_PASS

The most recent passed build will be kept no matter how old it is in comparison to the
failed builds.

=item ONLY_FAILED

Only failed builds will be purged automatically.

=item ONLY_PASSED

Only passed builds will be purged automatically.

=back

=item purgeType

This sets the rule for what is deleted when a build is purged.  It can be any of:

=over 8

=item CONSOLE_DATA

Removes the information for the build from the console, but leaves the files and directories
that were created for the build from the server(s) on which it was built.

=item LOGS_AND_FILES

Removes the build logs from the console and the directories and files created in the build
from the server(s) on which it was built, but leaves the build record on the console.  The build
record is moved to the "Archived" section.

=item FILES

Removes the directories and files created for the build from the server(s) on which it was built,
but leaves the build record and build logs on the console.  The build record is moved to the 
"Archived" section.

=item LOGS

Removes the build logs from the console, but leaves the build record on the console and the
directories and files created on the server(s) on which the build was built.  The build
record is moved to the "Archived" section.

=item EVERYTHING

All data for the build is deleted.

=back

=item purgeChainProjectId

The project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of this class is purged, or 0 (the default) if no project gets 
kicked off for this reason.

=item enterChainProjectId

The project ID (from $project->getProjectId()) for the project that gets kicked off when
a build gets changed from another class to this one, or 0 (the default) if no project gets 
kicked off for this reason.

=item exitChainProjectId

The project ID (from $project->getProjectId()) for the project that gets kicked off when
a build gets changed from this class to another one, or 0 (the default) if no project gets 
kicked off for this reason.

=back

=back


=head2 BuildForge::Services::DBO::BuildClass->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::BuildClass objects, one
for each entry in the database that the current user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::BuildClass->findByName(connection, className)

Returns the BuildForge::Services::DBO::BuildClass object that corresponds to the 
given name, or undef if no such BuildClass exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item className

The name of the desired class (from $class->getName()).

=back


=head2 $class->create()

Creates an entry for the build class in the database with the current settings.


=head2 $class->update()

Updates the build class in the database to correspond to the current
state of the BuildClass object.


=head2 $class->delete()

Removes the class from the database (if it exists).


=head2 BuildForge::Services::DBO::BuildClass->deleteByName(connection, className)

Deletes the class with the given name from the database (if it exists).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item className

The name of the desired class (from $class->getName()).

=back


=head2 $class->getName()

Returns the name / ID of the class.


=head2 $class->getLevel()

Returns the access group ID of the access group the class belongs to.  Users not in
the access group can not see, edit, or use the class.


=head2 $class->getMaxBuilds()

Returns the maximum number of builds of this class a given project may have before
excess builds become eligible for automatic purging.  A value of 0 means that unlimited
builds will be kept.


=head2 $class->getPurgeDays()

Returns the number of days a build  of this class will remain before it becomes eligible for automatic
purging.  A value of 0 means that builds of this class will not be deleted due to the length 
of time they have remained.


=head2 $class->getPurgeRule()

Returns the rule for what types of builds will be purged according to the maxBuilds and
purgeDays directives.  This can be any of:

=over 8

=item ANY_BUILD

Any build will be deleted.

=item KEEP_PASS

The most recent passed build will be kept no matter how old it is in comparison to the
failed builds.

=item ONLY_FAILED

Only failed builds will be purged automatically.

=item ONLY_PASSED

Only passed builds will be purged automatically.

=back


=head2 $class->getPurgeType()

Returns the rule for what is deleted when a build is purged.  It can be any of:

=over 8

=item CONSOLE_DATA

Removes the information for the build from the console, but leaves the files and directories
that were created for the build from the server(s) on which it was built.

=item LOGS_AND_FILES

Removes the build logs from the console and the directories and files created in the build
from the server(s) on which it was built, but leaves the build record on the console.  The build
record is moved to the "Archived" section.

=item FILES

Removes the directories and files created for the build from the server(s) on which it was built,
but leaves the build record and build logs on the console.  The build record is moved to the 
"Archived" section.

=item LOGS

Removes the build logs from the console, but leaves the build record on the console and the
directories and files created on the server(s) on which the build was built.  The build
record is moved to the "Archived" section.

=item EVERYTHING

All data for the build is deleted.

=back


=head2 $class->getEnterChainProjectId()

Returns the project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of another class gets changed to this class, or 0 if no project gets kicked off for this
reason.


=head2 $class->getExitChainProjectId()

Returns the project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of this class gets changed to another class, or 0 if no project gets kicked off for this
reason.


=head2 $class->getPurgeChainProjectId()

Returns the project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of this class is purged, or 0 if no project gets kicked off for this
reason.


=head2 $class->setName(className)

Sets the name / ID for the class.  $class->update() must be run before changes are 
replicated in the database.

=over 8

=item className

The new name for the class

=back


=head2 $class->setLevel(accessGroupId)

Sets the access group the class belongs to.  Users not belonging to the class's access
group will not be able to view, edit, or use the class.  $class->update() must be 
run before changes are replicated in the database.

=over 8

=item accessGroupId

The new access group ID for the class, from $accessGroup->getLevel().

=back


=head2 $class->setMaxBuilds(maxBuilds)

Sets the maximum number of builds of this class a project may have before older ones
become eligible for automatic deletion.  $class->update() must be 
run before changes are replicated in the database.

=over 8

=item maxBuilds

The number of builds allowed.  0 will make no build deletable for this reason.

=back


=head2 $class->setPurgeDays(purgeDays)

Sets the maximum number of days a build will remain before being eligible for
automatic deletion.  $class->update() must be run before changes are replicated 
in the database.

=over 8

=item purgeDays

The number of days, with 0 corresponding to a build being able to remain for unlimited
time.

=back


=head2 $class->setPurgeRule(purgeRule)

Sets the rule for what types of builds will be purged according to the maxBuilds and
purgeDays directives.  $class->update() must be run before changes are replicated in the database.

=over 8

=item purgeRule

The deletion rule.  It may be any of:

=over 8

=item ANY_BUILD

Any build will be deleted.

=item KEEP_PASS

The most recent passed build will be kept no matter how old it is in comparison to the
failed builds.

=item ONLY_FAILED

Only failed builds will be purged automatically.

=item ONLY_PASSED

Only passed builds will be purged automatically.

=back

=back


=head2 $class->setPurgeType(purgeType)

Sets the rule for what is deleted when a build is purged.  $class->update() must be 
run before changes are replicated in the database.

=over 8

=item purgeType

The deletion rule.  It may be any of:

=over 8

=item CONSOLE_DATA

Removes the information for the build from the console, but leaves the files and directories
that were created for the build from the server(s) on which it was built.

=item LOGS_AND_FILES

Removes the build logs from the console and the directories and files created in the build
from the server(s) on which it was built, but leaves the build record on the console.  The build
record is moved to the "Archived" section.

=item FILES

Removes the directories and files created for the build from the server(s) on which it was built,
but leaves the build record and build logs on the console.  The build record is moved to the 
"Archived" section.

=item LOGS

Removes the build logs from the console, but leaves the build record on the console and the
directories and files created on the server(s) on which the build was built.  The build
record is moved to the "Archived" section.

=item EVERYTHING

All data for the build is deleted.

=back

=back


=head2 $class->setEnterChainProjectId(projectID)

Sets the project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of another class gets changed to this class.  $class->update() must be 
run before changes are replicated in the database.

=over 8

=item projectID

The ID of the project to be kicked off (from $project->getProjectID()).  A value of 0 
causes no project to be kicked off for this reason.

=back


=head2 $class->setExitChainProjectId(projectID)

Sets the project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of this class gets changed to another class.  $class->update() must be 
run before changes are replicated in the database.

=over 8

=item projectID

The ID of the project to be kicked off (from $project->getProjectID()).  A value of 0 
causes no project to be kicked off for this reason.

=back


=head2 $class->setPurgeChainProjectId(projectID)

Sets the project ID (from $project->getProjectId()) for the project that gets kicked off when
a build of this class is purged.  $class->update() must be 
run before changes are replicated in the database.

=over 8

=item projectID

The ID of the project to be kicked off (from $project->getProjectID()).  A value of 0 
causes no project to be kicked off for this reason.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

