
package BuildForge::Services::DBO::Collector;
use BuildForge::Services::DBO::CollectorProperty;

use strict;
use overload '""' => \&toString;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(collectorId liveCollectorId);
my $prototype = {
	collectorId         => undef,
	liveCollectorId     => undef,
};
sub getTypeKey { 'Collector' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'properties'} = [];
	return $self;
}

sub getBuiltIns { ENUM_VALUES(ENUM_COLLECTORPROPERTY_BUILTIN); }

sub findById {
	my ($proto, $conn, $collectorName) = @_;
	$conn->request(COMMAND_COLLECTOR_FIND_ID);
	$conn->writeEntryString(KEY_COLLECTOR_ID, $collectorName);
	return _readDBOAndProperties($conn, $conn->call());
}

sub findAll {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_COLLECTOR_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_COLLECTOR_ADD);
	_writeDBOAndProperties($self);
	my $resp = $conn->call;
	my $dbo = $resp->{+KEY_COLLECTOR_DBO};
	!defined($dbo) || !isa($dbo, 'ARRAY') and die $self->missing(KEY_COLLECTOR_DBO);
	my $properties = $self->{'properties'};
	$self->fromArray($dbo);
	foreach my $prop (@$properties) { $prop->{'liveSequenceNumber'} = $prop->{'sequenceNumber'}; }
	$self->{'properties'} = $properties;
	return $self;
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $collectorId = $self->{'collectorId'};
	$conn->request(COMMAND_COLLECTOR_DEL);
	$conn->writeEntryString(KEY_COLLECTOR_ID, $collectorId);
	$conn->call();
	my $props = $self->{'properties'};
	foreach my $prop (@$props) { $prop->{'liveSequenceNumber'} = -1; }
	$self->{'liveCollectorId'} = '';
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_COLLECTOR_UPD);
	$conn->writeEntryDBO(KEY_COLLECTOR_DBO, $self);
	my $dbo = $conn->call()->{+KEY_COLLECTOR_DBO};
	!defined($dbo) || !isa($dbo, 'ARRAY') and die $self->missing(KEY_COLLECTOR_DBO);
	my $properties = $self->{'properties'};
	$self->fromArray($dbo);
	$self->{'properties'} = $properties;
	return $self;
}

sub addCollectorProperty {
	my ($self, $prop, $index) = @_;
	_dbo_addCollectorProperty($self, $prop, $index);
	my $liveCollectorId = $self->{'liveCollectorId'};
	defined($liveCollectorId) && length($liveCollectorId) and $prop->create();
	$prop->{'_parent'} = $self;
}

sub moveCollectorProperty {
	my ($self, $fromIndex, $toIndex) = @_;
	my $props = $self->{'properties'};
	$fromIndex < 0 || $fromIndex >= @$props
		and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex >= @$props
		and die $self->badIndex($toIndex);
	my $liveCollectorId = $self->{'liveCollectorId'};
	if (defined($liveCollectorId) && length($liveCollectorId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_COLLECTOR_MOVE_PROPERTY);
		$conn->writeEntryString(KEY_COLLECTOR_ID, $liveCollectorId);
		$conn->writeEntryString(KEY_COLLECTOR_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_COLLECTOR_TO_INDEX, $toIndex);
		$conn->call();
	}
	_dbo_moveCollectorProperty($self, $fromIndex, $toIndex);
	return $self;
}

sub copyCollectorProperty {
	my ($self, $fromIndex, $toIndex) = @_;
	my $conn = $self->{'_conn'};
	my $props = $self->{'properties'};
	$fromIndex < 0 || $fromIndex >= @$props
		and die $self->badIndex($fromIndex);
	!defined $toIndex
		and return $props->[$fromIndex]->clone();

	my $liveCollectorId = $self->{'liveCollectorId'};
	if (defined($liveCollectorId) && length($liveCollectorId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_COLLECTOR_COPY_PROPERTY);
		$conn->writeEntryString(KEY_COLLECTOR_ID, $liveCollectorId);
		$conn->writeEntryString(KEY_COLLECTOR_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_COLLECTOR_TO_INDEX, $toIndex);
		$conn->call;
	}
	_dbo_copyCollectorProperty($self, $fromIndex, $toIndex);
}

sub removeCollectorProperty {
	my ($self, $index) = @_;
	!defined($index) and return undef;
	if (isa($index, 'BuildForge::Services::DBO::CollectorProperty')) {
		# This is removeCollectorProperty(CollectorProperty)
		$index = $index->getSequenceNumber() - 1;
		# call to removeCollectorProperty(int) is handled by fall-through
	} elsif ($index !~ /^\d+$/) {
		# This is removeCollectorProperty(String)
		$index = $self->getCollectorProperty($index);
		!defined($index) and return undef;
		$index = $index->getSequenceNumber() - 1;
		# call to removeCollectorProperty(int) is handled by fall-through
	}
	# removeCollectorProperty(int)
	my $props = $self->{'properties'};
	$index < 0 || $index >= @$props
		and die $self->badIndex($index);

	my $prop = $props->[$index];
	my $liveCollectorId = $self->{'liveCollectorId'};
	if (defined($liveCollectorId) && length($liveCollectorId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_COLLECTOR_ENTRY_DEL);
		$conn->writeEntryString(KEY_COLLECTOR_ID, $liveCollectorId);
		$conn->writeEntryString(KEY_COLLECTOR_PROPERTY_SEQUENCE_NUMBER, $index);
		$conn->call();
	}
	_dbo_removeCollectorProperty($self, $index);
	$prop->{'sequenceNumber'} = -1;
	$prop->{'live'} = 0;
	return $prop;
}


sub _writeDBOAndProperties($) {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->writeEntryDBO(KEY_COLLECTOR_DBO, $self);
	my $props = $self->{'properties'};
	defined($props) && @$props > 0
		and $conn->writeEntryArray(KEY_COLLECTOR_PROPERTY_LIST, $props);
}

sub _readDBOAndProperties($$) {
	my ($conn, $resp) = @_;
	my $dbo = $resp->{+KEY_COLLECTOR_DBO};
	!defined($dbo) and return undef;
	my $collector = __PACKAGE__->new($conn)->fromArray($dbo);
	my $list = $resp->{+KEY_COLLECTOR_PROPERTY_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return $collector;
	my @props = map { BuildForge::Services::DBO::CollectorProperty->new($conn, $collector)->fromArray($_) } @$list;
	$collector->{'properties'} = \@props;
	return $collector;
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_COLLECTOR_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @collectors = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@collectors;
}

# wrap is inherited

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'});
	my @props = map { $_->clone } @{ $self->{'properties'} };
	$clone->{'properties'} = \@props;
	return $clone;
}

sub toString {
	my ($self) = @_;
	return '[CollectorProperty: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ', properties=' . scalar @{ $self->{'properties'} } . ']';
}

# Aliases for convenience
sub getCollectorId { getCollectorName(@_); }
sub setCollectorId { setCollectorName(@_); }

sub getCollectorName { defined($_[0]->{'collectorId'}) ? $_[0]->{'collectorId'} : $_[0]->{'liveCollectorId'}; }
sub setCollectorName {
	my ($self, $collectorId) = @_;
	my $liveCollectorId = $self->{'liveCollectorId'};
	$self->{'collectorId'} = $collectorId;
	defined($liveCollectorId) && length($liveCollectorId) and $self->update;
	my $props = $self->{'properties'};
	# Logic from CollectorDBO.setCollectorName(collectorId)
	if (defined($props)) {
		foreach (@$props) { $_->{'collectorId'} = $collectorId; }
	}
}


sub getProperties {
	my $list = $_[0]->{'properties'};
	my @props = @$list;
	return \@props;
}

sub getProperty {
	my ($self, $index) = @_;
	my $props = $self->{'properties'};
	# If the argument doesn't look like an index, or if it is out of range,
	# then assume it is the name of the property, instead.
	$index =~ /\D/ || $index < 0 || $index >= @$props
		and return $self->getCollectorProperty($index);

	# This is getProperty(int).  No need to check index, because we treat
	# an invalid index as a string, above...
	return $props->[$index];
}

#----------------------------------------------------------------
# DBO Methods
#----------------------------------------------------------------

# _dbo_setLiveCollectorId is not needed

sub _dbo_setCollectorId($$) {
	my ($self, $collectorId) = @_;
	$self->{'collectorId'} = $collectorId;
	my $props = $self->{'properties'};
	if (isa($props, 'ARRAY')) {
		if (defined($props) && @$props > 0) {
			foreach my $prop (@$props) { $prop->setCollectorId($collectorId); }
		}
	}
}

# _dbo_setProperties is not needed

sub _dbo_addCollectorProperty ($$;$) {
	my ($self, $prop, $index) = @_;
	my $props = $self->{'properties'};
	$props = $self->{'properties'} = [] unless isa($props, 'ARRAY');

	if (!defined($index) || $index >= @$props) {
		# This is addCollectorProperty(CollectorPropertyDBO) or
		# addCollectorProperty(CollectorPropertyDBO, int) with an
		# index that puts it at or past the end of the list
		$prop->{'sequenceNumber'} = @$props + 1;
		$prop->{'collectorId'} = $self->{'collectorId'};
		push @$props, $prop;
		return;
	}

	# This is addCollectorProperty(CollectorPropertyDBO, int)
	$index < 0 && die $self->badIndex($index);
	# $index >= @$props case handled above
	$prop->{'sequenceNumber'} = $index + 1;
	$prop->{'collectorId'} = $self->{'collectorId'};
	splice @{$self->{'properties'}}, $index, 0, $prop;
	_dbo_updateIndices($self, $index + 1);
}


sub getCollectorProperty {
	my ($self, $name) = @_;
	my $index = _dbo_findIndex($self, $name);
	my $props = $self->{'properties'};
	!isa($props, 'ARRAY') and return undef;
	return $index >= 0 ? $self->{'properties'}->[$index] : undef;
}

sub _dbo_removeCollectorProperty ($$) {
	my ($self, $index) = @_;
	my $props = $self->{'properties'};
	!isa($props, 'ARRAY') || $index < 0 || $index >= @$props
		and die $self->badIndex($index);
	my ($prop) = splice @$props, $index, 1;
	_dbo_updateIndices($self, $index);
	return $prop;
}

sub _dbo_moveCollectorProperty ($$$) {
	my ($self, $fromIndex, $toIndex) = @_;
	my $props = $self->{'properties'};
	# fromIndex is checked by _dbo_removeCollectorProperty
	# toIndex >= the array size means move to the end
	!isa($props, 'ARRAY') || $fromIndex < 0
		and die $self->badIndex($fromIndex);
	_dbo_addCollectorProperty($self, _dbo_removeCollectorProperty($self, $fromIndex), $toIndex);
}

sub _dbo_copyCollectorProperty ($$;$) {
	my ($self, $fromIndex, $toIndex) = @_;

	my $props = $self->{'properties'};
	!isa($props, 'ARRAY') || $fromIndex < 0 || $fromIndex >= @$props
		and die $self->badIndex($fromIndex);
	my $prop = $props->[$fromIndex];

	if (!defined($toIndex)) {
		# This is copyCollectorProperty(int);
		return $prop->clone;
	}

	# This is copyCollectorProperty(int, int);
	$toIndex < 0 || $toIndex > @$props
		and die $self->badIndex($toIndex);
	_dbo_addCollectorProperty($self, $prop->clone, $toIndex);
}

# _dbo_toString is not needed
# _dbo_clone is not needed
# getTypeCode, fromArray, and toArray are defined above

sub _dbo_findIndex ($$) {
	my ($self, $name) = @_;
	my $props = $self->{'properties'};
	!isa($props, 'ARRAY') and return -1;
	for (my $i=0; $i<@$props; $i++) {
		return $i if $props->[$i]->{'propertyName'} eq $name;
	}
	return -1;
}

sub _dbo_updateIndices ($$) {
	my ($self, $startingHere) = @_;
	my $liveCollectorId = $self->{'liveCollectorId'};
	my $props = $self->{'properties'};
	if (isa($props, 'ARRAY')) {
		for (my $i=$startingHere; $i<@$props; $i++) {
			my $prop = $props->[$i];
			$prop->{'sequenceNumber'} = $i + 1;
		}
	}
}

# BuiltIn enum is in Constants

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Collector


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing collectors
	$allCollectors = BuildForge::Services::DBO::Collector->findAll($conn);
	$oldcollector = BuildForge::Services::DBO::Collector->findById($conn, 'CollectorName');

	# Getter / setter functions
	$id = $oldcollector->getCollectorId();
	$name = $oldcollector->getCollectorName();
	$property1 = $oldcollector->getProperty(1);
	$property2 = $oldcollector->getProperty('BF_JOBS');
	$allProperties = $oldcollector->getProperties();
	$builtInPropertyNames = $oldcollector->getBuiltIns();
	
	$oldcollector->setCollectorId('New Name');
	$oldcollector->setCollectorName('Another New Name');
	$oldcollector->update();
	
	# Collector creation, modification, deletion
	$newcollector = new BuildForge::Services::DBO::Collector($conn);
	$newcollector->setCollectorName('New Collector');
	$newcollector->create();
	
	$newcollector2 = $newcollector->clone();
	$newcollector2->setCollectorName('Duplicate');
	$newcollector2->create();
	
	$newcollector->setCollectorName('New Name For New Collector');
	$newcollector->update();

	$property = new BuildForge::Services::DBO::CollectorProperty($conn);
	$property->setPropertyName('PROP_NAME');
	$property->setPropertyValue('Value');
	$newcollector->addCollectorProperty($property);
	$newcollector->copyCollectorProperty(1, 2);
	$newcollector->moveCollectorProperty(2, 1);
	$newcollector->removeCollectorProperty(2);
	$newcollector->removeCollectorProperty($property);
	$newcollector->removeCollectorProperty($property->getPropertyName());
	
	$newcollector->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Collector allows you to create and modify collectors for use with servers.


=head1 METHODS


=head2 new BuildForge::Services::DBO::Collector(connection, {CollectorParamsHashRef})

Returns a new instance of the Collector object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item CollectorParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item collectorId

The name/ID of the collector.

=back

=back


=head2 BuildForge::Services::DBO::Collector->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::Collector objects, one
for each entry in the database that the current user has access to.  These objects will not
contain their CollectorProperties.  To get the properties, use 
BuildForge::Services::DBO::Collector->findById with the collector's collector ID.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Collector->findById(connection, collectorId)

Returns the BuildForge::Services::DBO::Collector object that corresponds to the 
given ID, or undef if no such Collector exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item collectorId

The name/ID of the desired collector, from $collector->getCollectorId().

=back


=head2 $collector->create()

Creates a record for the collector in the database.


=head2 $collector->clone()

Returns a copy of the Collector object with all of the same properties.  This will not
exist in the database until create() gets called on the new object.


=head2 $collector->update()

Updates the collector in the database to correspond to the current
state of the Collector object.


=head2 $collector->addCollectorProperty(property, index)

Adds the property to the collector.  Adding it creates the property in the database.

=over 8

=item property

The BuildForge::Services::DBO::CollectorProperty object to be added.

=item index

The optional location at which to add the object.  0 will add the property at the front 
of the list, 1 will add it after the first property, etc.  Not setting a location will
add the property to the end of the list.

=back


=head2 $collector->moveCollectorProperty(from, to)

Moves a collector property within the property list.

=over 8

=item from

The index from which to move the property.  0 moves the first property, 1 moves the second 
property, etc.

=item to

The index to which to move the property.  0 moves it to the first position, 1 moves it to
the second position, and so on.

=back


=head2 $collector->copyCollectorProperty(from, to)

Copies a collector property to somewhere else within the property list.

=over 8

=item from

The index from which to copy the property.  0 copies the first property, 1 copies the second 
property, etc.

=item to

The index to which to copy the property.  0 copies it to the first position, 1 copies it to
the second position, and so on.

=back


=head2 $collector->removeCollectorProperty(property)

Deletes a collector property from the collector.

=over 8

=item property

property may be either a BuildForge::Services::DBO::CollectorProperty object, in which
that property gets deleted from the list, an index, in which case the property
at that location gets deleted, or a collector property name, in which case that property
gets deleted.

=back


=head2 $collector->delete()

Deletes the collector and its properties from the database.


=head2 $collector->getCollectorId()

Returns the ID / name for the collector.  An alias for $collector->getCollectorName().


=head2 $collector->getCollectorName()

Returns the ID / name for the collector.


=head2 $collector->getProperties()

Returns a reference to an array of BuildForge::Services::DBO::CollectorProperty objects
for all collector properties in the collector.


=head2 $collector->getProperty(property)

Returns the BuildForge::Services::DBO::CollectorProperty object for the given property
name / location, or undef if no such property exists.

=over 8

=item property

This may be either the name or the index of the property to return.

=back


=head2 $collector->getBuiltIns()

Returns a reference to an array of all the built-in collector property names.


=head2 $collector->setCollectorId(collectorId)

Sets the collector name / ID both within the object and (if the collector has been 
created) within the database.  An alias to $collector->setCollectorName(collectorId).  

=over 8

=item collectorId

The new name / ID for the collector.

=back


=head2 $collector->setCollectorName(collectorId)

Sets the collector name / ID both within the object and (if the collector has been 
created) within the database.

=over 8

=item collectorId

The new name / ID for the collector.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

