
package BuildForge::Services::DBO::CollectorProperty;

use strict;
use overload '""' => \&toString;

use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(
	collectorId propertyName propertyValue regex userDefined dirty
	sequenceNumber liveSequenceNumber);
my $prototype = {
	_parent             => undef,
	collectorId         => undef,
	propertyName        => undef,
	propertyValue       => undef,
	regex               => undef,
	userDefined         => 0,
	dirty               => 1,
	sequenceNumber      => -1,
	liveSequenceNumber  => -1,
};
sub getTypeKey { 'CollectorProp' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my ($newParent) = splice(@_, 1, 1);
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_parent'} = $newParent;
	Boolean(\$self->{'userDefined'});
	Boolean(\$self->{'dirty'});
	Integer(\$self->{'sequenceNumber'});
	Integer(\$self->{'liveSequenceNumber'});
	return $self;
}


# INTERNAL USE ONLY
sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->internal();
	$conn->request(COMMAND_COLLECTOR_ENTRY_ADD);
	$conn->writeEntryDBO(KEY_COLLECTOR_PROPERTY_DBO, $self);
	my $dbo = $conn->call()->{KEY_COLLECTOR_PROPERTY_DBO()};
	!defined($dbo) || !isa($dbo, 'ARRAY') and die $self->missing(KEY_COLLECTOR_PROPERTY_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $liveCollectorId = $self->{'_parent'}->{'liveCollectorId'};
	!defined($liveCollectorId) || !length($liveCollectorId) and die BuildForge::Ex::APIException->dead(__PACKAGE__);
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_COLLECTOR_ENTRY_UPD);
	$conn->writeEntryDBO(KEY_COLLECTOR_PROPERTY_DBO, $self);
	my $dbo = $conn->call()->{KEY_COLLECTOR_PROPERTY_DBO()};
	!defined($dbo) || !isa($dbo, 'ARRAY') and die $self->missing(KEY_COLLECTOR_PROPERTY_DBO);
	return $self->fromArray($dbo);
}

sub delete { $_[0]->{'_parent'}->removeCollectorProperty($_[0]); }

sub moveTo { $_[0]->{'_parent'}->moveCollectorProperty($_[0]->getSequenceNumber - 1, $_[1]); }

# wrap is inherited

sub getParent { $_[0]->{'_parent'}; }

# setParent is not needed

sub getCollectorName       { $_[0]->{'collectorId'}; }
sub getPropertyName        { $_[0]->{'propertyName'}; }
sub getPropertyValue       { $_[0]->{'propertyValue'}; }
sub getRegex               { $_[0]->{'regex'}; }
sub getUserDefined         { $_[0]->{'userDefined'}; }
sub getSequenceNumber      { $_[0]->{'sequenceNumber'}; }

sub setPropertyName        { $_[0]->{'propertyName'}  = $_[1]; }
sub setPropertyValue       { $_[0]->{'propertyValue'} = $_[1]; }
sub setRegex               { $_[0]->{'regex'}         = $_[1]; }
sub setUserDefined         { $_[0]->{'userDefined'}   = $_[1]; }
# setSequenceNumber is not needed


sub toString {
	my ($self) = @_;
	return '[CollectorProperty: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ']';
}

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self->{'_parent'});
	$clone->{'_parent'} = undef;
	return $clone;
}


1;



__END__

=pod

=head1 NAME

BuildForge::Services::DBO::CollectorProperty


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	$property = new BuildForge::Services::DBO::CollectorProperty($conn);

	# Getter / setter functions
	$collectorName = $property->getCollectorName();
	$propertyName = $property->getPropertyName();
	$value = $property->getPropertyValue();
	$regex = $property->getRegex();
	$userDefined = $property->getUserDefined();
	$sequencenumber = $property->getSequenceNumber();
	$parentCollector = $property->getParent();

	$property->setPropertyName('PROP_NAME');
	$property->setPropertyValue('New Value');
	$property->setRegex('version: ([0-9\.]+)');
	$property->setUserDefined(1);
	$property->update();
	
	# Collector property creation, modification, and deletion
	$collector = BuildForge::Services::DBO::Collector->findById($conn, $collId);
	$newproperty = new BuildForge::Services::DBO::CollectorProperty($conn);
	$newproperty->setPropertyName('PROPERTY_NAME');
	$newproperty->setPropertyValue('Value');
	$collector->addCollectorProperty($var);

	$newproperty->setPropertyValue('New Value');
	$newproperty->update();
	
	$newproperty2 = $newproperty->clone();
	$collector->addCollectorProperty($newproperty2);
	
	$newproperty->moveTo(2);

	$newproperty->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

CollectorProperty allows you to modify collector properties inside of a Collector.  To create
collector properties in the database, see the Collector->addCollectorProperty() function.

=head1 METHODS


=head2 new BuildForge::Services::DBO::CollectorProperty(connection, {collector}, {CollectorPropertyParamsHashRef})

Returns a new instance of a CollectorProperty object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item collector

This should be undef or not passed - it is only used internally.

=item CollectorPropertyParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item propertyName

The name of the collector property.

=item propertyValue

The collector property value.  If this is a "Run Command" collector property, this should be
the command to run.

=item regex

For "Run Command" collector properties, this is the regular expression to run on the output of the
command in order to get the actual value (which will be the contents of the first pair of 
parentheses in the regular expression).  For other types of properties, this should be left blank.

=item userDefined

1 to make this a "Run Command" collector property, 0 otherwise.

=back

=back


=head2 $collectorProperty->update()

Updates the collector property settings in the database.


=head2 $collectorProperty->clone()

Returns a CollectorProperty object which is a copy of the current one.  The new copy will not be created
in the database until it is added to a collector with $collector->addCollectorProperty().


=head2 $collectorProperty->moveTo(location)

Moves the collector property within its parent collector.

=over 8

=item location

The index of the new location in the collector.  0 moves it to be the first collector
property, 1 makes it the second, and so forth.

=back


=head2 $collectorProperty->delete()

Deletes the collector property from the database.


=head2 $collectorProperty->getCollectorName()

Returns the name/ID of the collector which this property is within.


=head2 $collectorProperty->getPropertyName()

Returns the name of this property.


=head2 $collectorProperty->getPropertyValue()

Returns the value of this property.


=head2 $collectorProperty->getRegex()

Returns the regular expression of this property (which should be an empty string if
this property is not a "Run Command" property.


=head2 $collectorProperty->getUserDefined()

Returns 1 if the property is a "Run Command" property or 0 otherwise.


=head2 $collectorProperty->getSequenceNumber()

Returns the location of the property within its parent collector.  0 means it's
the first property, 1 is the second property, etc.


=head2 $collectorProperty->getParent()

Returns the BuildForge::Services::DBO::Collector object containing the collector
in which this property is located.


=head2 $collectorProperty->setPropertyName(propertyName)

Sets the name of the collector property.
$collectorProperty->update() must be run before changes are replicated in the database.

=over 8

=item propertyName

The new name for the property.

=back


=head2 $collectorProperty->setPropertyValue(propertyValue)

Sets the value of the collector property.
$collectorProperty->update() must be run before changes are replicated in the database.

=over 8

=item propertyValue

The new value of the collector property.

=back


=head2 $collectorProperty->setRegex(regex)

Sets the regular expression for "Run Command" type collector properties.  
$collectorProperty->update() must be run before changes are replicated in the database.

=over 8

=item regex

The regular expression.

=back


=head2 $collectorProperty->setUserDefined(isRunCommand)

Sets whether or not the collector property is a "Run Command" property.  If so, 
the property value is run on the server and the output is used as the actual value
of this field.  If regex is also set, the output is filtered with the regular expression.
$collectorProperty->update() must be run before changes are replicated in the database.

=over 8

=item isRunCommand

1 for a "Run Command" property, 0 otherwise.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

