
package BuildForge::Services::DBO::Cron;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(cronId description projectId ownerId environmentGroupId
		environmentText active minute hour dayOfMonth month dayOfWeek
		fired nextRun selectorId buildClass);
my $prototype = {
	active                 => ENUM_CRON_ACTIVE->YES,
	buildClass             => '',
	cronId                 => 0,
	dayOfMonth             => '*',
	dayOfWeek              => '*',
	description            => '',
	environmentGroupId     => 0,
	environmentText        => undef,
	fired                  => 0,
	hour                   => '0',
	minute                 => '0',
	month                  => '*',
	nextRun                => 0,
	ownerId                => 0,
	projectId              => 0,
	selectorId             => '',
};
sub getTypeKey { 'Cron' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_envText'} = $self->{'environmentText'};
	$self->{'environmentText'} = undef;
	Enum(ENUM_CRON_ACTIVE, \$self->{'active'});
	Integer(\$self->{'cronId'});
	Integer(\$self->{'environmentGroupId'});
	Integer(\$self->{'fired'});
	Integer(\$self->{'nextRun'});
	Integer(\$self->{'ownerId'});
	Integer(\$self->{'projectId'});
	return $self;
}

sub _checkLive($) {
	my ($self) = @_;
	my $cronId = $self->{'cronId'};
	if ($cronId <= 0) {
		$cronId == 0 and die $self->dead(__PACKAGE__);
		die $self->invalid('ID');
	}
	return $cronId;
}


sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_CRON_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $cronId) = @_;
	$proto->checkConnection($conn);
	!defined($cronId) || $cronId <= 0 and die $proto->invalid('ID');
	$conn->request(COMMAND_CRON_FIND_ID);
	$conn->writeEntryString(KEY_CRON_ID, $cronId);
	my $dbo = $conn->call()->{KEY_CRON_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByName {
	my ($proto, $conn, $cronName) = @_;
	$proto->checkConnection($conn);
	!$cronName and die $proto->invalid('Name');
	$conn->request(COMMAND_CRON_FIND_NAME);
	$conn->writeEntryString(KEY_CRON_NAME, $cronName);
	my $dbo = $conn->call()->{KEY_CRON_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub deactivate { $_[0]->activate('NO'); }
# activate with no args is handled below
sub oneShot { $_[0]->activate('ONCE'); }
sub activate {
	my ($self, $active) = @_;
	my $conn = $self->{'_conn'};
	$active = defined($active) ? ENUM_CRON_ACTIVE($active) : ENUM_CRON_ACTIVE->YES;
	_checkLive($self);
	$conn->request(COMMAND_CRON_ACTIVATE);
	$conn->writeEntryString(KEY_CRON_ID, $self->{'cronId'});
	$conn->writeEntryString(KEY_CRON_ACTIVE, $active);
	$conn->call();
	$self->{'active'} = $active;
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $cronId = $self->{'cronId'};
	defined($cronId) && $cronId != 0 and die $self->live(__PACKAGE__);
	$conn->request(COMMAND_CRON_ADD);
	$conn->writeEntryDBO(KEY_CRON_DBO, $self);
	my $dbo = $conn->call()->{KEY_CRON_DBO()};
	!defined($dbo) and die $self->missing(KEY_CRON_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	_checkLive($self);
	$conn->request(COMMAND_CRON_UPD);
	$conn->writeEntryDBO(KEY_CRON_DBO, $self);
	my $dbo = $conn->call()->{KEY_CRON_DBO()};
	!defined($dbo) and die $self->missing(KEY_CRON_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	_checkLive($self);
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_CRON_DEL);
	$conn->writeEntryString(KEY_CRON_ID, $self->{'cronId'});
	$conn->call();
	$self->{'cronId'} = 0;
}
sub deleteById {
	my ($proto, $conn, $cronId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_CRON_DEL);
	$conn->writeEntryString(KEY_CRON_ID, $cronId);
	$conn->call;
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_CRON_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @crons = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@crons;
}

# wrap is inherited


sub getActive               { $_[0]->{'active'}; }
sub getBuildClass           { $_[0]->{'buildClass'}; }
sub getCronId               { $_[0]->{'cronId'}; }
sub getDayOfMonth           { $_[0]->{'dayOfMonth'}; }
sub getDayOfWeek            { $_[0]->{'dayOfWeek'}; }
sub getDescription          { $_[0]->{'description'}; }
sub getEnvironmentGroupId   { $_[0]->{'environmentGroupId'}; }
sub getEnvironmentText      { $_[0]->{'_envText'}; }
sub getFired                { $_[0]->{'fired'}; }
sub getHour                 { $_[0]->{'hour'}; }
sub getMinute               { $_[0]->{'minute'}; }
sub getMonth                { $_[0]->{'month'}; }
sub getNextRun              { $_[0]->{'nextRun'}; }
sub getOwnerId              { $_[0]->{'ownerId'}; }
sub getProjectId            { $_[0]->{'projectId'}; }
sub getSelectorId           { $_[0]->{'selectorId'}; }

sub setActive               { $_[0]->{'active'}             = $_[1]; }
sub setBuildClass           { $_[0]->{'buildClass'}         = $_[1]; }
sub setDayOfMonth           { $_[0]->{'dayOfMonth'}         = $_[1]; }
sub setDayOfWeek            { $_[0]->{'dayOfWeek'}          = $_[1]; }
sub setDescription          { $_[0]->{'description'}        = $_[1]; }
sub setEnvironmentGroupId   { $_[0]->{'environmentGroupId'} = $_[1]; }
sub setEnvironmentText      { $_[0]->{'environmentText'}    = $_[0]->{'_envText'} = $_[1]; }
sub setHour                 { $_[0]->{'hour'}               = $_[1]; }
sub setMinute               { $_[0]->{'minute'}             = $_[1]; }
sub setMonth                { $_[0]->{'month'}              = $_[1]; }
sub setProjectId            { $_[0]->{'projectId'}          = $_[1]; }
sub setSelectorId           { $_[0]->{'selectorId'}         = $_[1]; }

# Shorthands added for those who don't like typing out "Environment"
sub getEnvGroupId           { getEnvironmentGroupId(@_); }
sub getEnvText              { getEnvironmentText(@_); }
sub setEnvGroupId           { setEnvironmentGroupId(@_); }
sub setEnvText              { setEnvironmentText(@_); }

# toString is inherited

1;



__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Cron


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing schedules
	$allCrons = BuildForge::Services::DBO::Cron->findAll($conn);
	$oldCron = BuildForge::Services::DBO::Cron->findById($conn, 2);
	$anotherCron = BuildForge::Services::DBO::Cron->findByName($conn, 'MyCron');

	# Getter / setter functions
	$id = $cron->getCronId();
	$isActive = $cron->getActive();
	$class = $cron->getBuildClass();
	$description = $cron->getDescription();
	$owner = $cron->getOwnerId();
	$projectId = $cron->getProjectId();
	$selectorId = $cron->getSelectorId();
	$minute = $cron->getMinute();
	$hour = $cron->getHour();
	$dow = $cron->getDayOfWeek();
	$month = $cron->getMonth();
	$dom = $cron->getDayOfMonth();
	$envGroupId = $cron->getEnvironmentGroupId();
	$envGroupId = $cron->getEnvGroupId();
	$envText = $cron->getEnvironmentText();
	$envText = $cron->getEnvText();
	$lastFiredTimestamp = $cron->getFired();
	$nextFiredTimestamp = $cron->getNextRun();
	
	$cron->setActive('YES');
	$cron->setBuildClass('ClassName');
	$cron->setDescription('Schedule Description');
	$cron->setProjectId($projectId);
	$cron->setSelectorId($selectorId);
	$cron->setMinute('5');
	$cron->setHour('*');
	$cron->setDayOfWeek('1-5');
	$cron->setMonth('*/2');
	$cron->setDayOfMonth('1-6/3,29');
	$cron->setEnvironmentGroupId($envGroupId);
	$cron->setEnvGroupId($envGroupId);
	$cron->setEnvText($envText);
	$cron->setEnvironmentText($envText);
	$cron->update();
	
	# Schedule creation, modification, deletion
	$project = new BuildForge::Services::DBO::Project($conn);
	$project->setName('Test Project');
	$project->create();
	
	$newcron = new BuildForge::Services::DBO::Cron($conn);
	$newcron->setDescription('New Cron');
	$newcron->setProjectId($project->getProjectId());
	$newcron->create();

	$newcron->setDescription('New Cron Description');	
	$newcron->update();
	
	$newcron->activate();
	$newcron->oneShot();
	$newcron->deactivate();

	$newcron->delete();
	BuildForge::Services::DBO::Cron->deleteById($conn, $cron->getCronId());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Cron allows you to create, modify, and delete schedules in the BuildForge 
engine.


=head1 METHODS


=head2 new BuildForge::Services::DBO::Cron(connection, {CronParamsHashRef})

Returns a new instance of a Cron object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item CronParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item active

Whether the schedule is active ('YES', the default), inactive ('NO'), or run 
once only ('ONCE').

=item buildClass

The class of the build that is run via the schedule, gotten from $class->getName().  
If this is left unset or set to an empty string, the class of the scheduled project
is used.  If this is a class purge schedule, a class must be set.

=item description

The description of the schedule.

=item projectId

The project ID of the project to run (from $project->getProjectId()), or 0 to 
make this a class purge schedule (the default).

=item selectorId

The selector ID of the selector (from $selector->getSelectorId()) to run the scheduled 
build with.  If this is 0 or is not set, the build uses the selector of the project
being scheduled.  If this is a class purge schedule, this value is ignored.

=item minute

The minute(s) at which to run this schedule in crontab format.  Defaults to '0'.

=item hour

The hour(s) at which to run this schedule in crontab format.  Defaults to '0'.

=item dayOfWeek

The day(s) of the week in which to run this schedule in crontab format.  Defaults to '*'.

=item month

The month(s) in which to run this schedule in crontab format.  Defaults to '*'.

=item dayOfMonth

The day(s) of the month in which to run this schedule in crontab format.  Defaults to '*'.

=item environmentGroupId

The environment group ID (from $environment->getEnvironmentGroupId()) to run the scheduled
builds with.  The project's environment group is also applied - first the project's environment
group, then the schedule's environment group, so that the schedule's settings override the
project's settings if necessary.  Set this to 0 (the default) for no schedule-specific environment group
to be used.

=item environmentText

In order to set variables specifically in the schedule rather than in the environment
group, set this to a string consisting of newline-delimited S:VARIABLE_NAME:New variable value
strings.  This is the empty string by default.

=back

=back


=head2 BuildForge::Services::DBO::Cron->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::Cron objects corresponding
to all schedules in the database that the user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Cron->findById(connection, cronId)

Returns the BuildForge::Services::DBO::Cron object corresponding to the given
schedule ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item cronId

The schedule ID for the desired schedule, from $cron->getCronId().

=back


=head2 BuildForge::Services::DBO::Cron->findByName(connection, cronName)

Returns the BuildForge::Services::DBO::Cron object corresponding to the given
description, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item cronId

The schedule description for the desired schedule, from $cron->getDescription().

=back


=head2 $cron->create()

Creates the cron object within the database.


=head2 $cron->update()

Updates the database cron object with the current object settings.


=head2 $cron->activate()

Activates the schedule within the database.


=head2 $cron->oneShot()

Makes the schedule within the database a run once schedule.


=head2 $cron->deactivate()

Deactivates the schedule within the database.


=head2 $cron->delete()

Deletes the schedule from the database.


=head2 BuildForge::Services::DBO::Cron->deleteById(connection, cronId)

Deletes the schedule with the given ID from the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item cronId

The schedule ID (from $cron->getCronId()) to delete.

=back


=head2 $cron->getCronId()

Returns the ID of the schedule.


=head2 $cron->getActive()

Returns 'YES' if this is an active schedule, 'NO' if this is an inactive 
schedule, or 'ONCE' if this is a run once schedule.


=head2 $cron->getBuildClass()

Returns the class of the build being scheduled or '' to use the class of the
scheduled project for project schedules or the class of build to purge for
class purge schedules.


=head2 $cron->getDescription()

Returns the description of the schedule.


=head2 $cron->getOwnerId()

Returns the user ID of the owner of the schedule.  Builds kicked off by the 
schedule will run as if they had been run by that user.


=head2 $cron->getProjectId()

Returns the project ID of the project that the schedule will create a build
of, or 0 for class purge schedules.


=head2 $cron->getSelectorId()

Returns the selector ID of the selector that builds from this schedule will use.
An empty string will cause the builds to use the scheduled project's selector.


=head2 $cron->getMinute()

Returns the crontab string for the minute(s) in which the schedule will run.


=head2 $cron->getHour()

Returns the crontab string for the hour(s) in which the schedule will run.


=head2 $cron->getDayOfWeek()

Returns the crontab string for the day(s) of the week in which the schedule will run.


=head2 $cron->getMonth()

Returns the crontab string for the month(s) in which the schedule will run.


=head2 $cron->getDayOfMonth()

Returns the crontab string for the day(s) of the month in which the schedule will run.


=head2 $cron->getEnvironmentGroupId()

Returns the environment group ID for the environment that the schedule will use.
The project's environment group is also applied - first the project's environment
group, then the schedule's environment group, so that the schedule's settings override the
project's settings if necessary.


=head2 $cron->getEnvGroupId()

An alias for $cron->getEnvironmentGroupId().
Returns the environment group ID for the environment that the schedule will use.
The project's environment group is also applied - first the project's environment
group, then the schedule's environment group, so that the schedule's settings override the
project's settings if necessary.


=head2 $cron->getEnvironmentText()

Returns the text of the schedule's specific environment.  If this is used,
it'll consist of a string with newline-delimited S:VARIABLE_NAME:New variable value
strings.


=head2 $cron->getEnvText()

An alias for $cron->getEnvironmentText().
Returns the text of the schedule's specific environment.  If this is used,
it'll consist of a string with newline-delimited S:VARIABLE_NAME:New variable value
strings.


=head2 $cron->getFired()

Returns the timestamp for the time that the scheduled build was last kicked off.


=head2 $cron->getNextRun()

Returns the timestamp for the time that the scheduled build will next be kicked off.
This may take 15 seconds or so to update from the time that schedule settings are changed.
A value of 0 means that the next run time has not yet been calculated, -1 means that one or 
more of the date / time fields have invalid values, and -2 means that the date / time will never
be matched - for example, if $cron->getMonth() == 2 and $cron->getDayOfMonth() == 31.


=head2 $cron->setActive(active)

Sets whether the schedule is active, inactive, or run once.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item active

May be 'YES' for active, 'NO' for inactive, or 'ONCE' for run once.

=back


=head2 $cron->setBuildClass(className)

Sets the class of builds kicked off by this schedule for project schedules or the 
class of builds to be purged for class purge schedules.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item className

The name/ID of the class (from $class->getName()), or '' to use the class of the
project being kicked off.  This cannot be set to '' for class purge schedules.

=back


=head2 $cron->setDescription(desc)

Sets the description of the schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item desc

The new schedule description.

=back


=head2 $cron->setProjectId(projectId)

Sets the project ID of the project to be kicked off by the schedule or 
makes this a class purge schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item projectId

The project ID for project schedules (from $project->getProjectId()), or
0 to make this a class purge schedule.

=back


=head2 $cron->setSelectorId(selectorId)

Sets the selector that the kicked off builds will use.  This setting is 
ignored for class purge schedules.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item selectorId

The selector ID to use (from $selector->getSelectorId()), or '' to use the selector
of the project being scheduled.

=back


=head2 $cron->setMinute(minute)

Sets the minute(s) in which to run this schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item minute

The minute in which to run this schedule in crontab format.

=back


=head2 $cron->setHour(hour)

Sets the hour(s) in which to run this schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item hour

The hour in which to run this schedule in crontab format.

=back


=head2 $cron->setDayOfWeek(dow)

Sets the day(s) of the week in which to run this schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item dow

The day(s) of the week in which to run this schedule in crontab format.

=back


=head2 $cron->setMonth(month)

Sets the month(s) in which to run this schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item month

The month(s) in which to run this schedule in crontab format.

=back


=head2 $cron->setDayOfMonth(dom)

Sets the day(s) of the month in which to run this schedule.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item dom

The day(s) of the month in which to run this schedule in crontab format.

=back


=head2 $cron->setEnvironmentGroupId(envGroupId)

Sets the environment group that the kicked off build will use.
The project's environment group is also applied - first the project's environment
group, then the schedule's environment group, so that the schedule's settings override the
project's settings if necessary.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item envGroupId

The ID of the environment group to use (from $environment->getEnvGroupId()), or 0 to
not use a schedule-specific environment group.

=back


=head2 $cron->setEnvGroupId(envGroupId)

An alias for $cron->setEnvironmentGroupId(envGroupId).
Sets the environment group that the kicked off build will use.
The project's environment group is also applied - first the project's environment
group, then the schedule's environment group, so that the schedule's settings override the
project's settings if necessary.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item envGroupId

The ID of the environment group to use (from $environment->getEnvGroupId()), or 0 to
not use a schedule-specific environment group.

=back


=head2 $cron->setEnvironmentText(envText)

Sets the schedule-specifc environment text of the schedule, if having a schedule-specific
environment is not desired.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item envText

Set this to '' to use the environment group from $cron->getEnvironmentGroupId()).
In order to set variables specifically in the schedule rather than in the environment
group, set this to a string consisting of newline-delimited S:VARIABLE_NAME:New variable value
strings.

=back


=head2 $cron->setEnvText(envText)

An alias for $cron->setEnvironmentText(envText).
Sets the schedule-specifc environment text of the schedule, if having a schedule-specific
environment is not desired.  $cron->update() must be run 
before changes are replicated in the database.

=over 8

=item envText

Set this to '' to use the environment group from $cron->getEnvironmentGroupId()).
In order to set variables specifically in the schedule rather than in the environment
group, set this to a string consisting of newline-delimited S:VARIABLE_NAME:New variable value
strings.


=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

