
package BuildForge::Services::DBO::Environment;
use BuildForge::Services::DBO::EnvironmentEntry;

use strict;
use overload '""' => \&toString;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(envGroupId level name);
my $prototype = {
	envGroupId         => undef,
	level              => undef,
	name               => undef,
};
sub getTypeKey { 'Env'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_entries'} = [];
	Integer(\$self->{'envGroupId'});
	Integer(\$self->{'level'});
	return $self;
}

sub findAll {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_ENV_GROUP_FIND_ALL);
	return _readList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $envGroupId) = @_;
	$conn->request(COMMAND_ENV_GROUP_FIND_ID);
	$conn->writeEntryString(KEY_ENV_ID, $envGroupId);
	return _readEnvironment($conn, $conn->call());
}

sub findByName {
	my ($proto, $conn, $environmentName) = @_;
	$conn->request(COMMAND_ENV_GROUP_FIND_NAME);
	$conn->writeEntryString(KEY_ENV_NAME, $environmentName);
	return _readEnvironment($conn, $conn->call());
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_ENV_GROUP_ADD);
	_writeEnvironment($self, $conn);
	my $resp = $conn->call;
	my $dbo = $resp->{+KEY_ENV_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or die $self->missing(KEY_ENV_DBO);
	my $entries = $self->{'_entries'};
	$self->fromArray($dbo);
	foreach my $entry (@$entries) { $entry->{'envGroupId'} = $self->{'envGroupId'}; }
	$self->{'_entries'} = $entries;
	return $self;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_ENV_GROUP_UPD);
	$conn->writeEntryDBO(KEY_ENV_DBO, $self);
	my $dbo = $conn->call()->{+KEY_ENV_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or die $self->missing(KEY_ENV_DBO);
	my $entries = $self->{'_entries'};
	$self->fromArray($dbo);
	$self->{'_entries'} = $entries;
	return $self;
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $envGroupId = $self->{'envGroupId'};
	$conn->request(COMMAND_ENV_GROUP_DEL);
	$conn->writeEntryString(KEY_ENV_ID, $envGroupId);
	$conn->call();
	$self->{'envGroupId'} = undef;
}

sub addEnvironmentEntry {
	my ($self, $entry, $index) = @_;
	_dbo_addEnvironmentEntry($self, $entry, $index);
	my $envGroupId = $self->{'envGroupId'};
	$envGroupId and $entry->create();
	$entry->{'_parent'} = $self;
}

sub moveEnvironmentEntry {
	my ($self, $fromIndex, $toIndex) = @_;
	my $entries = $self->{'_entries'};
	$fromIndex < 0 || $fromIndex >= @$entries
		and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex >= @$entries
		and die $self->badIndex($toIndex);
	my $envGroupId = $self->{'envGroupId'};
	if (defined($envGroupId) && length($envGroupId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_ENV_MOVE_ENTRY);
		$conn->writeEntryString(KEY_ENV_ID, $envGroupId);
		$conn->writeEntryString(KEY_ENV_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_ENV_TO_INDEX, $toIndex);
		$conn->call();
	}
	_dbo_moveEnvironmentEntry($self, $fromIndex, $toIndex);
	return $self;
}

sub copyEnvironmentEntry {
	my ($self, $fromIndex, $toIndex) = @_;
	my $conn = $self->{'_conn'};
	my $entries = $self->{'_entries'};
	$fromIndex < 0 || $fromIndex >= @$entries
		and die $self->badIndex($fromIndex);
	!defined $toIndex
		and return $entries->[$fromIndex]->clone();

	my $envGroupId = $self->{'envGroupId'};
	if (defined($envGroupId) && length($envGroupId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_ENV_COPY_ENTRY);
		$conn->writeEntryString(KEY_ENV_ID, $envGroupId);
		$conn->writeEntryString(KEY_ENV_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_ENV_TO_INDEX, $toIndex);
		$conn->call;
	}
	_dbo_copyEnvironmentEntry($self, $fromIndex, $toIndex);
}

sub removeEnvironmentEntry {
	my ($self, $index) = @_;
	!defined($index) and return undef;
	if (isa($index, 'BuildForge::Services::DBO::EnvironmentEntry')) {
		# This is removeEnvironmentEntry(EnvironmentEntry)
		$index = $index->getEntryOrdinal() - 1;
		# call to removeEnvironmentEntry(int) is handled by fall-through
	} elsif ($index !~ /^\d+$/) {
		# This is removeEnvironmentEntry(String)
		$index = $self->getEnvironmentEntry($index);
		!defined($index) and return undef;
		$index = $index->getEntryOrdinal() - 1;
		# call to removeEnvironmentEntry(int) is handled by fall-through
	}
	# removeEnvironmentEntry(int)
	my $entries = $self->{'_entries'};
	$index < 0 || $index >= @$entries
		and die $self->badIndex($index);

	my $entry = $entries->[$index];
	my $envGroupId = $self->{'envGroupId'};
	if (defined($envGroupId) && length($envGroupId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_ENV_ENTRY_DEL);
		$conn->writeEntryString(KEY_ENV_ID, $envGroupId);
		$conn->writeEntryString(KEY_ENV_ENTRY_ID, $entry->{'entryOrdinal'} - 1);
		$conn->call();
	}
	_dbo_removeEnvironmentEntry($self, $index);
	$entry->{'entryOrdinal'} = -1;
	$entry->{'live'} = 0;
	return $entry;
}

sub _readList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_ENV_LIST};
	!defined($list) and return [];
	my @envs = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@envs;
}

sub _readEnvironment($$) {
	my ($conn, $resp) = @_;
	my $environment = $resp->{+KEY_ENV_DBO};
	defined($environment) or return undef;
	$environment = __PACKAGE__->new($conn)->fromArray($environment);

	my $list = $resp->{+KEY_ENV_ENTRY_LIST};
	defined($list) && isa($list, 'ARRAY') or return $environment;

	my @entries = map { BuildForge::Services::DBO::EnvironmentEntry->new($conn, $environment)->fromArray($_) } @$list;
	$environment->{'_entries'} = \@entries;
	return $environment;
}


sub _writeEnvironment($$) {
	my ($self, $conn) = @_;
	$conn->writeEntryDBO(KEY_ENV_DBO, $self);
	my $entries = $self->{'_entries'};
	defined($entries) && @$entries > 0
		and $conn->writeEntryArray(KEY_ENV_ENTRY_LIST, $entries);
}

# wrap is inherited

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self);
	my @entries = map { $_->clone } @{ $self->{'_entries'} };
	foreach my $entry (@entries)  { $entry->{'_parent'} = $clone; }
	$clone->{'_entries'} = \@entries;
	$clone->{'envGroupId'} = undef;
	$clone->{'name'} = undef;
	return $clone;
}

sub toString {
	my ($self) = @_;
	return '[EnvironmentEntry: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ', _entries=' . scalar @{ $self->{'_entries'} } . ']';
}

# Aliases for convenience
sub getEnvGroupId { $_[0]->{'envGroupId'}; }
sub getEnvironmentId { $_[0]->{'envGroupId'}; }
sub getEnvGroupName { $_[0]->{'name'}; }
sub getEnvironmentName { $_[0]->{'name'}; }
sub getLevel { $_[0]->{'level'}; }
sub getEntries {
	my $list = $_[0]->{'_entries'};
	return defined($list) ? [@{ $list }] : undef;
}

sub getEntry {
	my ($self, $index) = @_;
	my $entries = $self->{'_entries'};
	# If the argument doesn't look like an index, or if it is out of range,
	# then assume it is the name of the entry, instead.
	$index =~ /\D/ || $index < 0 || $index >= @$entries
		and return $self->getEnvironmentEntry($index);

	# This is getEntry(int).  No need to check index, because we treat
	# an invalid index as a string, above...
	return $entries->[$index];
}


sub setEnvGroupName { $_[0]->{'name'} = $_[1]; }
sub setEnvironmentName { $_[0]->{'name'} = $_[1]; }
sub setLevel { $_[0]->{'level'} = $_[1]; }


#----------------------------------------------------------------
# DBO Methods
#----------------------------------------------------------------

# _dbo_setLiveEnvironmentId is not needed

sub _dbo_setEnvironmentId($$) {
	my ($self, $envGroupId) = @_;
	$self->{'envGroupId'} = $envGroupId;
	my $entries = $self->{'_entries'};
	if (isa($entries, 'ARRAY')) {
		if (defined($entries) && @$entries > 0) {
			foreach my $entry (@$entries) { $entry->setEnvironmentId($envGroupId); }
		}
	}
}

sub _dbo_addEnvironmentEntry ($$;$) {
	my ($self, $entry, $index) = @_;
	my $entries = $self->{'_entries'};
	$entries = $self->{'_entries'} = [] unless isa($entries, 'ARRAY');
	$entry->{'_parent'} = $self;

	if (!defined($index) || $index >= @$entries) {
		# This is addEnvironmentEntry(EnvironmentEntryDBO) or
		# addEnvironmentEntry(EnvironmentEntryDBO, int) with an
		# index that puts it at or past the end of the list
		$entry->{'entryOrdinal'} = @$entries + 1;
		$entry->{'envGroupId'} = $self->{'envGroupId'};
		push @$entries, $entry;
		return;
	}

	# This is addEnvironmentEntry(EnvironmentEntryDBO, int)
	$index < 0 && die $self->badIndex($index);
	# $index >= @$entries case handled above
	$entry->{'entryOrdinal'} = $index + 1;
	$entry->{'envGroupId'} = $self->{'envGroupId'};
	splice @{$self->{'_entries'}}, $index, 0, $entry;
	_dbo_updateIndices($self, $index + 1);
}


sub getEnvironmentEntry {
	my ($self, $name) = @_;
	my $index = _dbo_findIndex($self, $name);
	my $entries = $self->{'_entries'};
	!isa($entries, 'ARRAY') and return undef;
	return $index >= 0 ? $self->{'_entries'}->[$index] : undef;
}

sub _dbo_removeEnvironmentEntry ($$) {
	my ($self, $index) = @_;
	my $entries = $self->{'_entries'};
	!isa($entries, 'ARRAY') || $index < 0 || $index >= @$entries
		and die $self->badIndex($index);
	my ($entry) = splice @$entries, $index, 1;
	_dbo_updateIndices($self, $index);
	return $entry;
}

sub _dbo_moveEnvironmentEntry ($$$) {
	my ($self, $fromIndex, $toIndex) = @_;
	my $entries = $self->{'_entries'};
	# fromIndex is checked by _dbo_removeEnvironmentEntry
	# toIndex >= the array size means move to the end
	!isa($entries, 'ARRAY') || $fromIndex < 0
		and die $self->badIndex($fromIndex);
	_dbo_addEnvironmentEntry($self, _dbo_removeEnvironmentEntry($self, $fromIndex), $toIndex);
}

sub _dbo_copyEnvironmentEntry ($$;$) {
	my ($self, $fromIndex, $toIndex) = @_;

	my $entries = $self->{'_entries'};
	!isa($entries, 'ARRAY') || $fromIndex < 0 || $fromIndex >= @$entries
		and die $self->badIndex($fromIndex);
	my $entry = $entries->[$fromIndex];

	if (!defined($toIndex)) {
		# This is copyEnvironmentEntry(int);
		return $entry->clone;
	}

	# This is copyEnvironmentEntry(int, int);
	$toIndex < 0 || $toIndex > @$entries
		and die $self->badIndex($toIndex);
	_dbo_addEnvironmentEntry($self, $entry->clone, $toIndex);
}

# _dbo_toString is not needed
# _dbo_clone is not needed
# getTypeCode, fromArray, and toArray are defined above

sub _dbo_findIndex ($$) {
	my ($self, $name) = @_;
	my $entries = $self->{'_entries'};
	!isa($entries, 'ARRAY') and return -1;
	for (my $i=0; $i<@$entries; $i++) {
		return $i if $entries->[$i]->{'entryName'} eq $name;
	}
	return -1;
}

sub _dbo_updateIndices ($$) {
	my ($self, $startingHere) = @_;
	my $envGroupId = $self->{'envGroupId'};
	my $entries = $self->{'_entries'};
	if (isa($entries, 'ARRAY')) {
		for (my $i=$startingHere; $i<@$entries; $i++) {
			my $entry = $entries->[$i];
			$entry->{'entryOrdinal'} = $i + 1;
		}
	}
}

__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Environment


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing environments
	$allEnvironments = BuildForge::Services::DBO::Environment->findAll($conn);
	$oldenvironment = BuildForge::Services::DBO::Environment->findById($conn, $envId);
	$oldenvironment = BuildForge::Services::DBO::Environment->findByName($conn, 'EnvironmentName');

	# Getter / setter functions
	$id = $oldenvironment->getEnvGroupId();
	$name = $oldenvironment->getEnvGroupName();
	$name = $oldenvironment->getEnvironmentName();
	$accessGroupId = $oldenvironment->getLevel();
	$variable1 = $oldenvironment->getEntry(0);
	$variable2 = $oldenvironment->getEntry('VARIABLE_NAME');
	$variables = $oldenvironment->getEntries();
	
	$oldenvironment->setEnvGroupName('New Name');
	$oldenvironment->setEnvironmentName('New Name');
	$level = BuildForge::Services::DBO::AccessGroup->findByName($conn, 'MyAccessGroup');
	$oldenvironment->setLevel($level->getLevel());
	$oldenvironment->update();
	
	# Environment creation, modification, deletion
	$newenvironment = new BuildForge::Services::DBO::Environment($conn);
	$newenvironment->setEnvironmentName('New Environment');
	$newenvironment->setLevel(4);
	$newenvironment->create();
	
	$newenvironment2 = $newenvironment->clone();
	$newenvironment2->setEnvironmentName('Duplicate');
	$newenvironment2->create();
	
	$newenvironment->setEnvironmentName('New Name For New Environment');
	$newenvironment->update();

	$var = new BuildForge::Services::DBO::EnvironmentEntry($conn);
	$var->setEntryName('VARIABLE_NAME');
	$var->setEntryValue('Value');
	$newenvironment->addEnvironmentEntry($var);
	$newenvironment->copyEnvironmentEntry(0, 1);
	$newenvironment->moveEnvironmentEntry(1, 0);
	$newenvironment->removeEnvironmentEntry(1);
	$newenvironment->removeEnvironmentEntry($var);
	
	$newenvironment->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Environment allows you to create, modify, and delete environment groups.


=head1 METHODS


=head2 new BuildForge::Services::DBO::Environment(connection, {EnvParamsHashRef})

Returns a new instance of an Environment object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item EnvParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item name

The name of the environment group.

=item level

The access group ID (from $accessGroup->getLevel()) of the access group the 
environment belongs to.  Users who do not belong to this access group will not
be able to view, edit, or use the environment.

=back

=back


=head2 BuildForge::Services::DBO::Environment->findAll(connection)

Returns a reference to an array of all the BuildForge::Services::DBO::Environment
objects the user has access to in the database.  These objects will not
contain their EnvironmentEntry subobjects.  To get the entries, use 
BuildForge::Services::DBO::Environment->findById() or findByName() with the environment's ID.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Environment->findById(connection, envGroupId)

Returns the BuildForge::Services::DBO::Environment object corresponding to the
given environment group ID, or undef if no such environment group exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item envGroupId

The ID of the desired environment group (from $environment->getEnvironmentGroupId()).

=back


=head2 BuildForge::Services::DBO::Environment->findByName(connection, envGroupName)

Returns the BuildForge::Services::DBO::Environment object corresponding to the
given environment group name, or undef if no such environment group exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item envGroupName

The name of the desired environment group (from $environment->getEnvironmentGroupName()).

=back


=head2 $environment->create()

Creates the environment group within the database.


=head2 $environment->clone()

Returns a copy of the BuildForge::Services::DBO::Environment object and all of its 
EnvironmentEntry variables.  The copy will not exist in the database unless it is created
with $environment->create().


=head2 $environment->update()

Updates the database with the new environment group settings.


=head2 $environment->addEnvironmentEntry(entry, index)

Adds an environment variable to the environment group within the database.

=over 8

=item entry

The BuildForge::Services::DBO::EnvironmentEntry object to be added.

=item index

The optional location at which to add the entry within the list.  0 adds it to the
front of the list, 1 adds it after the first variable, etc.  Leaving it unset
adds the variable to the end of the variable list.

=back


=head2 $environment->moveEnvironmentEntry(from, to)

Moves an environment variable within the list of variables in the database.

=over 8

=item from

The index of the variable to move.  0 moves the first variable, 1 moves the second, etc.

=item to

The index of the new location for the variable.  0 moves it to the first position,
1 to the second, etc.

=back


=head2 $environment->copyEnvironmentEntry(from, to)

Copies an environment variable within the environment group in the database.

=over 8

=item from

The index of the variable to be copied.  0 copies the first variable, 1 copies the second, etc.

=item to

The index of the location for the new variable copy.  0 copies it to the first position,
1 to the second, etc.

=back


=head2 $environment->removeEnvironmentEntry(variable)

Removes an environment variable from the variable list in the database.

=over 8

=item variable

This may be an index, in which case the variable at that position is deleted (0
deletes the first variable, 1 the second, etc.), or a BuildForge::Services::DBO::EnvironmentEntry
object, in which case that environment variable is the one deleted.

=back


=head2 $environment->delete()

Deletes the environment and all of its variables from the database.


=head2 $environment->getEnvGroupId()

Returns the ID of the environment group.  Identical to $environment->getEnvironmentId().


=head2 $environment->getEnvironmentId()

Returns the ID of the environment group.  Identical to $environment->getEnvGroupId().


=head2 $environment->getEnvGroupName()

Returns the name of the environment group.  Identical to $environment->getEnvironmentName().


=head2 $environment->getEnvironmentName()

Returns the name of the environment group.  Identical to $environment->getEnvGroupName().


=head2 $environment->getLevel()

Returns the access group ID of the access group the environment belongs to.
Users who are not in this access group cannot view, edit, or use the
environment group.


=head2 $environment->getEntry(variable)

Returns the BuildForge::Services::DBO::EnvironmentEntry object that corresponds
to the requested variable.

=over 8

=item variable

This may be an index, in which case that variable is the one returned (0 for the
first variable, 1 for the second, etc.), or a variable name, in which case the 
first variable with that name in the environment group is returned.

=back


=head2 $environment->getEntries()

Returns a reference to an array of BuildForge::Services::DBO::EnvironmentEntry 
objects for all variables in the environment group.


=head2 $environment->setEnvGroupName(envGroupName)

Sets the environment group's name.  Identical to $environment->setEnvironmentName().
$environment->update() must be run before changes are replicated in the database.

=over 8

=item envGroupName

The new name for the environment group.

=back


=head2 $environment->setEnvironmentName(envGroupName)

Sets the environment group's name.  Identical to $environment->setEnvGroupName().
$environment->update() must be run before changes are replicated in the database.

=over 8

=item envGroupName

The new name for the environment group.

=back


=head2 $environment->setLevel(accessGroupId)

Sets the environment group's access group.  Users who do not belong to this access group will not
be able to view, edit, or use the environment.  $environment->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) for the desired access group.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

