
package BuildForge::Services::DBO::EnvironmentEntry;

use strict;
use overload '""' => \&toString;

use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(envGroupId entryOrdinal entryName entryValue
	action mode);
my $prototype = {
	_parent             => undef,
	envGroupId          => undef,
	entryOrdinal			 => undef,
	entryName        => undef,
	entryValue       => undef,
	action            => ENUM_ENVIRONMENTENTRY_ACTION->SET,
	mode            => ENUM_ENVIRONMENTENTRY_MODE->NORMAL,
};
sub getTypeKey { 'EnvVar'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my ($newParent) = splice(@_, 1, 1);
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_parent'} = $newParent;
	Integer(\$self->{'envGroupId'});
	Integer(\$self->{'entryOrdinal'});
	Enum(ENUM_ENVIRONMENTENTRY_ACTION, \$self->{'action'});
	Enum(ENUM_ENVIRONMENTENTRY_MODE, \$self->{'mode'});
	return $self;
}


# INTERNAL USE ONLY
sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->internal();
	$conn->request(COMMAND_ENV_ENTRY_ADD);
	$conn->writeEntryDBO(KEY_ENV_ENTRY_DBO, $self);
	my $dbo = $conn->call()->{KEY_ENV_ENTRY_DBO()};
	defined($dbo) or die $self->missing(KEY_ENV_ENTRY_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $envGroupId = $self->{'_parent'}->{'envGroupId'};
	if (defined($envGroupId) && length($envGroupId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_ENV_ENTRY_UPD);
		$conn->writeEntryDBO(KEY_ENV_ENTRY_DBO, $self);
		my $dbo = $conn->call()->{KEY_ENV_ENTRY_DBO()};
		defined($dbo) or die $self->missing(KEY_ENV_ENTRY_DBO);
		return $self->fromArray($dbo);
	}
	return $self;
}

sub delete { $_[0]->{'_parent'}->removeEnvironmentEntry($_[0]); }

sub moveTo { $_[0]->{'_parent'}->moveEnvironmentEntry($_[0]->getEntryOrdinal - 1, $_[1]); }

# wrap is inherited

sub getParent { $_[0]->{'_parent'}; }

# setParent is not needed

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self->{'_parent'});
	$clone->{'_parent'} = undef;
	return $clone;
}

sub getEnvGroupId            { $_[0]->{'envGroupId'}; }
sub getEntryOrdinal            { $_[0]->{'entryOrdinal'}; }
sub getEntryName        { $_[0]->{'entryName'}; }
sub getEntryValue       { $_[0]->{'entryValue'}; }
sub getAction            { $_[0]->{'action'}; }
sub getMode          { $_[0]->{'mode'}; }

sub setEntryName        { $_[0]->{'entryName'}  = $_[1]; }
sub setEntryValue       { $_[0]->{'entryValue'} = $_[1]; }
sub setAction         { $_[0]->{'action'}      = $_[1]; }
sub setMode            { $_[0]->{'mode'}      = $_[1]; }


sub toString {
	my ($self) = @_;
	return '[EnvironmentEntry: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ']';
}


1;



__END__

=pod

=head1 NAME

BuildForge::Services::DBO::EnvironmentEntry


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	$entry = new BuildForge::Services::DBO::EnvironmentEntry($conn);

	# Getter / setter functions
	$environmentGroupId = $entry->getEnvGroupId();
	$ordinal = $entry->getEntryOrdinal();
	$name = $entry->getEntryName();
	$value = $entry->getEntryValue();
	$action = $entry->getAction();
	$mode = $entry->getMode();
	$envGroup = $entry->getParent();

	$entry->setEntryName('PROP_NAME');
	$entry->setEntryValue('New Value');
	$entry->setAction(ENUM_ENVIRONMENTENTRY_ACTION->APPEND);
	$entry->setMode(ENUM_ENVIRONMENTENTRY_MODE->READ_ONLY);
	$entry->update();
	
	# Entry creation, modification, and deletion.
	$envGroup = BuildForge::Services::DBO::Environment->findByName($conn, 'EnvironmentName');
	$newentry = new BuildForge::Services::DBO::EnvironmentEntry($conn);
	$newentry->setEntryName('VARIABLE_NAME');
	$newentry->setEntryValue('Value');
	$envGroup->addEnvironmentEntry($var);

	$newentry->setEntryValue('New Value');
	$newentry->update();
	
	$newentry2 = $newentry->clone();
	$envGroup->addEnvironmentEntry($newentry2);
	
	$newentry->moveTo(2);

	$newentry->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

EnvironmentEntry allows you to create and modify environment variables inside of an Environment.
To create the environment variables in the database, see the Environment->addEnvironmentEntry() function.


=head1 METHODS


=head2 new BuildForge::Services::DBO::EnvironmentEntry(connection, {environment}, {EnvEntryParamsHashRef})

Returns a new instance of an EnvironmentEntry object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item environment

This should be undef or not passed - it is only used internally.

=item EnvEntryParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item entryName

The name of the environment variable.

=item entryValue

The value of the environment variable, defaulting to none.

=item action

The variable's action.  This may be any of:

=over 8

=item SET

The default.  Will cause the variable to be set to its value.

=item SET_IF_NOT_SET

Will cause the variable to be set to the value if the variable value has
not already been set.

=item APPEND

Causes the variable value to be appended to the previous value separated by a system-specific
path delimiter (':' for Unix systems, ';' for Windows systems).

=item PREPEND

Causes the variable value to be prepended to the previous value separated by a system-specific
path delimiter (':' for Unix systems, ';' for Windows systems).

=item CLEAR

Sets the variable's value to the empty string.  The entryValue will be disregarded.

=item UNSET

Deletes the variable from the variable list.  This differs from clearing it in that the former
will print nothing when the variable is printed and the latter will (normally) print the 
variable name.

=item HIDDEN

Makes this variable a hidden variable.  Hidden variables do not appear in the list of variables
output by step results.

=back

=item mode

Affects what happens when a project is kicked off with the variable in the project's 
environment.  This may be any of:

=over 8

=item NORMAL

The default.  The variable acts normally.

=item MUST_CHANGE

The project cannot be quickstarted, and when the build is kicked off, the value
of the variable must change.

=item READ_ONLY

When the project build is being kicked off, the value of the variable cannot be
changed.

=item REQUIRED

The project cannot be quickstarted if the variable has no initial value, and when 
the build is kicked off, the value of the variable cannot be empty.

=item SUPPRESS_DISPLAY

The variable is not displayed in the project environment variable list on the 
project start page.

=back

=back

=back


=head2 $variable->clone()

A copy of the EnvironmentEntry object is returned without affecting the database.
To save the copy to an environment group, $environment->addEnvironmentEntry($copy) 
must be run on the copy.


=head2 $variable->update()

Updates the variable settings in the database.


=head2 $variable->moveTo(index)

Moves the variable within its parent environment group.

=over 8

=item index

The new location of the variable.  0 moves it to the first position, 1 to
the second position, etc.

=back


=head2 $variable->delete()

Deletes the variable from the database.


=head2 $variable->getParent()

Returns the BuildForge::Services::DBO::Environment object for the environment
group this variable belongs to.


=head2 $variable->getEnvGroupId()

Returns the environment group ID for the environment group this variable
belongs to.


=head2 $variable->getEntryOrdinal()

Returns the ordinal for the position of the variable in the variable list.
1 means it's the first variable, 2 means it's the second, etc.


=head2 $variable->getEntryName()

Returns the variable name.


=head2 $variable->getEntryValue()

Returns the variable value.


=head2 $variable->getAction()

Returns the variable action which controls how the variable value is set.  This may be 
any of 'SET', 'SET_IF_NOT_SET', 'APPEND', 'PREPEND', 'CLEAR', 'UNSET', or 'HIDDEN'.


=head2 $variable->getMode()

Returns the variable mode which controls how the variable behaves when it is in an
environment group that is connected to a project and a build of that project is 
kicked off.


=head2 $variable->setEntryName(name)

Sets the name of the variable.  $variable->update() must be run 
before changes are replicated in the database.

=over 8

=item name

The new name for the variable.

=back


=head2 $variable->setEntryValue(value)

Sets the value of the variable.  $variable->update() must be run 
before changes are replicated in the database.

=over 8

=item value

The new variable value.

=back


=head2 $variable->setAction(action)

Sets the variable action which controls how the variable value is set.   
$variable->update() must be run before changes are replicated in the database.

=over 8

=item action

May be any of:

=over 8

=item SET

Will cause the variable to be set to its value.

=item SET_IF_NOT_SET

Will cause the variable to be set to the value if the variable value has
not already been set.

=item APPEND

Causes the variable value to be appended to the previous value separated by a system-specific
path delimiter (':' for Unix systems, ';' for Windows systems).

=item PREPEND

Causes the variable value to be prepended to the previous value separated by a system-specific
path delimiter (':' for Unix systems, ';' for Windows systems).

=item CLEAR

Sets the variable's value to the empty string.  The entryValue will be disregarded.

=item UNSET

Deletes the variable from the variable list.  This differs from clearing it in that the former
will print nothing when the variable is printed and the latter will (normally) print the 
variable name.

=item HIDDEN

Makes this variable a hidden variable.  Hidden variables do not appear in the list of variables
output by step results.

=back

=back


=head2 $variable->setMode(mode)

Sets the variable mode which controls how the variable behaves when it is in an
environment group that is connected to a project and a build of that project is 
kicked off.  $variable->update() must be run 
before changes are replicated in the database.

=over 8

=item mode

May be any of:

=over 8

=item NORMAL

The default.  The variable acts normally.

=item MUST_CHANGE

The project cannot be quickstarted, and when the build is kicked off, the value
of the variable must change.

=item READ_ONLY

When the project build is being kicked off, the value of the variable cannot be
changed.

=item REQUIRED

The project cannot be quickstarted if the variable has no initial value, and when 
the build is kicked off, the value of the variable cannot be empty.

=item SUPPRESS_DISPLAY

The variable is not displayed in the project environment variable list on the 
project start page.

=back

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

