
package BuildForge::Services::DBO::Filter;
use BuildForge::Services::DBO::FilterPattern;

use strict;
use overload '""' => \&toString;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(filterId name level);
my $prototype = {
	filterId	=> undef,
	name		=> undef,
	level		=> undef,
};
sub getTypeKey { 'Filter'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_patterns'} = [];
	Integer(\$self->{'filterId'});
	Integer(\$self->{'level'});
	return $self;
}

sub findAll {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_FILTER_SET_FIND_ALL);
	return _readList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $filterId) = @_;
	$conn->request(COMMAND_FILTER_SET_FIND_ID);
	$conn->writeEntryString(KEY_FILTER_ID, $filterId);
	return _readFilter($conn, $conn->call());
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_FILTER_SET_ADD);
	_writeFilter($self, $conn);
	my $resp = $conn->call;
	my $dbo = $resp->{+KEY_FILTER_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or die $self->missing(KEY_FILTER_DBO);
	my $props = $self->{'_patterns'};
	$self->fromArray($dbo);
	foreach my $prop (@$props) { $prop->{'filterId'} = $self->{'filterId'}; }
	$self->{'_patterns'} = $props;
	return $self;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_FILTER_SET_UPD);
	$conn->writeEntryDBO(KEY_FILTER_DBO, $self);
	my $dbo = $conn->call()->{+KEY_FILTER_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or die $self->missing(KEY_FILTER_DBO);
	my $props = $self->{'_patterns'};
	$self->fromArray($dbo);
	$self->{'_patterns'} = $props;
	return $self;
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $filterId = $self->{'filterId'};
	$conn->request(COMMAND_FILTER_SET_DEL);
	$conn->writeEntryString(KEY_FILTER_ID, $filterId);
	$conn->call();
	$self->{'name'} = undef;
}


sub addFilterPattern {
	my ($self, $prop, $index) = @_;
	_dbo_addFilterPattern($self, $prop, $index);
	my $filterId = $self->{'filterId'};
	$filterId and $prop->create();
	$prop->{'_parent'} = $self;
}

sub moveFilterPattern {
	my ($self, $fromIndex, $toIndex) = @_;
	my $patterns = $self->{'_patterns'};
	$fromIndex < 0 || $fromIndex >= @$patterns
		and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex >= @$patterns
		and die $self->badIndex($toIndex);
	my $filterId = $self->{'filterId'};
	if (defined($filterId) && length($filterId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_FILTER_MOVE_ENTRY);
		$conn->writeEntryString(KEY_FILTER_ID, $filterId);
		$conn->writeEntryString(KEY_FILTER_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_FILTER_TO_INDEX, $toIndex);
		$conn->call();
	}
	_dbo_moveFilterPattern($self, $fromIndex, $toIndex);
	return $self;
}

sub copyFilterPattern {
	my ($self, $fromIndex, $toIndex) = @_;
	my $conn = $self->{'_conn'};
	my $patterns = $self->{'_patterns'};
	$fromIndex < 0 || $fromIndex >= @$patterns
		and die $self->badIndex($fromIndex);
	!defined $toIndex
		and return $patterns->[$fromIndex]->clone();

	my $filterId = $self->{'filterId'};
	if (defined($filterId) && length($filterId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_FILTER_COPY_ENTRY);
		$conn->writeEntryString(KEY_FILTER_ID, $filterId);
		$conn->writeEntryString(KEY_FILTER_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_FILTER_TO_INDEX, $toIndex);
		$conn->call;
	}
	_dbo_copyFilterPattern($self, $fromIndex, $toIndex);
}

sub removeFilterPattern {
	my ($self, $index) = @_;
	!defined($index) and return undef;
	if (isa($index, 'BuildForge::Services::DBO::FilterPattern')) {
		# This is removeFilterPattern(FilterPattern)
		$index = $index->getPatternOrdinal() - 1;
		# call to removeFilterPattern(int) is handled by fall-through
	} elsif ($index !~ /^\d+$/) {
		# This is removeFilterPattern(String)
		$index = $self->getFilterPattern($index);
		!defined($index) and return undef;
		$index = $index->getPatternOrdinal() - 1;
		# call to removeFilterPattern(int) is handled by fall-through
	}
	# removeFilterPattern(int)
	my $patterns = $self->{'_patterns'};
	$index < 0 || $index >= @$patterns
		and die $self->badIndex($index);

	my $prop = $patterns->[$index];
	my $filterId = $self->{'filterId'};
	if (defined($filterId) && length($filterId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_FILTER_ENTRY_DEL);
		$conn->writeEntryString(KEY_FILTER_PATTERN_FILTER_ID, $filterId);
		$conn->writeEntryString(KEY_FILTER_PATTERN_ORDINAL, $prop->{'filterOrdinal'} - 1);
		$conn->call();
	}
	_dbo_removeFilterPattern($self, $index);
	$prop->{'filterOrdinal'} = -1;
	$prop->{'live'} = 0;
	return $prop;
}

sub _readList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_FILTER_LIST};
	!defined($list) and return [];
	my @filters = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@filters;
}

sub _readFilter($$) {
	my ($conn, $resp) = @_;
	my $filter = $resp->{+KEY_FILTER_DBO};
	defined($filter) or return undef;
	$filter = __PACKAGE__->new($conn)->fromArray($filter);

	my $list = $resp->{+KEY_FILTER_PATTERN_LIST};
	defined($list) && isa($list, 'ARRAY') or return $filter;

	my @patterns = map { BuildForge::Services::DBO::FilterPattern->new($conn, $filter)->fromArray($_) } @$list;
	$filter->{'_patterns'} = \@patterns;
	return $filter;
}


sub _writeFilter($$) {
	my ($self, $conn) = @_;
	$conn->writeEntryDBO(KEY_FILTER_DBO, $self);
	my $patterns = $self->{'_patterns'};
	defined($patterns) && @$patterns > 0
		and $conn->writeEntryArray(KEY_FILTER_PATTERN_LIST, $patterns);
}

# wrap is inherited

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self);
	my @patterns = map { $_->clone } @{ $self->{'_patterns'} };
	$clone->{'_patterns'} = \@patterns;
	$clone->{'filterId'} = undef;
	$clone->{'name'} = undef;
	return $clone;
}

sub toString {
	my ($self) = @_;
	return '[FilterPattern: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ', _patterns=' . scalar @{ $self->{'_patterns'} } . ']';
}

# Aliases for convenience
sub getFilterId { $_[0]->{'filterId'}; }
sub getFilterName { $_[0]->{'name'}; }
sub getLevel { $_[0]->{'level'}; }
sub getPatterns {
	my $list = $_[0]->{'_patterns'};
	return defined($list) ? [@{ $list }] : undef;
}

sub getPattern {
	my ($self, $index) = @_;
	my $patterns = $self->{'_patterns'};
	# If the argument doesn't look like an index, or if it is out of range,
	# then assume it is the name of the pattern, instead.
	$index =~ /\D/ || $index < 0 || $index >= @$patterns
		and return $self->getFilterPattern($index);

	# This is getPattern(int).  No need to check index, because we treat
	# an invalid index as a string, above...
	return $patterns->[$index];
}


sub setFilterName { $_[0]->{'name'} = $_[1]; }
sub setLevel { $_[0]->{'level'} = $_[1]; }


#----------------------------------------------------------------
# DBO Methods
#----------------------------------------------------------------

# _dbo_setLiveFilterId is not needed

sub _dbo_setFilterId($$) {
	my ($self, $filterId) = @_;
	$self->{'filterId'} = $filterId;
	my $patterns = $self->{'_patterns'};
	if (isa($patterns, 'ARRAY')) {
		if (defined($patterns) && @$patterns > 0) {
			foreach my $prop (@$patterns) { $prop->setFilterId($filterId); }
		}
	}
}

# _dbo_setProperties is not needed

sub _dbo_addFilterPattern ($$;$) {
	my ($self, $prop, $index) = @_;
	my $patterns = $self->{'_patterns'};
	$patterns = $self->{'_patterns'} = [] unless isa($patterns, 'ARRAY');
	$prop->{'_parent'} = $self;

	if (!defined($index) || $index >= @$patterns) {
		# This is addFilterPattern(FilterPatternDBO) or
		# addFilterPattern(FilterPatternDBO, int) with an
		# index that puts it at or past the end of the list
		$prop->{'filterOrdinal'} = @$patterns + 1;
		$prop->{'filterId'} = $self->{'filterId'};
		push @$patterns, $prop;
		return;
	}

	# This is addFilterPattern(FilterPatternDBO, int)
	$index < 0 && die $self->badIndex($index);
	# $index >= @$patterns case handled above
	$prop->{'filterOrdinal'} = $index + 1;
	$prop->{'filterId'} = $self->{'filterId'};
	splice @{$self->{'_patterns'}}, $index, 0, $prop;
	_dbo_updateIndices($self, $index + 1);
}


sub getFilterPattern {
	my ($self, $name) = @_;
	my $index = _dbo_findIndex($self, $name);
	my $patterns = $self->{'_patterns'};
	!isa($patterns, 'ARRAY') and return undef;
	return $index >= 0 ? $self->{'_patterns'}->[$index] : undef;
}

sub _dbo_removeFilterPattern ($$) {
	my ($self, $index) = @_;
	my $patterns = $self->{'_patterns'};
	!isa($patterns, 'ARRAY') || $index < 0 || $index >= @$patterns
		and die $self->badIndex($index);
	my ($prop) = splice @$patterns, $index, 1;
	_dbo_updateIndices($self, $index);
	return $prop;
}

sub _dbo_moveFilterPattern ($$$) {
	my ($self, $fromIndex, $toIndex) = @_;
	my $patterns = $self->{'_patterns'};
	# fromIndex is checked by _dbo_removeFilterPattern
	# toIndex >= the array size means move to the end
	!isa($patterns, 'ARRAY') || $fromIndex < 0
		and die $self->badIndex($fromIndex);
	_dbo_addFilterPattern($self, _dbo_removeFilterPattern($self, $fromIndex), $toIndex);
}

sub _dbo_copyFilterPattern ($$;$) {
	my ($self, $fromIndex, $toIndex) = @_;

	my $patterns = $self->{'_patterns'};
	!isa($patterns, 'ARRAY') || $fromIndex < 0 || $fromIndex >= @$patterns
		and die $self->badIndex($fromIndex);
	my $prop = $patterns->[$fromIndex];

	if (!defined($toIndex)) {
		# This is copyFilterPattern(int);
		return $prop->clone;
	}

	# This is copyFilterPattern(int, int);
	$toIndex < 0 || $toIndex > @$patterns
		and die $self->badIndex($toIndex);
	_dbo_addFilterPattern($self, $prop->clone, $toIndex);
}

# _dbo_toString is not needed
# _dbo_clone is not needed
# getTypeCode, fromArray, and toArray are defined above

sub _dbo_findIndex ($$) {
	my ($self, $name) = @_;
	my $patterns = $self->{'_patterns'};
	!isa($patterns, 'ARRAY') and return -1;
	for (my $i=0; $i<@$patterns; $i++) {
		return $i if ($patterns->[$i]->{'pattern'} eq $name);
	}
	return -1;
}

sub _dbo_updateIndices ($$) {
	my ($self, $startingHere) = @_;
	my $filterId = $self->{'filterId'};
	my $patterns = $self->{'_patterns'};
	if (isa($patterns, 'ARRAY')) {
		for (my $i=$startingHere; $i<@$patterns; $i++) {
			my $prop = $patterns->[$i];
			$prop->{'filterOrdinal'} = $i + 1;
		}
	}
}

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Filter


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing filters
	$allFilters = BuildForge::Services::DBO::Filter->findAll($conn);
	$oldfilter = BuildForge::Services::DBO::Filter->findById($conn, $filterId);

	# Getter / setter functions
	$id = $oldfilter->getFilterId();
	$name = $oldfilter->getFilterName();
	$accessGroupId = $oldfilter->getLevel();
	$pattern1 = $oldfilter->getPattern(1);
	$pattern2 = $oldfilter->getPattern('VARIABLE_NAME');
	$patterns = $oldfilter->getPatterns();
	
	$oldfilter->setFilterName('New Name');
	$level = BuildForge::Services::DBO::AccessGroup->findByName($conn, 'MyAccessGroup');
	$oldfilter->setLevel($level->getLevel());
	$oldfilter->update();
	
	# Filter creation, modification, deletion
	$newfilter = new BuildForge::Services::DBO::Filter($conn);
	$newfilter->setFilterName('New Filter');
	$newfilter->create();
	
	$newfilter2 = $newfilter->clone();
	$newfilter2->setFilterName('Duplicate');
	$newfilter2->create();
	
	$newfilter->setFilterName('New Name For New Filter');
	$newfilter->update();

	$var = new BuildForge::Services::DBO::FilterPattern($conn);
	$var->setPattern('VARIABLE_NAME');
	$newfilter->addFilterPattern($var);
	$newfilter->copyFilterPattern(0, 1);
	$newfilter->moveFilterPattern(1, 0);
	$newfilter->removeFilterPattern(1);
	$newfilter->removeFilterPattern($var);
	
	$newfilter->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Filter allows you to create, modify, and delete filter groups.


=head1 METHODS


=head1 METHODS


=head2 new BuildForge::Services::DBO::Filter(connection, {FilterParamsHashRef})

Returns a new instance of a Filter object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item FilterParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item name

The name of the filter group.

=item level

The access group ID (from $accessGroup->getLevel()) of the access group the 
filter belongs to.  Users who do not belong to this access group will not
be able to view, edit, or use the filter.

=back

=back


=head2 BuildForge::Services::DBO::Filter->findAll(connection)

Returns a reference to an array of all the BuildForge::Services::DBO::Filter
objects the user has access to in the database.  These objects will not
contain their FilterPattern subobjects.  To get the patterns, use 
BuildForge::Services::DBO::Filter->findById() with the filter's ID.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Filter->findById(connection, filterId)

Returns the BuildForge::Services::DBO::Filter object corresponding to the
given filter group ID, or undef if no such filter group exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item filterId

The ID of the desired filter group (from $filter->getFilterId()).

=back


=head2 $filter->create()

Creates the filter group within the database.


=head2 $filter->clone()

Returns a copy of the BuildForge::Services::DBO::Filter object and all of its 
FilterPattern patterns.  The copy will not exist in the database unless it is created
with $filter->create().


=head2 $filter->update()

Updates the database with the new filter group settings.


=head2 $filter->addFilterPattern(pattern, index)

Adds a filter pattern to the filter group within the database.

=over 8

=item pattern

The BuildForge::Services::DBO::FilterPattern object to be added.

=item index

The optional location at which to add the pattern within the list.  0 adds it to the
front of the list, 1 adds it after the first pattern, etc.  Leaving it unset
adds the pattern to the end of the pattern list.

=back


=head2 $filter->moveFilterPattern(from, to)

Moves a filter pattern within the list of patterns in the database.

=over 8

=item from

The index of the pattern to move.  0 moves the first pattern, 1 moves the second, etc.

=item to

The index of the new location for the pattern.  0 moves it to the first position,
1 to the second, etc.

=back


=head2 $filter->copyFilterPattern(from, to)

Copies a filter pattern within the filter group in the database.

=over 8

=item from

The index of the pattern to be copied.  0 copies the first pattern, 1 copies the second, etc.

=item to

The index of the location for the new pattern copy.  0 copies it to the first position,
1 to the second, etc.

=back


=head2 $filter->removeFilterPattern(pattern)

Removes a filter pattern from the pattern list in the database.

=over 8

=item pattern

This may be an index, in which case the pattern at that position is deleted (0
deletes the first pattern, 1 the second, etc.), or a BuildForge::Services::DBO::FilterPattern
object, in which case that filter pattern is the one deleted.

=back


=head2 $filter->delete()

Deletes the filter and all of its patterns from the database.


=head2 $filter->getFilterId()

Returns the ID of the filter group.


=head2 $filter->getFilterName()

Returns the name of the filter group.


=head2 $filter->getLevel()

Returns the access group ID of the access group the filter belongs to.
Users who are not in this access group cannot view, edit, or use the
filter group.


=head2 $filter->getPattern(pattern)

Returns the BuildForge::Services::DBO::FilterPattern object that corresponds
to the requested pattern.

=over 8

=item pattern

This may be an index, in which case that pattern is the one returned (0 for the
first pattern, 1 for the second, etc.), or a pattern name, in which case the 
first pattern with that name in the filter group is returned.

=back


=head2 $filter->getPatterns()

Returns a reference to an array of BuildForge::Services::DBO::FilterPattern 
objects for all patterns in the filter group.


=head2 $filter->setFilterName(filterName)

Sets the filter group's name.  $filter->update() must be run 
before changes are replicated in the database.

=over 8

=item filterName

The new name for the filter group.

=back


=head2 $filter->setLevel(accessGroupId)

Sets the filter group's access group.  Users who do not belong to this access group will not
be able to view, edit, or use the filter.  $filter->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) for the desired access group.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

