
package BuildForge::Services::DBO::FilterPattern;

use strict;
use overload '""' => \&toString;

use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(filterId filterOrdinal pattern action notification);
my $prototype = {
	_parent             => undef,
	filterId          => undef,
	filterOrdinal			 => undef,
	pattern        => undef,
	action            => ENUM_FILTERPATTERN_ACTION->FAIL,
	notification   => undef,
};
sub getTypeKey { 'FilterPattern'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my ($newParent) = splice(@_, 1, 1);
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_parent'} = $newParent;
	Integer(\$self->{'filterId'});
	Integer(\$self->{'filterOrdinal'});
	Enum(ENUM_FILTERPATTERN_ACTION, \$self->{'action'});
	Integer(\$self->{'notification'});
	return $self;
}


# INTERNAL USE ONLY
sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->internal();
	$conn->request(COMMAND_FILTER_ENTRY_ADD);
	$conn->writeEntryDBO(KEY_FILTER_PATTERN_DBO, $self);
	my $dbo = $conn->call()->{KEY_FILTER_PATTERN_DBO()};
	defined($dbo) or die $self->missing(KEY_FILTER_PATTERN_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $filterId = $self->{'_parent'}->{'filterId'};
	if (defined($filterId) && length($filterId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_FILTER_ENTRY_UPD);
		$conn->writeEntryDBO(KEY_FILTER_PATTERN_DBO, $self);
		my $dbo = $conn->call()->{KEY_FILTER_PATTERN_DBO()};
		defined($dbo) or die $self->missing(KEY_FILTER_PATTERN_DBO);
		return $self->fromArray($dbo);
	}
	return $self;
}

sub delete { $_[0]->{'_parent'}->removeFilterPattern($_[0]); }

sub moveTo { $_[0]->{'_parent'}->moveFilterPattern($_[0]->getPatternOrdinal - 1, $_[1]); }

# wrap is inherited

sub getParent { $_[0]->{'_parent'}; }

# setParent is not needed

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self->{'_parent'});
	$clone->{'_parent'} = undef;
	return $clone;
}

sub getFilterId            { $_[0]->{'filterId'}; }
sub getPatternOrdinal            { $_[0]->{'filterOrdinal'}; }
sub getPattern        { $_[0]->{'pattern'}; }
sub getAction            { $_[0]->{'action'}; }
sub getNotification            { $_[0]->{'notification'}; }

sub setPattern        { $_[0]->{'pattern'}  = $_[1]; }
sub setAction         { $_[0]->{'action'}      = $_[1]; }
sub setNotification            { $_[0]->{'notification'}  = $_[1]; }


sub toString {
	my ($self) = @_;
	return '[FilterPattern: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ']';
}


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::FilterPattern


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	$pattern = new BuildForge::Services::DBO::FilterPattern($conn);

	# Getter / setter functions
	$filterId = $pattern->getFilterId();
	$ordinal = $pattern->getPatternOrdinal();;
	$patternRegex = $pattern->getPattern();
	$action = $pattern->getAction();
	$accessGroupId = $pattern->getNotification();

	$pattern->setPattern('Pattern to match');
	$pattern->setAction(ENUM_FILTERPATTERN_ACTION->CLEAR_AND_HALT);
	$pattern->setNotification($accessGroupId);
	$pattern->update();
	
	# Pattern creation, modification, and deletion.
	$envGroup = BuildForge::Services::DBO::Filter->findById($conn, filterId);
	$newpattern = new BuildForge::Services::DBO::FilterPattern($conn);
	$newpattern->setPattern('New pattern to match');
	$envGroup->addFilterPattern($var);

	$newpattern->setPattern('Different pattern');
	$newpattern->update();
	
	$newpattern2 = $newpattern->clone();
	$envGroup->addFilterPattern($newpattern2);
	
	$newpattern->moveTo(2);

	$newpattern->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

FilterPattern allows you to create and modify filter pattens inside of a Filter.
To create the patterns in the database, see the Filter->addFilterPattern() function.


=head1 METHODS


=head2 new BuildForge::Services::DBO::FilterPattern(connection, {filter}, {FilterPatternParamsHashRef})

Returns a new instance of an FilterPattern object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item filter

This should be undef or not passed - it is only used internally.

=item FilterPatternParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item pattern

The filter pattern itself.

=item action

The pattern's action when a log line matches the pattern.  This may be any of:

=over 8

=item FAIL

The default.  Will cause the step to fail (if not first cleared).

=item CLEAR

Causes the step to lose failure status.  Has no effect on warning status.

=item CLEAR_AND_HALT

Causes the step to lose failure status and the filter set to stop processing the current
log line and go on to the next.

=item FAIL_AND_HALT

Causes the step to fail and the filter set to stop processing the current
log line and go on to the next.

=item HALT

Causes the filter set to stop processing the current log line and go on to the next.

=item WARNING

Causes the step to pass with warning status if it doesn't end in failure.

=item CLEAR_WARNING

Clears the warning status of the step.  Has no effect on failure status.

=item CLEAR_WARNING_AND_HALT

Clears the warning status of the step and causes the filter set to stop processing the current
log line and go on to the next.

=item NOTIFY_CHANGERS

If the project has an adaptor attached which saves changed file information and the
people who changed it, this notifies the people that changed the build data that this filter
has been matched.

=item INCLUDE

Includes another filter set inside this one.  If this is set, the filter pattern must
be the name of the filter set to be included.

=back

=item notification

The access group ID (from $accessGroup->getLevel()) for the notification group for this pattern.  
If the pattern is matched on a step, the engine sends a notification email to all members of the access group.

=back

=back


=head2 $pattern->clone()

A copy of the FilterPattern object is returned without affecting the database.
To save the copy to an filter group, $filter->addFilterPattern($copy) 
must be run on the copy.


=head2 $pattern->update()

Updates the pattern settings in the database.


=head2 $pattern->moveTo(index)

Moves the pattern within its parent filter group.

=over 8

=item index

The new location of the pattern.  0 moves it to the first position, 1 to
the second position, etc.

=back


=head2 $pattern->delete()

Deletes the pattern from the database.


=head2 $pattern->getParent()

Returns the BuildForge::Services::DBO::Filter object for the filter
group this pattern belongs to.


=head2 $pattern->getFilterId()

Returns the filter group ID for the filter group this pattern
belongs to.


=head2 $pattern->getPatternOrdinal()

Returns the ordinal for the position of the pattern in the pattern list.
1 means it's the first pattern, 2 means it's the second, etc.


=head2 $pattern->getPattern()

Returns the actual pattern.


=head2 $pattern->getAction()

Returns the pattern action which controls how the pattern value is set.  This may be 
any of 'CLEAR', 'CLEAR_AND_HALT', 'FAIL', 'FAIL_AND_HALT', 'HALT', 'INCLUDE', 'WARNING',
'CLEAR_WARNING', 'CLEAR_WARNING_AND_HALT', or 'NOTIFY_CHANGERS'.


=head2 $pattern->getNotification()

Returns the notification group for this pattern.  If the pattern is matched on a step, 
the engine sends a notification email to all members of the access group.


=head2 $pattern->setPattern(value)

Sets the value of the pattern.  $pattern->update() must be run 
before changes are replicated in the database.

=over 8

=item value

The new regular expression.

=back


=head2 $pattern->setAction(action)

Sets the pattern action which controls how the filter reacts when a match is found
in a log line.   
$pattern->update() must be run before changes are replicated in the database.

=over 8

=item action

May be any of:

=over 8

=item FAIL

Will cause the step to fail (if not first cleared).

=item CLEAR

Causes the step to lose failure status.  Has no effect on warning status.

=item CLEAR_AND_HALT

Causes the step to lose failure status and the filter set to stop processing the current
log line and go on to the next.

=item FAIL_AND_HALT

Causes the step to fail and the filter set to stop processing the current
log line and go on to the next.

=item HALT

Causes the filter set to stop processing the current log line and go on to the next.

=item WARNING

Causes the step to pass with warning status if it doesn't end in failure.

=item CLEAR_WARNING

Clears the warning status of the step.  Has no effect on failure status.

=item CLEAR_WARNING_AND_HALT

Clears the warning status of the step and causes the filter set to stop processing the current
log line and go on to the next.

=item NOTIFY_CHANGERS

If the project has an adaptor attached which saves changed file information and the
people who changed it, this notifies the people that changed the build data that this filter
has been matched.

=item INCLUDE

Includes another filter set inside this one.  If this is set, the filter pattern must
be the name of the filter set to be included.

=back

=back


=head2 $pattern->setNotification(accessGroupId)

Sets the notification group for this pattern.  If the pattern is matched on a step, 
the engine sends a notification email to all members of the access group.
$pattern->update() must be run before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID for the access group to be notified, from $accessGroup->getLevel().

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

