
package BuildForge::Services::DBO::LDAP;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(domain
	adminCN authorizedGroupDN bindUserAccount displayName distinguishedName
	encryptedPassword groupName groupsSearchBase groupsUniqueIdentifier
	host mailName mapAccessGroups protocol searchBase shouldRecurse uniqueIdentifier liveDomain);
my $prototype = {
	'domain' => '',
	'adminCN' => '',
	'authorizedGroupDN' => '',
	'bindUserAccount' => 1,
	'displayName' => '',
	'distinguishedName' => '',
	'encryptedPassword' => '',
	'groupName' => '',
	'groupsSearchBase' => '',
	'groupsUniqueIdentifier' => '',
	'host' => '',
	'mailName' => '',
	'mapAccessGroups' => 0,
	'protocol' => 'ldap',
	'searchBase' => '',
	'shouldRecurse' => 1,
	'uniqueIdentifier' => '',
	'liveDomain' => '',
};
sub getTypeKey { 'LDAP' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Boolean(\$self->{'bindUserAccount'});
	Boolean(\$self->{'mapAccessGroups'});
	Boolean(\$self->{'shouldRecurse'});
	return $self;
}

sub findAllDomains {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LDAP_FIND_ALL);
	my $list = $conn->call()->{+KEY_LDAP_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	return \@$list;
}

sub findByDomain {
	my ($proto, $conn, $domain) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LDAP_FIND_DOMAIN);
	$conn->writeEntryString(KEY_LDAP_DOMAIN, $domain);
	my $dbo = $conn->call()->{+KEY_LDAP_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_LDAP_ADD);
	$conn->writeEntryDBO(KEY_LDAP_DBO, $self);
	my $dbo = $conn->call()->{+KEY_LDAP_DBO};
	defined($dbo) or die $self->missing(KEY_LDAP_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_LDAP_UPD);
	$conn->writeEntryDBO(KEY_LDAP_DBO, $self);
	my $dbo = $conn->call()->{+KEY_LDAP_DBO};
	defined($dbo) or die $self->missing(KEY_LDAP_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	$self->deleteByDomain($self->{'_conn'}, $self->{'domain'});
}

sub deleteByDomain {
	my ($proto, $conn, $domain) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_LDAP_DEL);
	$conn->writeEntryString(KEY_LDAP_DOMAIN, $domain);
	$conn->call();
}

# wrap is inherited

sub getAdminCN                 { $_[0]->{'adminCN'}; }
sub getAuthorizedGroupDN       { $_[0]->{'authorizedGroupDN'}; }
sub getBindUserAccount         { $_[0]->{'bindUserAccount'} ? 1 : 0; }
sub getDisplayName             { $_[0]->{'displayName'}; }
sub getDistinguishedName       { $_[0]->{'distinguishedName'}; }
sub getDomain                  { $_[0]->{'domain'} ? $_[0]->{'domain'} : $_[0]->{'liveDomain'}; }
sub getGroupName               { $_[0]->{'groupName'}; }
sub getGroupsSearchBase        { $_[0]->{'groupsSearchBase'}; }
sub getGroupsUniqueIdentifier  { $_[0]->{'groupsUniqueIdentifier'}; }
sub getHost                    { $_[0]->{'host'}; }
sub getMailName                { $_[0]->{'mailName'}; }
sub getMapAccessGroups         { $_[0]->{'mapAccessGroups'} ? 1 : 0; }
sub getSearchBase              { $_[0]->{'searchBase'}; }
sub getSearchesShouldRecurse   { $_[0]->{'shouldRecurse'}; }
sub getUniqueIdentifier        { $_[0]->{'uniqueIdentifier'}; }

sub setAdminCN                 { $_[0]->{'adminCN'}                 = $_[1]; }
sub setAuthorizedGroupDN       { $_[0]->{'authorizedGroupDN'}       = $_[1]; }
sub setBindUserAccount         { $_[0]->{'bindUserAccount'}         = $_[1]; }
sub setDisplayName             { $_[0]->{'displayName'}             = $_[1]; }
sub setDistinguishedName       { $_[0]->{'distinguishedName'}       = $_[1]; }
sub setDomain                  { $_[0]->{'domain'}                  = $_[1]; }
sub setEncryptedPassword       { $_[0]->{'encryptedPassword'}       = $_[1]; }
sub setGroupName               { $_[0]->{'groupName'}               = $_[1]; }
sub setGroupsSearchBase        { $_[0]->{'groupsSearchBase'}        = $_[1]; }
sub setGroupsUniqueIdentifier  { $_[0]->{'groupsUniqueIdentifier'}  = $_[1]; }
sub setHost                    { $_[0]->{'host'}                    = $_[1]; }
sub setMailName                { $_[0]->{'mailName'}                = $_[1]; }
sub setMapAccessGroups         { $_[0]->{'mapAccessGroups'}         = $_[1]; }
sub setSearchBase              { $_[0]->{'searchBase'}              = $_[1]; }
sub setSearchesShouldRecurse   { $_[0]->{'shouldRecurse'}           = $_[1]; }
sub setUniqueIdentifier        { $_[0]->{'uniqueIdentifier'}        = $_[1]; }

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::LDAP


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing domains
	$allDomains = BuildForge::Services::DBO::LDAP->findAllDomains($conn);
	$olddomain = BuildForge::Services::DBO::LDAP->findByDomain($conn, 'DomainName');

	# Getter / setter functions
	$id = $olddomain->getDomain();
	$adminCN = $olddomain->getAdminCN();
	$authGroupDN = $olddomain->getAuthorizedGroupDN();
	$bindUserAccount = $olddomain->getBindUserAccount();
	$displayName = $olddomain->getDisplayName();
	$distinguishedName = $olddomain->getDistinguishedName();
	$groupName = $olddomain->getGroupName();
	$groupSearchBase = $olddomain->getGroupsSearchBase();
	$groupUniqueIdentifier = $olddomain->getGroupsUniqueIdentifier();
	$hostName = $olddomain->getHost();
	$mailName = $olddomain->getMailName();
	$mapAccessGroups = $olddomain->getMapAccessGroups();
	$searchBase = $olddomain->getSearchBase();
	$uniqueIdentifier = $olddomain->getUniqueIdentifier();
	
	$olddomain->setDomain('New Name For Domain');
	$olddomain->setAdminCN('cn=Administrator,cn=users,dc=example,dc=com');
	$olddomain->setAuthorizedGroupDN('');
	$olddomain->setBindUserAccount(1);
	$olddomain->setDisplayName('displayname');
	$olddomain->setDistinguishedName('distinguishedname');
	$olddomain->setEncryptedPassword($password);
	$olddomain->setGroupName('memberof');
	$olddomain->setGroupsSearchBase('');
	$olddomain->setGroupsUniqueIdentifier('');
	$olddomain->setHost('ldap.example.com:389');
	$olddomain->setMailName('mail');
	$olddomain->setMapAccessGroups(1);
	$olddomain->setSearchBase('');
	$olddomain->setUniqueIdentifier('sAMAccountName=%');
	$olddomain->update();
	
	# Domain creation, updating, deletion
	$newdomain = new BuildForge::Services::DBO::LDAP($conn);
	$newdomain->setDomain('New Build Domain');
	$newdomain->create();

	$newdomain->setDomain('Different Name');
	$newdomain->update();
	
	$newdomain->delete();
	BuildForge::Services::DBO::LDAP->deleteByDomain($conn, $olddomain->getDomain());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

LDAP allows you to create, access, and delete BuildForge LDAP domains. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::LDAP(connection, {LDAPParamsHashRef})

Returns a new instance of an LDAP object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item LDAPParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item domain

The name/ID for this LDAP domain.

=item adminCN

The account to use when searching the LDAP server database for this LDAP domain.

=item authorizedGroupDN

The distinguished name of an LDAP group that should be allowed to log 
in to the Management Console.

=item bindUserAccount

This determines whether the system attempts to validate the user's credentials.
When this parameter evaluates to true (the default), the system uses the credentials supplied at 
login to re-bind with the LDAP server and validate the credentials.
Otherwise, the system assumes that the username passed 
in has already been validated (as in a single-signon environment) or that some 
external password validation mechanism has been supplied to the Management Console.

=item displayName

The keyname in the LDAP schema that holds the user's full name for this LDAP domain.

=item distinguishedName

The keyname in the LDAP schema that holds the Distinguished Name for a 
user account in this LDAP domain.

=item encryptedPassword

The password for logging in to this LDAP domain.

=item groupName

The keyname in the LDAP schema that holds the list of groups the user is a 
member of. This group list drives the authentication permissions and access 
permissions.

=item groupsSearchBase

The search string used to query LDAP records for group data.

=item groupsUniqueIdentifier

The field in the LDAP database that should be compared against the user 
login to uniquely identify the user, used to filter the user's data from the 
group table(s).

=item host

The hostname (and, optionally, port, separated by a ':') for the LDAP server.

=item mailName

The keyname in the LDAP schema that holds the user's email address.

=item mapAccessGroups

Whether or not to whether or not the system tries to map group information 
from the LDAP server to access groups in the Management Console to determine 
what groups an LDAP user should be a member of.  Yes if this evaluates to true,
no otherwise (the default).

=item protocol

Identifies the protocol Build Forge uses to read and write data from the directory 
service for the purpose of authenticating Build Forge users. The default is 'ldap'.

=item searchBase

The search string used to query LDAP records for users.

=item uniqueIdentifier

The field in the LDAP database that should be compared against the user login.

=back

=back


=head2 BuildForge::Services::DBO::LDAP->findAllDomains(connection)

Returns a reference to an array of the names of all LDAP domains
in the database that the user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::LDAP->findByDomain(connection, domainName)

Returns the BuildForge::Services::DBO::LDAP domain that has the given domain name
or undef if no such LDAP domain is found.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item domainName

The name of the given domain.

=back


=head2 $ldap->create()

Creates the LDAP domain within the database.


=head2 $ldap->update()

Updates the LDAP domain record within the database.


=head2 $ldap->delete()

Deletes the LDAP domain record from the database.


=head2 BuildForge::Services::DBO::LDAP->deleteByDomain(connection, domainName)

Deletes the LDAP domain record from the database that has the given domain name
(from $ldap->getDomain()).

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item domainName

The name / ID of the LDAP domain record to be deleted.

=back


=head2 $ldap->getDomain()

Returns the name / ID of the LDAP domain.


=head2 $ldap->getAdminCN()

Returns the account to use when searching the LDAP server database for this LDAP domain.


=head2 $ldap->getAuthorizedGroupDN()

Returns the distinguished name of an LDAP group that should be allowed to log 
in to the Management Console.


=head2 $ldap->getBindUserAccount()

Returns whether the system attempts to validate the user's credentials.
When it returns 1, the system uses the credentials supplied at 
login to re-bind with the LDAP server and validate the credentials.
If it returns 0, the system assumes that the username passed 
in has already been validated (as in a single-signon environment) or that some 
external password validation mechanism has been supplied to the Management Console.


=head2 $ldap->getDisplayName()

Returns the keyname in the LDAP schema that holds the user's full name for this LDAP domain.


=head2 $ldap->getDistinguishedName()

Returns the keyname in the LDAP schema that holds the Distinguished Name for a 
user account in this LDAP domain.


=head2 $ldap->getGroupName()

Returns the keyname in the LDAP schema that holds the list of groups the user is a 
member of. This group list drives the authentication permissions and access 
permissions.


=head2 $ldap->getGroupsSearchBase()

Returns the search string used to query LDAP records for group data.


=head2 $ldap->getGroupsUniqueIdentifier()

Returns the field in the LDAP database that should be compared against the user 
login to uniquely identify the user, used to filter the user's data from the 
group table(s).


=head2 $ldap->getHost()

Returns the hostname (and, optionally, port, separated by a ':') for the LDAP server.


=head2 $ldap->getMailName()

Returns the keyname in the LDAP schema that holds the user's email address.


=head2 $ldap->getMapAccessGroups()

Returns whether or not to whether or not the system tries to map group information 
from the LDAP server to access groups in the Management Console to determine 
what groups an LDAP user should be a member of.  Yes if this returns 1,
no if it returns 0.


=head2 $ldap->getSearchBase()

Returns the search string used to query LDAP records for users.


=head2 $ldap->getUniqueIdentifier()

Returns the field in the LDAP database that should be compared against the user login.


=head2 $ldap->setDomain(domainName)

Sets the name / ID of the LDAP domain.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item domainName

The new name.

=back


=head2 $ldap->setAdminCN(adminCN)

Sets the account to use when searching the LDAP server database for this LDAP domain.  
$ldap->update() must be run before changes are replicated in the database.

=over 8

=item adminCN

The new administration account login.

=back


=head2 $ldap->setAuthorizedGroupDN(authGroupDN)

Sets the distinguished name of an LDAP group that should be allowed to log 
in to the Management Console.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item authGroupDN

The new authorized group DN.

=back


=head2 $ldap->setBindUserAccount(bindUserAccount)

Sets whether the system attempts to validate the user's credentials.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item bindUserAccount

If this evaluates to true, the system uses the credentials supplied at 
login to re-bind with the LDAP server and validate the credentials.
If it evaluates to false, the system assumes that the username passed 
in has already been validated (as in a single-signon environment) or that some 
external password validation mechanism has been supplied to the Management Console.

=back


=head2 $ldap->setDisplayName(displayName)

Sets the keyname in the LDAP schema that holds the user's full name for this LDAP domain.
$ldap->update() must be run before changes are replicated in the database.

=over 8

=item displayName

The new display name.

=back


=head2 $ldap->setDistinguishedName(distinguishedName)

Sets the keyname in the LDAP schema that holds the Distinguished Name for a 
user account in this LDAP domain.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item distinguishedName

The new distinguished name.

=back


=head2 $ldap->setEncryptedPassword(password)

Sets the administration password used to log in to the LDAP server.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item password

The new password.

=back


=head2 $ldap->setGroupName(groupName)

Sets the keyname in the LDAP schema that holds the list of groups the user is a 
member of. This group list drives the authentication permissions and access 
permissions.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item groupName

The new group name.

=back


=head2 $ldap->setGroupsSearchBase(groupSearchBase)

Sets the search string used to query LDAP records for group data.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item groupSearchBase

The new group search string.

=back


=head2 $ldap->setGroupsUniqueIdentifier(groupUniqueIdentifier)

Sets the field in the LDAP database that should be compared against the user 
login to uniquely identify the user, used to filter the user's data from the 
group table(s).  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item groupUniqueIdentifier

The new group unique identifier.

=back


=head2 $ldap->setHost(host)

Sets the hostname for the LDAP server.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item host

The hostname of the LDAP server with an optional port separated by ':'.

=back


=head2 $ldap->setMailName(mailName)

Sets the keyname in the LDAP schema that holds the user's email address.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item mailName

The new keyname.

=back


=head2 $ldap->setMapAccessGroups(mapAccessGroups)

Sets whether or not to whether or not the system tries to map group information 
from the LDAP server to access groups in the Management Console to determine 
what groups an LDAP user should be a member of.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item mapAccessGroups

If this evaluates to true, it does map group information, and not if this evaluates
to false.

=back


=head2 $ldap->setSearchBase(searchBase)

Sets the search string used to query LDAP records for users.  $ldap->update() must be run 
before changes are replicated in the database.

=over 8

=item searchBase

The new search string.

=back


=head2 $ldap->setUniqueIdentifier(uniqueIdentifier)

Sets the field in the LDAP database that should be compared against the user login.  
$ldap->update() must be run before changes are replicated in the database.

=over 8

=item uniqueIdentifier

The new field name.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

