
package BuildForge::Services::DBO::Log;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(uid type lineId stamp messageText filterAction stepPart);
my $prototype = {
	'uid' => '',
	'type' => '',
	'lineId' => 0,
	'stamp' => 0,
	'messageText' => undef,
	'filterAction' => undef,
	'stepPart' => 0,
};
sub getTypeKey { 'Log' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'lineId'});
	Integer(\$self->{'stamp'});
	Integer(\$self->{'stepPart'});
	return $self;
}

sub findByResult {
	my ($proto, $conn, $buildId, $uid) = @_;
	$proto->checkConnection($conn);
	!defined($buildId) || $buildId <= 0 and die BuildForge::Services::DBO::Build->invalid('ID');
	!defined($uid) || !length($uid) and die BuildForge::Services::DBO::Result->invalid('ID');
	$conn->request(COMMAND_LOG_FIND_RESULT);
	$conn->writeEntryString(KEY_BUILD_ID, $buildId);
	$conn->writeEntryString(KEY_RESULT_UID, $uid);
	my $list = $conn->call()->{+KEY_LOG_LIST};
	!defined($list) and return undef;
	my @logs = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@logs;
}

sub getUid                     { $_[0]->{'uid'}; }
sub getType                    { $_[0]->{'type'}; }
sub getLineId                  { $_[0]->{'lineId'}; }
sub getStamp                   { $_[0]->{'stamp'}; }
sub getMessageText             { $_[0]->{'messageText'}; }
sub getFilterAction            { $_[0]->{'filterAction'}; }
sub getStepPart                { $_[0]->{'stepPart'}; }

# No setters... this kind of object is read-only for clients...

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Log


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting logs from existing builds.
	$project = new BuildForge::Services::DBO::Project($conn);
	$project->setName('Test Project');
	$project->create();
	$build = BuildForge::Services::DBO::Build->fire($conn, $project->getProjectId());
	$results = $build->getResults();
	$loglines1 = $results->[0]->getLogs();
	$loglines2 = BuildForge::Services::DBO::Log->findByResult($conn, $results->[1]->getBuildId(), 
			$results->[1]->getUid());
	$line = $loglines1->[0];

	# Getter / setter functions
	$uid = $line->getUid();
	$type = $line->getType();
	$lineNumber = $line->getLineId();
	$timestamp = $line->getStamp();
	$text = $line->getMessageText();
	$lineMatchedTypeOfFilter = $line->getFilterAction();
	$wasPartOfStepOutput = $line->getStepPart();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Log allows you to access a log line from the individual step results of a BuildForge build. 


=head1 METHODS


=head2 BuildForge::Services::DBO::Log->findByResult(connection, buildId, resultId)

Returns a reference to an array of BuildForge::Services::DBO::Log objects corresponding to
each individual line of output from the given step result.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The ID of the build the results are from (from $build->getBuildId()).

=item resultId

The ID of the step results to get the log lines from (from $result->getUid()).

=back


=head2 getUid()

Returns the step results ID for the step results this log line belongs to.


=head2 getType()

Returns the type of log output it is.  Possible values include 'AUTH', 'ENV', 'EXEC', 
'FILT', 'MANIFEST', 'MKDIR', 'SET', and many others.


=head2 getLineId()

The line number of the log line entry.


=head2 getStamp()

The timestamp at which this log line was logged.


=head2 getMessageText()

The text of the log line.


=head2 getFilterAction()

If this is a 'FILT' line, the filter action that took place is stored here.  Possible values 
and the filter actions they correspond to (see FilterPattern for more details):

=over 8

=item F	

'FAIL'

=item H	

'FAIL_AND_HALT'

=item C	

'CLEAR'

=item S	

'CLEAR_AND_HALT'

=item I	

'HALT'

=item A	

'INCLUDE'

=item N	

'WARNING'

=item U	

'CLEAR_WARNING'

=item G

'CLEAR_WARNING_AND_HALT'

=item R

'NOTIFY_CHANGERS'

=back


=head2 getStepPart()

Returns the block within the step command that this output log line relates to.
0 relates to pre-execution processing, 1 to the first block, etc.  Any dot command
takes place in its own block and non-dot commands are broken up into blocks by 
dot commands.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

