
package BuildForge::Services::DBO::Manifest;
use BuildForge::Services::DBO::ManifestProperty;

use strict;
use overload '""' => \&toString;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(name);
my $prototype = {
	name         => undef,
	properties   => [],
};
sub getTypeKey { 'Manifest' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'properties'} = {};
	return $self;
}

sub getServerName { $_[0]->{'name'}; }

sub getProperties {
	my $map = $_[0]->{'properties'};
	my @props = values %$map;
	return \@props;
}

sub getProperty {
	my ($self, $name) = @_;
	return $_[0]->{'properties'}->{$name};
}

sub getPropertyValue {
	my ($self, $name) = @_;
	my $prop = $self->getProperty($name);
	return defined($prop) ? $prop->{'propertyValue'} : undef;
}

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Manifest


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting manifest
	$servers = BuildForge::Services::DBO::Server->findAll($conn);
	$manifest = $servers->[0]->getManifest();

	# Getter functions
	$serverName = $manifest->getServerName();
	$properties = $manifest->getProperties();
	$property = $manifest->getProperty('BF_JOBS');
	$propertyValue = $manifest->getPropertyValue('BF_JOBS');
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Manifest allows you to access the manifest of a specific server.  You access the individual
manifest properties with ManifestProperty.  To get the manifest of a server, see 
BuildForge::Services::DBO::Server->getManifest().


=head1 METHODS


=head2 $manifest->getServerName()

Returns the name of the server this manifest is attached to.


=head2 $manifest->getProperties()

Returns a reference to an array of all BuildForge::Services::DBO::ManifestProperty objects that
this manifest contains.


=head2 $manifest->getProperty(propertyName)

Returns the BuildForge::Services::DBO::ManifestProperty object of the name in this manifest
or undef if none such property exists.

=over 8

=item propertyName

The name of the desired property.

=back


=head2 $manifest->getPropertyValue(propertyName)

Returns the value of the property with that name or undef if no such property exists
in the manifest.

=over 8

=item propertyName

The name of the desired property.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

