
package BuildForge::Services::DBO::Project;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info (DataDBO)
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(projectId level description buildClass selectorId tag
	environmentId active startNotify passNotify failNotify
	maxThread runLimit tagSync sticky passChainId failChainId geoId);
my $prototype = {
	'projectId' => 0,
	'level' => 0,
	'description' => '',
	'buildClass' => '',
	'selectorId' => '',
	'tag' => 'BUILD_$B',
	'environmentId' => 0,
	'active' => 1,
	'startNotify' => 0,
	'passNotify' => 0,
	'failNotify' => 0,
	'maxThread' => 0,
	'runLimit' => 0,
	'tagSync' => 0,
	'sticky' => 0,
	'passChainId' => 0,
	'failChainId' => 0,
	'geoId' => '',
};
sub getTypeKey { 'Project' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);

	Integer(\$self->{'projectId'});
	Integer(\$self->{'level'});
	Integer(\$self->{'environmentId'});
	Boolean(\$self->{'active'});
	Integer(\$self->{'startNotify'});
	Integer(\$self->{'passNotify'});
	Integer(\$self->{'failNotify'});
	Integer(\$self->{'maxThread'});
	Integer(\$self->{'runLimit'});
	Integer(\$self->{'tagSync'});
	Boolean(\$self->{'sticky'});
	Integer(\$self->{'passChainId'});
	Integer(\$self->{'failChainId'});
	$self->{'_steps'} = [];
	return $self;
}


sub _dbo_addStep ($$;$) {
	my ($self, $newStep, $index) = @_;
	my $steps = $self->{'_steps'};
	!isa($steps, 'ARRAY') and $steps = $self->{'_steps'} = [];
	if (!defined($index) || $index >= @$steps) {
		$index = @$steps;
		push @$steps, $newStep;
	} elsif ($index < 0) {
		die $self->badIndex($index);
	} else {
		splice @$steps, $index, 0, $newStep;
	}
	$newStep->{'projectId'} = $self->{'projectId'};
	_updateStepOrdinals($self, $index);
}

# Pre-declare prototype so it will be available for recursion
sub _dbo_removeStep ($$);
sub _dbo_removeStep ($$) {
	my ($self, $index) = @_;

	!length($index) and return undef;
	my $steps = $self->{'_steps'};
	!isa($steps, 'ARRAY') and $steps = $self->{'_steps'} = [];

	# removeStep(Step)
	if (isa($index, 'BuildForge::Services::DBO::Step')) {
		return _dbo_removeStep($self, $index->{'uid'});
	# removeStep(String)
	}  elsif ($index =~ /\D/) {
		for (my $i=0; $i <= @$steps; $i++) {
			my $step = $steps->[$i];
			$step->{'uid'} eq $index and return _dbo_removeStep($self, $i);
		}
		return undef;
	}
	# removeStep(int)
	$index < 0 || $index >= @$steps and die $self->badIndex($index);
	my ($step) = splice @$steps, $index, 1;
	_updateStepOrdinals($self, $index);
	return $step;
}

sub _dbo_copyStep ($$$$) {
	my ($self, $fromIndex, $toIndex, $newStepUid) = @_;
	my $steps = $self->{'_steps'};
	!isa($steps, 'ARRAY') and $steps = $self->{'_steps'} = [];

	$fromIndex < 0 || $fromIndex >= @$steps and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex > @$steps and die $self->badIndex($toIndex);
	my $step = $steps->[$fromIndex];
	my $clone = $step->clone();
	$clone->{'uid'} = $newStepUid;
	_dbo_addStep($self, $clone, $toIndex);
}

sub _dbo_moveStep ($$$) {
	my ($self, $fromIndex, $toIndex) = @_;
	my $steps = $self->{'_steps'};
	!isa($steps, 'ARRAY') and $steps = $self->{'_steps'} = [];
	$fromIndex < 0 || $fromIndex >= @$steps and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex >= @$steps and die $self->badIndex($toIndex);
	my ($step) = splice @$steps, $fromIndex, 1;
	_dbo_addStep($self, $step, $toIndex);
	_updateStepOrdinals($self, $fromIndex <= $toIndex ? $fromIndex : $toIndex);
}

sub _dbo_getStep ($$) {
	my ($self, $index) = @_;
	my $steps = $self->{'_steps'};
	!isa($steps, 'ARRAY') and $steps = $self->{'_steps'} = [];
	if ($index =~ /\D/) {
		for (my $i = 0; $i < @$steps; $i++) {
			my $step = $steps->[$i];
			$step->{'uid'} eq $index and return $step;
		}
		return undef;
	}
	return $index < 0 || $index >= @$steps ? undef : $steps->[$index];
}

sub _dbo_setSteps ($$) {
	my ($self, $steps) = @_;
	!isa($steps, 'ARRAY') and $steps = [];
	$self->{'_steps'} = [@$steps];
}

sub _updateStepOrdinals ($$) {
	my ($self, $startingHere) = @_;
	my $steps = $self->{'_steps'};
	$startingHere < 0 || !defined($steps) and return;
	for (my $i=$startingHere; $i < @$steps; $i++) {
		$steps->[$i]->{'ordinal'} = $i + 1;
	}
}


sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_PROJ_FIND_ALL);
	return _readList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $projectId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_PROJ_FIND_ID);
	$conn->writeEntryString(KEY_PROJECT_ID, $projectId);
	return _readProject($conn, $conn->call());
}


sub findByName {
	my ($proto, $conn, $projectName) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_PROJ_FIND_NAME);
	$conn->writeEntryString(KEY_PROJECT_NAME, $projectName);
	return _readProject($conn, $conn->call());
}

sub getBuilds {
	my ($self) = @_;
	my $projectId = $self->{'projectId'};
	if ($projectId <= 0) {
		$projectId == 0 and die $self->liveness(__PACKAGE__);
		die $self->invalid('ID');
	}
	return BuildForge::Services::DBO::Build->findByProjectId($self->{'_conn'}, $projectId);
}

sub clobber {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_PROJ_CLOBBER);
	$conn->writeEntryString(KEY_PROJECT_ID, $self->{'projectId'});
	$conn->call();
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_PROJ_ADD);
	$conn->writeEntryDBO(KEY_PROJECT_DBO, $self);
	_writeSteps($self);

	my $resp = $conn->call();
	my $dbo = $resp->{KEY_PROJECT_DBO()};
	defined($dbo) or die $self->missing(KEY_PROJECT_DBO);
	$self->fromArray($dbo);

	my $list = $resp->{+KEY_STEP_LIST};
	!isa($list, 'ARRAY') and return $self;

	my @steps = map { BuildForge::Services::DBO::Step->new($conn)->fromArray($_) } @$list;
	$self->{'_steps'} = \@steps;
	return $self;	
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_PROJ_UPD);
	$conn->writeEntryDBO(KEY_PROJECT_DBO, $self);
	my $dbo = $conn->call()->{KEY_PROJECT_DBO()};
	defined($dbo) or die $self->missing(KEY_PROJECT_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_PROJ_DEL);
	$conn->writeEntryString(KEY_PROJECT_ID, $self->{'projectId'});
	$conn->call();
	$self->{'projectId'} = 0;
}

sub deactivate { $_[0]->activate(0); }
sub activate {
	my ($self, $active) = @_;
	my $conn = $self->{'_conn'};
	$active = (!defined($active) || $active) ? 1 : 0;
	$conn->request(COMMAND_PROJ_ACTIVATE);
	$conn->writeEntryString(KEY_PROJECT_ID, $self->{'projectId'});
	$conn->writeEntryString(KEY_PROJECT_ACTIVE, $active);
	$conn->call();
	$self->{'active'} = $active;
}

sub inUse {
	my ($self, $list, $all) = @_;
	if (!isa($list, 'ARRAY')) {
		$list = undef;
		$all = 0;
	} elsif (!defined($all)) {
		$all = 1;
	} else {
		$all = $all ? 1 : 0;
	}

	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_PROJ_INUSE);
	$conn->writeEntryString(KEY_PROJECT_ID, $self->{'projectId'});
	$conn->writeEntryString(KEY_PROJECT_INUSE, $all);
	$conn->writeEntryString(KEY_PROJECT_INUSE_MSGS, defined($list) ? 1 : 0);
	my $resp = $conn->call();
	if (defined($list)) {
		my $msgs = $resp->{+KEY_PROJECT_INUSE_MSGS};
		isa($msgs, 'ARRAY') or return undef;
		push @$list, map { BuildForge::Services::DBO::Message->new($conn)->fromArray($_) } @$msgs;
	}
	return ENUM_PROJECT_INUSE($resp->{+KEY_PROJECT_INUSE});
}

sub _readList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_PROJECT_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @projs = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@projs;
}

sub _readProject($) {
	my ($conn, $resp) = @_;
	my $project = $resp->{+KEY_PROJECT_DBO};
	!defined($project) || !isa($project, 'ARRAY') and return undef;
	$project = __PACKAGE__->new($conn)->fromArray($project);

	my $list = $resp->{+KEY_STEP_LIST};
	!isa($list, 'ARRAY') and return $project;

	my @steps = map { BuildForge::Services::DBO::Step->new($conn, $project)->fromArray($_) } @$list;
	$project->{'_steps'} = \@steps;
	return $project;
}

sub _writeSteps($) {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $steps = $self->{'_steps'};
	isa($steps, 'ARRAY') && @$steps and $conn->writeEntryArray(KEY_STEP_LIST, $steps);
}

# wrap is inherited

sub getActive        { $_[0]->{'active'}; }
sub getBuildClass    { $_[0]->{'buildClass'}; }
sub getEnvironmentId { $_[0]->{'environmentId'}; }
sub getRunLimit      { $_[0]->{'runLimit'}; }
sub getFailNotify    { $_[0]->{'failNotify'}; }
sub getLevel         { $_[0]->{'level'}; }
sub getMaxThread     { $_[0]->{'maxThread'}; }
sub getName          { $_[0]->{'description'}; }
sub getPassNotify    { $_[0]->{'passNotify'}; }
sub getProjectId     { $_[0]->{'projectId'}; }
sub getSelectorId    { $_[0]->{'selectorId'}; }
sub getStartNotify   { $_[0]->{'startNotify'}; }
sub getSticky        { $_[0]->{'sticky'}; }
sub getTag           { $_[0]->{'tag'}; }
sub getTagSync       { $_[0]->{'tagSync'}; }
sub getPassChain     { $_[0]->{'passChainId'}; }
sub getFailChain     { $_[0]->{'failChainId'}; }
sub getGeoId         { $_[0]->{'geoId'}; }

sub setActive        { $_[0]->{'active'}        = $_[1]; }
sub setBuildClass    { $_[0]->{'buildClass'}    = $_[1]; }
sub setEnvironmentId { $_[0]->{'environmentId'} = $_[1]; }
sub setRunLimit      { $_[0]->{'runLimit'}      = $_[1]; }
sub setFailNotify    { $_[0]->{'failNotify'}    = $_[1]; }
sub setLevel         { $_[0]->{'level'}         = $_[1]; }
sub setMaxThread     { $_[0]->{'maxThread'}     = $_[1]; }
sub setName          { $_[0]->{'description'}   = $_[1]; }
sub setPassNotify    { $_[0]->{'passNotify'}    = $_[1]; }
sub setSelectorId    { $_[0]->{'selectorId'}    = $_[1]; }
sub setStartNotify   { $_[0]->{'startNotify'}   = $_[1]; }
sub setSticky        { $_[0]->{'sticky'}        = $_[1]; }
sub setTag           { $_[0]->{'tag'}           = $_[1]; }
sub setTagSync       { $_[0]->{'tagSync'}       = $_[1]; }
sub setPassChain     { $_[0]->{'passChainId'}   = $_[1]; }
sub setFailChain     { $_[0]->{'failChainId'}   = $_[1]; }
sub setGeoId         { $_[0]->{'geoId'}   = $_[1]; }

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self);
	my @steps = map { $_->clone } @{ $self->{'_steps'} };
	$clone->{'_steps'} = \@steps;
	_updateStepOrdinals($clone,0);
	$clone->{'projectId'} = 0;
	return $clone;
}

# toString is inherited



# Child handling methods

sub getSteps { 
	my $list = $_[0]->{'_steps'};
	return defined($list) ? [@{ $list }] : undef;
}

sub getStep { return _dbo_getStep($_[0], $_[1]); }

sub addStep {
	my ($self, $step, $index) = @_;
	_dbo_addStep($self, $step, $index);
	$self->{'projectId'} > 0 and $step->create();
	$step->{'_parent'} = $self;
}

sub moveStep {
	my ($self, $fromIndex, $toIndex) = @_;
	my $steps = $self->{'_steps'};
	!defined($steps) and $steps = $self->{'_steps'} = [];
	$fromIndex < 0 || $fromIndex >= @$steps and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex >= @$steps and die $self->badIndex($toIndex);
	if ($self->{'projectId'} > 0) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_STEP_MOVE_STEP);
		my $step = $steps->[$fromIndex];
		$conn->writeEntryString(KEY_STEP_UID, $step->{'uid'});
		$conn->writeEntryString(KEY_STEP_PROJECT_ID, $self->{'projectId'});
		$conn->writeEntryString(KEY_STEP_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_STEP_TO_INDEX, $toIndex);
		$conn->call();
	}
	_dbo_moveStep($self, $fromIndex, $toIndex);
}


sub copyStep {
	my ($self, $fromIndex, $toIndex) = @_;
	my $steps = $self->{'_steps'};
	!defined($steps) and $steps = $self->{'_steps'} = [];
	$fromIndex < 0 || $fromIndex >= @$steps and die $self->badIndex($fromIndex);
	$toIndex < 0 || $toIndex > @$steps and die $self->badIndex($toIndex);

	my $newStepUid = undef;
	if ($self->{'projectId'} > 0) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_STEP_COPY_STEP);
		my $step = $steps->[$fromIndex];
		$conn->writeEntryString(KEY_STEP_UID, $step->{'uid'});
		$conn->writeEntryString(KEY_STEP_PROJECT_ID, $self->{'projectId'});
		$conn->writeEntryString(KEY_STEP_FROM_INDEX, $fromIndex);
		$conn->writeEntryString(KEY_STEP_TO_INDEX, $toIndex);
		$newStepUid = $conn->call()->{+KEY_STEP_UID};
	}
	_dbo_copyStep($self, $fromIndex, $toIndex, $newStepUid);
}

sub removeStep {
	my ($self, $index) = @_;
	my $steps = $self->{'_steps'};
	!defined($steps) and $steps = $self->{'_steps'} = [];
	# removeStep(Step)
	if (isa($index, 'BuildForge::Services::DBO::Step')) {
		return $self->removeStep($index->{'uid'});
	# removeStep(String)
	}  elsif ($index =~ /\D/) {
		for (my $i=0; $i<@$steps; $i++) {
			$steps->[$i]->{'uid'} eq $index and return $self->removeStep($i);
		}
		die BuildForge::Services::DBO::Step->invalid('ID');
	}
	my $conn = $self->{'_conn'};
	$index < 0 || $index >= @$steps and die $self->badIndex($index);
	$self->{'projectId'} > 0 and BuildForge::Services::DBO::Step->_dbo_delete($conn, $steps->[$index]->{'uid'});
	_dbo_removeStep($self, $index);
}

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Project


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing projects
	$allProjects = BuildForge::Services::DBO::Project->findAll($conn);
	$oldproject = BuildForge::Services::DBO::Project->findById($conn, $pid);
	$oldproject2 = BuildForge::Services::DBO::Project->findByName($conn, 'MyProject');

	# Getter / setter functions
	$id = $oldproject->getProjectId();
	$name = $oldproject->getName();
	$isActive = $oldproject->getActive();
	$class = $oldproject->getBuildClass();
	$envId = $oldproject->getEnvironmentId();
	$maxRuns = $oldproject->getRunLimit();
	$accessGroupId = $oldproject->getLevel();
	$maxThreads = $oldproject->getMaxThread();
	$selectorId = $oldproject->getSelectorId();
	$isSticky = $oldproject->getSticky();
	$tagFormat = $oldproject->getTag();
	$tagSyncProjectId = $oldproject->getTagSync();
	$startNotifyAccessGroupId = $oldproject->getStartNotify();
	$failNotifyAccessGroupId = $oldproject->getFailNotify();
	$passNotifyAccessGroupId = $oldproject->getPassNotify();
	$allSteps = $oldproject->getSteps();
	$step1 = $oldproject->getStep(2);
	$step2 = $oldproject->getStep($stepUID);
	$allProjectBuilds = $oldproject->getBuilds();
	$isProjectInUseByOtherObjects = $oldproject->inUse();
	$passChainId = $oldproject->getPassChain();
	$failChainId = $oldproject->getFailChain();
	$geoId = $oldproject->getGeoId();
	
	$oldproject->setName('New Project Name');
	$oldproject->setActive(1);
	$oldproject->setBuildClass('ClassName');
	$oldproject->setEnvironmentId($envId);
	$oldproject->setRunLimit(5);
	$oldproject->setLevel($accessGroupId);
	$oldproject->setMaxThread(10);
	$oldproject->setSelectorId('SelectorName');
	$oldproject->setSticky(1);
	$oldproject->setTag('BF_$B');
	$oldproject->setTagSync($pid);
	$oldproject->setStartNotify($accessGroupId);
	$oldproject->setFailNotify($accessGroupId);
	$oldproject->setPassNotify($accessGroupId);
	$oldproject->setPassChain($projectId);
	$oldproject->setFailChain($projectId);
	$oldproject->setGeoId($geoId);
	$oldproject->update();
	
	# Project creation, updating, deletion
	$newproject = new BuildForge::Services::DBO::Project($conn);
	$newproject->setName('New Build Project');
	$newproject->setBuildClass('ClassName');
	$newproject->setLevel($accessGroupId);
	$newproject->create();
	
	$newproject2 = $newproject->clone();
	$newproject->setName('Duplicate');
	$newproject2->create();

	$newproject->setName('Different Name');
	$newproject->update();
	
	$newproject->deactivate();
	$newproject->activate();
	
	$step = new BuildForge::Services::DBO::Step($conn);
	$step->setDescription('Step Name');
	$step->setCommandText('echo hi');
	$newproject->addStep($step);
	$newproject->copyStep(0, 1);
	$newproject->moveStep(1, 0);
	$newproject->removeStep(1);
	$newproject->removeStep($step->getUid());
	$newproject->removeStep($step);
	
	$oldproject->delete();
	$newproject->clobber();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Project allows you to create, access, and delete BuildForge Project projects. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::Project(connection, {ProjectParamsHashRef})

Returns a new instance of a Project object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item ProjectParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item description

The name of the project.

=item level

The access group ID (from $accessGroup->getLevel()) of the access group this
project belongs to.  Users not in the class can not view, edit, or use this
project.

=item buildClass

The class ID (from $class->getName()) of the class that new builds of the 
project belong to by default.

=item selectorId

The selector ID (from $selector->getSelectorId()) of the selector that
new builds of the project belong to by default.  If this is left blank, 
the project is actually a library and will appear in that section of the
GUI.

=item tag

The tag format for builds of this project.  By default, this is set to
'BUILD_$B'.

=item environmentId

The environment ID (from $envGroup->getEnvGroupId()) for the environment
group that is applied to builds of this project.

=item active

When this evaluates to true, the project is active (the default).  When this
evaluates to false, the project is inactive and cannot be executed until it is
activated.

=item startNotify

The access group ID (from $accessGroup->getLevel()) for the access group which
gets notified when a build of the project is kicked off.  Set this to 0 (the default) to have
no such notification sent.

=item passNotify

The access group ID (from $accessGroup->getLevel()) for the access group which
gets notified when a build of the project completes successfully.  Set this to 0 (the default) to have
no such notification sent.

=item failNotify

The access group ID (from $accessGroup->getLevel()) for the access group which
gets notified when a build of the project fails.  Set this to 0 (the default) to have
no such notification sent.

=item maxThread

The maximum number of threaded steps the project is allowed to start simultaneously.
The default value of 0 allows unlimited threads to be run.

=item runLimit

The maximum number of simultaneous builds of this project allowed to be running.
The default value of 0 allows unlimited concurrent builds.

=item tagSync

The project ID (from $project->getProjectId()) of the project this project is tag 
synced to (or 0, the default, for no tag syncing).  Normally, auto-increment tag 
variables increase whenever a build of a project starts up.  When project A is
tag synced to project B, however, if a build of either project A _or_ project B occurs,
any tag variables that they share that are auto-increment increase.

=item sticky

If this evaluates to true, steps that use the project default selector will
all run on the same server.  If this evaluates to false (the default), each such step
will be chosen from the available servers in the project selector, and so may or may
not use the same servers.

=item passChainId

The project ID (from $project->getProjectId()) of a project to get kicked off if a 
build of this project completes successfully.  If this is set to 0 (the default),
no project gets kicked off.

=item failChainId

The project ID (from $project->getProjectId()) of a project to get kicked off if a 
build of this project fails.  If this is set to 0 (the default),
no project gets kicked off.

=item geoId

For GDD-aware projects only, this is the geo ID of the console on which the project
runs by default.  If this is set to the empty string (the default), it assumes the 
system is a non-GDD-aware system.

=back

=back


=head2 BuildForge::Services::DBO::Project->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::Project objects 
corresponding to all projects in the database that the user has access to.
These objects will not contain their Step subobjects.  To get the steps, use 
BuildForge::Services::DBO::Project->findById() or findByName() with the project's ID/name.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Project->findById(connection, projectId)

Returns the BuildForge::Services::DBO::Project object corresponding to the given
projectId, or undef if no such project exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The ID of the desired project (from $project->getProjectId()).

=back


=head2 BuildForge::Services::DBO::Project->findByName(connection, projectName)

Returns the BuildForge::Services::DBO::Project object corresponding to the given
project name, or undef if no such project exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The name of the desired project (from $project->getName()).

=back


=head2 $project->create()

Creates an entry for the project in the database with the current object settings.


=head2 $project->clone()

Returns a BuildForge::Services::DBO::Project object that is a copy of the current 
project.  The new project copy will not exist in the database unless created with 
$copy->create().


=head2 $project->update()

Updates the database with the current project settings.


=head2 $project->addStep(step, index)

Adds a step to the project in the database.

=over 8

=item step

The BuildForge::Services::DBO::Step object to be added to the project.

=item index

The optional location at which to add the new step.  0 makes the step the first step, 1 makes
it the second, etc.  If left unset, the step is added at the end of the step list.

=back


=head2 $project->moveStep(from, to)

Moves a step within the step list.

=over 8

=item from

The index of the step to be moved.  0 moves the first step, 1 the second step, etc.

=item to

The index of the location to move the step to.  0 makes it the first step, 1 the second step, etc.

=back


=head2 $project->copyStep(from, to)

Copies a step within the step list.

=over 8

=item from

The index of the step to be copied.  0 copies the first step, 1 the second step, etc.

=item to

The index of the location to copy the step to.  0 makes the copy the first step, 1 the second step, etc.

=back


=head2 $project->removeStep(step)

Removes a step from the step list.

=over 8

=item step

This may either be the index of the step to be deleted (0 to delete the first step, 
1 the second, etc.), the step ID of the step to be deleted (from $step->getUid()), or the
BuildForge::Services::DBO::Step to be removed.

=back


=head2 $project->activate()

Activates the project within the database.


=head2 $project->deactivate()

Deactivates the project within the database.  Deactivated projects cannot be run until
they are activated.


=head2 $project->delete()

Deletes the project from the database.  This will only work if there are no
current builds of the project existing in the database.


=head2 $project->clobber()

Starts a clobber project action.  After it is called, the engine will
clobber the project by deleting it and all its project from the database 
when all builds of the project are complete and when the engine has resources
available to do so.  The function will return when the clobbering has been readied to 
occur - it may be some time after that before the project is actually clobbered.


=head2 $project->getProjectId()

Returns the ID of the project.


=head2 $project->getName()

Returns the name of the project.


=head2 $project->getActive()

Returns 1 if the project is active or 0 if it is inactive.


=head2 $project->getBuildClass()

Returns the class ID (from $class->getName()) of the class that new builds of the 
project belong to by default.


=head2 $project->getEnvironmentId()

Returns the environment ID (from $envGroup->getEnvGroupId()) for the environment
group that is applied to builds of this project, or 0 if no environment is
assigned.


=head2 $project->getRunLimit()

Returns the maximum number of simultaneous builds of this project allowed to be running.
The default value of 0 allows unlimited concurrent builds.


=head2 $project->getLevel()

Returns the access group ID (from $accessGroup->getLevel()) of the access group this
project belongs to.  Users not in the class can not view, edit, or use this
project.


=head2 $project->getMaxThread()

Returns the maximum number of threaded steps the project is allowed to start simultaneously.
The default value of 0 allows unlimited threads to be run.


=head2 $project->getSelectorId()

Returns the selector ID (from $selector->getSelectorId()) of the selector that
new builds of the project belong to by default.  If this is an empty string, 
the project is actually a library and will appear in that section of the
GUI.


=head2 $project->getSticky()

If this returns 1, steps that use the project default selector will
all run on the same server.  If this returns 0, each such step
will be chosen from the available servers in the project selector, and so may or may
not use the same servers.


=head2 $project->getTag()

Returns the tag format for builds of this project.


=head2 $project->getTagSync()

Returns the project ID (from $project->getProjectId()) of the project this project is tag 
synced to (or 0 for no tag syncing).  Normally, auto-increment tag 
variables increase whenever a build of a project starts up.  When project A is
tag synced to project B, however, if a build of either project A _or_ project B occurs,
any tag variables that they share that are auto-increment increase.


=head2 $project->getStartNotify()

Returns the access group ID (from $accessGroup->getLevel()) for the access group which
gets notified when a build of the project is kicked off or 0 if
no such notification is sent.


=head2 $project->getPassNotify()

Returns the access group ID (from $accessGroup->getLevel()) for the access group which
gets notified when a build of the project completes successfully or 0 if
no such notification is sent.


=head2 $project->getFailNotify()

Returns the access group ID (from $accessGroup->getLevel()) for the access group which
gets notified when a build of the project fails or 0 if
no such notification is sent.


=head2 $project->getPassChain()

Returns the project ID (from $project->getProjectId()) of a project to get kicked off if a 
build of this project completes successfully or 0 if no such project is kicked off.


=head2 $project->getFailChain()

Returns the project ID (from $project->getProjectId()) of a project to get kicked off if a 
build of this project fails or 0 if no such project is kicked off.


=head2 $project->getGeoId()

This returns the geo ID of the console on which the project runs by default.  
It returns the empty string for non-GDD-aware projects.


=head2 $project->getSteps()

Returns a reference to an array of BuildForge::Services::DBO::Step objects for all the
steps in the project.


=head2 $project->getStep(step)

Returns the BuildForge::Services::DBO::Step object for the requested step.

=over 8

=item step

This may either be the index of the step to be returned (0 for the first step, 
1 the second, etc.), or the step's step ID (from $step->getUid()).

=back


=head2 $project->getBuilds()

Returns a reference to an array of BuildForge::Services::DBO::Build objects for
all builds of the project.


=head2 $project->inUse()

If the project is linked to by other objects (for example, if a class is set
to kick off this project when builds of the class are purged), returns 'YES', 
indicating that it cannot be deleted.  If the project is not being linked by 
other objects, but another project or step does chain the project, returns 'CHAIN'.
Otherwise, returns 'NO'.


=head2 $project->setName(projectName)

Sets the name of the project.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item projectName

The new project name.

=back


=head2 $project->setActive(isActive)

Sets whether or not the project is active.  If it is inactive, it cannot be
executed until it is activated.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item isActive

The project becomes active if this evaluates to true or inactive if this 
evaluates to false.

=back


=head2 $project->setBuildClass(className)

Sets the class that new builds of the project belong to by default.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item className

The class ID (from $class->getName()).

=back


=head2 $project->setEnvironmentId(envGroupId)

Sets the environment group that is applied to builds of this project.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item envGroupId

The environment group ID (from $envGroup->getEnvGroupId()).

=back


=head2 $project->setRunLimit(runLimit)

Sets the maximum number of simultaneous builds of this project allowed to be running.
$project->update() must be run before changes are replicated in the database.

=over 8

=item runLimit

The number of allowed builds.  Setting it to 0 allows unlimited concurrent builds.

=back


=head2 $project->setLevel(accessGroupId)

Sets the access group this project belongs to.  Users not in the class can not view, 
edit, or use this project.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()).

=back


=head2 $project->setMaxThread(maxThreads)

Sets the maximum number of threaded steps the project is allowed to start simultaneously.
$project->update() must be run before changes are replicated in the database.

=over 8

=item maxThreads

The maximum number of threads.  Setting it to 0 allows unlimited threads to be run.

=back


=head2 $project->setSelectorId(selectorId)

Sets the selector that new builds of the project belong to by default.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item selectorId

The selector ID (from $selector->getSelectorId()).  If this is set to 0,
the project is actually a library and will appear in that section of the
GUI.

=back


=head2 $project->setSticky(isSticky)

Sets whether or not the project is sticky.  If it is sticky, steps that use the 
project default selector will all run on the same server.  If not, each such step
will be chosen from the available servers in the project selector, and so may or may
not use the same servers.  $project->update() must be run before changes are 
replicated in the database.

=over 8

=item isSticky

The project becomes sticky if this evaluates to true and unsticky if this 
evaluates to false.

=back


=head2 $project->setTag(tagFormat)

Sets the tag format for builds of this project.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item tagFormat

The new tag format.

=back


=head2 $project->setTagSync(projectId)

Sets the project this project is tag 
synced to (or 0, the default, for no tag syncing).  Normally, auto-increment tag 
variables increase whenever a build of a project starts up.  When project A is
tag synced to project B, however, if a build of either project A _or_ project B occurs,
any tag variables that they share that are auto-increment increase.  $project->update() 
must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the synced project.

=back


=head2 $project->setStartNotify(accessGroupId)

Sets the access group which gets notified when a build of the project is 
kicked off.  $project->update() must be run before changes are replicated 
in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()), or 0 to have
no such notification sent.

=back


=head2 $project->setPassNotify(accessGroupId)

Sets the access group which gets notified when a build of the project 
completes successfully.  $project->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()), or 0 to have
no such notification sent.

=back


=head2 $project->setFailNotify(accessGroupId)

Sets the access group which gets notified when a build of the project fails.
$project->update() must be run before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()), or 0 to have
no such notification sent.

=back


=head2 $project->setPassChain(projectId)

Sets the project to get kicked off if a build of this project completes successfully.
$project->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) to be started, or 0 for no project
to get started.

=back


=head2 $project->setFailChain(projectId)

Sets the project to get kicked off if a build of this project fails.
$project->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) to be started, or 0 for no project
to get started.

=back


=head2 $project->setGeoId(geoId)

For GDD-aware projects only, this sets the geo ID of the console on which the project
runs by default.  $project->update() must be run before changes are replicated in the database.

=over 8

=item geoId

The ID of the console to use.  If this is set to the empty string, it assumes the system 
is a non-GDD-aware system.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

