
package BuildForge::Services::DBO::Register;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info (DataDBO)
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(buildId projectId userId register stamp dataText liveBuildId liveProjectId liveRegister);
my $prototype = {
	'buildId' => 0,
	'userId' => 0,
	'projectId' => 0,
	'register' => '',
	'stamp' => 0,
	'dataText' => '',
	'liveBuildId' => 0,
	'liveProjectId' => 0,
	'liveRegister' => undef,	
};
sub getTypeKey { 'Register' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'buildId'});
	Integer(\$self->{'userId'});
	Integer(\$self->{'projectId'});
	Integer(\$self->{'stamp'});
	Integer(\$self->{'liveBuildId'});
	Integer(\$self->{'liveProjectId'});
	return $self;
}


sub findAllRegistersByBuild {
	my ($proto, $conn, $buildId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_REGISTER_FND_BY_BUILD);
	$conn->writeEntryString(KEY_REGISTER_BUILD_ID, $buildId);
	return _getList($conn, $conn->call());
}

sub findAllRegistersByProject {
	my ($proto, $conn, $projectId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_REGISTER_FND_BY_PRJ);
	$conn->writeEntryString(KEY_REGISTER_PROJECT_ID, $projectId);
	return _getList($conn, $conn->call());
}

sub findBuildRegisterById {
	my ($proto, $conn, $buildId, $name) = @_;
	$conn->request(COMMAND_REGISTER_FND_BUILD_REG);
	$conn->writeEntry(KEY_REGISTER_BUILD_ID, $buildId);
	$conn->writeEntry(KEY_REGISTER_NAME, $name);
	my $dbo = $conn->call->{+KEY_REGISTER_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findProjectRegisterById {
	my ($proto, $conn, $projectId, $name) = @_;
	$conn->request(COMMAND_REGISTER_FND_PRJ_REG);
	$conn->writeEntry(KEY_REGISTER_PROJECT_ID, $projectId);
	$conn->writeEntry(KEY_REGISTER_NAME, $name);
	my $dbo = $conn->call->{+KEY_REGISTER_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}


sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_REGISTER_CREATE);
	$conn->writeEntry(KEY_REGISTER_DBO, $self);
	my $dbo = $conn->call->{+KEY_REGISTER_DBO};
	defined($dbo) or die $self->missing(KEY_REGISTER_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_REGISTER_UPDATE);
	$conn->writeEntry(KEY_REGISTER_DBO, $self);
	my $dbo = $conn->call->{+KEY_REGISTER_DBO};
	defined($dbo) or die $self->missing(KEY_REGISTER_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	if ($self->{'buildId'} > 0) {
		$self->deleteBuildRegister(
			$self->{'_conn'},
			$self->{'buildId'},
			$self->{'register'});
	} else {
		$self->deleteProjectRegister(
			$self->{'_conn'},
			$self->{'projectId'},
			$self->{'register'});
	}
}

sub deleteBuildRegister {
	my ($proto, $conn, $buildId, $register) = @_;
	$conn->request(COMMAND_REGISTER_DELETE);
	$conn->writeEntry(KEY_REGISTER_BUILD_ID, $buildId);
	$conn->writeEntry(KEY_REGISTER_NAME, $register);
	$conn->call();
}

sub deleteProjectRegister {
	my ($proto, $conn, $projectId, $register) = @_;
	$conn->request(COMMAND_REGISTER_DELETE);
	$conn->writeEntry(KEY_REGISTER_PROJECT_ID, $projectId);
	$conn->writeEntry(KEY_REGISTER_NAME, $register);
	$conn->call();
}

sub deleteAllRegistersByBuild {
	my ($proto, $conn, $buildId) = @_;
	$conn->request(COMMAND_REGISTER_DEL_BY_BUILD);
	$conn->writeEntry(KEY_REGISTER_BUILD_ID, $buildId);
	$conn->call();
}

sub deleteAllRegistersByProject {
	my ($proto, $conn, $projectId) = @_;
	$conn->request(COMMAND_REGISTER_DEL_BY_PROJECT);
	$conn->writeEntry(KEY_REGISTER_PROJECT_ID, $projectId);
	$conn->call();
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_REGISTER_LIST};
	!defined($list) and return undef;
	my @registers = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@registers;
}

sub getBuildId   { $_[0]->{'buildId'} ? $_[0]->{'buildId'} : $_[0]->{'liveBuildId'}; }
sub getProjectId { $_[0]->{'projectId'} ? $_[0]->{'projectId'} : $_[0]->{'liveProjectId'}; }
sub getUserId    { $_[0]->{'userId'}; }
sub getRegister  { $_[0]->{'register'} ? $_[0]->{'register'} : $_[0]->{'liveRegister'}; }
sub getStamp     { $_[0]->{'stamp'}; }
sub getDataText  { $_[0]->{'dataText'}; }

sub setBuildId   { $_[0]->{'buildId'}   = $_[1]; }
sub setProjectId { $_[0]->{'projectId'} = $_[1]; }
sub setRegister  { $_[0]->{'register'}  = $_[1]; }
sub setStamp     { $_[0]->{'stamp'}     = $_[1]; }
sub setDataText  { $_[0]->{'dataText'}  = $_[1]; }

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Register


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing registers
	$buildRegisters = BuildForge::Services::DBO::Register->findAllRegistersByBuild($conn, $buildId);
	$buildRegisters = BuildForge::Services::DBO::Register->findAllRegistersByProject($conn, $projectId);
	$register1 = BuildForge::Services::DBO::Register->findBuildRegisterById($conn, $buildId, 'REGNAME1');
	$register2 = BuildForge::Services::DBO::Register->findProjectRegisterById($conn, $projectId, 'REGNAME2');

	# Getter / setter functions
	$buildId = $register1->getBuildId();
	$projectId = $register1->getProjectId();
	$userId = $register1->getUserId();
	$registerName = $register1->getRegister();
	$registerValue = $register1->getDataText();
	$timestamp = $register1->getStamp();

	$register1->setBuildId($buildId);
	$register1->setProjectId($projectId);
	$register1->setRegister('NEWREGNAME');
	$register1->setDataText('Register Contents');
	$register1->setStamp($now);
	$register1->update();
	
	# Register creation, updating, deletion
	$newregister = new BuildForge::Services::DBO::Register($conn);
	$newregister->setRegister('New Build Register');
	$newregister->setDataText('Register Contents');
	$newregister->setProjectId($projectId);
	$newregister->create();

	$register1->setRegister('DIFFERENTREGNAME');
	$newregister->update();
	
	$newregister->delete();
	BuildForge::Services::DBO::Register->deleteBuildRegister($conn, $register1->getBuildId(), 
			$register1->getRegister());
	BuildForge::Services::DBO::Register->deleteProjectRegister($conn, $register2->getProjectId(), 
			$register2->getRegister());
	BuildForge::Services::DBO::Register->deleteAllRegistersByBuild($conn, $buildId);
	BuildForge::Services::DBO::Register->deleteAllRegistersByProject($conn, $projectId);
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Register allows you to create, access, and delete BuildForge project and build registers. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::Register(connection{RegisterParamsHashRef})

Returns a new instance of a Register object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item RegisterParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item buildId

0 to make this a project register, otherwise a build ID (from $build->getBuildId()) to 
make it a build register attached to that build.

=item projectId

For a build register, this value is ignored.  For a project register, a 
project ID (from $project->getProjectId()) to 
make it a project register attached to that project.

=item register

The name of the register.

=item dataText

The contents of the register.

=back

=back


=head2 BuildForge::Services::DBO::Register->findAllRegistersByBuild(connection, buildId)

Returns a reference to an array of all BuildForge::Services::DBO::Register objects
that refer to build registers associated with the given build.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The build ID (from $build->getBuildId()) for the desired build.

=back


=head2 BuildForge::Services::DBO::Register->findAllRegistersByProject(connection, projectId)

Returns a reference to an array of all BuildForge::Services::DBO::Register objects
that refer to project registers associated with the given project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) for the desired project.

=back


=head2 BuildForge::Services::DBO::Register->findBuildRegisterById(connection, buildId, registerName)

Returns the BuildForge::Services::DBO::Register object for the desired build register
or undef if it does not exist.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The build ID (from $build->getBuildId()) for the desired build.

=item registerName

The register name (from $register->getRegister()) of the desired register.

=back


=head2 BuildForge::Services::DBO::Register->findProjectRegisterById(connection, projectId, registerName)

Returns the BuildForge::Services::DBO::Register object for the desired project register
or undef if it does not exist.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectID

The project ID (from $project->getProjectId()) for the desired project.

=item registerName

The register name (from $register->getRegister()) of the desired register.

=back


=head2 $register->create()

Creates the register in the database.


=head2 $register->update()

Updates the register in the database.


=head2 $register->delete()

Deletes the register from the database.


=head2 BuildForge::Services::DBO::Register->deleteBuildRegister(connection, buildId, registerName)

Deletes the given build register from the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The build ID (from $build->getBuildId()) for the desired build.

=item registerName

The register name (from $register->getRegister()) of the desired register.

=back


=head2 BuildForge::Services::DBO::Register->deleteProjectRegister(connection, projectId, registerName)

Deletes the given project register from the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) for the desired project.

=item registerName

The register name (from $register->getRegister()) of the desired register.

=back


=head2 BuildForge::Services::DBO::Register->deleteAllRegistersByBuild(connection, buildId)

Deletes all build registers from the database attached to the given build.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The build ID (from $build->getBuildId()) for the desired build.

=back


=head2 BuildForge::Services::DBO::Register->deleteAllRegistersByProject(connection, projectId)

Deletes all project registers from the database attached to the given project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) for the desired project.

=back


=head2 $register->getBuildId()

Returns the register's build ID if it is a build register or 0 otherwise.


=head2 $register->getProjectId()

Returns the register's project ID if it is a project register or 0 otherwise.


=head2 $register->getUserId()

Returns the user ID of the user who created this register.


=head2 $register->getRegister()

Returns the register's name.


=head2 $register->getStamp()

Returns the timestamp for when the register value was last updated.


=head2 $register->getDataText()

Returns the value of the register.


=head2 $register->setBuildId(buildId)

Sets the build ID of the associated build for build registers.  $register->update() must be run 
before changes are replicated in the database.

=over 8

=item buildId

0 to make this a project register, otherwise a build ID (from $build->getBuildId()) to 
make it a build register attached to that build.

=back


=head2 $register->setProjectId(projectId)

Sets the project ID of the associated project for project registers.  If the register is a 
build register, this value will be ignored.  $register->update() must be run 
before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project to which to attach this 
project register.

=back


=head2 $register->setRegister(registerName)

Sets the name of the register.  $register->update() must be run 
before changes are replicated in the database.

=over 8

=item registerName

The new name for the register.

=back


=head2 $register->setDataText(value)

Sets the value of the register.  $register->update() must be run 
before changes are replicated in the database.

=over 8

=item value

The new value for the register.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

