
package BuildForge::Services::DBO::Selector;
use BuildForge::Services::DBO::SelectorProperty;

use strict;
use overload '""' => \&toString;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(selectorId level liveSelectorId);
my $prototype = {
	liveSelectorId	=> undef,
	selectorId		=> undef,
	level			=> undef,
};
sub getTypeKey { 'Selector'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_properties'} = [];
	Integer(\$self->{'level'});
	return $self;
}

sub findAll {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_SELECTOR_FIND_ALL);
	return _readList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $selectorName) = @_;
	$conn->request(COMMAND_SELECTOR_FIND_ID);
	$conn->writeEntryString(KEY_SELECTOR_ID, $selectorName);
	return _readSelector($conn, $conn->call());
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SELECTOR_ADD);
	_writeSelector($self, $conn);
	my $resp = $conn->call;
	my $dbo = $resp->{+KEY_SELECTOR_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or die $self->missing(KEY_SELECTOR_DBO);
	my $props = $self->{'_properties'};
	$self->fromArray($dbo);
	foreach my $prop (@$props) { $prop->{'_parent'} = $self; }
	$self->{'_properties'} = $props;
	return $self;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SELECTOR_UPD);
	$conn->writeEntryDBO(KEY_SELECTOR_DBO, $self);
	my $dbo = $conn->call()->{+KEY_SELECTOR_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or die $self->missing(KEY_SELECTOR_DBO);
	my $props = $self->{'_properties'};
	$self->fromArray($dbo);
	$self->{'_properties'} = $props;
	return $self;
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $selectorId = $self->{'liveSelectorId'};
	$conn->request(COMMAND_SELECTOR_DEL);
	$conn->writeEntryString(KEY_SELECTOR_ID, $selectorId);
	$conn->call();
	$self->{'selectorId'} = $self->{'liveSelectorId'};
	$self->{'liveSelectorId'} = undef;
}

sub addProperty {
	my ($self, $prop) = @_;
	_dbo_addProperty($self, $prop);
	my $liveSelectorId = $self->{'liveSelectorId'};
	defined($liveSelectorId) && length($liveSelectorId) and $prop->create();
	$prop->{'_parent'} = $self;
}

sub removeProperty {
	my ($self, $prop) = @_;
	!defined($prop) and return undef;
	!isa($prop, 'BuildForge::Services::DBO::SelectorProperty') and 
			return $self->removeProperty($self->getProperty($prop));
	!_dbo_removeProperty($self, $prop) and return undef;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SELECTOR_ENTRY_DEL);
	$conn->writeEntryDBO(KEY_SELECTOR_PROPERTY_DBO, $prop);
	my $dbo = $conn->call()->{+KEY_SELECTOR_PROPERTY_DBO};
	defined($dbo) or return undef;
	$prop->fromArray($dbo);
	$prop->{'livePropertyName'} = undef;
	return $prop;
}

sub _readList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_SELECTOR_LIST};
	!defined($list) and return [];
	my @sels = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@sels;
}

sub _readSelector($$) {
	my ($conn, $resp) = @_;
	my $selector = $resp->{+KEY_SELECTOR_DBO};
	defined($selector) or return undef;
	$selector = __PACKAGE__->new($conn)->fromArray($selector);

	my $list = $resp->{+KEY_SELECTOR_PROPERTY_LIST};
	defined($list) && isa($list, 'ARRAY') or return $selector;

	my @props = map { BuildForge::Services::DBO::SelectorProperty->new($conn)->fromArray($_) } @$list;
	foreach my $prop (@props)  { $prop->{'_parent'} = $selector; }
	$selector->{'_properties'} = \@props;
	return $selector;
}


sub _writeSelector($$) {
	my ($self, $conn) = @_;
	$conn->writeEntryDBO(KEY_SELECTOR_DBO, $self);
	my $props = $self->{'_properties'};
	defined($props) && @$props > 0
		and $conn->writeEntryArray(KEY_SELECTOR_PROPERTY_LIST, $props);
}

# wrap is inherited

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self);
	my @props = map { $_->clone } @{ $self->{'_properties'} };
	$clone->{'_properties'} = \@props;
	$clone->{'selectorId'} = undef;
	$clone->{'liveSelectorId'} = undef;
	return $clone;
}

sub toString {
	my ($self) = @_;
	return '[SelectorProperty: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ', _properties=' . scalar @{ $self->{'_properties'} } . ']';
}

# Aliases for convenience
sub getSelectorId { getSelectorName(@_); }
sub getSelectorName { defined($_[0]->{'selectorId'}) ? $_[0]->{'selectorId'} : $_[0]->{'liveSelectorId'}; }
sub getLevel { $_[0]->{'level'}; }
sub getProperties {
	my $list = $_[0]->{'_properties'};
	return defined($list) ? [@{ $list }] : undef;
}

sub getProperty {
	my ($self, $name) = @_;
	my $index = _dbo_findIndex($self, $name);
	my $props = $self->{'_properties'};
	!isa($props, 'ARRAY') and return undef;
	return $index >= 0 ? $self->{'_properties'}->[$index] : undef;
}

sub setSelectorId { setSelectorName(@_); }
sub setSelectorName {
	my ($self, $selectorId) = @_;
	$self->{'selectorId'} = $selectorId;
	my $props = $self->{'_properties'};
	if (defined($props)) {
		foreach (@$props) { $_->{'selectorId'} = $selectorId; }
	}
}
sub setLevel { $_[0]->{'level'} = $_[1]; }


#----------------------------------------------------------------
# DBO Methods
#----------------------------------------------------------------

sub _dbo_addProperty ($$) {
	my ($self, $prop, $index) = @_;
	my $props = $self->{'_properties'};
	$props = $self->{'_properties'} = [] unless isa($props, 'ARRAY');
	$prop->{'selectorId'} = $self->{'selectorId'};
	push @$props, $prop;
}

sub _dbo_removeProperty ($$) {
	my ($self, $prop) = @_;
	my $props = $self->{'_properties'};
	!isa($props, 'ARRAY') and return 0;
	for (my $i=0; $i<@$props; $i++) {
		next unless $props->[$i]->{'propertyName'} eq $prop->{'propertyName'};
		delete $props->[$i];
		return 1;
	}
	return 0;
}

sub _dbo_findIndex ($$) {
	my ($self, $name) = @_;
	my $props = $self->{'_properties'};
	!isa($props, 'ARRAY') and return -1;
	for (my $i=0; $i<@$props; $i++) {
		return $i if $props->[$i]->{'propertyName'} eq $name;
	}
	return -1;
}

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Selector


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing selectors
	$allSelectors = BuildForge::Services::DBO::Selector->findAll($conn);
	$oldselector = BuildForge::Services::DBO::Selector->findById($conn, 'SelectorName');

	# Getter / setter functions
	$id = $oldselector->getSelectorId();
	$name = $oldselector->getSelectorName();
	$accessGroupId = $oldselector->getLevel();
	$allProperties = $oldselector->getProperties();
	$property = $oldselector->getProperty('BF_JOBS');
	
	$oldselector->setSelectorId('New Name');
	$oldselector->setSelectorName('Another New Name');
	$oldselector->setLevel($accessGroupId);
	$oldselector->update();
	
	# Selector creation, modification, deletion
	$newselector = new BuildForge::Services::DBO::Selector($conn);
	$newselector->setSelectorName('New Selector');
	$newselector->setLevel($accessGroupId);
	$newselector->create();
	
	$newselector2 = $newselector->clone();
	$newselector2->setSelectorName('Duplicate');
	$newselector2->create();
	
	$newselector->setSelectorName('New Name For New Selector');
	$newselector->update();

	$property = new BuildForge::Services::DBO::SelectorProperty($conn);
	$property->setPropertyName('PROP_NAME');
	$property->setOperator(ENUM_SELECTORPROPERTY_OPERATOR->EQ);
	$property->setPropertyValue('Value');
	$newselector->addProperty($property);
	$newselector->removeProperty($property);
	$newselector->removeProperty($property->getPropertyName());
	
	$newselector->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Selector allows you to create, modify, and delete BuildForge selectors.


=head1 METHODS


=head2 new BuildForge::Services::DBO::Selector(connection, {SelectorParamsHashRef})

Returns a new instance of a Selector object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item SelectorParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item selectorId

The name / ID of the selector.

=item level

The access group ID (from $accessGroup->getLevel()) of the access group that the
selector belongs to.  If a user does not belong to this access group, they cannot
view, edit, or use the selector.

=back

=back


=head2 BuildForge::Services::DBO::Selector->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::Selector objects corresponding
to all selectors in the database that the user has access to.  These objects will not
contain their SelectorProperty subobjects.  To get the properties, use 
BuildForge::Services::DBO::Selector->findById() with the selector's ID.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Selector->findById(connection, selectorId)

Returns the BuildForge::Services::DBO::Selector object corresponding to the given
selector ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item selectorId

The selector ID / name of the desired selector (from $selector->getSelectorId()).

=back


=head2 $selector->create()

Creates the selector object inside the database.


=head2 $selector->clone()

Returns a copy of the BuildForge::Services::DBO::Selector object.  This copy
will not exist in the database until $copy->create() is called.


=head2 $selector->update()

Updates the selector record within the database.


=head2 $selector->addProperty(property)

Adds the given property to the selector within the database.

=over 8

=item property

The BuildForge::Services::DBO::SelectorProperty object to be added.

=back


=head2 $selector->removeProperty(property)

Removes the given property from the selector within the database.

=over 8

=item property

The BuildForge::Services::DBO::SelectorProperty object to be deleted or the property name.
If there is more than one property with the same name, the first one added to the selector 
will be deleted.

=back


=head2 $selector->delete()

Deletes the selector and all its properties from the database.


=head2 $selector->getSelectorId()

Returns the name / ID of the selector.  Alias for $selector->getSelectorName().


=head2 $selector->getSelectorName()

Returns the name / ID of the selector.


=head2 $selector->getLevel()

Returns the access group ID for the access group that controls the selector.  Users
not in the access group cannot view, edit, or use the selector.


=head2 $selector->getProperties()

Returns a reference to an array of BuildForge::Services::DBO::SelectorProperty objects
corresponding to all selector properties within the selector.


=head2 $selector->getProperty(propertyName)

Returns the BuildForge::Services::DBO::SelectorProperty object corresponding to the 
given name.  If there is more than one property with the same name, the first one added
to the selector will be deleted.

=over 8

=item propertyName

The name of the desired selector property.

=back


=head2 $selector->setSelectorId(selectorId)

Sets the selector ID / name.  Alias for $selector->setSelectorName().  $selector->update() must be run 
before changes are replicated in the database.

=over 8

=item selectorId

The new ID / name for the selector.

=back


=head2 $selector->setSelectorName(selectorId)

Sets the selector ID / name.  $selector->update() must be run 
before changes are replicated in the database.

=over 8

=item selectorId

The new ID / name for the selector.

=back


=head2 $selector->setLevel(accessGroupId)

Sets the access group that the selector belongs to.  Users who don't belong to 
this access group can't view, edit, or use this selector.  $selector->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The ID of the access group, from $accessGroup->getLevel().

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

