
package BuildForge::Services::DBO::SelectorProperty;

use strict;
use overload '""' => \&toString;

use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(selectorId propertyName propertyValue
	operator required livePropertyName);
my $prototype = {
	_parent             => undef,
	selectorId          => undef,
	propertyName        => undef,
	propertyValue       => undef,
	operator            => ENUM_SELECTORPROPERTY_OPERATOR->EQ,
	required            => 0,
	livePropertyName    => undef,
};
sub getTypeKey { 'SelectorProp'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my ($newParent) = splice(@_, 1, 1);
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_parent'} = $newParent;
	Enum(ENUM_SELECTORPROPERTY_OPERATOR, \$self->{'operator'});
	Boolean(\$self->{'required'});
	return $self;
}


# INTERNAL USE ONLY
sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->internal();
	$conn->request(COMMAND_SELECTOR_ENTRY_ADD);
	$conn->writeEntryDBO(KEY_SELECTOR_PROPERTY_DBO, $self);
	my $dbo = $conn->call()->{KEY_SELECTOR_PROPERTY_DBO()};
	defined($dbo) or die $self->missing(KEY_SELECTOR_PROPERTY_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $liveSelectorId = $self->{'_parent'}->{'liveSelectorId'};
	if (defined($liveSelectorId) && length($liveSelectorId)) {
		my $conn = $self->{'_conn'};
		$conn->request(COMMAND_SELECTOR_ENTRY_UPD);
		$conn->writeEntryDBO(KEY_SELECTOR_PROPERTY_DBO, $self);
		my $dbo = $conn->call()->{KEY_SELECTOR_PROPERTY_DBO()};
		defined($dbo) or die $self->missing(KEY_SELECTOR_PROPERTY_DBO);
		return $self->fromArray($dbo);
	}
	return $self;
}

sub delete { $_[0]->{'_parent'}->removeProperty($_[0]); }

# wrap is inherited

sub getParent { $_[0]->{'_parent'}; }

# setParent is not needed

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self->{'_parent'});
	$clone->{'_parent'} = undef;
	$clone->{'selectorId'} = undef;
	return $clone;
}

sub getOperator            { $_[0]->{'operator'}; }
sub getPropertyName        { $_[0]->{'propertyName'}; }
sub getPropertyValue       { $_[0]->{'propertyValue'}; }
sub getRequired            { $_[0]->{'required'} ? 1 : 0; }
sub getSelectorId          { $_[0]->{'selectorId'}; }

sub setOperator            { $_[0]->{'operator'}      = $_[1]; }
sub setPropertyName        { $_[0]->{'propertyName'}  = $_[1]; }
sub setPropertyValue       { $_[0]->{'propertyValue'} = $_[1]; }
sub setRequired            { $_[0]->{'required'}      = $_[1]; }


sub toString {
	my ($self) = @_;
	return '[SelectorProperty: ' . join(', ', map {
		$_ . '=' . (defined($self->{$_})
			? ('"' . $self->{$_} . '"')
			: 'undef')
	} @keys) . ']';
}


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::SelectorProperty


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	$property = new BuildForge::Services::DBO::SelectorProperty($conn);

	# Getter / setter functions
	$selectorName = $property->getSelectorId();
	$propertyName = $property->getPropertyName();
	$operator = $property->getOperator();
	$value = $property->getPropertyValue();
	$isRequired = $property->getRequired();
	$parentSelector = $property->getParent();

	$property->setPropertyName('PROP_NAME');
	$property->setOperator(ENUM_SELECTORPROPERTY_OPERATOR->EQ);
	$property->setPropertyValue('New Value');
	$property->setRequired(1);
	$property->update();
	
	# Selector property creation, modification, and deletion
	$selector = BuildForge::Services::DBO::Selector->findById($conn, $selectorId);
	$newproperty = new BuildForge::Services::DBO::SelectorProperty($conn);
	$newproperty->setPropertyName('PROPERTY_NAME');
	$newproperty->setOperator(ENUM_SELECTORPROPERTY_OPERATOR->EQ);
	$newproperty->setPropertyValue('Value');
	$selector->addProperty($var);

	$newproperty->setPropertyValue('New Value');
	$newproperty->update();
	
	$newproperty2 = $newproperty->clone();
	$selector->addProperty($newproperty2);

	$newproperty->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

SelectorProperty allows you to create and modify selector properties inside of a Selector.  To create
selector properties in the database, see the Selector->addSelectorProperty() function.


=head1 METHODS


=head2 new BuildForge::Services::DBO::SelectorProperty(connection, {selector}, {SelectorPropertyParamsHashRef})

Returns a new instance of a SelectorProperty object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item selector

This should be undef or not passed - it is only used internally.

=item SelectorPropertyParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item propertyName

The name of the selector property.

=item propertyValue

The selector property value.

=item operator

The selector property operator.  This may be one of 'EQ', 'NE', 'GT', 'GE', 'LT', 'LE', or
'CONTAINS', defaulting to 'EQ'.

=item required

This should be 1 if this selector property is required or 0 (the default) otherwise.

=back

=back


=head2 $property->update()

Updates the selector property within the database.


=head2 $property->delete()

Deletes the selector property from the database.


=head2 $property->clone()

Returns a copy of the BuildForge::Services::DBO::Selector object.  The object will not exist
in the database until added with $selector->addSelectorProperty($copy).


=head2 $property->getSelectorId()

Returns the name / ID of the selector that the property belongs to.


=head2 $property->getPropertyName()

Returns the name of the selector property.


=head2 $property->getOperator()

Returns the selector property operator.  This will be one of 'EQ', 'NE', 'GT', 'GE', 
'LT', 'LE', or 'CONTAINS'.


=head2 $property->getPropertyValue()

Returns the selector property value.


=head2 $property->getRequired()

Returns 1 if the selector property is required or 0 if it is optional.


=head2 $property->getParent()

Returns the BuildForge::Services::DBO::Selector object for the selector
this property belongs to.


=head2 $property->setPropertyName(propertyName)

Sets the name of the selector property.  $property->update() must be run 
before changes are replicated in the database.

=over 8

=item propertyName

The new name for the property.

=back


=head2 $property->setOperator(propertyOp)

Sets the property operator.  $property->update() must be run 
before changes are replicated in the database.

=over 8

=item propertyOp

The new operator.  This may be one of 'EQ', 'NE', 'GT', 'GE', 'LT', 'LE', or
'CONTAINS'.

=back


=head2 $property->setPropertyValue(propertyValue)

Sets the property value.  $property->update() must be run 
before changes are replicated in the database.

=over 8

=item propertyValue

The new value.

=back


=head2 $property->setRequired(isRequired)

Sets whether the property is required.  $property->update() must be run 
before changes are replicated in the database.

=over 8

=item isRequired

If this evaluates to true, the property will be required.  If false, the
property will be optional.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

