
package BuildForge::Services::DBO::Semaphore;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info (DataDBO)
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(name buildId userId timestamp active requestor liveName);
my $prototype = {
	'name' => '',
	'buildId' => 0,
	'userId' => '',
	'timestamp' => 0,
	'active' => 0,
	'requestor' => '',
	'liveName' => undef,
};
sub getTypeKey { 'Semaphore'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'buildId'});
	Integer(\$self->{'timestamp'});
	Boolean(\$self->{'active'});
	return $self;
}


sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SEM_FIND_ALL);
	return _getList($conn, $conn->call());
}


sub findById {
	my ($proto, $conn, $semaphoreId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SEM_FIND_NAME);
	$conn->writeEntryString(KEY_SEMAPHORE_NAME, $semaphoreId);
	my $dbo = $conn->call()->{+KEY_SEMAPHORE_DBO};
	return defined($dbo) ? $proto->new($conn)->fromArray($dbo) : undef;
}


sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SEM_ADD);
	$conn->writeEntryDBO(KEY_SEMAPHORE_DBO, $self);
	my $dbo = $conn->call()->{KEY_SEMAPHORE_DBO()};
	defined($dbo) or die $self->missing(KEY_SEMAPHORE_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SEM_DEL);
	$conn->writeEntryString(KEY_SEMAPHORE_NAME, $self->{'name'});
	$conn->call();
}

sub acquire {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SEM_GET);
	$conn->writeEntryDBO(KEY_SEMAPHORE_DBO, $self);
	my $active = 0;
	Boolean(\$active);
	$active = $conn->call()->{+KEY_SEMAPHORE_ACTIVE};
	$self->{'active'} = $active;
	return $active;
}

sub release {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SEM_PUT);
	$conn->writeEntryString(KEY_SEMAPHORE_NAME, $self->{'name'});
	$conn->call();
	my $active = 0;
	Boolean(\$active);
	$self->{'active'} = $active;
	return $active;
}


sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_SEMAPHORE_LIST};
	!isa($list, 'ARRAY') and return [];
	my @semas = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@semas;
}

sub getActive        { $_[0]->{'active'}; }
sub getBuildId       { $_[0]->{'buildId'}; }
sub getName          { $_[0]->{'name'}; }
sub getRequestor     { $_[0]->{'requestor'}; }
sub getTimestamp     { $_[0]->{'timestamp'}; }
sub getUserId        { $_[0]->{'userId'}; }

sub setActive        { $_[0]->{'active'}        = $_[1]; }
sub setBuildId       { $_[0]->{'buildId'}       = $_[1]; }
sub setName          { $_[0]->{'name'}          = $_[1]; }
sub setRequestor     { $_[0]->{'requestor'}     = $_[1]; }
sub setTimestamp     { $_[0]->{'timestamp'}     = $_[1]; }
sub setUserId        { $_[0]->{'userId'}        = $_[1]; }

# toString is inherited


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Semaphore


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing semaphores
	$allSemaphores = BuildForge::Services::DBO::Semaphore->findAll($conn);
	$semaphore = BuildForge::Services::DBO::Semaphore->findById($conn, 'SEM1');

	# Getter / setter functions
	$id = $semaphore->getName();
	$isActive = $semaphore->getActive();
	$buildId = $semaphore->getBuildId();
	$requestor = $semaphore->getRequestor();
	$timestamp = $semaphore->getTimestamp();
	
	# Semaphore releasing / deletion
	$newsemaphore->release();
	$newsemaphore->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Semaphore allows you to view, release, and delete BuildForge semaphores. 


=head1 METHODS

=head2 BuildForge::Services::DBO::Semaphore->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::Semaphore objects corresponding
to all semaphores in the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Semaphore->findById(connection, semaphoreId)

Returns the BuildForge::Services::DBO::Semaphore object corresponding to the given
semaphore ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item semaphoreId

The name / ID of the desired semaphore (from $semaphore->getName()).

=back


=head2 $semaphore->release()

Releases the semaphore while leaving it in the database.


=head2 $semaphore->delete()

Releases the semaphore and deletes it from the database.


=head2 $semaphore->getName()

Returns the ID / name of the semaphore.


=head2 $semaphore->getActive()

Returns 1 if the semaphore is currently active or 0 otherwise.


=head2 $semaphore->getBuildId()

Returns the build ID of the build that currently holds the semaphore.


=head2 $semaphore->getRequestor()

Returns a string describing who last requested the semaphore.


=head2 $semaphore->getTimestamp()

Returns the timestamp for the time the semaphore was last acquired or released.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

