
package BuildForge::Services::DBO::Server;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info (DataDBO)
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(serverId name level host path publish environmentId
	authId active collectorId error testText upstate);
my $prototype = {
	'serverId' => 0,
	'name' => '',
	'level' => 0,
	'host' => '',
	'path' => '/',
	'publish' => ENUM_SERVER_PUBLISH->NONE,
	'environmentId' => 0,
	'authId' => '',
	'active' => 1,
	'collectorId' => '',
	'error' => ENUM_SERVER_ERROR->TESTING,
	'testKey' => '',
	'testText' => '',
	'update' => ENUM_SERVER_UPSTATE->READY,
};
sub getTypeKey { 'Server'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'serverId'});
	Integer(\$self->{'level'});
	Enum(ENUM_SERVER_PUBLISH, \$self->{'publish'});
	Integer(\$self->{'environmentId'});
	Boolean(\$self->{'active'});
	Enum(ENUM_SERVER_ERROR, \$self->{'error'});
	Enum(ENUM_SERVER_UPSTATE, \$self->{'update'});
	return $self;
}


sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SERVER_FIND_ALL);
	return _getList($conn, $conn->call());
}


sub findById {
	my ($proto, $conn, $serverId) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SERVER_FIND_ID);
	$conn->writeEntryString(KEY_SERVER_ID, $serverId);
	my $dbo = $conn->call()->{KEY_SERVER_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByName {
	my ($proto, $conn, $serverName) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SERVER_FIND_NAME);
	$conn->writeEntryString(KEY_SERVER_NAME, $serverName);
	my $dbo = $conn->call()->{KEY_SERVER_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}


sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SERVER_ADD);
	$conn->writeEntry(KEY_SERVER_DBO, $self);
	my $dbo = $conn->call()->{KEY_SERVER_DBO()};
	defined($dbo) or die $self->missing(KEY_SERVER_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SERVER_UPD);
	$conn->writeEntry(KEY_SERVER_DBO, $self);
	my $dbo = $conn->call()->{KEY_SERVER_DBO()};
	defined($dbo) or die $self->missing(KEY_SERVER_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SERVER_DEL);
	$conn->writeEntryString(KEY_SERVER_ID, $self->{'serverId'});
	$conn->call();
	$self->{'serverId'} = 0;
}

sub getManifest {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_MANIFEST_FIND_BY_SERVER);
	$conn->writeEntry(KEY_MANIFEST_SERVER_NAME, $self->{'name'});
	return _readManifest($conn, $conn->call());
}


# wrap is inherited

sub _readManifest($$) {
	my ($conn, $resp) = @_;
	my $dbo = $resp->{+KEY_MANIFEST_DBO};
	defined($dbo) && isa($dbo, 'ARRAY') or return undef;
	my $manifest = BuildForge::Services::DBO::Manifest->new($conn)->fromArray($dbo);

	my $list = $resp->{+KEY_MANIFEST_PROPERTY_LIST};
	if (isa($list, 'ARRAY') && @$list) {
		my %props;
		foreach my $item (@$list) {
			my $prop = BuildForge::Services::DBO::ManifestProperty->new($conn)->fromArray($item);
			$props{$prop->{'propertyName'}} = $prop;
		}
		$manifest->{'properties'} = \%props;
	}
	return $manifest;
}

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_SERVER_LIST};
	!defined($list) and return undef;
	my @servs = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@servs;
}

sub getActive        { $_[0]->{'active'}; }
sub getAuthId        { $_[0]->{'authId'}; }
sub getCollectorId   { $_[0]->{'collectorId'}; }
sub getEnvironmentId { $_[0]->{'environmentId'}; }
sub getError         { $_[0]->{'error'}; }
sub getHost          { $_[0]->{'host'}; }
sub getLevel         { $_[0]->{'level'}; }
sub getName          { $_[0]->{'name'}; }
sub getPath          { $_[0]->{'path'}; }
sub getPublish       { $_[0]->{'publish'}; }
sub getServerId      { $_[0]->{'serverId'}; }
sub getTestText      { $_[0]->{'testText'}; }
sub getUpstate       { $_[0]->{'upstate'}; }

sub setActive        { $_[0]->{'active'}        = $_[1]; }
sub setAuthId        { $_[0]->{'authId'}        = $_[1]; }
sub setCollectorId   { $_[0]->{'collectorId'}   = $_[1]; }
sub setEnvironmentId { $_[0]->{'environmentId'} = $_[1]; }
sub setError         { $_[0]->{'error'}         = $_[1]; }
sub setHost          { $_[0]->{'host'}          = $_[1]; }
sub setLevel         { $_[0]->{'level'}         = $_[1]; }
sub setName          { $_[0]->{'name'}          = $_[1]; }
sub setPath          { $_[0]->{'path'}          = $_[1]; }
sub setPublish       { $_[0]->{'publish'}       = $_[1]; }
sub setTestText      { $_[0]->{'testText'}      = $_[1]; }
sub setUpstate       { $_[0]->{'upstate'}       = $_[1]; }

# toString is inherited


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Server


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing servers
	$allServers = BuildForge::Services::DBO::Server->findAll($conn);
	$oldserver1 = BuildForge::Services::DBO::Server->findById($conn, $serverId);
	$oldserver2 = BuildForge::Services::DBO::Server->findByName($conn, 'ServerName');

	# Getter / setter functions
	$id = $oldserver1->getServerId();
	$hostName = $oldserver1->getHost();
	$name = $oldserver1->getName();
	$isActive = $oldserver1->getActive();
	$serverAuthId = $oldserver1->getAuthId();
	$collectorId = $oldserver1->getCollectorId();
	$environmentId = $oldserver1->getEnvironmentId();
	$accessGroupId = $oldserver1->getLevel();
	$path = $oldserver1->getPath();
	$allowWhatOperations = $oldserver1->getPublish();
	$testResults = $oldserver1->getTestText();
	$currentState = $oldserver1->getUpstate();
	$errorState = $oldserver1->getError();
	$manifest = $oldserver1->getManifest();

	$oldserver1->setName('New Name');
	$oldserver1->setHost('hostname');
	$oldserver1->setActive(1);
	$oldserver1->setAuthId($authId);
	$oldserver1->setCollectorId($collectorId);
	$oldserver1->setEnvironmentId($environmentId);
	$oldserver1->setLevel($accessGroupId);
	$oldserver1->setPath('/tmp');
	$oldserver1->setPublish(ENUM_SERVER_PUBLISH->GET);
	$oldserver1->update();
	
	# Server creation, updating, deletion
	$newserver = new BuildForge::Services::DBO::Server($conn);
	$newserver->setName('New Build Server');
	$newserver->setLevel(1); # Guest access
	$newserver->create();

	$newserver->setLevel($accessGroupId);
	$newserver->update();
	
	$newserver->delete();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Server allows you to create, access, and delete BuildForge servers. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::Server(connection, {ServerParamsHashRef})

Returns a new instance of a Server object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item ServerParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item name

The name of the server.

=item level

The access group ID (from $accessGroup->getLevel()) for the access group that the
server belongs to.  Users who do not belong to this access group cannot view, edit,
or use the server.

=item host

The hostname of the server (with an optional port number separated by ':').

=item path

The path to the server's root directory on the remote server.

=item publish

Determines which kinds of operations are allowed on the server.  May be any of:

=over 8

=item NONE

Allows neither get nor put operations.

=item GET

Allows files to be gotten from the server.

=item PUT

Allows files to be loaded onto the server.

=item ALL

Allows both get and put operations.

=back

=item environmentId

The environment group ID (from $envGroup->getEnvironmentGroupId()) for the environment
applied to jobs run on the server.

=item authId

The server authorization ID (from $serverAuth->getAuthId()) for the server authorization
that is used to log in to the server.

=item active

If this evaluates to true, the server is enabled.  If this evaluates to false, the 
server is disabled and no jobs will be run on it.

=item collectorId

The collector ID (from $collector->getCollectorName()) of the collector that operates
on the selector in order to get manifest properties.

=back

=back


=head2 BuildForge::Services::DBO::Server->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::Server objects corresponding
to all servers in the database that the user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Server->findById(connection, serverId)

Returns the BuildForge::Services::DBO::Server object corresponding to the given
server ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item serverId

The server ID (from $server->getServerId()) for the server to return.

=back


=head2 BuildForge::Services::DBO::Server->findByName(connection, serverName)

Returns the BuildForge::Services::DBO::Server object corresponding to the given
server name, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item serverName

The server name (from $server->getName()) for the server to return.

=back


=head2 $server->create()

Creates the server record in the database.


=head2 $server->update()

Updates the server record in the database.


=head2 $server->delete()

Deletes the server record from the database.


=head2 $server->getServerId()

Returns the ID of the server.


=head2 $server->getName()

Returns the name of the server.


=head2 $server->getHost()

Returns the hostname of the server.


=head2 $server->getActive()

Returns 1 if the server is active and jobs can be run on it or 0 otherwise.


=head2 $server->getAuthId()

Returns the server auth ID for the server authorization the server uses to
authenticate when connecting.


=head2 $server->getCollectorId()

Returns the collector ID for the collector that is used to gather manifest data
about the server.


=head2 $server->getEnvironmentId()

Returns the enviroment group ID for the environment group that is applied to 
jobs run on this server.


=head2 $server->getLevel()

Returns the access group ID for the access group the server belongs to.  Users
who do not belong to this access group cannot view, edit, or use the server.


=head2 $server->getPath()

Returns the path to the server base directory.


=head2 $server->getPublish()

Returns which kinds of operations are allowed on the server.  May be any of:

=over 8

=item NONE

Allows neither get nor put operations.

=item GET

Allows files to be gotten from the server.

=item PUT

Allows files to be loaded onto the server.

=item ALL

Allows both get and put operations.

=back


=head2 $server->getTestText()

Returns the output of the last time the server was tested.


=head2 $server->getUpstate()

This will return 'UPDATING' if the server is currently being tested or 'READY' otherwise.


=head2 $server->getError()

Returns 'TESTING' if the server is currently being tested, 'NO' if there is no error, or 'YES'
if there is an error and the server has been disabled.  To find out the actual error, use 
$server->getTestText() for test details.


=head2 $server->getManifest()

Returns the BuildForge::Services::DBO::Manifest object that holds the server's
current manifest.


=head2 $server->setName(serverName)

Sets the name of the server.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item serverName

The new server name.

=back


=head2 $server->setHost(host)

Sets the hostname of the server.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item host

The new server hostname.

=back


=head2 $server->setActive(isActive)

Sets whether or not the server is active.  Inactive servers will not have jobs run
on them.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item isActive

If this evaluates to true, the server is activated.  If not, the server is deactivated.

=back


=head2 $server->setAuthId(authId)

Sets the server auth that the engine uses to authenticate when contacting the 
server.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item authId

The server auth ID (from $serverAuth->getAuthId()).

=back


=head2 $server->setCollectorId(collectorId)

Sets the collector which is used to gather the server manifest.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item collectorId

The collector ID (from $collector->getCollectorId()).

=back


=head2 $server->setEnvironmentId(envGroupId)

Sets the environment which is applied to any jobs that run on 
this server.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item envGroupId

The environment group ID (from $envGroup->getEnvironmentGroupId()).

=back


=head2 $server->setLevel(accessGroupId)

Sets the access group which the server belongs to.  Users that do not belong
to this access group cannot view, edit, or use the server.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The access group's ID (from $accessGroup->getLevel()).

=back


=head2 $server->setPath(path)

Sets the path to the server's base directory.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item path

The new server path.

=back


=head2 $server->setPublish(publish)

Sets what operations are allowed on the server.  $server->update() must be run 
before changes are replicated in the database.

=over 8

=item publish

May be any of:

=over 8

=item NONE

Allows neither get nor put operations.

=item GET

Allows files to be gotten from the server.

=item PUT

Allows files to be loaded onto the server.

=item ALL

Allows both get and put operations.

=back

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

