
package BuildForge::Services::DBO::ServerAuth;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(liveAuthId authId username password level);
my $prototype = {
	liveAuthId => undef,
	authId     => '',
	username   => '',
	password   => '',
	level      => 0,
};
sub getTypeKey { 'ServerAuth'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }




sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'level'});
	return $self;
}


sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SERVERAUTH_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $authId) = @_;
	# $proto->checkConnection($conn);
	$conn->request(COMMAND_SERVERAUTH_FIND_ID);
	$conn->writeEntryString(KEY_SERVERAUTH_ID, $authId);
	my $dbo = $conn->call()->{+KEY_SERVERAUTH_DBO};
	return defined($dbo) ? $proto->new($conn)->fromArray($dbo) : undef;
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $liveAuthId = $self->{'liveAuthId'};
	my $password = $self->{'password'};
	defined($liveAuthId) && length($liveAuthId) and die $self->live(__PACKAGE__);
	#!defined($password) || !length($password) and die $self->invalid('Password');
	$conn->request(COMMAND_SERVERAUTH_ADD);
	$conn->writeEntryDBO(KEY_SERVERAUTH_DBO, $self);
	my $auth = $conn->call()->{+KEY_SERVERAUTH_DBO};
	!defined($auth) and die $self->missing(KEY_SERVERAUTH_DBO);
	$self->fromArray($auth);
	$self->{'password'} = $password;
	return $self;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $liveAuthId = $self->{'liveAuthId'};
	my $password = $self->{'password'};
	!defined($liveAuthId) || !length($liveAuthId) and die $self->dead(__PACKAGE__);
	!defined($password) || !length($password) and die $self->invalid('Password');
	$conn->request(COMMAND_SERVERAUTH_UPD);
	$conn->writeEntryDBO(KEY_SERVERAUTH_DBO, $self);
	my $auth = $conn->call()->{+KEY_SERVERAUTH_DBO};
	!defined($auth) and die $self->missing(KEY_SERVERAUTH_DBO);
	$self->fromArray($auth);
	$self->{'password'} = $password;
	return $self->wrap($auth);
}

sub delete {
	my ($self) = @_;
	my $liveAuthId = $self->{'liveAuthId'};
	!defined($liveAuthId) || !length($liveAuthId) and die $self->dead(__PACKAGE__);
	deleteById($self, $self->{'_conn'}, $liveAuthId);
	$self->{'liveAuthId'} = undef;
}

sub deleteById {
	my ($proto, $conn, $authId) = @_;
	$proto->checkConnection($conn);
	!defined($authId) || !length($authId) and die $proto->invalid('ID');
	$conn->request(COMMAND_SERVERAUTH_DEL);
	$conn->writeEntryString(KEY_SERVERAUTH_ID, $authId);
	return $conn->call();
}

# wrap is inherited

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_SERVERAUTH_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @auths = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@auths;
}


sub getAuthId   { $_[0]->{'authId'}; }
sub getUsername { $_[0]->{'username'}; }
sub getLevel    { $_[0]->{'level'}; }

sub setAuthId   { $_[0]->{'authId'}    = $_[1]; }
sub setUsername { $_[0]->{'username'}  = $_[1]; }
sub setPassword { $_[0]->{'password'}  = bf_encrypt($_[1]); }
sub setLevel    { $_[0]->{'level'}     = $_[1]; }

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::ServerAuth


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing serverauths
	$allServerAuthes = BuildForge::Services::DBO::ServerAuth->findAll($conn);
	$oldauth = BuildForge::Services::DBO::ServerAuth->findById($conn, $authId);

	# Getter / setter functions
	$id = $oldauth->getAuthId();
	$username = $oldauth->getUsername();
	$accessGroupId = $oldauth->getLevel();

	$oldauth->setAuthId('Server Authorization Name');
	$oldauth->setUsername('username');
	$oldauth->setPassword('password');
	$oldauth->setLevel($accessGroupId);
	$oldauth->update();
	
	# ServerAuth creation, updating, deletion
	$newserverauth = new BuildForge::Services::DBO::ServerAuth($conn);
	$newserverauth->setAuthId('New Build ServerAuth');
	$newserverauth->setLevel($accessGroupId);
	$newserverauth->create();

	$newserverauth->setLevel(2);
	$newserverauth->update();
	
	$newserverauth->delete();
	BuildForge::Services::DBO::ServerAuth->deleteById($conn, $oldauth->getAuthId());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

ServerAuth allows you to create, access, and delete BuildForge server authorization objects. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::ServerAuth(connection, {ServerAuthParamsHashRef})

Returns a new instance of a ServerAuth object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item ServerAuthParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item authId

The name / ID of the new server auth.

=item username

The username used to connect to the server.

=item password

The password used to connect with the server.

=item level

The access group ID (from $accessGroup->getLevel()) of the access group this server
auth belongs to.  If a user does not belong to this access group, they cannot view, edit,
or use this server auth.

=back

=back


=head2 BuildForge::Services::DBO::ServerAuth->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::ServerAuth objects corresponding
to all server auths in the database that the user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::ServerAuth->findById(connection, serverAuthId)

Returns the BuildForge::Services::DBO::ServerAuth object corresponding to the given
server auth ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item serverAuthId

The desired server auth ID (from $serverauth->getAuthId()).

=back


=head2 $serverauth->create()

Creates a record for the server auth in the database.


=head2 $serverauth->update()

Updates the server auth record in the database.


=head2 $serverauth->delete()

Deletes the server auth from the database.


=head2 BuildForge::Services::DBO::ServerAuth->deleteById(connection, serverAuthId)

Deletes the server auth from the database which has the specified ID.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item serverAuthId

The server auth ID (from $serverauth->getAuthId()) to be deleted.

=back


=head2 $serverauth->getAuthId()

Returns the server auth name / ID.


=head2 $serverauth->getUsername()

Returns the username used to connect to servers with this server auth.


=head2 $serverauth->getLevel()

Returns the access group ID of the access group this server auth belongs to.
Users not in the access group cannot view, edit, or use the server auth.


=head2 $serverauth->setAuthId(serverAuthId)

Sets the ID / name for this server auth.  $serverAuth->update() must be run 
before changes are replicated in the database.

=over 8

=item serverAuthId

The new server auth ID / name.

=back


=head2 $serverauth->setUsername(username)

Sets the new username used to connect to servers with this server auth.  $serverAuth->update() must be run 
before changes are replicated in the database.

=over 8

=item username

The new user name.

=back


=head2 $serverauth->setPassword(password)

Sets the new password used to connect to servers with this server auth.  $serverAuth->update() must be run 
before changes are replicated in the database.

=over 8

=item password

The new login password.

=back


=head2 $serverauth->setLevel(accessGroupId)

Sets the access group this server auth belongs to.  Users who do not belong
to this access group cannot view, edit, or use this server auth.  $serverAuth->update() must be run 
before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()).

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

