
package BuildForge::Services::DBO::Step;

use strict;
use constant FLAG_DISABLED => 'D';

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info (DataDBO)
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(uid projectId ordinal environmentId filterSetId inlineChainId
	description directory passNotify failNotify onFail resource threaded
	absolute timeout level broadcast commandText passChainId failChainId
	active passWait failWait);
my $prototype = {
	'uid' => '',
	'projectId' => 0,
	'ordinal' => -1,
	'environmentId' => 0,
	'filterSetId' => 0,
	'inlineChainId' => 0,
	'description' => '',
	'directory' => '/',
	'passNotify' => 0,
	'failNotify' => 0,
	'onFail' => ENUM_STEP_ONFAIL->HALT,
	'resource' => '',
	'threaded' => ENUM_STEP_THREADED->NO,
	'absolute' => 0,
	'timeout' => 300,
	'level' => 0,
	'broadcast' => 0,
	'commandKey' => '',
	'commandText' => '',
	'passChainId' => 0,
	'failChainId' => 0,
	'active' => 1,
	'passWait' => 0,
	'failWait' => 0,
};
sub getTypeKey { 'Step'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my ($newParent) = splice(@_, 1, 1);
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_parent'} = $newParent;
	Integer(\$self->{'projectId'});
	Integer(\$self->{'ordinal'});
	Integer(\$self->{'environmentId'});
	Integer(\$self->{'filterSetId'});
	Integer(\$self->{'inlineChainId'});
	Integer(\$self->{'passNotify'});
	Integer(\$self->{'failNotify'});
	Enum(ENUM_STEP_ONFAIL, \$self->{'onFail'});
	Enum(ENUM_STEP_THREADED, \$self->{'threaded'});
	Boolean(\$self->{'absolute'});
	Integer(\$self->{'timeout'});
	Integer(\$self->{'level'});
	Boolean(\$self->{'broadcast'});
	Integer(\$self->{'passChainid'});
	Integer(\$self->{'failChainid'});
	Boolean(\$self->{'active'});
	Boolean(\$self->{'passWait'});
	Boolean(\$self->{'failWait'});
	return $self;
}

# INTERNAL USE ONLY
sub create {
	my ($self) = @_;
	$self->internal;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_STEP_ADD);
	$conn->writeEntry(KEY_STEP_DBO, $self);
	my $dbo = $conn->call()->{+KEY_STEP_DBO};
	defined($dbo) or die $self->missing(KEY_STEP_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $parent = $self->{'_parent'};
	!defined($parent) || $parent->{'projectId'} <= 0
		and return $self;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_STEP_UPD);
	$conn->writeEntry(KEY_STEP_DBO, $self);
	my $dbo = $conn->call()->{+KEY_STEP_DBO};
	defined($dbo) or die $self->missing(KEY_STEP_DBO);
	return $self->fromArray($dbo);
}

sub _dbo_delete {
	my ($proto, $conn, $uid) = @_;
	return if (!$uid);
	
	$conn->request(COMMAND_STEP_DEL);
	$conn->writeEntryString(KEY_STEP_UID, $uid);
	$conn->call();
}

sub delete {
	my ($self, $uid) = @_;
	$uid = defined($uid) ? $uid : $self->{'uid'};

	if (!defined($self->{'_parent'}) || (!isa($self->{'_parent'}, 'BuildForge::Services::DBO::Project')))  {
		return $self->_dbo_delete($self->{'_conn'}, $uid);
	}
	return $self->{'_parent'}->removeStep($uid);
}

sub disable { $_[0]->enable(0); }
sub enable {
	my ($self, $enable) = @_;
	my $conn = $self->{'_conn'};
	$enable = !defined($enable) || ($enable && !($enable =~ /^[0NnFf]/));
	$conn->request(COMMAND_STEP_ACTIVATE);
	$conn->writeEntryString(KEY_STEP_UID, $self->{'uid'});
	!$enable and $conn->writeEntryString(KEY_FLAGS, FLAG_DISABLED);
	$conn->call();
	$self->{'active'} = $enable;
}

# wrap is inherited

sub _readStepList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_PROJECT_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @steps = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@steps;
}

sub getAbsolute      { $_[0]->{'absolute'}; }
sub getActive        { $_[0]->{'active'}; }
sub getBroadcast     { $_[0]->{'broadcast'}; }
sub getCommandText   { $_[0]->{'commandText'}; }
sub getDescription   { $_[0]->{'description'}; }
sub getDirectory     { $_[0]->{'directory'}; }
sub getEnvironmentId { $_[0]->{'environmentId'}; }
sub getFailChainId   { $_[0]->{'failChainId'}; }
sub getFailNotify    { $_[0]->{'failNotify'}; }
sub getFailWait      { $_[0]->{'failWait'}; }
sub getFilterSetId   { $_[0]->{'filterSetId'}; }
sub getInlineChainId { $_[0]->{'inlineChainId'}; }
sub getLevel         { $_[0]->{'level'}; }
sub getOnFail        { $_[0]->{'onFail'}; }
sub getPassChainId   { $_[0]->{'passChainId'}; }
sub getPassNotify    { $_[0]->{'passNotify'}; }
sub getPassWait      { $_[0]->{'passWait'}; }
sub getProjectId     { $_[0]->{'projectId'}; }
sub getResource      { $_[0]->{'resource'}; }
sub getStepOrdinal   { $_[0]->{'ordinal'}; }
sub getThreaded      { $_[0]->{'threaded'}; }
sub getTimeout       { $_[0]->{'timeout'}; }
sub getUid           { $_[0]->{'uid'}; }

sub setAbsolute      { $_[0]->{'absolute'}      = $_[1]; }
sub setActive        { $_[0]->{'active'}        = $_[1]; }
sub setBroadcast     { $_[0]->{'broadcast'}     = $_[1]; }
sub setCommandText   { $_[0]->{'commandText'}   = $_[1]; }
sub setDescription   { $_[0]->{'description'}   = $_[1]; }
sub setDirectory     { $_[0]->{'directory'}     = $_[1]; }
sub setEnvironmentId { $_[0]->{'environmentId'} = $_[1]; }
sub setFailChainId   { $_[0]->{'failChainId'}   = $_[1]; }
sub setFailNotify    { $_[0]->{'failNotify'}    = $_[1]; }
sub setFailWait      { $_[0]->{'failWait'}      = $_[1]; }
sub setFilterSetId   { $_[0]->{'filterSetId'}   = $_[1]; }
sub setInlineChainId { $_[0]->{'inlineChainId'} = $_[1]; }
sub setLevel         { $_[0]->{'level'}         = $_[1]; }
sub setOnFail        { $_[0]->{'onFail'}        = $_[1]; }
sub setPassChainId   { $_[0]->{'passChainId'}   = $_[1]; }
sub setPassNotify    { $_[0]->{'passNotify'}    = $_[1]; }
sub setPassWait      { $_[0]->{'passWait'}      = $_[1]; }
sub setProjectId     { $_[0]->{'projectId'}     = $_[1]; }
sub setResource      { $_[0]->{'resource'}      = $_[1]; }
sub setStepOrdinal   { $_[0]->{'ordinal'}       = $_[1]; }
sub setThreaded      { $_[0]->{'threaded'}      = $_[1]; }
sub setTimeout       { $_[0]->{'timeout'}       = $_[1]; }

# toString is inherited

sub clone {
	my ($self) = @_;
	my $clone = $self->new($self->{'_conn'}, $self->{'_parent'});
	$clone->{'_parent'} = undef;
	$clone->{'uid'} = '';
	$clone->{'projectId'} = 0;
	$clone->{'ordinal'} = -1;
	return $clone;
}

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Step


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing steps
	$project = BuildForge::Services::DBO::Project->findById($conn, $pid);
	$allSteps = $project->getSteps();
	$step1 = $project->getStep(2);
	$step2 = $project->getStep($stepUID);

	# Getter / setter functions
	$id = $step1->getUid();
	$projectId = $step1->getProjectId();
	$description = $step1->getDescription();
	$directory = $step1->getDirectory();
	$command = $step1->getCommandText();
	$isAbsolute = $step1->getAbsolute();
	$isActive = $step1->getActive();
	$isBroadcast = $step1->getBroadcast();
	$envGroupId = $step1->getEnvironmentId();
	$failChainProjectId = $step1->getFailChainId();
	$failNotifyAccessGroupId = $step1->getFailNotify();
	$failWait = $step1->getFailWait();
	$filterId = $step1->getFilterSetId();
	$inlineProjectId = $step1->getInlineChainId();
	$accessGroupId = $step1->getLevel();
	$continueOnFail = $step1->getOnFail();
	$passChainProjectId = $step1->getPassChainId();
	$passNotifyAccessGroupId = $step1->getPassNotify();
	$passWait = $step1->getPassWait();
	$selectorId = $step1->getResource();
	$stepNumber = $step1->getStepOrdinal();
	$thread = $step1->getThreaded();
	$timeoutInSeconds = $step1->getTimeout();

	$step1->setDescription('New Step Description');
	$step1->setDirectory('/');
	$step1->setCommandText('echo hi');
	$step1->setAbsolute(1);
	$step1->setActive(1);
	$step1->setBroadcast(1);
	$step1->setEnvironmentId($envGroupId);
	$step1->setFailChainId($projectId);
	$step1->setFailNotify($accessGroupId);
	$step1->setFailWait(1);
	$step1->setFilterSetId($filterId);
	$step1->setInlineChainId($projectId);
	$step1->setLevel($accessGroupId);
	$step1->setOnFail(ENUM_STEP_ONFAIL->CONTINUE);
	$step1->setPassChainId($projectId);
	$step1->setPassNotify($accessGroupId);
	$step1->setPassWait(1);
	$step1->setResource($selectorId);
	$step1->setThreaded(ENUM_STEP_THREADED->YES);
	$step1->setTimeout(300);
	$step1->update();
	
	# Step creation, updating, deletion
	$newstep = new BuildForge::Services::DBO::Step($conn);
	$newstep->setDescription('New Step Description');
	$newstep->setDirectory('/');
	$newstep->setCommandText('echo hi');
	$project->addStep($newstep);
	
	$newstep2 = $newstep->clone();
	$newstep2->setDescription('Duplicate Step Description');
	$project->addStep($newstep2);

	$newstep->setLevel(2);
	$newstep->update();
	
	$newstep->disable();
	$newstep->enable();
	
	$newstep->delete();
	$project->removeStep($step1->getUid());

	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Step allows you to access, and delete BuildForge project steps.  To create steps within
the database, see $project->addStep().


=head1 METHODS


=head2 new BuildForge::Services::DBO::Step(connection, {project}, {StepParamsHashRef})

Returns a new instance of a Step object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item environment

This should be undef or not passed - it is only used internally.

=item StepParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item description

The step name.

=item directory

The directory the step commands will be executed in.  This is relative to 
the server base directory if the step is absolute or to the build directory if the step
is relative.  For example, if the server base directory is "/tmp", the project is "Project",
the build tag is "BUILD_57", and the step path is "/Step", the step will execute in "/tmp/Step"
if the step is absolute and in "/tmp/Project/BUILD_57/Step" if the step is relative.  While 
the build directory is created by the engine, the step directory is not.  In the above example,
in order for the step not to fail due to a nonexistant directory, a previous step will need to
create the "Step" directory.  This defaults to "/" if unset and may be either in Unix or Windows format.

=item commandText

The commands that will be run within the step separated by newlines.

=item absolute

If this evaluates to true, the step will be absolute, and the step commands will execute inside
the server base directory.  If this evaluates to false, the step will be relative and the
step commands will execute inside of the build directory.  It defaults to relative if unset.

=item active

If this evaluates to true the step is enabled and will run on build execution.  
If this evaluates to false the step is disabled and will not run until enabled.
It defaults to active if unset.

=item broadcast

If this evaluates to true the step is a broadcast step.  If false (the default)
the step is not.  Where non-broadcast steps operate on a single server in the selector,
broadcast steps run on every server that the selector matches.

=item environmentId

The environment group ID (from $environmentGroup->getEnvironmentGroupId()) of the environment
group that gets applied to the step, or 0 (the default) to just use the project's environment.
If an environment group is set, first the server's environment is applied, then the project's, 
then the step's, so that the project variables overwrite the server variables if they duplicate, then
the step's variables overwrite both the others if they duplicate.

=item failChainId

The project ID (from $project->getProjectId()) of the project that gets run if this
step fails or 0 (the default) to have no project start up.

=item failNotify

The access group ID (from $accessGroup->getLevel()) of the access group that gets notified if this
step fails or 0 (the default) to have no access group be notified.

=item failWait

If this evaluates to true, fail wait is set.  If false, fail wait is not set.
When a project gets kicked off due to this step failing and failChainId being set,
if fail wait is set, the execution of this step will pause until that project finishes
before continuing.  If fail wait isn't set, the other project will get kicked off and 
then execution of this project will continue immediately.

=item filterSetId

The filter set ID (from $filter->getFilterId()) for the filter to use to set this step's
result status.  If this is set to 0 (the default), no filter set is used and the return
code of the step's commands is used to determine step result status.

=item inlineChainId

The project ID (from $project->getProjectId()) of the project to inline from this step.
Inlined projects have their steps inserted after this one when executing this project.  
If this is 0 (the default), no project is inlined by this step.

=item level

The access group ID (from $accessGroup->getLevel()) of the access group this step belongs
to, or 0 (the default) to use the project's access group.  If a user does not belong to 
this access group, they cannot view or edit this step,
and when they start this project this step will only be executed if the user has
the "Execute Inaccessible Steps" permission.

=item onFail

This may either be 'HALT', the default, in which case if this step fails the project execution
stops immediately with a failure, or 'CONTINUE', in which case if the step fails the project 
execution continues and (if it encounters no further failures) finishes with a warning.

=item passChainId

The project ID (from $project->getProjectId()) of the project that gets run if this
step succeeds or 0 (the default) to have no project start up.

=item passNotify

The access group ID (from $accessGroup->getLevel()) of the access group that gets notified if this
step succeeds or 0 (the default) to have no access group be notified.

=item passWait

If this evaluates to true, pass wait is set.  If false, pass wait is not set.
When a project gets kicked off due to this step succeeding and passChainId being set,
if pass wait is set, the execution of this step will pause until that project finishes
before continuing, and then this step will pass or fail depending on the result of that
project execution.  If pass wait isn't set, the other project will get kicked off and 
then execution of this project will continue immediately and this step's result will be 
unaffected by the operation of the kicked off project.

=item resource

The selector ID (from $selector->getSelectorId()) of the selector that this step runs on.  
If this is set to the empty string (the default), the step uses the project's selector.

=item threaded

This may be 'NO', the default, for no threading, 'YES' for threading, or 'JOIN' to make this
a join step.  Threaded/join steps in the same thread block will run simultaneously up to the project's
limit of concurrent threads.  A thread block is delimited by either a non-threading step or
a join step.

=item timeout

The time in seconds before no response from the server causes a step failure.  It defaults
to 300.

=back

=back


=head2 $step->clone()

Returns a BuildForge::Services::DBO::Step object that is a duplicate of this one.
For the new step to exist in the database, it must be added to a project with
$project->addStep($copy).


=head2 $step->update()

Updates the step record in the database.


=head2 $step->enable()

Enables the step within the database.  Similar to $step->setActive(1), except that
$step->enable() doesn't need to have $step->update() run afterwards to update the 
database record.


=head2 $step->disable()

Disables the step within the database.  Similar to $step->setActive(0), except that
$step->disable() doesn't need to have $step->update() run afterwards to update the 
database record.


=head2 $step->delete()

Deletes the step from the database.


=head2 $step->getUid()

Returns the step's unique ID.


=head2 $step->getProjectId()

Returns the project ID of the project this step belongs to.


=head2 $step->getDescription()

Returns the step's description.


=head2 $step->getDirectory()

Returns the directory the step commands will be executed in.  This is relative to 
the server base directory if the step is absolute or to the build directory if the step
is relative.  For example, if the server base directory is "/tmp", the project is "Project",
the build tag is "BUILD_57", and the step path is "/Step", the step will execute in "/tmp/Step"
if the step is absolute and in "/tmp/Project/BUILD_57/Step" if the step is relative.  While 
the build directory is created by the engine, the step directory is not.  In the above example,
in order for the step not to fail due to a nonexistant directory, a previous step will need to
create the "Step" directory.


=head2 $step->getCommandText()

Returns the commands that will be run within the step separated by newlines.


=head2 $step->getAbsolute()

Returns 1 if the step is absolute and the step commands will execute inside
the server base directory.  Returns 0 if the step is relative and the
step commands will execute inside of the build directory.


=head2 $step->getActive()

Returns 1 if the step is enabled and will run on build execution or 0 if
the step is disabled and will not run until enabled.


=head2 $step->getBroadcast()

Returns 1 if the step is a broadcast step or 0 otherwise.
Where non-broadcast steps operate on a single server in the selector,
broadcast steps run on every server that the selector matches.


=head2 $step->getEnvironmentId()

Returns the environment group ID (from $environmentGroup->getEnvironmentGroupId()) of the environment
group that gets applied to the step, or 0 if it just uses the project's environment.
If an environment group is set, first the server's environment is applied, then the project's, 
then the step's, so that the project variables overwrite the server variables if they duplicate, then
the step's variables overwrite both the others if they duplicate.


=head2 $step->getFailChainId()

Returns the project ID (from $project->getProjectId()) of the project that gets run if this
step fails or 0 if no such project gets run.


=head2 $step->getFailNotify()

Returns the access group ID (from $accessGroup->getLevel()) of the access group that gets notified if this
step fails or 0 if no such access group is notified.


=head2 $step->getFailWait()

Returns 1 if fail wait is set or 0 if fail wait is not set.
When a project gets kicked off due to this step failing and failChainId being set,
if fail wait is set, the execution of this step will pause until that project finishes
before continuing.  If fail wait isn't set, the other project will get kicked off and 
then execution of this project will continue immediately.


=head2 $step->getFilterSetId()

Returns the filter set ID (from $filter->getFilterId()) for the filter to use to set this step's
result status or 0 if no filter set is used and the return
code of the step's commands is used to determine step result status.


=head2 $step->getInlineChainId()

Returns the project ID (from $project->getProjectId()) of the project to inline from this step 
or 0 if no project is inlined.
Inlined projects have their steps inserted after this one when executing this project.  


=head2 $step->getLevel()

Returns the access group ID (from $accessGroup->getLevel()) of the access group this step belongs
to or 0 if it belongs to the project's access group.  If a user does not belong to this access 
group, they cannot view or edit this step,
and when they start this project this step will only be executed if the user has
the "Execute Inaccessible Steps" permission.


=head2 $step->getOnFail()

Returns either 'HALT' in which case if this step fails the project execution
stops immediately with a failure, or 'CONTINUE', in which case if the step fails the project 
execution continues and (if it encounters no further failures) finishes with a warning.


=head2 $step->getPassChainId()

Returns the project ID (from $project->getProjectId()) of the project that gets run if this
step succeeds or 0 if no such project gets run.


=head2 $step->getPassNotify()

Returns the access group ID (from $accessGroup->getLevel()) of the access group that gets notified if this
step succeeds or 0 if no such access group is notified.


=head2 $step->getPassWait()

Returns 1 if pass wait is set or 0 if pass wait is not set.
When a project gets kicked off due to this step succeeding and passChainId being set,
if pass wait is set, the execution of this step will pause until that project finishes
before continuing, and then this step will pass or fail depending on the result of that
project execution.  If pass wait isn't set, the other project will get kicked off and 
then execution of this project will continue immediately and this step's result will be 
unaffected by the operation of the kicked off project.


=head2 $step->getResource()

Returns the selector ID (from $selector->getSelectorId()) of the selector that this step runs on
or the empty string if the step uses the project's selector.


=head2 $step->getStepOrdinal()

Returns the step's step number within this project.  The first step returns 1,
the second returns 2, etc.


=head2 $step->getThreaded()

Returns either 'NO' for no threading, 'YES' for threading, or 'JOIN' for join steps.
Threaded/join steps in the same thread block will run simultaneously up to the project's
limit of concurrent threads.  A thread block is delimited by either a non-threading step or
a join step.


=head2 $step->getTimeout()

Returns the time in seconds before no response from the server causes a step failure.


=head2 $step->setDescription(description)

Sets the step's description.
$step->update() must be run before changes are replicated in the database.

=over 8

=item description

The new step description.

=back


=head2 $step->setDirectory(directory)

Sets the directory the step commands will be executed in.  This is relative to 
the server base directory if the step is absolute or to the build directory if the step
is relative.  For example, if the server base directory is "/tmp", the project is "Project",
the build tag is "BUILD_57", and the step path is "/Step", the step will execute in "/tmp/Step"
if the step is absolute and in "/tmp/Project/BUILD_57/Step" if the step is relative.  While 
the build directory is created by the engine, the step directory is not.  In the above example,
in order for the step not to fail due to a nonexistant directory, a previous step will need to
create the "Step" directory.
$step->update() must be run before changes are replicated in the database.

=over 8

=item directory

The new step directory.  This may be either in Unix or Windows format.

=back


=head2 $step->setCommandText(command)

Sets the step's commands.
$step->update() must be run before changes are replicated in the database.

=over 8

=item command

The commands that will be run within the step separated by newlines.

=back


=head2 $step->setAbsolute(isAbsolute)

Sets whether this step is absolute or relative.
If absolute, the step commands will execute inside
the server base directory.  If relative, the
step commands will execute inside of the build directory.
$step->update() must be run before changes are replicated in the database.

=over 8

=item isAbsolute

If this evaluates to true, the step will be absolute.  If this evaluates
to false, the step will be relative.

=back


=head2 $step->setActive(isActive)

Sets if this step is active or not.  If the step is active,
it runs when this project is executed.  If inactive, the step will not run
until it is activated.
$step->update() must be run before changes are replicated in the database.

=over 8

=item isActive

If this evaluates to true, the step is active.  If false, the step is inactive.

=back


=head2 $step->setBroadcast(isBroadcast)

Sets whether or not the step is a broadcast step.
Where non-broadcast steps operate on a single server in the selector,
broadcast steps run on every server that the selector matches.
$step->update() must be run before changes are replicated in the database.

=over 8

=item isBroadcast

If this evaluates to true the step becomes a broadcast step.  If false
the step becomes not a broadcast step..  

=back


=head2 $step->setEnvironmentId(envGroupId)

Sets the step's environment.
If an environment group is set, first the server's environment is applied, then the project's, 
then the step's, so that the project variables overwrite the server variables if they duplicate, then
the step's variables overwrite both the others if they duplicate.
$step->update() must be run before changes are replicated in the database.

=over 8

=item envGroupId

The environment group ID (from $environmentGroup->getEnvironmentGroupId()) of the environment
group that gets applied to the step, or 0 (the default) to just use the project's environment.

=back


=head2 $step->setFailChainId(projectId)

Sets a project to run if the execution of this step fails.
$step->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project that gets run or 0
to have no project start up.

=back


=head2 $step->setFailNotify(accessGroupId)

Sets an access group to notify if the execution of this step fails.
$step->update() must be run before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) of the access group to notify 
or 0 to have no access group be notified.

=back


=head2 $step->setFailWait(isFailWait)

Sets whether or not fail waiting is set.
When a project gets kicked off due to this step failing and failChainId being set,
if fail wait is set, the execution of this step will pause until that project finishes
before continuing.  If fail wait isn't set, the other project will get kicked off and 
then execution of this project will continue immediately.
$step->update() must be run before changes are replicated in the database.

=over 8

=item isFailWait

If this evaluates to true, fail wait is set.  If false, fail wait is not set.

=back


=head2 $step->setFilterSetId(filterId)

Sets the filter set for the step.  If a step has a filter set, that filter set is
used to set the step's result status.  If not, the return
code of the step's commands is used to determine step result status.
$step->update() must be run before changes are replicated in the database.

=over 8

=item filterId

The filter set ID (from $filter->getFilterId()) for the filter to use or 0 to
use the step command's return code.

=back


=head2 $step->setInlineChainId(projectId)

Sets the project to inline from this step.
Inlined projects have their steps inserted after this one when executing this project.  
$step->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project to inline from this step
or 0 to not inline a project.

=back


=head2 $step->setLevel(accessGroupId)

Sets the access group this step belongs to.
If a user does not belong to 
this access group, they cannot view or edit this step,
and when they start this project this step will only be executed if the user has
the "Execute Inaccessible Steps" permission.
$step->update() must be run before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) of the access group this step should
belong to or 0 to use the project's access group.

=back


=head2 $step->setOnFail(onFail)

Sets the step's on fail settings.  
This may either be 'HALT', in which case if this step fails the project execution
stops immediately with a failure, or 'CONTINUE', in which case if the step fails the project 
execution continues and (if it encounters no further failures) finishes with a warning.
$step->update() must be run before changes are replicated in the database.

=over 8

=item onFail

Should either be 'HALT' or 'CONTINUE'

=back


=head2 $step->setPassChainId(projectId)

Sets the project to get chained if this step succeeds.
$step->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project to get chained
or 0 to have no project be chained.

=back


=head2 $step->setPassNotify(accessGroupId)

Sets the access group that gets notified if this step succeeds.
$step->update() must be run before changes are replicated in the database.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) of the access group 
or 0 to have no access group be notified.

=back


=head2 $step->setPassWait(isPassWait)

Sets the step's pass wait setting.
When a project gets kicked off due to this step succeeding and passChainId being set,
if pass wait is set, the execution of this step will pause until that project finishes
before continuing, and then this step will pass or fail depending on the result of that
project execution.  If pass wait isn't set, the other project will get kicked off and 
then execution of this project will continue immediately and this step's result will be 
unaffected by the operation of the kicked off project.
$step->update() must be run before changes are replicated in the database.

=over 8

=item isPassWait

If this evaluates to true, pass wait gets set.  If false, pass wait gets unset.

=back


=head2 $step->setResource(selectorId)

Sets the selector for this step to run on.
$step->update() must be run before changes are replicated in the database.

=over 8

=item selectorId

The selector ID (from $selector->getSelectorId()) of the selector to use or the 
empty string to cause the step to use the project's selector.

=back


=head2 $step->setThreaded(threading)

Sets the threading status of the step.
Threaded/join steps in the same thread block will run simultaneously up to the project's
limit of concurrent threads.  A thread block is delimited by either a non-threading step or
a join step.
$step->update() must be run before changes are replicated in the database.

=over 8

=item threading

This may be 'NO' for no threading, 'YES' for threading, or 'JOIN' to make this
a join step.

=back


=head2 $step->setTimeout(timeout)

Sets the time before no response from the server causes a step failure.
$step->update() must be run before changes are replicated in the database.

=over 8

=item timeout

The timeout time in seconds.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

