
package BuildForge::Services::DBO::SysConfig;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(name value public defaultValue title description
	dataType validate);
my $prototype = {
	'name' => '',
	'value' => '',
	'public' => 0,
	'defaultValue' => '',
	'title' => '',
	'description' => '',
	'dataType' => '',
	'validate' => '',
};
sub getTypeKey { 'SysConf'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Boolean(\$self->{'public'});
	return $self;
}

sub findById {
	my ($proto, $conn, $param) = @_;
	$proto->checkConnection($conn);
	!defined($param) || !length($param) and die $proto->invalid('Name');
	$conn->request(COMMAND_SYSCONF_FIND);
	$conn->writeEntryString(KEY_SYSCONFIG_ID, $param);
	my $dbo = $conn->call()->{+KEY_SYSCONFIG_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_SYSCONF_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_SYSCONF_UPD);
	$conn->writeEntryDBO(KEY_SYSCONFIG_DBO, $self);
	my $dbo = $conn->call()->{+KEY_SYSCONFIG_DBO};
	defined($dbo) or die $self->missing(KEY_SYSCONFIG_DBO);
	return $self->fromArray($dbo);
}

# Not Implemented - SysConfig property creation is unsupported from the
# client.  TODO Connections auth'd as SYSTEM will need to be able to do
# this.  On the other hand, this probably should be in SimpleCommands
# rather than handled by an entire DBO.
sub create { return undef; }

# Not Implemented - SysConfig property deletion is unsupported from the
# client.  TODO Connections auth'd as SYSTEM will need to be able to do
# this.  On the other hand, this probably should be in SimpleCommands
# rather than handled by an entire DBO.
sub delete { return; }

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_SYSCONFIG_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @sysconfs = map { __PACKAGE__->new($conn, $_)->fromArray($_) } @$list;
	return \@sysconfs;
}



# wrap is inherited

sub getDataType      { $_[0]->{'dataType'}; }
sub getDefaultValue  { $_[0]->{'defaultValue'}; }
sub getDescription   { $_[0]->{'description'}; }
sub getIsPublic      { $_[0]->{'isPublic'} ? 1 : 0; }
sub getName          { $_[0]->{'name'}; }
sub getTitle         { $_[0]->{'title'}; }
sub getValidate      { $_[0]->{'validate'}; }
sub getValue         { $_[0]->{'value'}; }

sub setValue         { $_[0]->{'value'}        = $_[1]; }

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::SysConfig


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing sysconfigs
	$allSysconfs = BuildForge::Services::DBO::SysConfig->findAll($conn);
	$sysconf = BuildForge::Services::DBO::SysConfig->findById($conn, 'param_name');

	# Getter functions
	$id = $sysconf->getName();
	$title = $sysconf->getTitle();
	$description = $sysconf->getDescription();
	$value = $sysconf->getValue();
	$datatype = $sysconf->getDataType();
	$defaultValue = $sysconf->getDefaultValue();
	$isPublic = $sysconf->getIsPublic();
	$validationString = $sysconf->getValidate();

	# SysConfig updating
	$sysconf->setValue('new value for system parameter');
	$sysconf->update();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

SysConfig allows you to view and update BuildForge system configuration settings.


=head1 METHODS


=head2 BuildForge::Services::DBO::SysConfig->findAll(connection)

Returns a reference to an array of all BuildForge::Services::DBO::SysConfig objects 
that the current user has access to.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::SysConfig->findById(connection, sysConfId)

Returns the BuildForge::Services::DBO::SysConfig object corresponding to the given
system configuration ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item sysConfId

The ID of the desired system configuration setting (from $sysconf->getName()).

=back


=head2 $sysconf->update()

Updates the system configuration setting in the database.


=head2 $sysconf->getName()

Returns the name / ID of the system configuration setting.


=head2 $sysconf->getTitle()

Returns the title text for the system configuration setting.


=head2 $sysconf->getDescription()

Returns the description of what the system configuration setting does.


=head2 $sysconf->getValue()

Returns the current value of the setting.


=head2 $sysconf->getDataType()

Returns the datatype of the setting, limiting what values are acceptable.  The possible
return values are:

=over 8

=item ''

This is not a publically editable setting.

=item 'bool'

Possible values of this are 'Y' and 'N'.

=item 'email'

The value of this setting should be a valid email address.

=item 'formatdate'

Used for date formatting strings consisting of 'y', 'm', and 'd' with appropriate
separators.  For the date July 20, 2007, for example, the format string 'ymd' will
result in '20070720', and the format string 'm-d-y' will result in '07-20-2007'.

=item 'formatpasswd'

Used for the system setting controlling the requirements for user passwords.
See the online documentation on the Administration->System topic regarding the 
"Password Format" system setting for details on the format of this.

=item 'formattime'

Used for time formatting strings consisting of 'h', 'm', and 's' with appropriate
separators.  For the time 11:35:58 PM, for example, the format string 'hms' will
result in '233558', and the format string 's/m/h' will result in '58/35/11'.

=item 'formattimestamp'

Used for date/time formatting strings.  It is built on the POSIX strftime()
function, so see your local machine's documentation for that to see what settings this field 
accepts.

=item 'hostip'

Accepts a host ip address and optional port number separated by a ':'.

=item 'integer'

Accepts any integer value.

=item 'passwd'

Accepts any string.

=item 'real'

Accepts any floating point or integer value.

=item 'string'

Accepts any string.

=item 'timedays'

Accepts any positive integer value or 0.

=item 'timemins'

Accepts any positive integer value or 0.

=item 'timesecs'

Accepts any positive integer value or 0.

=item 'timestamp'

Accepts any positive integer value or 0.

=item 'url'

Accepts any URL.

=back


=head2 $sysconf->getDefaultValue()

Returns the default value for this field.


=head2 $sysconf->getIsPublic()

Returns 1 if the value is publically accessible or 0 if it is an internal system
setting.


=head2 $sysconf->getValidate()

Returns the validation string for the setting, currently unused.


=head2 $sysconf->setValue(value)

Sets the value for the system configuration setting.  $sysconf->update() must be run 
before changes are replicated in the database.

=over 8

=item value

The new value for the setting.  This value must fit the datatype rules for the particular
system variable (see $sysconf->getDataType()).

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

