
package BuildForge::Services::DBO::TagVar;

use strict;
use constant ILLEGAL_TAG_NAME_CHARS => "}:=\r\n\0";

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(projectId tagId name value padding autoInc liveProjectId);
my $prototype = {
	'projectId' => 0,
	'tagId' => 0,
	'name' => '',
	'value' => 1,
	'padding' => 0,
	'autoInc' => 0,
	'liveProjectId' => 0,
};
sub getTypeKey { 'TagVar'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'projectId'});
	Integer(\$self->{'tagId'});
	Integer(\$self->{'padding'});
	Boolean(\$self->{'autoInc'});
	return $self;
}

# wrap is inherited

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_TAGVAR_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @tags = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@tags;
}



sub findByPid {
	my ($proto, $conn, $projectId) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId <= 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	$conn->request(COMMAND_TAGVAR_FIND_PID);
	$conn->writeEntryString(KEY_TAGVAR_PROJECT_ID, $projectId);
	return _getList($conn, $conn->call());
}

sub findByPidName {
	my ($proto, $conn, $projectId, $name) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId <= 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($name) || !length($name) and die $proto->invalid('Name');
	$conn->request(COMMAND_TAGVAR_FIND_PID_NAME);
	$conn->writeEntryString(KEY_TAGVAR_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_TAGVAR_TAG_NAME, $name);
	my $dbo = $conn->call()->{+KEY_TAGVAR_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByPidTid {
	my ($proto, $conn, $projectId, $tagId) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId <= 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($tagId) || $tagId <= 0 and die $proto->invalid('ID');
	$conn->request(COMMAND_TAGVAR_FIND_PID_TID);
	$conn->writeEntryString(KEY_TAGVAR_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_TAGVAR_TAG_ID, $tagId);
	my $dbo = $conn->call()->{+KEY_TAGVAR_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_TAGVAR_UPD);
	$conn->writeEntryDBO(KEY_TAGVAR_DBO, $self);
	my $dbo = $conn->call()->{+KEY_TAGVAR_DBO};
	defined($dbo) or die $self->missing(KEY_TAGVAR_DBO);
	return $self->fromArray($dbo);
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_TAGVAR_ADD);
	$conn->writeEntryDBO(KEY_TAGVAR_DBO, $self);
	my $dbo = $conn->call()->{+KEY_TAGVAR_DBO};
	defined($dbo) or die $self->missing(KEY_TAGVAR_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->deleteById($conn, $self->{'liveProjectId'}, $self->{'tagId'});
	$self->{'tagId'} = 0;
}
sub deleteById {
	my ($proto, $conn, $projectId, $tagId) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId <= 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($tagId) || $tagId <= 0 and die $proto->invalid('ID');
	$conn->request(COMMAND_TAGVAR_DEL_PID_TID);
	$conn->writeEntryString(KEY_TAGVAR_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_TAGVAR_TAG_ID, $tagId);
	$conn->call();
}


sub getAutoInc    { $_[0]->{'autoInc'}; }
sub getName       { $_[0]->{'name'}; }
sub getPadding    { $_[0]->{'padding'}; }
sub getProjectId  { $_[0]->{'projectId'} ? $_[0]->{'projectId'} : $_[0]->{'liveProjectId'}; }
sub getTagId      { $_[0]->{'tagId'}; }
sub getValue      { $_[0]->{'value'}; }

sub setAutoInc    { $_[0]->{'autoInc'}   = $_[1]; }
sub setName       { $_[0]->{'name'}      = $_[1]; }
sub setPadding    { $_[0]->{'padding'}   = $_[1]; }
sub setProjectId  { $_[0]->{'projectId'} = $_[1]; }
sub setValue      { $_[0]->{'value'}     = $_[1]; }


# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::TagVar


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing tag variables
	$allProjectTagVars = BuildForge::Services::DBO::TagVar->findByPid($conn, $projectId);
	$tagvar1 = BuildForge::Services::DBO::TagVar->findByPidName($conn, $projectId, $tagId);
	$tagvar2 = BuildForge::Services::DBO::TagVar->findByPidTid($conn, $projectId, 'TagVarName');

	# Getter / setter functions
	$projectId = $oldtagvar->getProjectId();
	$tagId = $oldtagvar->getTagId();
	$name = $oldtagvar->getName();
	$currentValue = $oldtagvar->getValue();
	$isAutoIncrement = $oldtagvar->getAutoInc();
	$amountOfPadding = $oldtagvar->getPadding();

	$oldtagvar->setProjectId($projectId);
	$oldtagvar->setName('NewName');
	$oldtagvar->setValue(5);
	$oldtagvar->setAutoInc(1);
	$oldtagvar->setPadding(5);
	$oldtagvar->update();
	
	# TagVar creation, updating, deletion
	$newtagvar = new BuildForge::Services::DBO::TagVar($conn);
	$newtagvar->setProjectId($projectId);
	$newtagvar->setName('Name');
	$newtagvar->create();

	$newtagvar->setValue(5);
	$newtagvar->update();
	
	$newtagvar->delete();
	BuildForge::Services::DBO::TagVar->deleteById($conn, $projectId, $oldtagvar->getTagId());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

TagVar allows you to create, access, and delete BuildForge tag variables. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::TagVar(connection, {TagVarParamsHashRef})

Returns a new instance of a TagVar object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item TagVarParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project the tag variable is
attached to.

=item name

The name of the tag variable.

=item value

The value of the tag variable.  Must be a positive integer or 0.  Defaults to 1.

=item padding

The field length of the value, with initial zeroes added to make up the difference.  
If the value is 17, for example, padding between 0 and 2 will result in '17', padding of 3 will result
in '017', and so forth.  Defaults to 0.

=item autoInc

Whether or not to auto-increment this tag variable after builds of this project.
If this evaluates to true, auto-increment it, and if this evaluates to false (the
default), don't auto-increment.

=back

=back


=head2 BuildForge::Services::DBO::TagVar->findByPid(connection, projectId)

Returns a reference to an array of BuildForge::Services::DBO::TagVar objects corresponding
to all tag variables in the database that are in the given project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) of the project to check for tag
variables.

=back


=head2 BuildForge::Services::DBO::TagVar->findByPidName(connection, projectId, tagVarName)

Returns the BuildForge::Services::DBO::Cron object corresponding to the given
project and name, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) of the project to check for tag
variables.

=item tagVarName

The tag variable name to look for (from $tagvar->getName()).

=back


=head2 BuildForge::Services::DBO::TagVar->findByPidTid(connection, projectId, tagVarId)

Returns the BuildForge::Services::DBO::Cron object corresponding to the given
project and ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) of the project to check for tag
variables.

=item tagVarId

The tag variable ID to look for (from $tagvar->getTagId()).

=back


=head2 $tagvar->create()

Creates the tag variable within the database.


=head2 $tagvar->update()

Updates the tag variable record within the database.


=head2 $tagvar->delete()

Deletes the tag variable record from the database.


=head2 BuildForge::Services::DBO::TagVar->deleteById(connection, projectId, tagVarId)

Deletes the tag variable record from the database with the given ID in the given project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID (from $project->getProjectId()) of the project.

=item tagVarId

The tag variable ID to look for (from $tagvar->getTagId()).

=back


=head2 $tagvar->getProjectId()

Returns the project ID of the project the tag variable is connected to.


=head2 $tagvar->getTagId()

Returns the tag variable ID.


=head2 $tagvar->getName()

Returns the tag variable's name.


=head2 $tagvar->getValue()

Return the tag variable's current value.


=head2 $tagvar->getAutoInc()

Returns 1 if the tag variable auto-increments after each build of the project 
it is attached to, or 0 otherwise.


=head2 $tagvar->getPadding()

Returns the field width of the tag variable.  If the current value of the variable
has fewer characters than the field width, zeroes are added on to the front to make up 
the difference.


=head2 $tagvar->setProjectId(projectId)

Sets which project this tag variable is attached to.
$tagvar->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project to attach to.

=back


=head2 $tagvar->setName(name)

Sets the name of the tag variable.
$tagvar->update() must be run before changes are replicated in the database.

=over 8

=item name

The new name for the tag variable.

=back


=head2 $tagvar->setValue(value)

Sets the value of the tag variable.
$tagvar->update() must be run before changes are replicated in the database.

=over 8

=item value

The new value for the tag variable.  This value must be a positive integer or 0.

=back


=head2 $tagvar->setAutoInc(autoInc)

Sets whether or not to auto-increment this tag variable after builds of this project.
$tagvar->update() must be run before changes are replicated in the database.

=over 8

=item autoInc

If this evaluates to true, auto-increment it, and if this evaluates to false (the
default), don't auto-increment.

=back


=head2 $tagvar->setPadding(fieldLength)

Sets the field length of the value, with initial zeroes added to make up the difference.  
If the value is 17, for example, padding between 0 and 2 will result in '17', padding of 3 will result
in '017', and so forth.  
$tagvar->update() must be run before changes are replicated in the database.

=over 8

=item fieldLength

The new field length for the tag variable.  This must be a positive integer or 0.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

