
package BuildForge::Services::DBO::TimeZone;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(zoneId description
	gmtOffset dstStartPos dstStartDay dstStartMon
	dstEndPos dstEndDay dstEndMon dstOffset liveZoneId);
my $prototype = {
	zoneId      => '',
	description => '',
	gmtOffset   => 0,
	dstStartPos => 0,
	dstStartDay => -1,
	dstStartMon => -1,
	dstEndPos   => 0,
	dstEndDay   => -1,
	dstEndMon   => -1,
	dstOffset   => 0,
	liveZoneId  => undef,
};
sub getTypeKey { 'TimeZone'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'gmtOffset'});
	Integer(\$self->{'dstStartPos'});
	Integer(\$self->{'dstStartDay'});
	Integer(\$self->{'dstStartMon'});
	Integer(\$self->{'dstEndPos'});
	Integer(\$self->{'dstEndDay'});
	Integer(\$self->{'dstEndMon'});
	Integer(\$self->{'dstOffset'});
	return $self;
}

sub findAll {
	my ($proto, $conn) = @_;
	$proto->checkConnection($conn);
	$conn->request(COMMAND_TIMEZONE_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $zoneId) = @_;
	$proto->checkConnection($conn);
	!defined($zoneId) || !length($zoneId) and die $proto->invalid('ID');
	$conn->request(COMMAND_TIMEZONE_FIND_ID);
	$conn->writeEntryString(KEY_TIMEZONE_ID, $zoneId);
	my $dbo = $conn->call()->{+KEY_TIMEZONE_DBO};
	return defined($dbo) ? $proto->new($conn)->fromArray($dbo) : undef;
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $zoneId = $self->{'zoneId'};
	!defined($zoneId) || !length($zoneId) and die $self->invalid('ID');
	$conn->request(COMMAND_TIMEZONE_ADD);
	$conn->writeEntryDBO(KEY_TIMEZONE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_TIMEZONE_DBO};
	defined($dbo) or die $self->missing(KEY_TEMPLATE_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $zoneId = $self->{'zoneId'};
	!defined($zoneId) || !length($zoneId) and die $self->invalid('ID');
	$conn->request(COMMAND_TIMEZONE_UPD);
	$conn->writeEntryDBO(KEY_TIMEZONE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_TIMEZONE_DBO};
	defined($dbo) or die $self->missing(KEY_TEMPLATE_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->deleteById($conn, $self->{'zoneId'});
}
sub deleteById {
	my ($proto, $conn, $zoneId) = @_;
	$proto->checkConnection($conn);
	!defined($zoneId) || !length($zoneId) and die $proto->invalid('ID');
	$conn->request(COMMAND_TIMEZONE_DEL);
	$conn->writeEntryString(KEY_TIMEZONE_ID, $zoneId);
	$conn->call();
}


sub _getList ($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_TIMEZONE_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @zones = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@zones;
}

# wrap is inherited


sub getZoneId               { $_[0]->{'zoneId'} ? $_[0]->{'zoneId'} : $_[0]->{'liveZoneId'}; }
sub getDescription          { $_[0]->{'description'}; }
sub getDstEndDay            { $_[0]->{'dstEndDay'}; }
sub getDstEndMon            { $_[0]->{'dstEndMon'}; }
sub getDstEndPos            { $_[0]->{'dstEndPos'}; }
sub getDstOffset            { $_[0]->{'dstOffset'}; }
sub getDstStartDay          { $_[0]->{'dstStartDay'}; }
sub getDstStartMon          { $_[0]->{'dstStartMon'}; }
sub getDstStartPos          { $_[0]->{'dstStartPos'}; }
sub getGmtOffset            { $_[0]->{'gmtOffset'}; }

sub setZoneId               { $_[0]->{'zoneId'}      = $_[1]; }
sub setDescription          { $_[0]->{'description'} = $_[1]; }
sub setDstEndDay            { $_[0]->{'dstEndDay'}   = $_[1]; }
sub setDstEndMon            { $_[0]->{'dstEndMon'}   = $_[1]; }
sub setDstEndPos            { $_[0]->{'dstEndPos'}   = $_[1]; }
sub setDstOffset            { $_[0]->{'dstOffset'}   = $_[1]; }
sub setDstStartDay          { $_[0]->{'dstStartDay'} = $_[1]; }
sub setDstStartMon          { $_[0]->{'dstStartMon'} = $_[1]; }
sub setDstStartPos          { $_[0]->{'dstStartPos'} = $_[1]; }
sub setGmtOffset            { $_[0]->{'gmtOffset'}   = $_[1]; }

# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::TimeZone


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing timezones
	$allTimeZones = BuildForge::Services::DBO::TimeZone->findAll($conn);
	$oldtimezone = BuildForge::Services::DBO::TimeZone->findById($conn, 'DST');

	# Getter / setter functions
	$id = $oldtimezone->getZoneId();
	$description = $oldtimezone->getDescription();
	$differenceFromGMTInSeconds = $oldtimezone->getGmtOffset();
	$whichWeekStartsDaylightSavingsTime = $oldtimezone->getDstStartPos();
	$whichDayStartsDaylightSavingsTime = $oldtimezone->getDstStartDay();
	$whichMonthStartsDaylightSavingsTime = $oldtimezone->getDstStartMon();
	$whichWeekEndsDaylightSavingsTime = $oldtimezone->getDstEndPos();
	$whichDayEndsDaylightSavingsTime = $oldtimezone->getDstEndDay();
	$whichMonthEndsDaylightSavingsTime = $oldtimezone->getDstEndMon();
	$daylightSavingsTimeOffsetInSeconds = $oldtimezone->getDstOffset();

	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

TimeZone allows you to create, access, and delete BuildForge time zones.  As the time zones already
in the database encompass most standard time zones, this should rarely be useful.


head1 METHODS


=head2 BuildForge::Services::DBO::TimeZone->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::TimeZone objects corresponding
to all timezones in the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::TimeZone->findById(connection, timeZoneId)

Returns the BuildForge::Services::DBO::TimeZone object corresponding to the given
timezone ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item timeZoneId

The ID of the timezone to return (from $timezone->getZoneId()).

=back


=head2 $timezone->getZoneId()

Returns the timezone's ID.


=head2 $timezone->getDescription()

Returns the timezone's description.


=head2 $timezone->getGmtOffset()

Returns the amount of offset from GMT in seconds.  Timezones that are ahead of GMT have
positive values and timezones that are behind GMT have negative values.


=head2 $timezone->getDstStartPos()

If this timezone has daylight savings time, returns the week within the month
that it starts.  For example, if DST starts on the 2nd Tuesday in a month, this will
be 2.  The last week of the month is value 6.  If this timezone lacks DST, returns 0.


=head2 $timezone->getDstStartDay()

Returns the day of the week on which daylight savings time starts, going from 0 (Sunday) to 6 (Saturday).
A value of -1 means this time zone lacks DST.


=head2 $timezone->getDstStartMon()

Returns the month in which daylight savings time starts, going from 0 (January) to 11 (December).
A value of -1 means this time zone lacks DST.


=head2 $timezone->getDstEndPos()

If this timezone has daylight savings time, returns the week within the month
that it ends.  For example, if DST ends on the 3rd Thursday in a month, this will
be 3.  The last week of the month is value 6.  If this timezone lacks DST, returns 0.


=head2 $timezone->getDstEndDay()

Returns the day of the week on which daylight savings time ends, going from 0 (Sunday) to 6 (Saturday).
A value of -1 means this time zone lacks DST.


=head2 $timezone->getDstEndMon()

Returns the month in which daylight savings time ends, going from 0 (January) to 11 (December).
A value of -1 means this time zone lacks DST.


=head2 $timezone->getDstOffset()

Returns the amount of offset to be added to the time when daylight savings time is in effect
in seconds.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

