
package BuildForge::Services::DBO::User;

use strict;

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
our @keys = qw(userId levelHint name email login password maxBuilds
		passwordUpdated passwordExpires passwordExpired
		timeZone dateFormat priority locale data ldap);
my $prototype = {
	userId          => UID_NONE,
	levelHint       => 0,
	name            => '',
	email           => '',
	login           => '',
	password        => undef,
	maxBuilds       => 0,
	passwordUpdated => 0,
	passwordExpires => 1,
	passwordExpired => 0,
	timeZone        => '',
	dateFormat      => '',
	priority        => 0,
	locale          => 'en_US',
	data            => undef,
	ldap            => 0,
};
sub getTypeKey { 'User'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }



sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'levelHint'});
	Integer(\$self->{'maxBuilds'});
	Integer(\$self->{'passwordUpdated'});
	Boolean(\$self->{'passwordExpires'});
	Boolean(\$self->{'passwordExpired'});
	Boolean(\$self->{'priority'});
	Boolean(\$self->{'ldap'});
	return $self;
}

sub _checkLive($) {
	my ($self) = @_;
	my $userId = $self->{'userId'};
	!defined($userId) || $userId == UID_NONE and die $self->dead(__PACKAGE__);
	$userId <= 0 and die $self->invalid('ID');
	return $userId;
}



sub findAll {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_USER_FIND_ALL);
	return _getList($conn, $conn->call());
}

sub findById {
	my ($proto, $conn, $userId) = @_;
	!defined($userId) || $userId <= 0 and die $proto->invalid('ID');
	$conn->request(COMMAND_USER_FIND_ID);
	$conn->writeEntryString(KEY_USER_ID, $userId);
	my $dbo = $conn->call()->{KEY_USER_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByLogin {
	my ($proto, $conn, $login) = @_;
	!defined($login) || !length($login) and die $proto->invalid('Login');
	$conn->request(COMMAND_USER_FIND_LOGIN);
	$conn->writeEntryString(KEY_USER_LOGIN, $login);
	my $dbo = $conn->call()->{KEY_USER_DBO()};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub create {
	my ($self) = @_;
	my $userId = $self->{'userId'};
	defined($userId) && $userId != UID_NONE and die $self->live(__PACKAGE__);

	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_USER_ADD);
	$conn->writeEntryDBO(KEY_USER_DBO, $self);

	my $dbo = $conn->call()->{KEY_USER_DBO()};
	!defined($dbo) and die $self->missing(KEY_USER_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self, $group) = @_;
	_checkLive($self);

	my $conn = $self->{'_conn'};
	$conn->request(COMMAND_USER_UPD);
	$conn->writeEntryDBO(KEY_USER_DBO, $self);

	my $dbo = $conn->call()->{KEY_USER_DBO()};
	!defined($dbo) and die $self->missing(KEY_USER_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $userId = _checkLive($self);
	deleteById($self, $self->{'_conn'}, $userId);
	$self->{'userId'} = UID_NONE;
}

sub deleteById {
	my ($proto, $conn, $userId) = @_;
	!defined($userId) || $userId <= 0 and die $proto->invalid('ID');
	$userId == UID_ROOT and die BuildForge::Ex::APIException->new('UserRootIsMagic');
	$conn->request(COMMAND_USER_DEL);
	$conn->writeEntryString(KEY_USER_ID, $userId);
	$conn->call();
}

sub resetLicense {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $userId = _checkLive($self);
	$conn->request(COMMAND_USER_RESET_LICENSE);
	$conn->writeEntryString(KEY_USER_ID, $userId);
	$conn->call();
}

sub resetLogin {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $userId = _checkLive($self);
	$conn->request(COMMAND_USER_RESET_LOGIN);
	$conn->writeEntryString(KEY_USER_ID, $userId);
	$conn->call();
}

# Implemented through the access group's add user command
sub addAccessGroup {
	my ($self, $level) = @_;
	my $conn = $self->{'_conn'};
	my $userId = _checkLive($self);
	$level <= 0 and die BuildForge::Services::DBO::AccessGroup->invalid('ID');

	$conn->request(COMMAND_ACCESS_ADD_USER);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_ACCESS_USER_ID, $userId);
	$conn->call();
}

# Implemented through the access group's del user command
sub deleteAccessGroup {
	my ($self, $level) = @_;
	my $conn = $self->{'_conn'};
	my $userId = _checkLive($self);
	$level <= 0 and die BuildForge::Services::DBO::AccessGroup->invalid('ID');

	$conn->request(COMMAND_ACCESS_DEL_USER);
	$conn->writeEntryString(KEY_ACCESS_GROUP_ID, $level);
	$conn->writeEntryString(KEY_ACCESS_USER_ID, $userId);
	$conn->call();
}


# wrap is inherited

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_USER_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @users = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@users;
}

sub getUserId          { $_[0]->{'userId'}; }
sub getLevelHint       { $_[0]->{'levelHint'}; }
sub getName            { $_[0]->{'name'}; }
sub getEmail           { $_[0]->{'email'}; }
sub getLogin           { $_[0]->{'login'}; }
# sub getPassword        { $_[0]->{'password'}; }  Never sent by server, anyway
sub getMaxBuilds       { $_[0]->{'maxBuilds'}; }
sub getPasswordUpdated { $_[0]->{'passwordUpdated'}; }
sub getPasswordExpires { $_[0]->{'passwordExpires'}; }
sub getPasswordExpired { $_[0]->{'passwordExpired'}; }
sub getTimeZone        { $_[0]->{'timeZone'}; }
sub getDateFormat      { $_[0]->{'dateFormat'}; }
sub getPriority        { $_[0]->{'priority'}; }
sub getLocale          { $_[0]->{'locale'}; }
# sub getData            { $_[0]->{'data'}; }
sub getLdap            { $_[0]->{'ldap'}; }

# sub setUserId -- not needed
sub setLevelHint       { $_[0]->{'levelHint'}       = $_[1]; }
sub setName            { $_[0]->{'name'}            = $_[1]; }
sub setEmail           { $_[0]->{'email'}           = $_[1]; }
sub setLogin           { $_[0]->{'login'}           = $_[1]; }
sub setPassword        {
	$_[0]->{'password'} = bf_encrypt(defined($_[1]) ? $_[1] : '');
	$_[0]->{'passwordUpdated'} = time;
}
sub setMaxBuilds       { $_[0]->{'maxBuilds'}       = $_[1]; }
# sub setPasswordUpdated { $_[0]->{'passwordUpdated'} = $_[1] + 0; }
sub setPasswordExpires { $_[0]->{'passwordExpires'} = $_[1]; }
sub setPasswordExpired { $_[0]->{'passwordExpired'} = $_[1]; }
sub setTimeZone        { $_[0]->{'timeZone'}        = $_[1]; }
sub setDateFormat      { $_[0]->{'dateFormat'}      = $_[1]; }
sub setPriority        { $_[0]->{'priority'}        = $_[1]; }
sub setLocale          { $_[0]->{'locale'}          = $_[1]; }
sub setData            { $_[0]->{'data'}            = $_[1]; }
# sub setLdap            { $_[0]->{'ldap'}            = $_[1]; }

# toString is inherited


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::User


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing users
	$allUsers = BuildForge::Services::DBO::User->findAll($conn);
	$olduser = BuildForge::Services::DBO::User->findById($conn, $userId);
	$olduser2 = BuildForge::Services::DBO::User->findByLogin($conn, 'login');

	# Getter / setter functions
	$id = $olduser->getUserId();
	$username = $olduser->getName();
	$login = $olduser->getLogin();
	$email = $olduser->getEmail();
	$maximumDailyBuilds = $olduser->getMaxBuilds();
	$timezone = $olduser->getTimeZone();
	$dateFormat = $olduser->getDateFormat();
	$locale = $olduser->getLocale();
	$wasPasswordUpdated = $olduser->getPasswordUpdated();
	$doesPasswordExpire = $olduser->getPasswordExpires();
	$hasPasswordExpired = $olduser->getPasswordExpired();
	$isPriorityUser = $olduser->getPriority();
	$isLDAPUser = $olduser->getLdap();
	
	$olduser->setName('New User Name');
	$olduser->setLogin('newlogin');
	$olduser->setPassword('password');
	$olduser->setEmail('login@company.com');
	$olduser->setMaxBuilds(5);
	$olduser->setTimeZone('PDT');
	$olduser->setDateFormat('%D %I:%M%p');
	$olduser->setLocale('en_US');
	$olduser->setPasswordExpires(1);
	$olduser->setPasswordExpired(0);
	$olduser->setPriority(1);
	$olduser->update();
	
	# User creation, updating, access group control, deletion
	$newuser = new BuildForge::Services::DBO::User($conn);
	$newuser->setName('Newly Created User');
	$newuser->setLogin('newerlogin');
	$newuser->setPassword('password');
	$newuser->create();

	$newuser->setEmail('newerlogin@company.com');
	$newuser->update();
	
	$newuser->resetLicense();
	$newuser->resetLogin();
	
	$accessGroup = BuildForge::Services::DBO::AccessGroup->findByName($conn, 'MyAccessGroup');
	$newuser->addAccessGroup($accessGroup->getLevel());
	$newuser->deleteAccessGroup($accessGroup->getLevel());
	
	$newuser->delete();
	BuildForge::Services::DBO::User->deleteById($conn, $olduser->getUserId());
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

User allows you to create, access, and delete BuildForge users. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::User(connection, {UserParamsHashRef})

Returns a new instance of a User object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item UserParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item name

The displayed name of the user.

=item login

The user's login name.

=item password

The user's login password.  This must match the engine's password security settings.
See the "Password Format" system configuration setting and the related online help information
for details.

=item email

The user's email address where notification emails should be sent.

=item maxBuilds

The user's maximum number of daily builds.  A value of 0 (the default) allows the 
user to start unlimited builds.

=item passwordExpires

If this evaluates to true (the default), the user's password expires after a number of days set
in the "Password Expiration Days" system configuration setting.  If this evaluates to 
false, the user's password doesn't expire.

=item timeZone

The user's timezone (from $timezone->getZoneId()).

=item dateFormat

The date format for date strings that appear in the user interface for the user.  
This uses the POSIX strftime() function - check your local system implementation
of this to see what values it accepts.

=item priority

The user is a priority user if this evaluates to true and isn't if this evaluates to false.
When a priority user logs in, if there are too many users currently connected to the BuildForge
console due to license restrictions, one of those users gets bumped off to make way for the 
priority user to log in.

=item locale

The user's locale, such as 'en_US' (the default), which controls the user's language
settings.

=back

=back


=head2 BuildForge::Services::DBO::User->findAll(connection)

Returns a reference to an array of BuildForge::Services::DBO::User objects corresponding
to all users in the database.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::User->findById(connection, userId)

Returns the BuildForge::Services::DBO::User object corresponding to the given
user ID, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item userId

The user ID (from $user->getUserId()) for the desired user.

=back


=head2 BuildForge::Services::DBO::User->findByLogin(connection, login)

Returns the BuildForge::Services::DBO::User object corresponding to the given
user login, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item userId

The user login (from $user->getLogin()) for the desired user.

=back


=head2 $user->create()

Creates the user record within the database.


=head2 $user->update()

Updates the user record within the database.


=head2 $user->resetLicense()

Resets the user's license information, causing them to no longer take up a license seat
until they log back in.  Note that this does not log the user out of the services layer - the
user's session within the services layer is valid until the user logs out.


=head2 $user->resetLogin()

Logs the user out of the BuildForge system.  This does not log the user out of the 
services layer, just out of the web-based UI.


=head2 $user->addAccessGroup(accessGroupId)

Adds the user to an access group.  Access group membership controls both 
user permissions and what objects the user has access to within the system.
The root user cannot get added to any access groups.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) of the access group.

=back


=head2 $user->deleteAccessGroup(accessGroupId)

Removes the user from an access group.  Access group membership controls both 
user permissions and what objects the user has access to within the system.

=over 8

=item accessGroupId

The access group ID (from $accessGroup->getLevel()) of the access group.

=back


=head2 $user->delete()

Deletes the user record from the database.
The root user cannot get deleted.


=head2 BuildForge::Services::DBO::User->deleteById(connection, userId)

Deletes the user record with the given ID from the database.
The root user (ID of 1) cannot get deleted.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item userId

The user ID (from $user->getUserId()) to delete.

=back


=head2 $user->getUserId()

Returns the user's ID.


=head2 $user->getName()

Returns the user's name.


=head2 $user->getLogin()

Returns the user's login name.


=head2 $user->getEmail()

Returns the user's email address.


=head2 $user->getMaxBuilds()

Returns the user's maximum daily builds or 0 if the user has unlimited builds.


=head2 $user->getTimeZone()

Returns the user's timezone.


=head2 $user->getDateFormat()

Returns the date format for date strings that appear in the user interface for the user.  
This uses the POSIX strftime() function - check your local system implementation
of this to see what values it accepts.


=head2 $user->getLocale()

Returns the user's language settings.


=head2 $user->getPasswordUpdated()

Returns the timestamp for the last time the user was updated.  This is only logged
if the system expires passwords with the "Password Expiration Days" system configuration
setting.


=head2 $user->getPasswordExpires()

Returns 1 if the user's password expires and 0 if it doesn't.



=head2 $user->getPasswordExpired()

Returns 1 if the user's password has expired and 0 if it hasn't.


=head2 $user->getPriority()

Returns 1 if the user is a priority user and 0 if it isn't.
When a priority user logs in, if there are too many users currently connected to the BuildForge
console due to license restrictions, one of those users gets bumped off to make way for the 
priority user to log in.


=head2 $user->getLdap()

Returns 1 if the user is an LDAP user and 0 if they aren't.


=head2 $user->setName(userName)

Sets the user's name.
$user->update() must be run before changes are replicated in the database.

=over 8

=item userName

The new user name string.

=back


=head2 $user->setLogin(login)

Sets the user's login.
$user->update() must be run before changes are replicated in the database.

=over 8

=item login

The new login name string.

=back


=head2 $user->setPassword(password)

Sets the user's password.  This must match the engine's password security settings.
See the "Password Format" system configuration setting and the related online help information
for details.
$user->update() must be run before changes are replicated in the database.

=over 8

=item password

The new password string.

=back


=head2 $user->setEmail(email)

Sets the user's email address which notification emails get sent to.
$user->update() must be run before changes are replicated in the database.

=over 8

=item email

The new email address string.

=back


=head2 $user->setMaxBuilds(maxBuilds)

Sets the users maximum daily builds.
$user->update() must be run before changes are replicated in the database.

=over 8

=item maxBuilds

The new number of daily builds, or 0 to allow unlimited builds.

=back


=head2 $user->setTimeZone(timeZoneId)

Sets the user's timezone.
$user->update() must be run before changes are replicated in the database.

=over 8

=item timeZoneId

The timezone ID (from $timezone->getZoneId()) for the new user timezone.

=back


=head2 $user->setDateFormat(dateFormat)

Sets the date format for date strings that appear in the user interface for the user.  
$user->update() must be run before changes are replicated in the database.

=over 8

=item dateFormat

The new date format.
This uses the POSIX strftime() function - check your local system implementation
of this to see what values it accepts.

=back


=head2 $user->setLocale(locale)

Sets the user's locale, which controls the user's language
settings.
$user->update() must be run before changes are replicated in the database.

=over 8

=item locale

The new locale, such as 'en_US'.

=back


=head2 $user->setPasswordExpires(doesPasswordExpire)

Sets whether the user's password expires on a schedule controlled by the
"Password Expiration Days" system configuration setting.
$user->update() must be run before changes are replicated in the database.

=over 8

=item doesPasswordExpire

If this evaluates to true, the password does expire, and if not, it doesn't.

=back


=head2 $user->setPasswordExpired(hasPasswordExpired)

Sets whether the user's password has expired.
$user->update() must be run before changes are replicated in the database.

=over 8

=item hasPasswordExpired

If this evaluates to true, the password has expired, and the user must
change their password when next they log in.  If this evaluates to false,
the password hasn't expired.

=back


=head2 $user->setPriority(isPriorityUser)

Sets whether or not the user is a priority user.
When a priority user logs in, if there are too many users currently connected to the BuildForge
console due to license restrictions, one of those users gets bumped off to make way for the 
priority user to log in.
$user->update() must be run before changes are replicated in the database.

=over 8

=item isPriorityUser

The user becomes a priority user if this evaluates to true and doesn't if this evaluates to false.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

