<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
 owns the copyright and other intellectual property rights in this
 software.
 Duplication or use of the Software is not permitted except as expressly
 provided in a written agreement between your company and BuildForge,
 Inc.
 */
/*
* BFColumnTypeCache
 *
 * Utilities for checking on type and length of columns in buildforge tables.
 * Also serializes/unserializes this from disk or generates it if it's not there.
 * <code>
 *    $typeCache = BFColumnTypeCache::getCache();
 *    $type = $typeCache->getColumnType('bf_table', 'bf_column');
 *    $len  = $typeCache->getColumnLength('bf_table', 'bf_column');
 * </code>
 *
 * @package Console
 * @subpackage Core
 */


class BFColumnTypeCache {
 	public $version = "0";
 	public $database = "";
 	private $allTables;
 	public $cache_file;
 	public $isAPI = false;
 	protected $tables;
 	protected $db;

	/* This method is called to regenerate and re-serialize the cache to disk. */
 	public function populate() {
		// get a list of all the tables we want to cache.
 		if (!is_array($this->tables)) {
 			if ($this->isAPI) {
 				$this->tables = $this->db->listTables();
 			} else {
 				$this->tables = BFQueryBuilder::listTables($this->db);
 			}
 		}
		
 		foreach ($this->tables as $tbl) {
			// get column name, type, len info for each column in the table.
 			$info = $this->db->tableInfo($tbl);
			if(empty($info)) { //this attempt to prevent a freaky error in DB2 when the DB isn't completly ready
				  BFExceptionHandler::printDumbErrorPage("There was an error preparing your database, your system may not be finished installing.\nIf you continue to see this error message, contact customer service\n");
				  exit();
			}	
			
 			foreach ($info as $i) {
 				$this->allTables[strtolower($tbl)][strtolower($i['name'])] = 
 						array('type' => $i['type'], 'len' => $i['len']);
 			}
 		}
 		$this->updateVersionInfo();
 		$this->serializeObject();
 	}
	
	public function repopulate() {
		$this->tables = null;
		$this->populate();
	}


 	public function setTables($tables_array) {
 		$this->tables = $tables_array;
 	}

	public function getColumns($table) {
		return (is_array($this->allTables) && isset($this->allTables[strtolower($table)])) ? 
				array_keys($this->allTables[strtolower($table)]) : 
				array();
	}


 	/* Get the Database Type of the given tables column */
 	public function getColumnType($table, $column) {
 		$table = strtolower($table);
 		$column = strtolower($column);
	 	if (isset($table) && isset($column) && isset($this->allTables[$table]) &&
	 			array_key_exists($column, $this->allTables[$table])) {
 			return $this->allTables[$table][$column]['type'];
		}
 	}

 	/* Get the maximum length of the given tables column.
		* Note: This is really only useful for data types that specify a length.   */
 	public function getColumnLength($table, $column) {
 		return $this->allTables[strtolower($table)][strtolower($column)]['len'];
 	}


 	/* Serialize ourselves */
 	public function serializeObject() {
		global $controller;
 		$tmpdb = $this->db;
 		$this->db = null;
		$serialized = serialize($this);
 		if (file_put_contents($this->cache_file.$this->dbtype.$this->version, $serialized, LOCK_EX) < 1) {
 			throw new BFException(translate('2149'));
 		}
		if ($this->isAPI) {
			$this->db = $tmpdb;
			return;
		}
		$controller->delPCache($this->cache_file.$this->dbtype.$this->version);
		$controller->setPCache($this->cache_file.$this->dbtype.$this->version, $this);
 		$this->db = $tmpdb;
 	}


 	/* Factory method for returning a BFColumnTypeCache object */
	/*
		* Unserialize the cached column_type object from disk if available,
	 * otherwise populate a new object.
	 * Pass a db object as the argument.
	 */
 	public static function getCache($db, $db_type, $version, $isapi = false) {
		global $controller;
		static $data;
		if (!isset($data)) {
			$data = new BFColumnTypeCache($db, $db_type, $version);
			$dbType = $data->dbtype;
			if (file_exists($data->cache_file.$db_type.$version)) {

				if (!$isapi) {	
					$data = $controller->getPCache($data->cache_file.$db_type.$version);
				}
				if (!$data) {
					$data = @unserialize(implode("", @file($data->cache_file.$db_type.$version)));
				}
				if (!$data) {
					$data = new BFColumnTypeCache($db, $db_type, $version);
					$data->isAPI = $isapi;
					$data->populate();
				} else {
					$data->dbtype = $dbType;
					$data->db = $db;
					if (!$data->checkVersion()) {
						if ($isapi) {
							$tables_array = $db->listTables();
						} else {
							$tables_array = BFQueryCompiler::listTables($db);
						}
						$data = new BFColumnTypeCache($db, $db_type, $version);
						$data->isAPI = $isapi;
					        	
						$data->setTables($tables_array);
						$data->populate();
					}
				}
			} else {
				$data->isAPI = $isapi;
				$data->populate();
			}
		}

	 	return $data;
	}


	/* Checks the existing version in the object against the databases.
		* If the database is newer we call populate.
		*/
	private function checkVersion() {
		global $GLOBAL_DB_TABLES;
		if ($this->version < $this->dbversion || $this->database != $this->dbtype) {
			return false;
		} else {
			$diff = array_diff($GLOBAL_DB_TABLES, $this->tables);
			if (count($diff) > 0) {
				throw new BFDBException(translate('2157'));
			}
		}
		return true;
	}

	private function setVersion($ver) {
		$this->dbversion = $ver;
		$this->database = $this->dbtype;
	}

	/* used to internally update the version of the schema to perform need-to-update checks with later */
	private function updateVersionInfo() {
		$this->version = $this->dbversion;
		$this->database = $this->dbtype;
	}



	/* base constructor... not much here other than setting up the filename */
	/* accepts a db connection object as argument. */
	function __construct($db, $db_type, $version) {
		//parent::__construct();

		$this->db = $db;
		$this->dbtype = $db_type;
		$this->dbversion = $version;
		// Setup the file path that will always be used for the cache.  This should be globaly used.
		$this->cache_file = ini_get('upload_tmp_dir') . DIRECTORY_SEPARATOR . 'column_type_cache';
	}

}


?>
