<?php

/**
 * Constants for use in specifying options
 */
define('DB_FETCHMODE_ASSOC', 1);
define('DB_FETCHMODE_OBJECT', 2);
define('DB_FETCHMODE_ARRAY', 3);

/**
 * BFDB
 * 
 * Emulate PEAR::DB (or rather, what we use from it) for the Oracle connection. We needed
 * to pull this out for Oracle, since PEAR::DB's Oracle implementation would take longer
 * to fix than to rewrite...
 *
 * It turns out this runs about twice as fast, anyway. PEAR::DB likes to run queries on
 * its own, several of which bog down the databases...lovely.
 * 
 * @package Console
 * @subpackage Core
 */
class DB {
	// Methods not yet implemented from PEAR::DB
	// string apiVersion ()
	// boolean isManip (string $query)
	// string errorMessage (integer $value)
	// object a &factory (string $type, [array $options = false])
	// bool isConnection (mixed $value)
	// array parseDSN (string $dsn)
	
	static public function connect($dsn, $options = array()) {
		global $BF_REGEX_DSN, $controller;
		if (isset($dsn['provider'])) {
			$class = 'DB_' . strtolower($dsn['provider']);
			$path = 'BFDB' . DIRECTORY_SEPARATOR . $class . '.php';
			if (include_once $path) {
				try {
					return new $class($dsn, $options);
				} catch (Exception $e) {
					BFExceptionHandler::printDumbErrorPage($e->getMessage());
				}
			} else {
				throw new BFDBException(translate('2151', $class));
			}
		} else {
			throw new BFDBException(translate('2152'));
		}
	}
	
	static public function isError($value) {
		return $value instanceof DB_Error;
	}
}

class DB_common {
	protected $handle;
	protected $options = array(
							'mode' => DB_FETCHMODE_OBJECT
						);

	/**
	 * Array, not string, of the DSN
	 */
	protected $dsn_parsed;
/*	
	public function affectedRows() -- Finds number of rows affected by a data changing query
	public function autoCommit() -- Turns auto-commit on or off
	public function autoExecute() --  Prepares and runs an INSERT or UPDATE query based on variables you supply
	public function autoPrepare() --  Prepares an INSERT or UPDATE statement based on variables you supply
	public function commit() -- Commits the current transaction
	public function createSequence() -- Creates a new sequence
	public function disconnect() -- Disconnects from a database
	public function dropSequence() -- Deletes a sequence
	public function escapeSimple() -- Escapes a string according to the current DBMS's standards
	public function execute() -- Executes a prepared SQL statment
	public function executeMultiple() -- Runs a prepared SQL statment for each element of an array
	public function freePrepared() -- Releases resources associated with a prepared SQL statement
	public function getAll() -- Runs a query and returns all the data as an array
	public function getAssoc() -- Runs a query and returns the data as an array
	public function getCol() -- Runs a query and returns the data from a single column
	public function getListOf() -- Views database system information
	public function getOne() -- Runs a query and returns the first column of the first row
	public function getOption() --  Determines current state of a PEAR DB configuration option
	public function getRow() -- Runs a query and returns the first row
	public function limitQuery() -- Sends a LIMIT query to the database
	public function nextId() -- Returns the next number from a sequence
	public function prepare() -- Prepares a SQL statement for later execution
	public function provides() -- Checks if the DBMS supports a particular feature
	public function query() -- Sends a query to the database
	public function quote() -- DEPRECATED: Quotes a string
	public function quoteIdentifier() -- Formats a string so it can be safely used as an identifier
	public function quoteSmart() -- Formats input so it can be safely used as a literal
	public function rollback() -- Rolls back the current transaction
	public function setOption() -- Sets run-time configuration options for PEAR DB
	public function tableInfo()
*/

	public function setFetchMode($mode) {
		$this->options['mode'] = $mode;
	}

	/**
	 * Returns the string indicating the type of database engine
	 * it has connected to (ie. db2, mysqli, sybase)
	 */
	public function getType() {
		return (isset($this->dsn_parsed['provider'])) ? $this->dsn_parsed['provider'] : false;
	}
	
	/**
	 * Yeah, PHPType...since we can have a SQL Server connection
	 * via ODBC, the driver differs from the SQL that needs to
	 * get generated to pass off to the driver in the first place.
	 */
	public function getPHPType() {
		return (isset($this->dsn_parsed['php_type'])) ? $this->dsn_parsed['php_type'] : false;
	}
	
	function __construct($dsn, $options = null) {
		$this->dsn_parsed = $dsn;
		if (is_array($options)) {
			$this->options = $options;
		}
		if (!$this->connect()) {
			return new DB_Error(translate('2153'));
		}
	}
}

class DB_Error {
	protected $message;
	protected $options;
	
	public function getMessage() {
		return $this->message;
	}
	
	function __construct($msg) {
		$this->message = $msg;
	}
}

class DB_Result {
	protected $result;
	protected $query;
	protected $handle;
	protected $params;
	
	public function setQuery($query) {
		$this->query = $query;
	}

	public function setHandle($handle) {
		$this->handle = $handle;
	}

	public function setParameters($params = null) {
		$this->params = $params;
	}
	
	function __construct($res, $opts) {
		$this->result = $res;
		$this->options = $opts;
	}
}

?>
