<?php

class DB_pgsql extends DB_common {
	public function query($query, $params = null) {
		if ($result = pg_query($this->handle, $query)) {
			if ($result === true) {
				return true;
			} else if ($result === false) {
				return false;
			} else {
				return new DB_result_pgsql($result, $this->options);
			}
		} else {
			return new DB_error($query);
		}
	}
	
	public function getRow($query, $params = null, $fetchmode = null) {
		if ($result = pg_query($this->handle, $query)) {
			$db_result = new DB_result_pgsql($result, $this->options);
			return $db_result->fetchRow($fetchmode);
		}
		return false;
	}
	
	public function getCol($query, $index = 0) {
		$column = array();
		if ($result = pg_query($this->handle, $query)) {
			while ($row = pg_fetch_array($result)) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				$column[] = $row[$index];
			}
		}
		return $column;
	}
	
	public function getOne($query, $index = 0) {
		if ($result = pg_query($this->handle, $query)) {
			if ($row = pg_fetch_array($result)) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				return $row[$index];
			}
		}
		return false;
	}
	
	public function getSpecialQuery($token) {
		switch ($token) {
			case 'tables':
                return 'SELECT c.relname AS "Name"'
                        . ' FROM pg_class c, pg_user u'
                        . ' WHERE c.relowner = u.usesysid'
                        . " AND c.relkind = 'r'"
                        . ' AND NOT EXISTS'
                        . ' (SELECT 1 FROM pg_views'
                        . '  WHERE viewname = c.relname)'
                        . " AND c.relname !~ '^(pg_|sql_)'"
                        . ' UNION'
                        . ' SELECT c.relname AS "Name"'
                        . ' FROM pg_class c'
                        . " WHERE c.relkind = 'r'"
                        . ' AND NOT EXISTS'
                        . ' (SELECT 1 FROM pg_views'
                        . '  WHERE viewname = c.relname)'
                        . ' AND NOT EXISTS'
                        . ' (SELECT 1 FROM pg_user'
                        . '  WHERE usesysid = c.relowner)'
                        . " AND c.relname !~ '^pg_'";
			default:
				exit($token);
		}
	}
	
	public function quoteIdentifier($ident) {
		return '"' . str_replace('"', '""', $ident) . '"';
	}
	
	public function quoteSmart($in, $type = null) {
	    if ($type != null) {
		    if ($type == 'string') {
			    return '\'' . pg_escape_string($in) . '\'';
		    } else {
			    return $in;
		    }
	    }

        if (is_int($in) || is_double($in)) {
            return $in;
        } elseif (is_bool($in)) {
            return $in ? 1 : 0;
        } elseif (is_null($in)) {
            return 'NULL';
        } else {
            return '\'' . pg_escape_string($in) . '\'';
        }
    }
	
	public function tableInfo($param) {
		$info = array();
		if (is_string($param)) {
			$query = 'SELECT * FROM ' . $this->quoteIdentifier($param) . ' LIMIT 0';
			if ($result = pg_query($this->handle, $query)) {
				$pgsql_result = new DB_result_pgsql($result, $this->options);
				$info = $pgsql_result->tableInfo();
			}
		} else if ($param instanceof DB_result_pgsql) {
			$temp_info = $param->tableInfo();
			foreach ($temp_info as $k => $v) {
				$info[$k] = array();
				foreach ($v as $kk => $vv) {
					$info[$k][$kk] = $vv;
				}
			}
		}
		return $info;
	}
	
	protected function connect() {
		if (!function_exists('pg_connect')) {
			throw new BFDBException(translate('database extension not found', 'pgsql'));
		}

		if (array_key_exists('persistent', $this->options) && $this->options['persistent'] === true) {
			$this->handle = pg_pconnect('host=' . $this->dsn_parsed['hostname'] . ' dbname=' . $this->dsn_parsed['database'] . ' user=' . $this->dsn_parsed['username'] . ' password=' . $this->dsn_parsed['password']);
		} else {
			$this->handle = pg_connect('host=' . $this->dsn_parsed['hostname'] . ' dbname=' . $this->dsn_parsed['database'] . ' user=' . $this->dsn_parsed['username'] . ' password=' . $this->dsn_parsed['password']);
		}
		if ($this->handle) {
			return true;
		} else {
			throw new BFDBException(pg_last_notice($this->handle));
		}
	}

	public function close() {
		pg_close($this->handle);
	}
}

class DB_result_pgsql extends DB_result {
	public function fetchRow($fetch_mode = false, $rownum = null) {
		if (!is_null($rownum)) {
			pg_result_seek($this->result, $rownum);
		}
		return $this->fetchNext();
	}
	
	public function fetchNext($fetch_mode = false) {
		if (!$fetch_mode) {
			$fetch_mode = $this->options['mode'];
		}
		switch ($fetch_mode) {
			case DB_FETCHMODE_ARRAY:
				return pg_fetch_array($this->result);
			case DB_FETCHMODE_ASSOC:
				return pg_fetch_assoc($this->result);
			case DB_FETCHMODE_OBJECT:
				return pg_fetch_object($this->result);
		}
	}
	
	public function numRows() {
		return pg_num_rows($this->result);
	}
	
	public function free() {
		pg_free_result($this->result);
	}
	
	public function tableInfo() {
		$info = array();
		
		$field_count = pg_num_fields($this->result);
		for ($i = 0; $i < $field_count; $i++) {
			$info[] = array(
							'name' => pg_field_name($this->result, $i),
							'type' => pg_field_type($this->result, $i),
							'len' => pg_field_size($this->result, $i)
							);
		}
		
		return $info;
	}
}

?>
