<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * CentralController
 * This provides the access point for just about anything to get to just about
 * anything else. It initializes and passes references to the default database
 * connection, provides access to modules, files, HTTP headers, initiates the
 * calls to modules for handling requests and answers questions such as:
 * - Does the license allow for this?
 * - Should I allow the user to do this?
 * Some of this functionality should get broken out into other objects
 * and modules over time, but for now it makes for a convenient central
 * location to control aspects of the user interface.
 * @package Console
 * @subpackage Core
 */

class CentralController {
	protected $modules_available;
	protected $modules_viewable;
	protected $modules_loaded;
	protected $plugins_global;
	protected $plugins_module;
	protected $plugins;
	protected $permissions;
	public $master = false;
	public $local = false;
	public $geo;
	protected $db;
	protected $master_db;
	protected $dbpool = array();
	protected $sysconfig;
	public $configuration;
	protected $master_configuration;
	protected $time_offset;
	protected $active_module;
	protected $default_module = 'home'; // To come from the user''s pref, defaults to this
	protected $headers;
	protected $content_type_set;
	protected $messages;
	protected $errors;
	protected $template;
	protected $raw_request;
	protected $product;
	protected $users_count;
	public $request_type = BF_REQUEST_XHTML;
	protected $included_files = array(); // An array of all files included -- test against instead of require_once
	protected $request_headers;
	protected $shmoppable;
	protected $fileCachePrefix;
	public $license_error = false;

	/**
	 * Tied to Apache. If we switch, you just won't get any headers
	 * 
	 * @param string $header For example: 'If-Modified-Since'
	 * @return string|boolean Returns the corresponding header value or false on failure
	 */
	public function getRequestHeader($header) {
		// If not set, attempt to load the headers first
		if (!is_array($this->request_headers)) {
			if (function_exists('apache_request_headers')) {
				$this->request_headers = apache_request_headers();
			} else {
				// If no function, no headers
				$this->request_headers = array();
			}
		}
		// Actually try to return the header
		if (array_key_exists($header, $this->request_headers)) {
			return $this->request_headers[$header];
		} else {
			return false;
		}
	}

	/**
	 * You never know when you want to override something like this
	 * for the sake of permissions, logging, etc.
	 * 
	 * @param string $file The filename or path to include
	 * @return boolean Success
	 */
	public function include_file($file) {
		if (!isset($this->included_files[$file])) {
			$this->included_files[$file] = true;
			if (include($file)) {
				return true;
			} else {
				BFExceptionHandler::logError(new BFException(BF_LANG_ERR_FILE_INCLUDE . ': \'' . $file . '\''));
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * Instead of using require_one (from the PHP library), use include_once and throw
	 * our OWN exception if it fails. This not only means that the page doesn''t just crap out
	 * if it fails to include the file, but the error gets logged with everything else and it
	 * gives the caller much more control about how to handle the failed file include.
	 * 
	 * @param string $file The filename or path to include
	 * @return boolean Success
	 */
	public function require_file($file) {
		if (!isset($this->included_files[$file])) {
			$this->included_files[$file] = $file;
			if (include($file)) {
				return true;
			} else {
				throw new BFException(BF_LANG_ERR_FILE_INCLUDE . ': \'' . $file . '\'');
			}
		} else {
			return true;
		}
	}

	/**
	 * Basically a wrapper for {@link require_file()} to include a couple of extra bits just
	 * for modules:
	 *
		1. Check that the module requested exists, which only has the modules allowed for the user
			based on permission settings.
		2. Attempt to use {@link require_file()} to actually include the file - this makes the
			pathname like "modules/(classname)/(classname).php" so ALWAYS make your modules'
			file structure conform to that. (If I don't remember to stick this info in the main
			module notes, you have permission to come over here and smack me upside the head)
		3. If it works, cool! If not, either throw the Exception handed off by
			{@link require_file()} so the caller can decide what to do, or throw an exception that
			the user doesn't have permission to access the module in question.
	 *
	 * @param string $mod The module name, as defined in bf_modules
	 * @return boolean Success
	 */

	public function require_module($mod) {
		if (array_key_exists($mod, $this->modules_available)) {
			try {
				$this->require_file(

						'modules/'
						. $this->modules_available[$mod]['class']
						. '/'
						. $this->modules_available[$mod]['class'] . '.php'
					);
				return true;
			} catch (Exception $e) {
				throw $e;
			}
		} else {
			throw new BFPermissionException($mod);
		}

		return false;
	}

	/**
	 * Use this instead of calling header() directly. Reasoning: if we ever want to log headers,
	 * filter them, do anything else with them - this makes it easier... Later on in execution,
	 * {@link display()} calls {@link sendHeaders()}, which actually goes through and sends them.
	 * 
	 * @param string $header The full header (ex: 'Location: http://whatever/wherever/')
	 * @return boolean
	 */
	public function addHeader($header) {
		if (strpos(strtolower($header), 'content-type') !== false) {
			if ($this->content_type_set) {
				return true;
			} else {
				$this->content_type_set = true;
			}
		}
		$this->headers[] = $header;
		return true;
	}

	/**
	 * Adds an informational message to the output for the template to decide how to display.
	 * Useful for indicating the success of an action or a system-wide message.
	 * 
	 * @param string $msg
	 */
	public function addMessage($msg) {
		if (!is_array($this->messages)) {
			$this->messages = array($msg);
		} else {
			$this->messages[] = $msg;
		}
	}

	/**
	 * Adds an error message to the output for the template to decide how to display. Useful for
	 * indicating the failure of an action.
	 * 
	 * @param string $err
	 */
	public function addError($err) {
		if (!is_array($this->errors)) {
			$this->errors = array($err);
		} else {
			$this->errors[] = $err;
		}
	}

    public function getUsersCount() {
        if (!isset($this->users_count)) {
            $qb = BFQueryBuilder::select();
            $qb->addColumn('bf_userid', 'bf_sessions', 'bf_userid', 'COUNT');
            if ($this->getSysconfig('feature_fixedusers') != '1') {
                $qb->where(
                    array(
                        array('!=', 'bf_sessions', 'bf_id', ''),
                        array('!=', 'bf_sessions', 'bf_userid', 1)
                    )
                );
            } else {
                $qb->where(
                    array(
                        array('!=', 'bf_sessions', 'bf_userid', 1)
                    )
                );
            }
            // If this fails, you get NO USERS!
            $this->users_count = intval($qb->getOne());
        }
        return $this->users_count;
    }

    public function currentUserHasSeat($userid) {
        $qb = bfquerybuilder::select();
        $qb->addcolumn('bf_userid', 'bf_sessions', 'bf_userid', 'count');
        $qb->where(
            array(
                array('=', 'bf_sessions', 'bf_userid', $userid)
            )
        );
        $this->userHasSeat = intval($qb->getone());
        if ($this->userHasSeat != 0) {
            $userHasLicense = 1;
        }
        return $userHasLicense;
    }

	/**
	 * Used to compensate for the strange database records we have around DST.  Bah.
	 */
	private function get_nth_day($yr, $pos, $day, $mon)  {
		if ($pos != 6)  {
			$offset		= date('w', strtotime("$yr-$mon-1"));
			$date		= $day - $offset;
			if ($date < 0) {
				$date += 7;
			}
			$date = $date + 1 + (($pos - 1) * 7);
		}  else  {
			$numdays	= date('t', strtotime("$yr-$mon-1"));
			$offset		= date('w', strtotime("$yr-$mon-$numdays"));
			$date		= $offset - $day;
			if ($date < 0)  {
				$date += 7;
			}
			$date = $numdays - $date;
		}
		return(strtotime("$yr-$mon-$date"));
	}
	
	private function is_dst($stamp, $spos, $sday, $smon, $fpos, $fday, $fmon)  {
		if (!$spos)  {
			return false;
		}
		$smon++;
		$fmon++;
		$yr	= date('Y', (int)$stamp);
		return($stamp > $this->get_nth_day($yr, $spos, $sday, $smon) &&
			$stamp < $this->get_nth_day($yr, $fpos, $fday, $fmon));
	}
		

	/**
	 * Returns the time offset between the user's timezone and GMT.  Only used because it's 
	 * easier than trying to figure out what a timezone of "TAJ" translates to in PHP-land.
	 */
	public function getTimeOffset($stamp = null, $userid = null) {
		global $user;
		if (!$userid)  {
			$userid = $user;
		}
		if (!$stamp)  {
			$stamp = time();
		}
		if (!isset($this->time_offset) && $userid) {
			$uid = (int)$userid->get('bf_userid');

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_userid', 'bf_users', 'userid');
			$qb->addColumn('bf_gmtoffset', 'bf_tzones', 'gmtoffset');
			$qb->addColumn('bf_dstart_pos', 'bf_tzones', 'spos');
			$qb->addColumn('bf_dstart_day', 'bf_tzones', 'sday');
			$qb->addColumn('bf_dstart_mon', 'bf_tzones', 'smon');
			$qb->addColumn('bf_dend_pos', 'bf_tzones', 'fpos');
			$qb->addColumn('bf_dend_day', 'bf_tzones', 'fday');
			$qb->addColumn('bf_dend_mon', 'bf_tzones', 'fmon');
			$qb->addColumn('bf_dst_offset', 'bf_tzones', 'dstoffset');
			$qb->where(array(
							 array('=', 'bf_users', 'bf_tzone', 'bf_tzones', 'bf_zone'),
							 array('=', 'bf_users', 'bf_userid', $uid)
							 ));
			
			$res = $qb->getResult();
			if (!DB::isError($res)) {
				$row = $res->fetchRow();
				$this->time_offset = $row->gmtoffset;


				if ($this->is_dst($stamp, $row->spos, $row->sday, $row->smon, $row->fpos, $row->fday, $row->fmon))  {
					$this->time_offset += $row->dstoffset;
				}

			}  else {
				$this->time_offset = 0;
			}
		}
		return $this->time_offset;
	}

	/**
	 * Return a reference to the current Pear::DB object. The original thought: use this to access
	 * the database so that if you don't need it for anything it doesn''t need to get instantiated
	 * in the first place.
	 *
	 * (*Shawn = idiot*) <i>Everything</i> uses the database connection, which the sole exception of the
	 * bland error page that tells you that the system can''t connect to a database.
	 * 
	 * Takes an optional geo_id argument to get a database to a specific location from the pool
	 */
	public function getDB($geo_id = null) {
		if (isset($geo_id)) {
			if (!isset($this->dbpool[$geo_id])) {
				$this->require_module('BFGeo');
				$dsn = BFGeo::getDSNFromGeoID($geo_id);
				if ($dsn) {
					$this->loadDBConnection($dsn, $geo_id);
				} else {
					return false;
				}
			}
			return $this->dbpool[$geo_id];
		} else {
			if (!isset($this->db)) {
				$this->loadDBConnection();
			}
			return $this->db;
		}
	}

	/**
	 * Either returns a reference to the master DB connection or
	 * simply the usual $this->db if either:
		1. No master; or
		2. $this->db used the master configuration
	 */
	public function getMasterDB() {
		if ($this->master || !$this->local) {
			return $this->db;
		} else {
			return $this->master_db;
		}
	}

	public function getLocationName() {
		if ($this->master) {
			return translate('master');
		} else if ($this->local) {
			return $this->configuration->getName();
		} else {
			return false;
		}
	}
	
	/**
	 * Nastiness to get around a chicken and egg issue in GDD setups...
	 */
	protected function sneakSysConfig($param) {
		$q = BFQueryBuilder::select();
		$q->addColumn('bf_value', 'bf_sysconfig', 'bf_value');
		$q->where(array('=', 'bf_sysconfig', 'bf_param', $param));
		return $q->getOne();
	}

	public function getColumnCache() {
		if (!isset($this->column_cache)) {
			try {
				$this->column_cache = BFColumnTypeCache::getCache(
																$this->getMasterDB(),
																$this->getMasterDB()->getType(),
																BFQueryBuilder::getDBVersion($this->getMasterDB())
																  );
				if ($this->sneakSysConfig('columncache_dirty') != 'No') {
					$this->column_cache->repopulate();
					$q = BFQueryBuilder::update();
					$q->setColumn('bf_value', 'bf_sysconfig', 'No');
					$q->where(array('=', 'bf_sysconfig', 'bf_param', 'columncache_dirty'));
					$q->getResult();
					// Not pretty, but it MUST update bf_strings cache later on rather than now
					$GLOBALS['UPDATED_COLUMN_CACHE'] = true;
					$this->delPCache('plugins');
					$this->delPCache('plugins_global');
					$this->delPCache('modules_available');
					$this->delPCache('navigation_modules');
					$this->delPCache('modules_viewable');
				}
			} catch(Exception $e) {
				$upload_tmp_dir = ini_get('upload_tmp_dir');
				if ($upload_tmp_dir == '') {
					BFExceptionHandler::printDumbErrorPage(translate('2156'));
				}
				BFExceptionHandler::printDumbErrorPage($e->getMessage());//translate('2157'));
			}
		}

		return $this->column_cache;
	}

	/**
	 * By default, try to retrieve a database handle based on the DSN
	 * as constructed from the configuration.
	 * 
	 * Can take an optional argument of a DSN to override using the config
	 */
	public function loadDBConnection($dsn = null, $id = null) {
		if (isset($dsn) && isset($id)) {
			// If not using a configuration file, any exception simply gets thrown
			// Additionally, it gets set to the pool rather than its own variable
			$this->dbpool[$id] = $this->DBFromDSN($dsn);
		} else {
			$dsn = $this->configuration->getDSN();
			try {
				$this->db = $this->DBFromDSN($dsn);
			} catch (Exception $e) {
				BFExceptionHandler::printDumbErrorPage($e->getMessage());
			}
		}
	}

	public function loadMasterDBConnection() {
		if ($this->master_configuration) {
			try {
				// Only load the secondary master connection if not a master setup
				if (!$this->master) {
					$this->local = true;
					$this->master_db = $this->DBFromDSN($this->master_configuration->getDSN());
					// Load in the geo record for the geo_id, etc.
					$this->require_module('BFGeo');
					$this->geo = new BFGeo();
					$this->geo->select('bf_location', $this->configuration->getName());
				}
			} catch (Exception $e) {
				$this->errorPage($e);
			}
		}
	}

	/**
	 * Actually connect to the database based on the DSN constructed by the Configuration object.
	 * This dense little block of code:
	 *
		1. Gets access to the Pear::DB options, configurable in includes/config.php
		2. Next it tries to {@link require_file()} in the actual DB PHP.
		3. If that works, try to connect using the DSN from the Configuration object (NOTE: if
			this fails, Configuration probably got a bad file or didn''t get setup first)
		4. If that works, set the default fetch mode to object. Since most modules tend to
			set this for themselves whenever they query, it probably wouldn''t take too much
			effort to change this (if necessary) down the road.
		5. If that works, it all worked!
	 *
	 * If any of that failed, it pretty much throws a very particular exception, since our support
	 * guys will need to know exactly <i>why</i> it failed when and if it ever does. This
	 * gets thrown as a PHP error, which should get thrown up at the browser during development
	 * and spat into the standard PHP error log in production.
	 */
	public function DBFromDSN($dsn) {
		global $GLOBAL_DB_ATTEMPTS, $GLOBAL_DB_OPTIONS, $GLOBAL_DB_ENGINE;
		$this->require_file($GLOBAL_DB_ENGINE);
		$e = null;
		$db = null;

		for($t = 0; $t < $GLOBAL_DB_ATTEMPTS; $t++) {
			try {
				$db = DB::connect($dsn, $GLOBAL_DB_OPTIONS);
			} catch (Exception $exception) {
				$e = $exception->getMessage();;
				// Log the attempt as a warning, so the script keeps going
				@trigger_error($e, E_WARNING);
				sleep(1);
			}
			break;
		}

		if (!$db) {
			throw new BFDBException($e);
		}

		$db->setFetchMode(DB_FETCHMODE_OBJECT);
		return $db;
	}

	public function getDBType($db = null) {
		if (isset($db)) {
			return $db->getType();
		} else {
			return $this->configuration->getDBType();
		}
	}

	public function getMasterDBType() {
		return $this->master_configuration->getDBType();
	}

	/**
	 * Not to get confused with the Configuration object (which manages config stuff for the UI,
	 * like the database connection to use, etc.), the SysConfig that this returns actually comes
	 * from the database table used to set configuration options for the product as a whole.
	 *
	 * Also acts as an alias for actually retrieving sysconfig settings.
	 */
	public function getSysconfig($param = false) {
		if (!isset($this->sysconfig)) $this->loadSysConfig();

		if ($param) {
			return $this->sysconfig->get($param);
		} else {
			return $this->sysconfig;
		}
	}

	public function setSysconfig($param, $value) {
		if (!isset($this->sysconfig)) $this->loadSysConfig();

		if ($param) {
			return $this->sysconfig->set($param, $value);
		}
	}

	/**
	 * Get the file and make the new manager. As mentioned with {@link getSysconfig()}:
	 * Not to get confused with the Configuration object (which manages config stuff for the UI,
	 * like the database connection to use, etc.), the SysConfig that this loads actually comes
	 * from the database table used to set configuration options for the product as a whole.
	 */
	public function loadSysConfig() {
		$this->require_file('modules/BFDBO/BFDBO.php');
		$this->require_file('modules/BFSysConfig/BFSysConfig.php');
		$this->sysconfig = new BFSysConfigManager();
	}

	/**
	 * This needs to hand out simple answers to whether or not a user
	 * can do something, based on the license, permissions, etc.
	 *
	 * It may also cease to exist, since using a method like this to check for the ability
	 * to access something doesn''t actually tell you <i>why</i> you can''t access it.
	 **/
	public function allowAccessTo($widget) {
		if (!$this->licenseHasAccessTo($widget))	return false;
		else if ($this->userHasAccessTo($widget))   return false;
	}

	/**
	 * So this one actually works - it checks the list of features allowed by the license (held
	 * in an associative array), loaded in from the bf_sysconf database table.
	 *
	 * This assumes that if a piece of functionality has <i>not</i> had a record put in for
	 * management by the license, then it should grant access.
	 */
	public function licenseHasAccessTo($widget) {
		if (array_key_exists($widget, $this->licensed_features)
				&& !$this->licensed_features[$widget]) {
			return false;
		}
	}

	/**
	 * This one doesn''t actually work yet...in part from the schema only recently getting sorted
	 * for how it will store nested group information.
	 *
	 * This will assume that if a piece of functionality has <i>not</i> had a record put in for
	 * management by permissions, it should <i>not</i> grant access (except for root, of course).
	 *
	 * Get your $widgets from bf_perms/bf_ptable.
	 */
	public function userHasAccessTo($widget) {
		return ($GLOBALS['user']->get('bf_userid') == 1 || in_array($widget, $this->permissions));
	}

	/**
	 * A magical method by which PHP can run code as another user (root, most likely) in order
	 * to accomplish something not otherwise allowed to the user currently logged in. I initially
	 * wrote this in order to create access groups and auto-add the user to those groups when
	 * using LDAP authentication.
	 */
	public function sudo($callback, $params, $super_user_id = 1) {
		global $user;
		$result = false;
		$current_user_id = $user->get($user->primary_key);
		$user->set($user->primary_key, $super_user_id);

		try {
			$result = call_user_func_array($callback, $params);
		} catch (Exception $e) {
			$this->addError($e->getMessage());
		}

		// No matter what, the user's id MUST GET RETURNED TO ITS PREVIOUS STATE unless it
		// changed in the process (only happens when a user creates themselves).
		if ($user->get($user->primary_key) == $super_user_id) {
			$user->set($user->primary_key, $current_user_id);
		}
		return $result;
	}

	/**
	 * Check to see if the requested module can get viewed by the user by checking the associative
	 * array of allowed modules, loaded in previously. If the user tries to put anything funny in
	 * the request, panic.
	 */
	public function moduleAllowed($mod) {
		if (preg_match(BF_REGEXP_IS_WORD, $mod)) {
			return array_key_exists($mod, $this->modules_available);
		} else {
			throw new BFException(BF_LANG_ERR_UNVIEWABLE_MODULE);
		}
		return false;
	}

	/**
	 * Check the module to see if it supports the type of request...uh...requested. Technically,
	 * this could also check the interfaces implemented by the module instead of calling one of
	 * the module''s methods, since this does give the module the option of lying.
	 */
	public function requestTypeAllowed($req) {
		if ($this->active_module->handlesRequestType($req)) {
			return true;
		}

		return false;
	}

	/**
	 * The request does not have to come from an HTTP GET command. It <i>does</i>, however, need
	 * to consist of an associative array by this point.
	 *
	 * This method also decides on the request type to use for the overall execution. If it finds
	 * a key setting a request type, it will use that. If not, it sets it as the default.
	 *
	 * Lastly, it triggers the request filters.
	 */
	public function parseRequest($req, $post = null) {
		if (array_key_exists(BF_REQUEST_KEY_TYPE, $req)) {
			$this->request_type = $req[BF_REQUEST_KEY_TYPE];
		}

		// Dirty, dirty request!
		$this->raw_request = $req;
		$this->raw_post = $post;
		
		$this->checkLicense();
	}

	/**
	 * The method, prepare(), does two things:
		1. Call {@link assignModule()} so the controller actually has a module to work with
		2. Call {@link assignTemplate()} so the controller actually has a template to use, based on
			the request type.
	 */
	public function prepare() {
		$this->runRequestFilters();
		$this->assignModule();
		$this->assignTemplate($this->request_type);
	}

	/**
	 * Check the request array for a module. If it specifies one, try to set it. If not, set the
	 * default module.
	 */
	public function assignModule() {
		// Begin: Styx Hack -- either old way "styx styx" in .conf or new way, the license key == BF_STYX_LICENSE_KEY
		if ($this->configuration->styx() || $this->getSysconfig('license_key') == BF_STYX_LICENSE_KEY) {
			$this->default_module = 'projects';
		}
		// End: Styx Hack

		if (array_key_exists(BF_REQUEST_KEY_MODULE, $this->raw_request)) {
			$this->setActiveModule($this->raw_request[BF_REQUEST_KEY_MODULE], $this->raw_request, $this->raw_post);
		} else {
			$this->setActiveModule($this->default_module, $this->raw_request, $this->raw_post);
		}
	}

	/**
	 * Based on the request type passed, set the template (if it exists) and set raw if it doesn''t.
	 *
	 * NOTE: It does try to assign the default, because modules have the ability to decide the
	 * request types allowed <i>including</i> request types not inherently supported by the
	 * central controller.
	 */
	protected function assignTemplate($request_type) {
		if (file_exists(BF_PRODUCT_ROOT . DIRECTORY_SEPARATOR . 'templates' . DIRECTORY_SEPARATOR . $request_type . '.tpl')) {
			$this->template = BF_PRODUCT_ROOT . DIRECTORY_SEPARATOR . 'templates' . DIRECTORY_SEPARATOR . $request_type . '.tpl';
		} else if (file_exists(BF_ROOT . DIRECTORY_SEPARATOR . 'templates' . DIRECTORY_SEPARATOR . $request_type . '.tpl')) {
			$this->template = $request_type . '.tpl';
		} else {
			$this->template = $GLOBALS['GLOBAL_TEMPLATES'][BF_REQUEST_RAW];
		}
	}

	/**
	 * After:
		1. Checking that the user can access the module, this tries to require the starting file
			for the module (described in {@link require_module()}) and
		2. Checking that the module implements the Viewable interface,
	 * the method then either calls handleRequest on the default module or the requested module.
	 */
	protected function setActiveModule($mod, $req, $post = null) {
		$function = null;

		if ($this->moduleAllowed($mod) && array_key_exists($mod, $this->modules_viewable)) {
			try {
				$this->require_module($mod);
			} catch (Exception $e) {
				$this->exitWithError($e);
				return false;
			}
			if (!in_array('Viewable', class_implements(new $mod()))) {
				$this->errorPage(new BFException(BF_LANG_ERR_UNVIEWABLE_MODULE));
				return false;
			}

			$function = array($this->modules_viewable[$mod]['class'], 'handleRequest');
		} else {
			$this->require_module($this->default_module);
			$function = array($this->modules_viewable[$this->default_module]['class'], 'handleRequest');
		}
		return $this->active_module = call_user_func($function, $req, $this->request_type, $post);
	}

	/**
	 * Allow products to override templates, etc. But we need to know where to find them
	 */
	public function setProduct($prod) {
		if (file_exists(BF_ROOT . DIRECTORY_SEPARATOR . 'public' . DIRECTORY_SEPARATOR . $prod)
			&& is_dir(BF_ROOT . DIRECTORY_SEPARATOR . 'public' . DIRECTORY_SEPARATOR . $prod)) {
			$this->product = $prod;
			define('BF_PRODUCT_ROOT', BF_ROOT . DIRECTORY_SEPARATOR . 'public' . DIRECTORY_SEPARATOR . $prod);
		}
	}

	public function setDefaultModule($module) {
		$this->default_module = $module;
	}

	/**
	 * Request filters run before the request gets anywhere near the module. That way, they can
	 * <i>filter</i> the request, either removing from or adding to the requested data. This method
	 * gets used by essentially passing the request "through" the filters, all called here.
	 *
	 * The list of filters gets held in an associative array, part of the multi-dimensional array
	 * holding all plugins for the product.
	 */
	public function runRequestFilters() {
		if (!array_key_exists('requestfilter', $this->plugins)) return true;

		foreach ($this->plugins['requestfilter'] as $plugin) {
			if ($this->require_module($plugin['class'])) {

				$temp_plug = new $plugin['class']();

				if (!in_array('BFRequestFilter', class_implements($temp_plug))) {
					throw new BFException();
				} else {
					$this->raw_request = $temp_plug->handleRequest($this->raw_request);
					$this->raw_post = $temp_plug->handlePost($this->raw_post);
				}
			}
		}
		
		return true;
	}

	/**
	 * After checking that the user has permission to use the request type, set the variable. If
	 * permission got denied, return false and set the request to the default.
	 */
	protected function setRequestType($req) {
		if ($this->requestTypeAllowed($req)) {
			$this->request_type =   $req;
			return true;
		} else {
			$this->request_type =   $this->default_request_type;
			return false;
		}
	}

	/**
	 * Really, this just instantiates a Configuration object. But since that loads in a config
	 * setup (or the default if you don''t pass it one), it really does load the config.
	 * 
	 * Passes the path to the file to use, after looking in the places where it 
	 * would expect to file one. This could later allow random things like remote
	 * configuration files.
	 */
	protected function loadConfig() {
		global $GLOBAL_CONFIGS;
		$this->require_file('Configuration.php');

		// First, get the configuration path for the local database
		foreach ($GLOBAL_CONFIGS as $path) {
			if (file_exists($path)) {
				$this->configuration = new Configuration($path);
				if ($this->configuration->master) {
					$this->master = true;
				}
				return true;
			}
		}
		
		// If you got this far, you either have a master config or a broken system
		return $this->loadMasterConfig();
	}

	protected function loadMasterConfig() {
		if ($this->master && !is_null($this->configuration)) {
			return true;
		}
		
		global $GLOBAL_MASTERS;
		$this->require_file('Configuration.php');

		foreach ($GLOBAL_MASTERS as $path) {
			if (file_exists($path)) {
				// If no local config, treat master as local
				if (is_null($this->configuration)) {
					$this->master = true;
					$this->local = false;
					$this->configuration = new Configuration($path);
				} else {
					$this->master = false;
					// We need to assume that the setup only supports the local database until
					// the last possible second.
					$this->local = false;
					$this->master_configuration = new Configuration($path);
				}
				break;
			}
		}
	}

	/**
	 * TODO: stop loading all at once and break it up by group.
	 */
	public function loadPermissions($userid, $force = null) {
		$this->permissions = $this->getPCache('permissions.'.$userid);
		if ($this->permissions === false || $force) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tag', 'bf_ptable', 'tag');
			$qb->where(array(
							 array('=', 'bf_ptable', 'bf_level', 'bf_accesscache', 'bf_level'),
							 array('=', 'bf_accesscache', 'bf_userid', $userid)
							 ));

			$perms = $qb->getCol();

			if (DB::isError($perms)) {
				throw new BFException(translate('2159'));
			}

			$this->permissions = $perms;
			$this->delPCache('permissions.'.$userid);
			$this->setPCache('permissions.'.$userid, $this->permissions);
		}
	}

	/**
	 * Load the modules available into an associative array, then make another array listing the
	 * viewable modules. Then make another array to use with the navigation (XHTML requests).
	 */
	protected function loadAvailableModules() {
		// Try cache first
		$this->modules_available = $this->getPCache('modules_available');
		$this->navigation_modules = $this->getPCache('navigation_modules');
		$this->modules_viewable = $this->getPCache('modules_viewable');
		// If missing the modules, regenerate all
		if (!$this->modules_available) {
			$this->modules_available = array();
			$this->navigation_modules = array();
			$this->modules_viewable = array();
	
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', 'bf_modules', 'name');
			$qb->addColumn('bf_title', 'bf_modules', 'title');
			$qb->addColumn('bf_classname', 'bf_modules', 'class');
			$qb->addColumn('bf_viewable', 'bf_modules', 'viewable');
			// Don't use 'order' as an alias.  It's a keyword and most DBs won't like it.
			$qb->addColumn('bf_order', 'bf_modules', 'bf_order');
			$qb->where(array('=', 'bf_modules', 'bf_active', 'Y'));
			$qb->orderBy('ASC', 'bf_modules', 'bf_order');
	
			$res = $qb->getResult();
	
			if (DB::isError($res)) return false;
			while ($row = $res->fetchRow()) {
				if (DB::isError($row)) {
					throw new BFDBException($row->getMessage());
				}
				$tmp_array = array(
					'class'	 => $row->class,
					'title'	 => $row->title,
					'viewable'  => ($row->viewable == 'Y'),
					'order'	 => $row->bf_order
				);
				$this->addAvailableModule($row->name,$tmp_array);
			}
			$this->delPCache('modules_available');
			$this->delPCache('navigation_modules');
			$this->delPCache('modules_viewable');
			$this->setPCache('modules_available', $this->modules_available);
			$this->setPCache('navigation_modules', $this->navigation_modules);
			$this->setPCache('modules_viewable', $this->modules_viewable);
		}
	}

	public function addAvailableModule($module_name,$module_info) {
		$this->modules_available[$module_name] = $module_info;

		// Set viewable modules
		if($module_info['viewable']) {
			$this->modules_viewable[$module_name] = $module_info;
			if ($module_info['order'] != '') {
				$this->navigation_modules[$module_name] = $module_info;
			}
		}
	}

	/**
	 * Get the list of plugins for use by the system.
	 *
	 * This not only gets the plugins, but also splits them up into "global" plugins (which affect
	 * all modules) and plugins for specific modules.
	 */
	protected function loadGlobalPlugins() {
		// Try cache first
		$this->plugins = $this->getPCache('plugins');
		$this->plugins_global = $this->getPCache('plugins_global');
		// If missing the plugins, regenerate all
		if (!$this->plugins) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', 'bf_modules', 'bf_name');
			$qb->addColumn('bf_title', 'bf_modules', 'bf_title');
			$qb->addColumn('bf_classname', 'bf_modules', 'bf_classname');
			$qb->addColumn('bf_type', 'bf_plugins', 'bf_type');
			$qb->addColumn('bf_order', 'bf_plugins', 'bf_order');
			$qb->where(array(
							 array('=', 'bf_modules', 'bf_name', 'bf_plugins', 'bf_sourcemodule'),
							 array('=', 'bf_modules', 'bf_active', 'Y'),
							 array('=', 'bf_plugins', 'bf_active', 'Y'),
							 array('=', 'bf_plugins', 'bf_targetmodule', '')
							 ));
			$qb->orderBy('ASC', 'bf_plugins', 'bf_order');
			$res = $qb->getResult();
	
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
			$this->plugins_global = array();
			$this->plugins =	array();
			while ($row = $res->fetchRow()) {
				if (DB::isError($row)) throw new BFDBException($row->getMessage());
				$this->plugins_global[$row->bf_name] = array('class' => $row->bf_classname, 'title' => $row->bf_title, 'type' => $row->bf_type);
				if (!array_key_exists($row->bf_type, $this->plugins)) $this->plugins[$row->bf_type] = array();
				$this->plugins[$row->bf_type][$row->bf_name] = array('class' => $row->bf_classname, 'title' => $row->bf_title);
			}
			$this->delPCache('plugins');
			$this->delPCache('plugins_global');
			$this->setPCache('plugins', $this->plugins);
			$this->setPCache('plugins_global', $this->plugins_global);
		}
	}

	/**
	 * Run throught the list of headers and send them all in the order set. Of course, when sent to
	 * the browser, they arrive in reverse order, so we keep things consistent...
	 */
	protected function sendHeaders() {
		if (is_array($this->headers)) {
			foreach ($this->headers as $header) header($header);
		}
	}

	/**
	 * (*deep breath*)
	 *
	 * Okay: after getting the global jTE object, settings array and BFUser object:
		1. Ask the active module to render itself and assign the module in the template object
		2. If an XHTML request, assign all sorts of things that the global templates need. This
			ranges from the global navigation from the module, to the global navigation list, to
			the list of includes (scripts, stylesheets, etc.) from the module, to the BFUser object.
		3. Pass the resulting template object to {@link runRenderFilters()} for "post processing"
	 */
	public function render() {
		global $bfjte, $GLOBAL_SETTINGS, $user, $_REQUEST;
		$bfjte->assign('product',  $this->product);
		if (isset($this->active_module)) {
			$this->active_module->render();
			$bfjte->assign('module',  $this->active_module);
		}
		if ($this->request_type == BF_REQUEST_XHTML && $this->active_module) {
			if (!in_array('ViewableXHTML', class_implements($this->active_module))) {
				$this->errorPage(new BFException(BF_LANG_ERR_UNVIEWABLE_MODULE));
				return false;
			}

			$bfjte->assign('nav_tree',			$this->active_module->getNavigation());

			// Begin: Styx Hack -- either old way "styx styx" in .conf or new way, the license key == BF_STYX_LICENSE_KEY
			if ($this->configuration->styx() || $this->getSysconfig('license_key') == BF_STYX_LICENSE_KEY) {
				$bfjte->assign('is_styx', true);
				$styxModules = array('projects', 'library', 'jobs', 'environments', 'servers', 'help');
				foreach($this->navigation_modules as $k=>$v) {
					if (in_array($k, $styxModules)) {
						$styx_navigation_modules[$k] = $v;
					}
				}
				$this->navigation_modules = $styx_navigation_modules;
			}
			// End: Styx Hack

			$bfjte->assign('global_navigation',	$this->navigation_modules);
			$bfjte->assign('bfversion',			BF_CONSOLE_VERSION);

			// The request can specify to ONLY retrieve the navigation for an XHTML request
			if (array_key_exists(BF_REQUEST_XHTML_NAV, $_REQUEST)) {
				// Special template
				$this->template = 'xml_global_navigation.tpl';
				// XML Header
				$this->request_type = BF_REQUEST_XML;
			} else {
				$bfjte->assign('includes',		   $this->active_module->getIncludes());
				// System wide message
				$msg = $this->getSysconfig('user_message');
				if (!empty($msg)) {
					$this->addMessage($msg);
				}
			}
			
		}  else if ($this->request_type == BF_REQUEST_XML && $this->active_module && 
				array_key_exists(BF_REQUEST_XML_NAV, $_REQUEST)) {
			$this->active_module->content = $this->active_module->nav_content;
		} else if ($this->request_type == BF_REQUEST_RSS2 && $this->active_module) {
			// Look for If-Modified-Since and return a 304 if possible
			if ($last_build_date = $this->active_module->getLastBuildDate()) {
				if ($if_modified_since = $this->getRequestHeader('If-Modified-Since')
						&& isset($if_modified_since)
						&& strtotime($last_build_date) < strtotime($if_modified_since)) {
					$this->addHeader('HTTP/1.1 304 Not Modified');
					exit();
				}
			}
		}

		// Assign errors
		if (is_array($this->errors) && count($this->errors) > 0) {
			$bfjte->assign('errors',  $this->errors);
		}

		// Check if the engine is paused - and if it is tell everyone
		$engineStatus = $this->getSysconfig('engine_pause');
		if (strpos(strtolower($engineStatus), 'y') !== false && $this->request_type == BF_REQUEST_XHTML) {
			$this->addMessage(translate('2160'));
		}

		// Assign messages
		if (is_array($this->messages) && count($this->messages) > 0) {
			$bfjte->assign('messages',  $this->messages);
		}

		// Run through the renderfilter plugins
		$this->runRenderFilters($bfjte);
	}

	/**
	 * Render filters work in the opposite way that request filters do, in that the render filters
	 * run <i>after</i> the module renders itself initially. This allows for all sorts of cool
	 * plugins to get created that I haven't thought of yet.
	 */
	protected function runRenderFilters($jte) {
		if (!array_key_exists('renderfilter', $this->plugins)) return true;

		if (array_key_exists('renderfilter', $this->plugins)) {
			foreach ($this->plugins['renderfilter'] as $plugin) {
				$this->require_module($plugin['class']);
				$temp_plug = new $plugin['class']();
				if (!in_array('BFRenderFilter', class_implements($temp_plug))) throw new BFException();
				else $temp_plug->render($GLOBALS['bfjte']);
			}
		}
	}

	/**
	 * Actually send the headers and call display on the jTE object, sending info to the browser.
	 */
	public function display() {
		// If you don't know what to make of it, use the raw header
		if (array_key_exists($this->request_type, $GLOBALS['GLOBAL_HEADERS'])) {
			$this->addHeader($GLOBALS['GLOBAL_HEADERS'][$this->request_type]);
		} else {
			$this->addHeader($GLOBALS['GLOBAL_HEADERS'][BF_REQUEST_RAW]);
		}

		$this->sendHeaders();
		$GLOBALS['bfjte']->display($this->template);
		
	}

	/**
	 * Since it takes an Exception as a parameter, the caller doesn''t have to worry about logging
	 * the error (or knowing how to log it in the first place), and then calls for an error page.
	 */
	public function exitWithError(Exception $e) {
		if (!($e instanceof BFDBException)) {
			BFExceptionHandler::logError($e, $this->getDB());
		}
		if ($this instanceof CentralController) {
			$this->errorPage($e);
		} else {
			BFExceptionHandler::printDumbErrorPage($e->toString());
		}
	}

	/**
	 * First, try to use jTE to send out the error page (so we can still hopefully get the
	 * navigation, etc.), but resort to printing out a flat, useless error page. Regardless, since
	 * this came from something crapping itself, exit() out to stop everything short.
	 */
	public function errorPage($e) {	
		if (!$this->printTemplatedErrorPage($e->getMessage())) {
			// If you hit this, someone really screwed up and you'll need the full dump.
			BFExceptionHandler::printDumbErrorPage($e->toString());
		}
	}

	/**
	 * Using the jTE object, assign just enough to get the global navigation and site settings so
	 * the user can easily navigate away from the error page.
	 */
	protected function printTemplatedErrorPage($msg) {
		global $bfjte, $GLOBAL_SETTINGS;
		if (class_exists('BFJTE') && $bfjte instanceof BFJTE) {
			if (is_array($this->modules_viewable)) {
				$bfjte->assign('global_navigation', $this->modules_viewable);
			}
			$bfjte->assign('site', $GLOBAL_SETTINGS['site']);
			$bfjte->assign('content', $msg);
			$error_tpl = $this->request_type . '.' . $GLOBAL_SETTINGS['site']['errortemplate'];
			// If you don't know what to make of it, use the raw header
			if (array_key_exists($this->request_type, $GLOBALS['GLOBAL_HEADERS'])) {
				$this->addHeader($GLOBALS['GLOBAL_HEADERS'][$this->request_type]);
			} else {
				$this->addHeader($GLOBALS['GLOBAL_HEADERS'][BF_REQUEST_RAW]);
			}
			$this->sendHeaders();
			$bfjte->display($error_tpl);
			exit();
		}

		return false;
	}
	
	public function logAudit($message, $params) {
		$this->logMessage($message, $params, 'A');
	}
	
	public function logError($message, $params) {
		$this->logMessage($message, $params, 'E');
	}
	
	public function logInfo($message, $params) {
		$this->logMessage($message, $params, 'I');
	}
	
	public function logWarning($message, $params) {
		$this->logMessage($message, $params, 'W');
	}
	
	/**
	 * logMessage
	 * 
	 * The $message parameter actually means "message key" since the strings all get held
	 * in another database table.
	 * 
	 * The $params parameter...this needs to get passed as an array of strings to get inserted
	 * into the message when it gets pulled out and displayed.
	 */
	public function logMessage($message, $params, $severity) {
		global $user;
		
		// Get the UID if possible
		$uid = isset($user) ? $user->get('bf_userid') : 0;
		if (is_null($uid)) {
			$uid = 0;
		}
		
		// Construct the bf_store out of the parameters. Don't blame me for this, the
		// other guys thought up this magical storage technique all on their own...
		$this->require_module('BFStore');
		$store = new BFStore();
		foreach ($params as $i => $param) {
			// Hooray for special characters.
			if (isset($param[0]) && $param[0] == '!')  {
				$param = substr($param, 1);
				$prefix = '!';
			}  else {
				$prefix = '';
			}
			$hextext = unpack("H*hex", $param);
			$params[$i] = $hextext['hex'];
			// Random NULLs at the end = bad
			$len = strlen($params[$i]);
			if ($len > 2* strlen($param) && $params[$i]{$len - 1} == '0') {
				$params[$i] = substr($params[$i], 0, -1);
			}
			$params[$i] = $prefix . $params[$i];
		}
		$stored_params = implode(',', $params);
		$key = $store->setStorage($stored_params);
		
		$qb = BFQueryBuilder::insert($this->db);
		$qb->setColumn('bf_stamp', 'bf_messages', time());
		$qb->setColumn('bf_userid', 'bf_messages', $uid);
		$qb->setColumn('bf_pid', 'bf_messages', 0);
		$qb->setColumn('bf_severity', 'bf_messages', $severity);
		$qb->setColumn('bf_type', 'bf_messages', 'UserInterface');
		$qb->setColumn('bf_msgkey', 'bf_messages', $message);
		$qb->setColumn('bf_msgargs', 'bf_messages', $key);
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}
	}
	
	/**
	 * Checks various aspects of the setup and pukes appropriately
	 */
	protected function checkEnvironment() {
		if (get_magic_quotes_gpc()) {
			BFExceptionHandler::printDumbErrorPage(translate(2270));
		}
	}

	protected function checkVersion() {
		global $GLOBAL_VERSION;
		// Don't bother checking the source, we don't care. You should know your own database.
		if ($GLOBAL_VERSION['CURRENT'] == 0) {
			return true;
		}
		$schema_complete = $this->getSysconfig('schema_complete');
		if ($schema_complete != 'Y') {
			BFExceptionHandler::printDumbErrorPage(translate('2157'));
		}
		$database_version = $this->getSysconfig('schema_version');
		$codebase_version = $GLOBAL_VERSION['CURRENT'];
		if ($codebase_version != $database_version) {
			BFExceptionHandler::printDumbErrorPage(translate('2161', $codebase_version, $database_version));
		}
	}

	/**
	 * Check for a license error and spit it out at the user if one exists
	 */
	protected function checkLicense() {
		$license_error = $this->getSysconfig('license_error');
		if ($license_error != '' && $this->request_type == BF_REQUEST_XHTML) {
			$this->license_error = translate('license error', $license_error);
			$this->addError($this->license_error);
		} else {
			$this->license_error = false;
		}
	}

	/** Set/Get Cache Methods
	 * This is my poor mans cache system that basically stuffs information into memory.. 
	 * It can be used for anything as the key is MD5'd.. if you have anything that you want to be
	 * quickly cached for this page build, stuff it here.. Note; GetRow/GetCol/GetOne already uses this.
	 */
	public function setCache($key, $value) {
		$md5Key = md5($key);
		$GLOBALS['PoorMansCache'][$md5Key] = array( 'query' => $key, 'result' => $value);
		return true;
	}
	public function getCache($key) {
		$md5Key = md5($key);
		if (isset($GLOBALS['PoorMansCache'][$md5Key])) {
			return $GLOBALS['PoorMansCache'][$md5Key]['result'];
		}
		return false;
	}
	public function delCache($key) {
		$md5Key = md5($key);
		unset($GLOBALS['PoorMansCache'][$md5Key]);
		return true;
	}
	public function clearCache() {
		$GLOBALS['PoorMansCache'] = array();
		return true;
	}
	public function setPCache($key, $value) {
		global $user;
$key .= '_7.02.02';
		if ($this->shmoppable) {
			$crc32Key = crc32($key);
			$serialized = serialize($value);
			$size = strlen($serialized) + 10;
			if (($source = @shmop_open($crc32Key, 'w', 0, 0)) || ($source = @shmop_open($crc32Key, 'n', 0600, $size))) {
				$now = time();
				shmop_write($source, str_pad($now, 10, '0', STR_PAD_LEFT), 0);
				$bytes = shmop_write($source, $serialized, 10);
				shmop_close($source);
				@shmop_close($crc32Key);
				return true;
			} else {
				@shmop_close($crc32Key);
				if ($user && $user->getPreference('debug') > 0) {
					$this->addMessage('Failed to write pcache "' . $key . '"');
				}
			}
		}
		return false;
	}
	public function getPCache($key, $death = null) {
		global $GLOBAL_CACHE_AGE, $user;
$key .= '_7.02.02';
		if ($this->shmoppable) {
			$crc32Key = crc32($key);
			$min_age = time() - (is_null($death) ? $death : $GLOBAL_CACHE_AGE);
			if ($source = @shmop_open($crc32Key, 'a', 0, 0)) {
				$age = shmop_read($source, 0, 10);
				$res = false;
				// If the data got created 
				if ($age !== 0 && $age < $min_age) {
					$size = shmop_size($source) - 10;
					$res = @unserialize(shmop_read($source, 10, $size));
				}
				@shmop_close($source);
				@shmop_close($crc32Key);
				return $res;
			}
		}
		return false;
	}
	public function delPCache($key) {
		global $user;
$key .= '_7.02.02';
		if ($this->shmoppable) {
			$crc32Key = crc32($key);
			if ($source = @shmop_open($crc32Key, 'w', 0, 0)) {
				if (!shmop_delete($source)) {
					$this->addMessage('Failed to delete pcache "' . $key . '"');
				}
				@shmop_close($source);
			}
			@shmop_close($crc32Key);
		}
		return true;
	}

	/** setFileCache
	 * In the event that Shmop is unshmopperific we can use a file cache instead
	 * You will only get an "error" message if debug is turned on - otherwise it will fail 
	 * silently and the UI will just be slower for some reason.. 
	 */
	public function setFileCache($key, $value) {
		global $user;
		if (@file_put_contents($this->fileCachePrefix.md5($key), serialize($value), LOCK_EX) < 1) {
			if ($user && $user->getPreference('debug') > 0) {
				$this->addMessage('Unable to set file cache "'. $key .'"');
			}
		}
	}

	/** getFileCache
	 * Get file cache works very much like getPCache but to the file system.
	 * The only major difference is that age is <= to min age instead of < cause file transactions can be
     *   slow and its possible that the cache you're testing against happened the same second as the test.
	 * Again we only error out if in debug mode 
	 */
	public function getFileCache($key, $death = null) {
		global $GLOBAL_CACHE_AGE, $user;
		$results = false;
		if (file_exists($this->fileCachePrefix.md5($key))) {
			$age = filemtime($this->fileCachePrefix.md5($key));
			$min_age = time() . (is_null($death) ? $death : $GLOBAL_CACHE_AGE);
			if ($age !== 0 && $age <= $min_age) {
				if ($source = file_get_contents($this->fileCachePrefix.md5($key))) {
					$results = @unserialize($source);
				} else {
					if ($user && $user->getPreference('debug') > 0) {
						$this->addMessage('Unable to get file cache "'. $key .'"');
					}
				}
			}
		}
		return $results;		
	}

	

	/**
	 * TODO: stop the two-step construct...
	 */
	public function init() {
		$this->loadConfig();
		$this->loadDBConnection();
		// We'll need this in order to do pretty much anything other than display a panic page.
		$this->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
		$this->loadSysConfig();
		$this->loadAvailableModules();
		$this->loadGlobalPlugins();
		
		$this->loadMasterConfig();
		$this->loadMasterDBConnection();
		
		$this->checkEnvironment();
		$this->checkVersion();
		if (isset($GLOBALS['UPDATED_COLUMN_CACHE'])) {
			bf_update_strings();
		}
	}

	/**
	 * Just in case the database extension doesn't actually close the connection
	 * when the request completes and PHP exits
	 */
	public function cleanup() {
		// Close the default connection
		$this->db->close();
		// Close the master database connection
		if ($this->master_db) {
			$this->master_db->close();
		}
		// Close any connections open in the connection pool
		foreach ($this->dbpool as $id => $db) {
			$this->dbpool[$id]->close();
		}
	}

	/**
	 * The constructor for the CentralController object loads in the configuration, database and
	 * everything it needs to manage itself and the modules under it.
	 */
	function __construct() {
		// shmop?
		$this->shmoppable = extension_loaded('shmop');
		// File/Key caching
		$this->fileCachePrefix = ini_get('upload_tmp_dir') . DIRECTORY_SEPARATOR . 'key_cache_';
	}
}

/**
 * BFPermissionException
 *
 * When you need to let a calling function know that they can''t do that,
 * throw it a new BFPermissionException
 * @package Console
 * @subpackage Core
 */
class BFPermissionException extends BFException {
	public function __construct($attept, $code = 0) {
		parent::__construct(BF_LANG_ERR_PERMISSION . ' ' . $attept, $code);
	}
}

/**
 * BFLicenseException
 *
 * When you need to let a calling function know that their license
 * doesn''t let them do that, throw it a new BFLicenseException
 * @package Console
 * @subpackage Core
 */
class BFLicenseException extends BFPermissionException {
	public function __construct($attept, $code = 0) {
		parent::__construct(BF_LANG_ERR_LICENSE . ' ' . $attept, $code);
	}
}

?>
