<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Configuration
 * @package Console
 * @subpackage Core
 */

class BFConfigurationException extends BFDBException { }

/**
 * Configuration
 * An object to handle storage and retrieval of options held in buildforge.conf
 * files. As an added bonus, it can return a constructed DSN string and also
 * supports layering configuration options. In other words, you could (with the
 * same instantiation of the object) load in additional conf files. These
 * would take priority in reverse order of loading.
 * Later on down the road this may prove useful to subclass in order to support
 * different configuration formats.
 * @package Console
 * @subpackage Core
 */
class Configuration {
	protected $options;
	protected $dsn;
	protected $type;
	public $master = false;
	public $local = false;

	/**
	 * If it has everything it needs to make a full DSN, it makes one with the info out of the
	 * configuration file. If not, it makes one with the system defaults.
	 */
	protected function reconstructDSN() {
		global $GLOBAL_DATABASE, $GLOBAL_DB_CHARSET;
		if (is_array($this->options) && array_key_exists('db_type', $this->options)
				&& ($this->options['db_type'] == 'local' || array_key_exists('db_provider', $this->options))
				&& ($this->options['db_type'] == 'local' || array_key_exists('db_hostname', $this->options))
				&& array_key_exists('db_username', $this->options)
				&& array_key_exists('db_password', $this->options)
				&& array_key_exists('db_database', $this->options)) {
			$provider = null;
			$hostname = null;
			$this->options['db_type'] = strtolower($this->options['db_type']);

			if ($this->options['db_type'] == 'local') {
				$this->type = $provider = 'db2';
				$hostname = 'localhost';
			} else {
				$this->options['db_provider'] = strtolower($this->options['db_provider']);
				$hostname = $this->options['db_hostname'];
				switch ($this->options['db_provider']) {
					case 'local':
						$this->type = $this->options['db_provider'] = 'db2';
						break;
					case 'mysql':
						$this->type = $this->options['db_provider'] = 'mysqli';
						break;
					case 'pgsql':
					case 'postgres':
						$this->type = $this->options['db_provider'] = 'pgsql';
						break;
					case 'sybase':
					case 'sqlserver':
						$this->type = $this->options['db_provider'];
						$this->options['db_provider'] = 'odbc';
						break;
					case 'oracle':
						$this->type = $this->options['db_provider'] = 'oci8';
						break;
					default:
						$this->type = $this->options['db_provider'];
						break;
				}
			}
			$this->dsn = array(
				'php_type' => $this->type,
				'provider' => $this->options['db_provider'],
				'username' => $this->options['db_username'],
				'password' => $this->options['db_password'],
				'hostname' => $this->options['db_hostname'],
				'tcp_port' => (isset($this->options['db_tcp_port']) ? $this->options['db_tcp_port'] : null),
				'database' => $this->options['db_database']
			);
		} else {
			$this->dsn = array(
				'php_type' => $this->type,
				'provider' => $GLOBAL_DATABASE['provider'],
				'username' => $GLOBAL_DATABASE['username'],
				'password' => $GLOBAL_DATABASE['password'],
				'hostname' => $GLOBAL_DATABASE['hostname'],
				'tcp_port' => $GLOBAL_DATABASE['tcp_port'],
				'database' => $GLOBAL_DATABASE['database']
			);
			$this->type = 'db2';
		}
	}

	public function getName() {
		if (!isset($this->dsn)) {
			$this->reconstructDSN();
		}
		return (isset($this->options['location_name']) ? $this->options['location_name'] : false);
	}
		

	public function getDSN() {
		if (!isset($this->dsn)) {
			$this->reconstructDSN();
		}
		return $this->dsn;
	}

	public function getDBType() {
		if (!isset($this->dsn)) {
			$this->reconstructDSN();
		}
		return $this->type;
	}

	/**
	 * If the configuration file exists, looks like a file and PHP has permission to read it,
	 * try to parse it. Otherwise, puke meaningfully.
	 */
	public function load($path) {
		$msg = null;

		if (file_exists($path)) {
			if (is_file($path)) {
				if (is_readable($path)) {
					return $this->parseFile($path);
				} else $msg = BF_LANG_ERR_CONF_READABLE . ' \'' . $path . '\'';
			} else $msg = BF_LANG_ERR_CONF_NOTFILE . ' \'' . $path . '\'';
		} else $msg = BF_LANG_ERR_CONF_MISSING . ' \'' . $path . '\'';

		throw new BFConfigurationException($msg);
	}

	/**
	 * Rather than use a file format already made for things like this (INI, XML, etc.), we've
	 * used OMA format.
	 *
	 * After parsing a file, {@link reconstructDSN()} for use in the system if needed.
	 */
	protected function parseFile($path) {
		if (!is_array($this->options)) $this->options = array();
		$optsfile = fopen($path, 'r');

		// This currently does no validation, so make your conf files right.
		while (!feof($optsfile)) {
			$matches = array();
			if (preg_match_all(BF_REGEXP_MATCH_CONF, fgets($optsfile), $matches)) {
				// This goes through and will overwrite exiting entries
				// So...you could technically load in a secondary conf file,
				// overriding the entries in the first
				$optName = $matches[1][0];
				$optValue = rtrim($matches[2][0]);
				$this->options[$optName] = $optValue;
			}
		}

		fclose($optsfile);
		$this->reconstructDSN();
		if (isset($this->options['gdd'])) {
			if ($this->options['gdd'] == 'primary') {
				$this->master = true;
			} else if ($this->options['gdd'] == 'secondary') {
				$this->local = true;
			}
		}

	}

	// Begin: Styx Hack -- optional method of enforcing the styx UI is to add "styx styx" to the buildforge.conf
	public function styx() {
		return (isset($this->options['styx']));
	}	
	// End: Styx Hack

	/**
	 * Pass the path to the file to use. If for any reason, the object fails to
	 * make a set of options after looking for and hopefully parsing the file
	 * in question, return false.
	 */
	function __construct($path) {
		$this->load($path);
	}
}

?>
