<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFDBO
 * The parent database object for the web app, providing simple CRUDing to
 * various objects in the database. Most objects extend this, but theoretically
 * they could also use it to create dynamically defined database objects:
 * <code>
 * $mydbo = new BFDBO();
 * $mydbo->table = 'some_new_table';
 * $mydbo->primary_key = 'its_pk';
 * $mydbo->select($mydbo->primary_key, 1);
 * </code>
 * When extending the BFDBO, all objects MUST define a table and primary_key
 * in order for things to work. These variables get defined publically in order
 * for other objects to use them effectively.
 * @package Console
 * @subpackage Core
 */

$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

class BFDBO {
	/**
	 * @var string The identifier for the database table.
	 */
	public $table;
	/**
	* @var string The identifier for the primary key.
	 */
	public $primary_key;
	/**
	* @var string The column cache version of what the primary key is.
	 */
	public $primary_key_type;
	/**
	 * @var string The primary key override tells what type to assume the primary key is.
	 * valid types = 'generic','env','dn','message','vars','command','body','data','group','interface','text','step'	 
	 */
	public $primary_key_type_override;	 	

	/**
	 * @var string The identifier for the object descriptor.  May or may not be the same as primary_key.
	 */
	public $descriptor_key;

	/**
	 * @var string The name of the object type.
	 */
	public $type_name;

	/**
	 * @var boolean Set to false to turn off audit logging on this object.
	 */
	public $log_changes = true;

	/**
	 * @var array An array of the field types to use when deciding how to handle fields.
	 */
	public $field_types = array();
	
	/**
	 * @var array An array of field descriptions to be used when logging field changes.
	 */
	public $field_descriptions = array();

	/**
	 * @var object A reference to the global controller object.
	 */
	protected $querybuilder;
	/**
	 * @var array An associative array using the field identifiers as keys and values as values.
	 */
	protected $fields;
	/**
	 * @var array A quick array defining the available fields of the database table.
	 * @access private
	 */
	protected $tablefields;
	/**
	 * @var array A simple array to indicate which fields have changed
	 */
	protected $updated;
	/**
	 * @var boolean A simple flag to indicate whether or not the database object exists in the database.
	 */
	protected $inserted = false;
	/**
	 * @var array When overriding BFDBO, include any fields here that (when empty) throw an error
	 */
	protected $required_fields = array();
	/**
	 * @var array When overriding BFDBO, include any default values here: array('field' => value)
	 */
	protected $field_defaults = array();
	/**
	 * @var array When overriding BFDBO, include any fields here that get stored in bf_store
	 */
	protected $bfstores = array();
	/**
	 * @var array When overriding BFDBO, include any fields here that should remember their last values
	 */
	public $stored_values = array();
	/**
	 * @var object A reference to the global controller object.
	 */
	protected $controller;

	/**
	 * Yay for faked (well, totally hacked) multiple, dynamic inheritance!
	 * Unfortunately, since we need determine which class to "extend" at
	 * runtime, we can't actually really extend it. So we make the current
	 * object into more of less of a proxy object.
	 *
	 * @param string $method The method something called that BFDBO doesn't define.
	 * @param array $args An array of arguments passed to the undefined method.
	 * @access private
	 */
	public function __call($method, $args) {
		// First, try it on the current (and parent) object
		if (method_exists($this, $method)) {
			return call_user_func_array(array($this, $method), $args);
		// Next, try it on the BFQueryBuilder instance
		} else if (method_exists($this->querybuilder, $method)) {
			return call_user_func_array(array($this->querybuilder, $method), $args);
		// Since we like consistency, panic if you can't find anything
		} else {
			trigger_error('Call to undefined method ' . get_class($this) . '::' . $method . '()');
		}
	}

	public function get($field) {
		if (is_array($this->fields) && array_key_exists($field, $this->fields)) {
			// Check for bfstores...they load dynamically since they take up loads of memory
			if (array_key_exists($field, $this->bfstores)) {
				// Only load if you haven't already and we have something to load
				if (!empty($this->fields[$field]) && !is_array($this->bfstores[$field])) {
					$this->bfstores[$field] = $this->bfstore->getStorage($this->fields[$field]);
				}
				return $this->bfstores[$field]['TEXT'];
			} else {
				return $this->fields[$field];
			}
		}

		return false;
	}

	public function set($field, $value) {
		if (!is_array($this->fields)) {
			$this->loadFields();
		}
		if (array_key_exists($field, $this->fields)) {
			// Check for bfstores...they load dynamically since they take up loads of memory
			if (array_key_exists($field, $this->bfstores)) {
				// Only use the current copy if one exists
				if (!empty($this->fields[$field])) {
					// Next, we'll need to go get the current version (if possible)
					if (!isset($this->bfstores[$field])) {
						$this->bfstores[$field] = $this->bfstore->getStorage($this->fields[$field]);
					}
					// Only edit if different
					if (strcmp($this->bfstores[$field]['TEXT'], $value) !== 0) {
						$this->logChangedFields($field, $this->bfstores[$field]['TEXT'], $value);
						$this->bfstores[$field]['TEXT'] = $value;
						$this->updated[$field]  = true;
					}
				// If no entry exists for the store, create a new one
				} else {
					$this->logChangedFields($field, '', $value);
					$this->bfstores[$field] = array(
													'UID' => null,
													'PART' => null,
													'TEXT' => $value,
													'NUM' => null
													);
					$this->updated[$field]  = true;

				}
			// Seperate these tests out since only one returns false
			} else if ((is_int($this->fields[$field]) && $this->fields[$field] != $value) || (!is_int($this->fields[$field]) && $this->fields[$field] !== $value)) {
				$this->logChangedFields($field, $this->fields[$field], $value);
				$this->fields[$field]   = $value;
				$this->updated[$field]  = true;
			}

			// No error? Return successful.
			return true;
		}

		// Only return false when something bad happens
		return false;
	}

	/**
	 * Stupid method to get the inserted status of a database object
	 */
	public function inserted() {
		return $this->inserted;
	}

	/**
	 * I've realized that I want to get REALLY lazy and
	 * simply hand off an associative array for the database
	 * object to sort out for itself.
	 */
	public function setMany($values) {
		if (!is_array($this->fields)) {
			$this->loadFields();
		}

		if (is_array($values)) {
			foreach ($values as $var => $val) {
				// BFDBO::set() takes care of validation
				$this->set($var, $val);
			}
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Either use two arrays or two non-array variables
	 * The arrays must have the same length.
	 **/
	public function select($fields, $values, $comp = '=') {
		if (is_array($fields) && is_array($values) && count($fields) === count($values)) {
			return $this->selectByFields($fields, $values, $comp);
		} else if (!is_array($fields) && !is_array($values)) {
			return $this->selectByField($fields, $values, $comp);
		}

		return false;
	}

	/**
	 * Selects all columns from the table as specified in the WHERE clause, LIMIT 1, since we
	 * don''t need any more. The comparison operator selection doesn't really scale at the moment...
	 **/
	public function selectByFields($fields, $values, $comp = '=') {
		$c = count($fields);
		for ($i = 0; $i < $c; $i++) {
			$this->querybuilder->where(array($comp, $this->table, $fields[$i], $values[$i]));
		}
		return $this->fetch();
	}

	/**
	 * Fetch the database object's record from the BFQueryBuilder result
	 *
	 * At this point, you will not have the ability to select based on
	 * BFStore contents, because it holds hex-encoded, split-up, large
	 * quantities of text that you should never have to use for selecting.
	 *
	 * @access private
	 */
	public function fetch() {
		if (!is_array($this->fields)) {
			$this->loadFields();
		}

		$res = $this->querybuilder->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else if ($cols = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$this->updated  = array();  // Keeps track of which fields get modified
			$this->inserted = true;
			foreach ($cols as $col => $val) {
				$this->fields[$col]	 = $val;
				$this->updated[$col]	= false;
				// Keep a stored copy where specified
				if (array_key_exists($col, $this->stored_values)) {
					$this->stored_values[$col] = $val;
				}
			}
			return true;
		}

		// If you made it here, something went wrong...
		return false;
	}

	/**
	 * Just a little alias for selectByFields, instead of
	 * duplicating functionality. Might change later, hance
	 * the new function, rather than an inline forwarding.
	 *
	 * @access private
	 **/
	public function selectByField($field, $value, $comp) {
		return $this->selectByFields(array($field), array($value), $comp);
	}

	/**
	 * search
	 * This goes a little against the normal use of a BFDBO,
	 * in that it returns an array of objects. Definitely only
	 * for use in light-weight searching! If you need a bigger
	 * result set (or results with heavier objects), use the
	 * BFReport and BFReportResults objects.
	 *
	 * I''d make this a static method, but it needs to use the
	 * table and primary key of the object extending this. And
	 * since those can''t exist statically...
	 */
	public function search($fields, $values, $comp = '=') {
		if (is_array($fields) && is_array($values) && count($fields) === count($values)) {
			return $this->searchByFields($fields, $values, $comp);
		} else if (!is_array($fields) && !is_array($values)) {
			return $this->searchByField($fields, $values, $comp);
		}

		return false;
	}

	public function searchByField($field, $value, $comp = '=') {
		return $this->searchByFields(array($field), array($value), $comp);
	}

	public function searchByFields($fields, $values, $comp = '=') {
		if (!is_array($this->fields)) {
			$this->loadFields();
		}

		$results = array();

		$qb = BFQueryBuilder::select();
		foreach ($this->fields as $field => $value) {
			if (is_string($field)) {
				$qb->addColumn($field, $this->table, $field);
			}
		}

		$c = count($fields);
		for ($i = 0; $i < $c; $i++) {
			$qb->where(array('=', $this->table, $fields[$i], $values[$i]));
		}

		if ($res = $qb->getResult($qb)) {
			$classname = get_class($this);
			while ($cols = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				if (DB::isError($cols)) {
					BFExceptionHandler::logError(new BFDBException($cols), $GLOBALS['controller']->getDB());
				} else {
					$results[$c] = new $classname();
					$results[$c]->setMany($cols);
					$results[$c++]->inserted = true;
				}
			}
		}

		// Even if you get here without a results set, return the empty results
		return $results;
	}

	/**
	 * Only you can prevent table fires.
	 *
	 * Wait...no...THIS prevents table fires.
	 *
	 * @return boolean
	 * @access private
	 */
	public function hasRequiredFields() {	
		foreach ($this->required_fields as $field) {
			if (array_key_exists($field, $this->fields)) {
				if (array_key_exists($field, $this->bfstores)) {
					// bf_store-ed values only get considered empty when
					// neither the BFUID reference nor the value exist yet.
					if ($this->fields[$field] == '' && $this->bfstores[$field] == '') {
						return false;
					}
				} else {
					if ($this->fields[$field] == '') {
						return false;
					}
				}
			}
		}

		return true;
	}

	/**
	 * Alias for insert or update, depending
	 * 
	 * @return boolean
	 **/
	public function save() {
		if ($this->inserted) {
			return $this->update();
		} else {
			return $this->insert();
		}
	}

	/**
	 * insert
	 *
	 * If the object doesn't have a primary key defined, or it can't generate a new
	 * numeric primary key, just give up. Otherwise, create and attempt to execute
	 * an INSERT query.
	 *
	 * @return boolean
	 * @access private
	 */
	public function insert() {
		if (!$this->generatePrimaryKey()) {
			return false;
		}
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object			
			throw new BFException(translate('2181'));
		}

		$qb = BFQueryBuilder::insert();
			
		foreach ($this->tablefields as $field) {
			// If a bfstored value, need to save that first to get the ID
			if (array_key_exists($field, $this->bfstores)) {
				$this->fields[$field] = $this->bfstore->setStorage($this->bfstores[$field]['TEXT']);
			}
			$qb->setColumn($field, $this->table, $this->fields[$field]);
		}
		$res = $qb->getResult();

		if (DB::isError($res)) {
			$this->controller->addError($res->getMessage());
			return false;
		} else {
			$this->inserted = true;
			$this->logChange('Created');
			return true;
		}
	}

	/**
	 * saveAsNew
	 * 
	 * Copies the object by saving it with a new primary key.
	 */
	public function saveAsNew() {
		if ($this->generatePrimaryKey(true)) {
			$this->inserted = false;
			return $this->save();
		}
	}

	/**
	 * copy
	 * 
	 * Attempts to copy the current object.  Calls generateNewDescriptor to call it by a new name.
	 * Then it generates a new primary key (if necessary) and saves it as a new object.
	 */
	public function copy($newdescriptor = true)  {
		if ($newdescriptor)  {
			$this->generateDescriptorKey();
		}
		if ($this->updated[$this->primary_key] || $this->generatePrimaryKey(true)) {
			//  Lets copy all the bf_store _contents_ instead of the ids.
			foreach ($this->bfstores as $field => $storage) {
				$this->bfstores[$field] = $this->bfstore->getStorage($this->fields[$field]);
			}
			
			$this->inserted = false;
			return $this->save();
		}
		return false;
	}
	
	
	/**
	 * generatePrimaryKey
	 *
	 * If the object already has a primary key set, simply return true. Otherwise,
	 * as long as the column type = int, get the next available ID and set the
	 * object's primary key to it.
	 * 
	 * if the primary_key_override = 'uid' then generate one of those - type = string	 	 
	 *
	 * @param boolean $force When true, attempt to generate a new key even when already set
	 * @return boolean
	 */
	public function generatePrimaryKey($force = false) {	
		// Define the various valid key types
		global $GLOBAL_DATABASE;
			
		if (is_array($this->fields) && isset($this->fields[$this->primary_key]) && 
				!empty($this->fields[$this->primary_key]) && !$force) {
			return true;
		} else if ($this->primary_key_type == 'int' || strtolower($this->primary_key_type) == 'number') {
			$qb = BFQueryBuilder::select();
			$qb->addColumn($this->primary_key, $this->table, $this->primary_key, 'MAX');
			$key = $qb->getOne(false);

			if (DB::isError($key)) {

				throw new BFDBException($key->getMessage());
			} else {
				$this->fields[$this->primary_key] = intval($key) + 1;
				return true;
			}
			
		// Primary Key is a UID as defined by its type_override so generate a uid using that type
		} else if (isset($this->primary_key_type_override) && in_array($this->primary_key_type_override, $GLOBAL_DATABASE['valid_types'])) {
			$s = explode(" ", microtime());	$m = explode(".", $s[0]);
			$uidLock = (substr($s[1], 4, 4).substr($m[1], 0, 4));
			
			// Now define the lockObject either nicely or through brute force.			
			$lockObject = $this->setUIDLock($this->primary_key_type_override, $uidLock);			
			if (!is_object($lockObject)) {
				$lockObject = $this->forceUIDLock($this->primary_key_type_override, $uidLock);
				if (!is_object($lockObject)) {
					throw new BFException(translate('2186'));
					die;
				}
			}
			$this->resetUIDLock($this->primary_key_type_override, $uidLock);
	
			// Now after all that mess, set the UID
			$this->fields[$this->primary_key] = $lockObject->bf_stamp.$this->primary_key_type_override.$lockObject->bf_num;			
			return true;
		}

		return false;
	}


	/**
	 * generateDescriptorKey
	 *
	 * For (usually) the descriptor field, tacks ' COPY' onto it, then searches for all objects with that field.  
	 * If any exist, it adds a number on the end ('COPY 2', 'COPY 3', etc.).  Then it sets the field to that 
	 * value.
	 */	
	public function generateDescriptorKey($field = null)  {
		if (!$field || !isset($this->fields[$field]))  {
			$field = $this->descriptor_key;
		}
		if ($field)  {
			$newdesc = $this->get($field) . ' ' . translate('copy (n)');
			$qb = BFQueryBuilder::select();
			$qb->addColumn($field, $this->table, 'desc', 'DISTINCT');
			$qb->where(array('LIKE', $this->table, $field, $newdesc . '%'));
			$res = $qb->getCol();
			if (!DB::isError($res))  {
				$descs = array();
				foreach ($res as $v)  {
					$descs[$v] = 1;
				}
				if (isset($descs[$newdesc]))  {
					for ($n = 2; isset($descs[$newdesc . ' ' . $n]); $n++)   {}
					$newdesc .= ' ' . $n;
				}
			}
			return $this->set($field, $newdesc);
		}
		return true;
	}


	/** set UID lock
	 * This is part of the generate uid process.. we lock the entry we are working with so that no one
	 * else can come along and use the same one.  The engine uses this alot so we have to be careful here	 
	 */
	protected function setUIDLock($uidType, $uidLock) {
		// Attempt to get a lock on the db 5 times
		$result = '';
		for ($t=0;$t<5;$t++) {

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num', 'MAX');
			$qb->where(array('=', 'bf_uid', 'bf_type', $uidType));
			$num = $qb->getOne(false);

			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_lock', 'bf_uid', $uidLock);
			$qb->setColumn('bf_num', 'bf_uid', ++$num);
			$qb->where(array(
					array('=', 'bf_uid', 'bf_type', $uidType),
					array('=', 'bf_uid', 'bf_lock', '0')
				));
			$resA = $qb->getResult();

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_stamp', 'bf_uid', 'bf_stamp');
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num');
			$qb->where(array(
					array('=', 'bf_uid', 'bf_type', $uidType),
					array('=', 'bf_uid', 'bf_lock', $uidLock),
				));

			$resB = $qb->getResult();

			if (DB::isError($resA) || DB::isError($resB)) {
				throw new BFException(translate('2187'));
				return false;
				}
			$result = $resB->fetchRow();
			if (is_object($result)) {
				// this is a good break - we want this break
				break;
				}
			usleep(10000);
		}
		return $result;
	}
	
	/** force UID Lock
	 * part of the UID locking cycle with the engine, if we haven't gotten the lock by now then
	 * most likely the engine forgot to unlock it so force the issue.	 	
	 */
	protected function forceUIDLock($uidType, $uidLock) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num', 'MAX');
			$qb->where(array('=', 'bf_uid', 'bf_type', $uidType));
			$num = $qb->getOne(false);

			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_lock', 'bf_uid', $uidLock);
			$qb->setColumn('bf_num', 'bf_uid', ++$num);
			$qb->where(array('=', 'bf_uid', 'bf_type', $uidType));
			$resA = $qb->getResult();

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_stamp', 'bf_uid', 'bf_stamp');
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num');
			$qb->where(array(
					array('=', 'bf_uid', 'bf_type', $uidType),
					array('=', 'bf_uid', 'bf_lock', $uidLock),
				));
			$resB = $qb->getResult();

		if (DB::isError($resA) || DB::isError($resB)) {
			throw new BFException(translate('2188'));
			return false;
			}
		$result = $resB->fetchRow();
		return $result;
	}
	
	/** reset UID Lock
	 * Okay unlock it for the rest of the world to use.	
	 */
	public function resetUIDLock($uidType, $uidLock) {
		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_lock', 'bf_uid', '0');
		$qb->where(array(
				array('=', 'bf_uid', 'bf_type', $uidType),
				array('=', 'bf_uid', 'bf_lock', $uidLock)
			));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFException(translate('2189'));
			return false;
			}
		return true;
	}
	

	/**
	 * update
	 * 
	 * Create and execute an UPDATE query, updating all modified fields.
	 * 
	 * @return boolean
	 * @access private
	 */
	public function update() {
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object
			throw new BFException(translate('2181'));
		}
		$qb = BFQueryBuilder::update();

		// No sense in sending empty queries. Just return true to keep everyone happy
		$updated = false;
		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				if (!$updated) {
					$updated = true;
				}
				// If a bfstored value, need to save that first to get the ID
				if (array_key_exists($field, $this->bfstores)) {
					$this->fields[$field] = $this->bfstore->setStorage($this->bfstores[$field]['TEXT'], $this->bfstores[$field]['UID']);
				}
				$qb->setColumn($field, $this->table, $this->fields[$field]);
			}
		}

		if ($updated) {
			$qb->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));
			$res = $qb->getResult();
			if (!DB::isError($res))  {
				$this->logChange('Updated');
				return true;
			}  else {
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * delete
	 * 
	 * Create and execute a DELETE query.
	 * 
	 * @return boolean
	 */
	public function delete() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));

		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}

	/**
	 * loadFields
	 * 
	 * Retrieves the columns from the ColumnCache object for the table and assigns
	 * them as fields of the object. When extending BFDBO, you can override this
	 * to add JOIN-ed fields to make the object behave as if it "natively" owns
	 * the fields.
	 * 
	 * @return boolean
	 * @access private
	 */
	protected function loadFields() {
		$info = $this->controller->getColumnCache();
		$this->fields = $this->tablefields = $info->getColumns($this->table);

		foreach ($this->fields as $field) {
			// Populate defaults where specified
			if (array_key_exists($field, $this->field_defaults)) {
				$this->fields[$field] = $this->field_defaults[$field];
			} else {
				$this->fields[$field] = null;
			}
			$this->querybuilder->addColumn($field, $this->table, $field);
		}
		// Primary key
		$this->primary_key_type = $info->getColumnType($this->table, $this->primary_key);;
	}

	/**
	 * Dynamically load the Database Handle and lookup table for the Keyed SQL.
	 * Used ONLY from the Reports area, this should really live somewhere else,
	 * since it doesn't have anything to do with database objects.
	 * 
	 * @access private
	 */
	private function initSQLLookup() {
		$DBType = $GLOBALS['controller']->getDBType()."SQL";

		$GLOBALS['controller']->require_file("modules/BFDBO/SQL/CommonSQL.php");
		$GLOBALS['controller']->require_file("modules/BFDBO/SQL/" . $DBType . ".php");

		$this->lookup = new $DBType;
	}
	
	/**
	 * Perform a lookup in the ugliness that they stuck into our pristine BFDBO.
	 * Used ONLY from the Reports area, this should really live somewhere else,
	 * since it doesn't have anything to do with database objects.
	 * 
	 * @param string $key A key to get the actual SQL query
	 * @return string The actual SQL query
	 * @access private
	 */
	protected function Lookup($key) {
		$this->initSQLLookup();
		return $this->lookup->getSQL($key);
	}
	
	/**
	 * A BFDBO-centered logging function.  Override as necessary.  $type should (currently) be one of 
	 * 'Created', 'Updated', or 'Deleted'.
	 */
	public function logChange($type)  {
		if ($this->log_changes && ($type != 'Updated' || 
				$this->controller->getSysconfig('log_all_changes') != 'Y'))  {
			$this->controller->logAudit($this->getObjectTypeName() . $type, array($this->getObjectName()));
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->log_changes && $this->inserted && $fieldname &&
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			$this->controller->logAudit($this->getObjectTypeName() . 'UpdatedField', array($fieldname,
					$this->getObjectName(), $before, $after));
		}
	}

	/**
	 * Gets the translated name of the object type ('Class', 'Interface', etc.), usually for logging 
	 * purposes.
	 */
	public function getObjectTypeName()  {
		if (isset($this->type_name))  {
			return $this->type_name;
		}  else {
			return $this->table;
		}
	}
	
	/**
	 * Gets the name of the object ('Bill', 'My First Project Which I Love Very Much', etc.), usually for 
	 * logging purposes.
	 */
	public function getObjectName()  {
		if (isset($this->descriptor_key))  {
			return $this->fields[$this->descriptor_key];
		}  elseif (is_array($this->primary_key))  {
			return $this->fields[$this->primary_key[0]];
		}  else {
			return $this->fields[$this->primary_key];
		}
	}

	/**
	 * The constructor sets up some of the core variables used in the BFDBO.
	 * It creates references to the global database connection object, the controller
	 * object, and creates a BFQueryBuilder object. If the object contains any fields
	 * stored in bf_store, it includes the file and creates an internal BFStore
	 * instance.
	 */
	function __construct() {
		global $controller;
		$this->controller 		= $controller;
		$this->querybuilder 	= BFQueryBuilder::select();
		// Only need this if we'll use it
		if (count($this->bfstores) > 0) {
			$this->controller->require_module('BFStore');
			$this->bfstore = new BFStore();
		}
	}
}

?>
