<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * In an effort to pull database-specific code (read: anything with SQL in it)
 * out of objects that have it fairly isolated to begin with, this object's
 * raison d'etre (not sure how to get accents in here, but I digress): take
 * a database connection to decide the type of database involved, take calls
 * to form chunks of queries and then assemble it all together.
 *
 * @package Console
 * @subpackage Core
 */

$GLOBALS['controller']->require_file('includes/BFColumnTypeCache.php');

// Supporting things like different query types will help a lot...
define('BFQUERYBUILDER_INSERT', 1);
define('BFQUERYBUILDER_SELECT', 2);
define('BFQUERYBUILDER_UPDATE', 3);
define('BFQUERYBUILDER_DELETE', 4);

// Table locking.. joy..
define('TABLE_LOCKED', true);
define('TABLE_UNLOCKED', false);

/**
 * The class itself extends nothing, in an effort to simplify things... It
 * needs to inherit NOTHING and it gets what it asked for.
 */
class BFQueryBuilder {
	protected $compiler;
	protected $querytype;
	protected $db;
	protected $db_type;
	protected $using_master;
	protected $presql;
	protected $sql;
	protected $sql_count;
	protected $res;
	protected $res_count;
	protected $numrows;
	protected $tablelock;
	protected $sessionCache;

	/**
	 * After verifying all sorts of things (no objects, no trying to cheat, etc),
	 * bring in the presql array from a serialized state.
	 * 
	 * Usage:
	 * <pre>
	 * $qb1 = BFQueryBuilder::select();
	 * $qb1->addColumn("bf_userid", "bf_users", "id");
	 * $qb1->addColumn("bf_name", "bf_users", "name");
	 * // Generally, you'll store the serialized value somewhere
	 * $ser = $qb1->serialize();
	 * // Then retrieve it from storage and apply it
	 * $qb2 = BFQueryBuilder::select();
	 * $qb2->unserialize($ser);
	 * </pre>
	 */
	public function unserialize($serialized) {
		if (preg_match(BF_REGEXP_MATCH_QRY, $serialized)) {
			$this->presql = unserialize($serialized);
			return true;
		}
		return false;
	}

	/**
	 * Returns a string representation of the "presql" array making up
	 * the entire query as it currently stands.
	 * 
	 * The following...
	 * <pre>
	 * $qb = BFQueryBuilder::select();
	 * $qb->addColumn("bf_userid", "bf_users", "id");
	 * $qb->addColumn("bf_name", "bf_users", "name");
	 * print $qb->serialize();
	 * </pre>
	 * ...print:
	 * <pre>
	 * a:2:{s:6:"tables";a:1:{i:0;s:8:"bf_users";}s:6:"fields";a:2:{i:0;a:5:{i:0;s:8:"bf_users";i:1;s:9:"bf_userid";i:2;s:2:"id";i:3;N;i:4;N;}i:1;a:5:{i:0;s:8:"bf_users";i:1;s:7:"bf_name";i:2;s:4:"name";i:3;N;i:4;N;}}}
	 * </pre>
	 * 
	 * @see BFQueryBuilder::unserialize()
	 */
	public function serialize() {
		return serialize($this->presql);
	}

	/**
	 * Returns a reference to the "presql" array making up the query
	 * as it currently stands.
	 */
	public function getPreSQL() {
		return $this->presql;
	}
	
	public function getSQL() {
		if (!isset($this->sql)) {
			$this->compile();
		}
		$params = $this->compiler->getParameters();
		if ($params)  {
			$param_str = ' - ';
			foreach ($params as $k => $v)  {
				$param_str .= $k . ' = ' . (is_array($v) ? $v[0] : $v) . '; ';
			}
			return $this->sql . $param_str;
		}  else {
			return $this->sql;
		}
	}

	public function getSQLCount() {
		if (!isset($this->sql_count)) {
			$this->compile();
		}
		return $this->sql_count;
	}

	public function getCount() {
		if (!isset($this->res_count)) {
			$cparms = $this->compiler->getCountParameters();
			$this->res_count = $this->db->getOne($this->sql_count, null, (is_array($cparms)) ? $cparms : null);
		}
		return (DB::isError($this->res_count) ? false : $this->res_count);
	}

	public function getInfo() {
		if (isset($this->res)) {
			return $this->db->tableInfo($this->res);
		} else {
			return false;
		}
	}

	public function getLimitOffset() {
		if (array_key_exists('limit', $this->presql) && array_key_exists('offset', $this->presql['limit'])) {
			return $this->presql['limit']['offset'];
		} else {
			return false;
		}
	}

	public function getLimitCount() {
		if (array_key_exists('limit', $this->presql) && array_key_exists('count', $this->presql['limit'])) {
			return $this->presql['limit']['count'];
		} else {
			return false;
		}
	}

	/**
	 * Actually run the query and return the database result object
	 */
	public function getResult() {
		if (is_null($this->db_type)) {
			$this->compile();
		}

		// Lock the table if we are supposed to or return false;
		if ($this->tablelock == TABLE_LOCKED) {
			if ($this->compiler->dbLock() == TABLE_UNLOCKED) {
				// ERROR - didn't lock
				return false;
			}
		}

		$parms = $this->compiler->getParameters();

		// Only do this when selecting...
		if ($this->querytype == BFQUERYBUILDER_SELECT) {
			$cparms = $this->compiler->getCountParameters();
			$this->res_count = $this->db->getOne($this->sql_count, null, (is_array($cparms)) ? $cparms : null);
		}

		if (isset($parms) && is_array($parms)) {
			$this->res = $this->db->query($this->sql, $parms);
		} else {
			$this->res = $this->db->query($this->sql);
		}

		// unLock the tables
		if ($this->tablelock == TABLE_LOCKED) {
			if (! $this->compiler->dbUnlock()) { return false; }
		}
		
		// I don't care if it worked. The caller can deal with that.
		return $this->res;
	}

	/**
	 * Can I help that I don't feel like going through a huge song and
	 * dance just to get a column into an array? This saves time!
	 */
	public function getCol($col = 0, $cache = true) {
		$this->compile();

		$parms = $this->compiler->getParameters();

		// This cached?
		if ($cache)  {
			$params = serialize($parms);
			if ($res = $GLOBALS['controller']->getCache($this->sql . $params)) {
				return $res;
			}
		}

		// Lock the table if we are supposed to or return false;
		if ($this->tablelock == TABLE_LOCKED) {
			if ($this->compiler->dbLock() == TABLE_UNLOCKED) {
				// ERROR - didn't lock
				return false;
			}
		}

		// Only do this when selecting...
		if ($this->querytype == BFQUERYBUILDER_SELECT) {
			$cparms = $this->compiler->getCountParameters();
			$this->res_count = $this->db->getOne($this->sql_count, null, (is_array($cparms)) ? $cparms : null);
		}

		if (isset($parms) && is_array($parms)) {
			$this->res = $this->db->getCol($this->sql, $col, $parms);
		} else {
			$this->res = $this->db->getCol($this->sql, $col);
		}

		// unLock the tables
		if ($this->tablelock == TABLE_LOCKED) {
			if (! $this->compiler->dbUnlock()) { return false; }
		}

		// Set the cache
		if ($cache) {
			$GLOBALS['controller']->setCache($this->sql . $params, $this->res);
		}
		
		// I don't care if it worked. The caller can deal with that.
		return $this->res;
	}

	/**
	 * Can I help that I don't feel like going through a huge song and
	 * dance just to get a single row? This saves time!
	 */
	public function getRow($cache = true) {
		$this->compile();

		$parms = $this->compiler->getParameters();

		// This cached?
		if ($cache)  {
			$params = serialize($parms);
			if ($res = $GLOBALS['controller']->getCache($this->sql . $params)) {
				return $res;
			}
		}

		// Lock the table if we are supposed to or return false;
		if ($this->tablelock == TABLE_LOCKED) {
			if ($this->compiler->dbLock() == TABLE_UNLOCKED) {
				// ERROR - didn't lock
				return false;
			}
		}

		$this->res = $this->db->getRow($this->sql, $parms, DB_FETCHMODE_ASSOC);

		// unLock the tables
		if ($this->tablelock == TABLE_LOCKED) {
			if (! $this->compiler->dbUnlock()) { return false; }
		}

		// Set the cache
		if ($cache) {
			$GLOBALS['controller']->setCache($this->sql . $params, $this->res);
		}
		
		// I don't care if it worked. The caller can deal with that.
		return $this->res;
	}

	/**
	 * Can I help that I don't feel like going through a huge song and
	 * dance just to get a single item? This saves time!
	 */
	public function getOne($cache = true) {
		$this->compile();

		$parms = $this->compiler->getParameters();

		// This cached?
		if ($cache)  {
			$params = serialize($parms);
			if ($res = $GLOBALS['controller']->getCache($this->sql . $params)) {
				return $res;
			}
		}

		// Lock the table if we are supposed to or return false;
		if ($this->tablelock == TABLE_LOCKED) {
			if ($this->compiler->dbLock() == TABLE_UNLOCKED) {
				// ERROR - didn't lock
				return false;
			}
		}

		$this->res = $this->db->getOne($this->sql, null, $parms);

		// unLock the tables
		if ($this->tablelock == TABLE_LOCKED) {
			if (! $this->compiler->dbUnlock()) { return false; }
		}

		// Set the cache
		if ($cache) {
			$GLOBALS['controller']->setCache($this->sql . $params, $this->res);
		}
		
		// I don't care if it worked. The caller can deal with that.
		return $this->res;
	}

	/**
	 * This is the new AddColumn which replaces every other kind.
	 *
	 * Below are valid calls and what they do
	 * <pre>
	 *  $rep->addColumn('bf_class', 'bf_projects');							 ~ Adds "table.field"
	 *  $rep->addColumn('bf_class', 'bf_projects', 'Class');					~ Adds "table.field AS alias"
	 *  $rep->addColumn('bf_start', 'bf_builds', 'Date', 'FORMAT_DATE');		~ Adds "DATE_FORMAT(FROM_UNIXTIME(table.field,'%b %e, %Y - %H:%i') AS alias"
	 *  $rep->addColumn('bf_start', 'bf_builds', 'Date', 'FORMAT_DATE', '$H');  ~ Adds "DATE_FORMAT(FROM_UNIXTIME(table.field,'%H') AS alias"
	 *  $rep->addColumn('bf_durat', 'bf_builds', 'time', 'SECONDS_TO_TIME');	~ Adds "SEC_TO_TIME(table.field) AS alias"
	 *  $rep->addColumn('bf_param', 'bf_env', 'Env', 'CONDITIONAL',  array(array('!=','bf_projects','bf_eid','0')), '');
		 ~ A conditional expression that works similar to the WHERE clause but inline.  Come see me for questions.. Below is actual SQL for above
		   IF (`bf_projects`.`bf_eid` != 0, `bf_env`.`bf_param`, '' ) AS `Env`
	 * </pre>
	 */
	public function addColumn($field, $table, $alias = null, $option = null, $args = null) {
		// PreSQL Fields and Tables
		$this->addTable($table);
		$this->presql['fields'][] = array($table,$field,$alias,$option,$args);
	}

	/**
	 * Similarly to addColumn, setColumn receives fields and tables, but also
	 * receives values, rather than aliases. This ONLY gets used with UPDATE
	 * and INSERT types of queries. I mean, you can call it for other types
	 * if you really want, but it won't do a thing but add memory usage.
	 */
	public function setColumn($field, $table, $value = null) {
		$this->addTable($table);
		if (!array_key_exists('setfields', $this->presql)) {
			$this->presql['setfields'] = array();
		}
		// Check the column width
		if ($width = $this->columnInfo->getColumnLength($table, $field) < strlen($value)) {
			$value = bf_substr($value, 0, $width);
		}
		$this->presql['setfields'][] = array($table,$field,$value);
	}

	/**
	 * This is the all inclusive WHERE statement now - no more different functions for each type of WHERE
	 * Use this for every where there is - it's all one big happy array.
	 * <pre>
	 * BOOLEAN (=,!=,<,>,<=,>=) etc..
	 * table1.field1 = table2.field2	~ array('=', 'table1','field1','table2','field2');
	 * table1.field1 = value			~ array('=', 'table1','field1','value');
	 * alias1 = alias2				  ~ array('=', 'alias1','alias2');
	 * alias1 = value				   ~ array('=', 'alias1','value');
	 *
	 * LIKE:
	 * table1.field1 LIKE %..%		  ~ array('LIKE', 'table1', 'field1', '...');
	 * alias1 LIKE %..%				 ~ array('LIKE', 'alias1', '...');
	 *
	 * IN:
	 * table1.field1 IN (.....)		 ~ array('IN','table1','field1', array(.,.,.));
	 * alias1 IN (.....)				~ array('IN','alias1', array(.,.,.));
	 * </pre>
	 * Examples:
	 * <pre>
		$rep->where(
			array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level')
		);

		$rep->where(
			array(
				array(
					array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level'),
					array('=', 'bf_projects', 'bf_eid', 'bf_env', 'bf_bid'),
					array('=', 'bf_env', 'bf_eid', '0')
				),
				'OR',
				array(
					array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level'),
					array('=', 'bf_projects', 'bf_eid', '0')
				)
			)
		);
	 * </pre>
	 */
	public function where($whereArray) {
		if (is_array($whereArray)) {
			if (!array_key_exists('where', $this->presql)) {
				$this->presql['where'] = array();
			}
			$this->presql['where'][] = $whereArray;
			$this->parseWhereForTables($whereArray);
		} else {
			$this->presql['where'] = '1';
		}
	}

	/**
	 * This gets called from within the object, since we have no reason
	 * to keep putting the same things inline all over the place.
	 */
	protected function addTable($table) {
		if (!array_key_exists('tables', $this->presql)) {
			$this->presql['tables'] = array($table);
		} else if ((!is_array($this->presql['tables']) || (!in_array($table, $this->presql['tables'])))) {
			$this->presql['tables'][] = $table;
		}
	}

	/**
	 * Adding where clauses should work just like a straight join, as far as
	 * not having to addColumn from the table for it to show up in FROM
	 * 
	 * NOTE: Due to the nature of this beast any where clause that uses IN must be nested in an array.
	 * Example:
	 * <pre>
	 *  // With other types, either of these are valid	 
	 *  $qb->where(array('=', 'bf_projects', 'bf_pid', '100')); 		IS VALID
	 *  $qb->where(array(array('=', 'bf_projects', 'bf_pid', '100'))); 	IS VALID	  
	 *  However if using an IN you must have it in the second level
	 *	$qb->where(array('IN', 'bf_projects', 'bf_level', array(0,1,2,3)));			IS NOT VALID
	 *	$qb->where(array(array('IN', 'bf_projects', 'bf_level', array(0,1,2,3))));	IS VALID	 	 	 	 	 	 
	 * </pre>
	 */
	protected function parseWhereForTables($whereArray) {
		$sigils = array ('=','!=','<>','<','>','<=','>=','IN', 'LIKE', 'LIKE BINARY', 'IS NULL', 'IS NOT NULL');
		$results = '';
		foreach($whereArray as $k=>$v) {				
			if (is_array($v)) {
				if (in_array($v[0], $sigils)) {

					if (strtoupper($v[0]) != 'IN') {
						// Handle the query based on the number of elements in the array
						switch (count($v)) {
							case(3):
							case(4):
								$this->addTable($v[1]);
								break;
							case(5):
								$this->addTable($v[1]);
								$this->addTable($v[3]);
								break;
						}
					}
				} else {
					$this->parseWhereForTables($v);
				}
			} else if (in_array($v,$sigils)) {
				$this->parseWhereForTables(array($whereArray));
			}
		}
	}

	/**
	 * This acts as your one-stop shop for joins, with which we'll have a field
	 * day in compile() to implement across databases.
	 *
	 * The parameter works similarly to where(), in that it takes one glorious
	 * array and decides what to do with it from there. In fact, join requires
	 * a where() parameter as its third item:
	 * <pre>
	 join(
		array(
				TYPE (LEFT, STRAIGHT, OUTER...just make it a string),
				table (just like any other table getting passed around here)
				where (the where clause)
			  );
	 * </pre>
	 * Example:
	 * <pre>
		  $rep->join(
				 array(
					   'LEFT',
					   'bf_env',
					   array(
						   array('=', 'bf_env', 'bf_eid', 0),
						   array('=', 'bf_env', 'bf_bid', 'bf_servers', 'bf_eid')
					   );
					)
					 );
	 * </pre>
	 */
	public function join($joinArray) {
		if (is_array($joinArray)) {
			if (!array_key_exists('join', $this->presql)) {
				$this->presql['join'] = array();
			}
			$this->presql['join'][] = $joinArray;
			return true;
		} else {
			return false;
		}
	}

	// WHERE for the filter
	public function filter($filterArray) {
		if (is_array($filterArray)) {
			$this->presql['filter'] = $filterArray;
		}
	}

	/**
	 * If you do this with joins, it will choke. So don't.
	 */
	public function cleanWhere() {
		if (is_array($this->presql) && array_key_exists('where', $this->presql)) {
			unset($this->presql['where']);
			unset($this->sql);
		}
		// It doesn't matter whether we had to - same end result
		return true;
	}

	public function limit($count, $offset = 0) {
		if (is_int($count) && is_int($offset) && $count > -1 && $offset > -1) {
			$this->presql['limit'] = array(
									'count'  =>	$count,
									'offset' =>	$offset
								);
			return true;
		}
		return false;
	}

	public function groupBy($tableOrAlias, $field = null) {
		$this->presql['groupby'] = array($tableOrAlias,$field);
	}

	public function orderBy($direction, $tableOrAlias, $field = null) {
		$this->presql['orderby'][] = array($direction,$tableOrAlias,$field);
	}

	/*
	 * Some databases (MySQL for example) will require a specific field to count by
	 * or the results you will get when you count(*) will be Waaaay off.
	 * This error usually crops up when you use joins and distinct with counts
	 * This is an optional field.
	 */
	public function countBy($field, $table) {
		$this->presql['countby'] = array($table, $field);
	}

	public function valid() {
		return isset($this->sql);
	}
	
	public function compile() {
		// Get the necessary BFQueryCompiler at the last possible second
		// before compiling the SQL specifically for the connection
		$this->loadCompiler();
		$this->compiler->setPreSQL($this->presql);
		if ($this->compiler->compile()) {
			$this->sql = $this->compiler->getSQL();
			$this->sql_count = $this->compiler->getSQLCount();
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * If using a local machine, decided which connection to use.
	 */
	public function loadCompiler() {
		global $controller, $GDD_LOCAL_TABLES;
		
		$this->db_type = null;
		$local = count(array_intersect(isset($this->presql['tables']) ? $this->presql['tables'] : array(), 
				$GDD_LOCAL_TABLES));

		// Use the specified connection rather than a global one
		if (isset($this->db)) {
			$this->using_master = false;
			$this->db_type = $controller->getDBType($this->db);
		// Only use the master database connection when using a local/master system
		// and the presql touches locally managed tables.
		} else if ($controller->local && $local == 0) {
			$this->using_master = true;
			$this->db_type = $controller->getMasterDBType();
			$this->db = $controller->getMasterDB();
//		} else if ($controller->local && $local != count($this->presql['tables'])) {
//			print_r(new BFException('GDD pukage'));
//			exit();
		} else {
			$this->using_master = false;
			$this->db_type = $controller->getDBType();
			$this->db = $controller->getDB();
		}
		
		// Yank in the appropriate class
		if ($controller->require_file('modules/BFQueryBuilder/BFQueryCompiler_' . $this->db_type . '.php')) {
			$classname = 'BFQueryCompiler_' . $this->db_type;
			$this->compiler = new $classname($this->db, $this->querytype, $this->tablelock);
		} else {
			throw new BFException('Not a valid db type :: '.'BFQueryCompiler_' . $this->db_type . '.php');
		}
	}
	
	/**
	 * Retrieve the database schema_version
	 * note this must be straight sql (not generated) because the builders rely on the cache can't have your
	 * chicken before the egg...
	 */
	public static function getDBVersion($db) {
		$res = $db->getOne('SELECT bf_value FROM bf_sysconfig WHERE bf_param = \'schema_version\'');
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			return $res;
		}
	}
	
	
	public function getColumnCache() {
		return $this->columnInfo;
	}
	
	protected function loadColumnCache() {
		$this->columnInfo = $GLOBALS['controller']->getColumnCache();
	}
	
	/* convenience method for grabbing all 'BF_%' tables.  Currently just used for cacheing the schema. */
	/* note this must be straight sql (not generated) because the builders rely on the cache can't have your
		* chicken before the egg...
		*/
	public static function listTables($db) {
		$tables = Array();
		$sql = $db->getSpecialQuery("tables");
		// odbc handles this differently.  It will do the odbc_tables function in this case
		// and return an array of table names.
		if (is_array($sql)) {
			return $sql;
		}
		$res = $db->query($sql);
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$cols = Array();
			while(($cols = $res->fetchRow(DB_FETCHMODE_ASSOC)) && $cols && !DB::isError($cols)) {
				foreach ($cols as $col => $val) {
					if (strtoupper(substr($val, 0, 2)) == 'BF') {
						array_push($tables, strtolower($val));
					}
				}
			}
			return $tables;
		}
	}
	
	/**
	 * @param int $type One of the constants specifying the type of query
	 * @param bool $tablelock Whether or not to "lock" the table for the query
	 * @param BFDBO $db The database connection (contained in the BFDBO instance) to use
	 * @construct
	 */
	public function __construct($type, $tablelock, $db = null) {
		$this->querytype = $type;
		$this->tablelock = $tablelock;
		$this->presql = array();
		// Allow the forcing of one kind of database connection
		if (isset($db)) {
			$this->db = $db;
		}
		$this->loadColumnCache();
	}

	/**
	 * BFQueryCompiler factory for various types of queries. The database
	 * connection passed gets ignored now that we need to late-connect depending
	 * on the database we need to connect to for a given table set.
	 */
	public static function insert($db = null, $tablelock = TABLE_UNLOCKED) {
		return new BFQueryBuilder(BFQUERYBUILDER_INSERT, $tablelock, $db);
	}

	public static function select($db = null, $tablelock = TABLE_UNLOCKED) {
		return new BFQueryBuilder(BFQUERYBUILDER_SELECT, $tablelock, $db);
	}

	public static function update($db = null, $tablelock = TABLE_UNLOCKED) {
		return new BFQueryBuilder(BFQUERYBUILDER_UPDATE, $tablelock, $db);
	}

	public static function delete($db = null, $tablelock = TABLE_UNLOCKED) {
		return new BFQueryBuilder(BFQUERYBUILDER_DELETE, $tablelock, $db);
	}
}

/**
 * Wise, old, learned QueryCompiler
 */
abstract class BFQueryCompiler {
	protected $db;
	protected $sql;
	protected $sql_count;
	protected $parameters;
	protected $count_parameters;
	protected $presql;
	protected $res;
	protected $res_count;
	protected $numrows;
	protected $tablelock;
	protected $sessionCache;

	/**
	 * BFQueryBuilder::compile() gets called auto-magically the first time.
	 * You need to call it explicitly after that if you need a different result
	 * set from a modified BFQueryBuilder instance.
	 */
	abstract public function compile();

	/**
	 * BFQueryBuilder::dbLock() locks all the tables used for this query ($this->presql['tables'])
	 * BFQueryBuilder::dbUnlock() .. guess..
	 * BFQueryBuilder::setLock() locks this table - forces after a number of tries or returns unlocked
	 * BFQueryBuilder::getLock() finds out if this table is already locked
	 */
	abstract public function dbLock();
	abstract public function dbUnlock();
	abstract protected function setLock($tableName);
	abstract protected function getLock($tableName);
	
	public function getParameters() {
		return $this->parameters;
	}
	
	public function getCountParameters() {
		return $this->count_parameters;
	}
	
	public function setPreSQL(&$presql) {
		$this->presql = $presql;
	}
	
	/**
	 * This calls whatever implementation of compile() the child object
	 * has. It had better work.
	 */
	public function getSQL() {
		return $this->sql;
	}
	
	/**
	 * This calls whatever implementation of compile() the child object
	 * has. It had better work.
	 */
	public function getSQLCount() {
		return $this->sql_count;
	}

	public function __construct($db, $type = BFQUERYBUILDER_SELECT, $tableLock = TABLE_UNLOCKED) {
		global $controller;

		$this->db = $db;
		$this->querytype = $type;
		$this->presql = array();
		$this->tablelock = $tableLock;
		$this->columnInfo = $controller->getColumnCache();
	}
}

?>
