<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/


/**
 * BFQueryCompiler_oci8 a database specific extension of BFQueryCompiler
 *
 * @package Console
 * @subpackage Core
 */

// Set TRUE to have files written to temp directory for each query type.
define('DATABASE_DEBUG', false);

$GLOBALS['controller']->require_file('includes/BFColumnTypeCache.php');

class BFQueryCompiler_oci8 extends BFQueryCompiler {
	protected $quotedTypes;
	protected $paramname = 'param';
	protected $paramcount = 0;

	/**
	 * compile
	 *
	 * The compile functionality is database type specific because SQL isn't
	 * standardized between the different database types.
	 * As such, this function will be used to actualy build the SQL.
	 */
	public function compile() {
		$this->parameters = array();
		$this->count_parameters = array();
		switch ($this->querytype) {
			case BFQUERYBUILDER_INSERT:
				return $this->compileInsert();
			case BFQUERYBUILDER_SELECT:
				return $this->compileSelect();
			case BFQUERYBUILDER_UPDATE:
				return $this->compileUpdate();
			case BFQUERYBUILDER_DELETE:
				return $this->compileDelete();
		}
	}

	/**
	 * Compile INSERT queries
	 *
	 * You'd better do this from a database object (since it uses all
	 * fields) or know exactly what the query will do. I don't care
	 * if fields can accept a null value from not getting set, but
	 * the database may or may not throw up on you.
	 */
	protected function compileInsert() {
		$QUERY =	'INSERT INTO %s (%s) VALUES (%s)';
		$TABLE =	null;
		$FIELDS =   '';
		$VALUES =   '';

		// Build fields to insert to
		if (!empty($this->presql['setfields'])) {
			foreach($this->presql['setfields'] as $k=>$v) {

				if (is_null($TABLE)) {
					$TABLE = $v[0];
				}
				$FIELDS .= (($k=='0') ? ' ' : ', ')
								. $v[1];
				$name = $this->paramname . ($this->paramcount++);
				$VALUES .= ($k == 0) ? ' :' . $name . ' ' : ', :' . $name . ' ';
				$columnType = $this->columnInfo->getColumnType($v[0], $v[1]);
				$this->parameters[$name] = array($v[2], $columnType);

			}

			$this->sql = sprintf($QUERY, $TABLE, $FIELDS, $VALUES);
			
			if (DATABASE_DEBUG) {
				$out = $this->sql . "\n\t(";
				foreach ($this->parameters as $name => $value) $out .= "'{$name}' => '{$value[0]}', ";
				$out .= ")\n";
				file_put_contents("/tmp/oci8_insert.log", $out, FILE_APPEND);
			}

			return true;
		}
		// Not about to insert only blank fields
		return false;
	}

	/**
	 * Compile SELECT queries
	 */
	protected function compileSelect() {
		// Filter out the tables used in JOINs. We need to do this first,
		// since it can't exist in two places...
		if (array_key_exists('join', $this->presql)) {
			$this->filterTables();
		}

		$SELECT = '';
		// Build fields to select from
		if (!empty($this->presql['fields'])) {
			foreach($this->presql['fields'] as $k=>$v) {
				$prepender = ''; $appender = '';
				if (!empty($v[3])) {
					// Handle different options if there are any, otherwise drive on!
					switch($v[3]){
						case('MAX'): // Get the macimum value from the selected results
							$prepender = 'MAX(';
							$appender = ')';
							break;
						case('SUBSTRING'): // Selects a specified chunk of string
							$prepender = 'SUBSTR(';
							$appender = ','.$v[4][0] . ((isset($v[4][1]))?','.$v[4][1]:'') . ')';
							break;
						case('COUNT'): // The total number of a certain field within results
							$prepender = 'COUNT(';
							$appender = ')';
							break;
						case('DISTINCT'): // The total number of a certain field within results
							$prepender = 'DISTINCT ';
							$appender = '';
							break;
						default:
							$prepender = '';
							$appender = '';
							break;
					}
				}

				$SELECT .= (($k=='0')?' ':', ').$prepender
											.$v[0].'.'
											.$v[1].$appender
											.((!empty($v[2]))?' AS '.$this->db->quoteIdentifier($v[2]):'');
			}
		} else {
			$SELECT .= ' * ';
		}

		// Build from
		$FROM = ' FROM ';
		foreach($this->presql['tables'] as $k=>$v) {
			$add = true;
			if (array_key_exists('join', $this->presql) && is_array($this->presql['join']))  {
				foreach ($this->presql['join'] as $join_arr) {
					// If this table is being joined, oracle doesn't work if it's in the from clause.
					if (is_array($join_arr)
							&& is_string($join_arr[0])
							&& is_string($join_arr[1])
							&& is_array($join_arr[2])) {
						if ($v == $join_arr[1])  {
							$add = false;
						}
					}
				}
			}
			if ($add)  {
				$FROM .= (($k=='0')?' ':', ').$v;
			}
		}
		$FROM .= ' ';

		// Build joins
		$JOIN = '';
		if (array_key_exists('join', $this->presql) && is_array($this->presql['join'])) {
			$JOIN = $this->parseJoins($this->presql['join']);
		}

		// Build where
		$WHERE = '';
		if (array_key_exists('where', $this->presql) && is_array($this->presql['where'])) {
			foreach ($this->presql['where'] as $where_seg) {
				 if ($WHERE == '') {
					$WHERE  = ' WHERE ';
				} else {
					$WHERE .= ' AND ';
				}
				$WHERE .= $this->parseWhere($where_seg);
			}
		}

		// Build filter (second part to where)
		$FILTER = (!empty($this->presql['filter']))?
				(!$WHERE ? ' WHERE ' : ' AND ') . $this->parseWhere($this->presql['filter']):'';

		// Group by
		$GROUP = '';
		if (array_key_exists('groupby', $this->presql) && is_array($this->presql['groupby'])) {
			$GROUP  = ' GROUP BY ';
			$GROUP .= $this->presql['groupby'][0].
					  ((isset($this->presql['groupby'][1]))?'.'.$this->presql['groupby'][1]:'');
		}

		// Order by
		$ORDER = '';
		if (array_key_exists('orderby', $this->presql) && is_array($this->presql['orderby'])) {
			$ORDER  = ' ORDER BY ';
			foreach($this->presql['orderby'] as $k=>$v) {
				if ($k > 0) {
					$ORDER .= ',';
				}
				if (strtoupper($v[0]) == 'DESC' || strtoupper($v[0]) == 'ASC')  {
					$asc = array_shift($v);
				}  else {
					$asc = 'ASC';
				}
				if (isset($v[1]))  {
					$id = $v[0].'.'.$v[1];
				}  else {
					$id = '';
					if (isset($this->presql['fields']))  {
						foreach($this->presql['fields'] as $fieldArr) {
							if ($fieldArr[1] == $v[0])  {
								$id = $v[0];
							}
						}
					}						
					if (!$id)  {
						$id = $this->db->quoteIdentifier($v[0]);
					}

				}
				$ORDER .= $id . ' ' . $asc;
			}
		}
		$this->count_parameters = $this->parameters;

		// LIMITS are done differently in Oracle. Very, very differently...
		if (array_key_exists('limit', $this->presql) && is_array($this->presql['limit']) && array_key_exists('count', $this->presql['limit'])) {
			// Bit of a hack to pass the limit as a parameter to that the
			// database abstraction layer can prefetch the expected number of rows
			if ($prefetch = (int)$this->presql['limit']['count'] > 0) {
				$this->parameters['prefetch'] = $prefetch;
			}
			// Need offset no matter what
			$offset = (array_key_exists('offset', $this->presql['limit'])) ? $this->presql['limit']['offset'] : 0;
			// Need the list of aliases for each and every field
			$aliases = '';
			foreach ($this->presql['fields'] as $k => $v) {
				if ($k > 0) {
					$aliases .= ',';
				}
				// Use the alias, if available. If not, use the field name
				$aliases .= (array_key_exists(2, $v) && $v[2]) ? $this->db->quoteIdentifier($v[2]) : 
						$this->db->quoteIdentifier($v[1]);
			}
			// Round one: the real query
			$this->sql = 'SELECT '.$SELECT.$FROM.$JOIN.$WHERE.$FILTER.$GROUP.$ORDER;
			// Round two: select the same thing, this time, with row numbers
			$this->sql = 'SELECT '.$aliases.',ROWNUM n FROM ('.$this->sql.')';
			// Round three: select the same thing, this time LIMITED by said row numbers
			$name = $this->paramname . ($this->paramcount++);
			$this->parameters[$name] = array((int)$offset + 1, 'NUMBER');
			$this->sql = 'SELECT '.$aliases.' FROM ('.$this->sql.') WHERE n BETWEEN :'.$name;
			$name = $this->paramname . ($this->paramcount++);
			$this->parameters[$name] = array((((int)$this->presql['limit']['count'])+((int)$offset)), 'NUMBER');
			$this->sql .= ' AND :'.$name;
		} else {
			$this->sql = 'SELECT '.$SELECT.$FROM.$JOIN.$WHERE.$FILTER.$GROUP.$ORDER;
		}

		//DISTINCT
		$COUNTBY = (!empty($this->presql['countby']))?'DISTINCT '
			.$this->presql['countby'][0].'.'.$this->presql['countby'][1]:'*';

		$this->sql_count = 'SELECT count( '.$COUNTBY.' )'.$FROM.$JOIN.$WHERE.$FILTER.$GROUP;

		if (DATABASE_DEBUG) {
			$out = $this->sql . "\n\t(";
			foreach ($this->parameters as $name => $value) $out .= "'{$name}' => '{$value[0]}', ";
			$out .= ")\n";
			file_put_contents("/tmp/oci8_select.log", $out, FILE_APPEND);
		}

		return true;
	}

	/**
	 * Compile UPDATE queries
	 */
	protected function compileUpdate() {
		// Build table list
		$TABLES = ' ';
		foreach($this->presql['tables'] as $k=>$v) {
			$TABLES .= (($k=='0')?' ':', ').$v;
		}
		$TABLES .= '';

		$UPDATE = '';
		// Build fields to update
		if (!empty($this->presql['setfields'])) {
			foreach($this->presql['setfields'] as $k=>$v) {
				$name = $this->paramname . ($this->paramcount++);
				$columnType = $this->columnInfo->getColumnType($v[0], $v[1]);
				$this->parameters[$name] = array($v[2], $columnType);
				$UPDATE .= (($k=='0')?' ':', ')
											.$v[0] . '.'
											.$v[1] . ' = '
											.':'.$name;
			}
		} else {
			return false;
		}

		// Build where
		$WHERE = '';
		if (array_key_exists('where', $this->presql) && is_array($this->presql['where'])) {
			foreach ($this->presql['where'] as $where_seg) {
				 if ($WHERE == '') {
					$WHERE  = ' WHERE ';
				} else {
					$WHERE .= ' AND ';
				}
				$WHERE .= $this->parseWhere($where_seg);
			}
		}

		// Build filter (second part to where) - why not? It could happen.
		$FILTER = (!empty($this->presql['filter']))?' AND '.$this->parseWhere($this->presql['filter']):'';

		$this->sql	   = 'UPDATE ' . $TABLES . ' SET ' . $UPDATE . $WHERE . $FILTER;

		if (DATABASE_DEBUG) {
			$out = $this->sql . "\n\t(";
			foreach ($this->parameters as $name => $value) $out .= "'{$name}' => '{$value[0]}', ";
			$out .= ")\n";
			file_put_contents("/tmp/oci8_update.log", $out, FILE_APPEND);
		}

		return true;
	}

	/**
	 * Compile DELETE queries
	 *
	 * MySQL allows DELETE queries spanning tables.
	 *
	 * DON'T FORGET YOUR WHERE CLAUSE OR WE'LL ALL DIE HORRIBLY
	 */
	protected function compileDelete() {
		// Build table list
		$TABLES = ' ';
		foreach($this->presql['tables'] as $k=>$v) {
			$TABLES .= (($k=='0')?' ':', ').$v;
		}
		$TABLES .= ' ';

		// Build where
		$WHERE = "";
		if (array_key_exists('where', $this->presql) && is_array($this->presql['where'])) {
			foreach ($this->presql['where'] as $where_seg) {
				 if ($WHERE == '') {
					$WHERE  = ' WHERE ';
				} else {
					$WHERE .= ' AND ';
				}
				$WHERE .= $this->parseWhere($where_seg);
			}
		}

		$this->sql	   = 'DELETE FROM ' . $TABLES . $WHERE;

		if (DATABASE_DEBUG) {
			$out = $this->sql . "\n\t(";
			foreach ($this->parameters as $name => $value) $out .= "'{$name}' => '{$value[0]}', ";
			$out .= ")\n";
			file_put_contents("/tmp/oci8_delete.log", $out, FILE_APPEND);
		}

		return true;
	}

	protected function filterTables() {
		foreach ($this->presql['join'] as $join_arr) {
			if (is_array($join_arr) && is_string($join_arr[1]) && $k = array_search($join_arr[1], $this->presql['tables'])) {
				unset($this->presql['tables'][$k]);
			}
		}
	}

	/**
	 * May get split up by types of joins later on. Or not.
	 */
	protected function parseJoins($joins) {
		$JOIN = '';
		foreach ($this->presql['join'] as $join_arr) {
			// Joins need a type, a table and a where clause
			if (is_array($join_arr)
					&& is_string($join_arr[0])
					&& is_string($join_arr[1])
					&& is_array($join_arr[2])) {
				$temp_join = '';

				$type = strtoupper($join_arr[0]);
/*
				$temp_join .= ' AND ' . $this->parseWhere($join_arr[2],(($type == 'LEFT')?true:false));

				$JOIN .= $temp_join;
*/
				$type = strtoupper($join_arr[0]);
				if ($type == 'STRAIGHT') {
					$temp_join  .= ' NATURAL JOIN ' . $join_arr[1];
				} else if ($type == 'INNER') {
					$temp_join  .= ' INNER JOIN ' . $join_arr[1];
				} else if ($type == 'LEFT') {
					$temp_join  .= ' LEFT OUTER JOIN ' . $join_arr[1];
				} else {
					continue;
				}
				$temp_join .= ' ON ' . $this->parseWhere($join_arr[2]);
				$JOIN .= $temp_join;

			}
		}

		return $JOIN;
	}

	// Does the job of recursively parsing the WHERE clause
	// ALSO allows yo uto use in conditionals in the select portion of the SQL statement.. *gleee*
	private function parseWhere($whereArray, $joinhack=false) {
		$sigils = array ('=','!=','<>','<','>','<=','>=','IN', 'LIKE', 'LIKE BINARY', 'IS NULL', 'IS NOT NULL');

		$results = '';
		foreach($whereArray as $k=>$v) {
			if (is_array($v)) {
				if (in_array($v[0], $sigils)) {

					// WHERE foo in ()  fails badly - so stop it here
					if ($v[0] == 'IN') {
						if (!is_array($v[3])) {
							$v[3] = array($v[3]);
						}
						// Filter out empties
						foreach ($v[3] as $tk => $tv) {
							if (!$tv && $tv !== 0)  {
								unset($v[3][$tk]);
							}
						}
						// If nothing left, switch it to an "="
						if (empty($v[3])) {
							$v[0] = "=";
							$v[3] = "";
						}
					}

					// If this isn't the first item, and the next item after this one is an array, or if the next one isn't set
					// or if the previous one wasn't an array and is set
					if ($k!='0' && !in_array(trim(substr($results, -3, 3)),array('AND','OR','('))) {
						$results .= ' AND';
					}

					// If the query is a LIKE query add % around the value
					$wrap = (strtoupper($v[0] == 'LIKE'))?'%':'';

					// Handle the query based on the number of elements in the array
					switch (count($v)) {
						case(2):
							if ($v[0] == 'IS NULL' || $v[0] == 'IS NOT NULL') {
								$results .= ' '.$v[1].' '.$v[0];
							}
							break;
						case(3):
							if ($v[0] == 'IS NULL' || $v[0] == 'IS NOT NULL') {
								$results .= ' '.$v[1] .'.'.
											$v[2].' '.$v[0];
							}
							break;
						case(4):
							if ($v[0] == 'IN') {
								$results .= ' ' . $v[1] . '.' . $v[2] . ' ' . $v[0] .' ( ';
								foreach ($v[3] as $tk => $tv) {
									$name = $this->paramname . ($this->paramcount++);
									$columnType = $this->columnInfo->getColumnType($v[1], $v[2]);
									$this->parameters[$name] = array($tv, $columnType);
									$results .= ($tk === 0) ? ':' . $name : ', :' . $name;
								}
								$results .= ' ) ';
							} else {
								$name = $this->paramname . ($this->paramcount++);
								$columnType = $this->columnInfo->getColumnType($v[1], $v[2]);
								$this->parameters[$name] = array($wrap . $v[3] . $wrap, $columnType);
								$results .= ' ' . $v[1] . '.' . $v[2] . ' ' . $v[0] . ' :' . $name;
							}
							break;
						case(5):
							$results .= ' '.$v[1] .'.'.
											$v[2] .' '.$v[0].' '.
											$v[3] .'.'.
											$v[4];
							break;
					}

				} else {
					$results .= ' ( ' . $this->parseWhere($v) . ' ) ';
				}
			} else {
				if (in_array($v,$sigils)) {
					return $this->parseWhere(array($whereArray));
				} else {
					$results .= ' '.$v.' ';
				}
			}
		}

		// Can't have ORs running amok
		return ''.$results.(($joinhack==true)?'(+)':'').'';

		// Amok amok amok!!
	}

	/** dbLock
	 *  Attempts to lock all of the tables we are working with in bf_sysconfig
	 * if it can't lock one it will push back onto stack and wait a second and try again
	 * if it can't lock one after ten retries, it will return unlocked
	 * after 8 tries it will try to force the issue.
	 */
	public function dbLock() {
		// Get list of tables to lock - in the event there are more than one..
		$tableStack = $this->presql['tables'];
		if (!is_array($tableStack)) { return false; }

		$retries = 0;
		while(!empty($tableStack) && $retries < 10) {
			$tableName = array_pop($tableStack);
			if (!empty($tableName)) {
				$lock = $this->getLock($tableName);
				if (!DB::isError($lock) && ($lock == TABLE_UNLOCKED || $retries > 5)) {
					$lock = $this->setLock($tableName);
					if (DB::isError($lock) || $lock == TABLE_UNLOCKED) {
						$retries++;
						array_push($tableStack, $tableName);
						// ERROR: wasn't able to lock the table.. Why fer?
						// Push back onto stack.. let it try some more.. but at 10 tries.. FAIL
					}						
				} else {
					// Table is locked currently, push back on stack
					$retries++;
					array_push($tableStack, $tableName);
					sleep(1);
				}
			}
		}

		return (empty($tableStack))?TABLE_LOCKED:TABLE_UNLOCKED;
	}

	public function dbUnlock() {
		// Get list of tables to lock - in the event there are more than one..
		$tableStack = $this->presql['tables'];
		if (!is_array($tableStack)) { return false; }

		foreach ($tableStack as $tableName) {
			$res = $this->db->query(
							"UPDATE bf_sysconfig SET bf_value=:value WHERE bf_param=:param",
							array('value' => array('0', 'CHAR'), 'param' => array('lock_' . $tableName, 'CHAR'))
						);
			if (DB::isError($res)) { return false; }
		}
		return true;
	}

	protected function setLock($tableName) {
		$res = $this->db->getOne(
							'SELECT count(*) FROM bf_sysconfig WHERE bf_param=:param', 0,
							array('param' => array('lock_' . $tableName, 'CHAR'))
						);
		if (DB::isError($res))  {
			return TABLE_UNLOCKED;
		}  elseif ($res == '0') {
			$res = $this->db->query(
								'INSERT INTO bf_sysconfig VALUES(:a,:b,:c,:d,:e,:f,:g,:h,:i)',
								array(
									'a'=>array('N', 'CHAR'),
									'b'=>array('lock_'.$tableName, 'CHAR'),
									'c'=>array('1', 'CHAR'),
									'd'=>array('0', 'CHAR'),
									'e'=>array(' ', 'CHAR'),
									'f'=>array(' ', 'CHAR'),
									'g'=>array(' ', 'CHAR'),
									'h'=>array(' ', 'CHAR'),
									'i'=>array(' ', 'CHAR')
								)
							);
			return (! DB::isError($res))?TABLE_LOCKED:TABLE_UNLOCKED;
		} else {
			$res = $this->db->query(
								'UPDATE bf_sysconfig SET bf_value=:value WHERE bf_param=:param',
								array('value'=>array('1', 'CHAR'),'param'=>array('lock_'.$tableName, 'CHAR'))
							);
			return (! DB::isError($res))?TABLE_LOCKED:TABLE_UNLOCKED;
		}
	}

	protected function getLock($tableName) {
		$res = $this->db->getOne(
							'SELECT count(*) FROM bf_sysconfig WHERE bf_param=:param AND bf_value=:value', 0,
							array('param'=>array('lock_'.$tableName, 'CHAR'),'value'=>array('1', 'CHAR'))
						);
		return $res;
	}

	// overriden from Base Class BFQueryBuilder in order to handle empty string workaround for oracle.
	public function getOne() {
		if (!isset($this->sql)) {
			$this->compile();
		}

		// Lock the table if we are supposed to or return false;
		if ($this->tablelock == TABLE_LOCKED) {
			if ($this->dbLock() == TABLE_UNLOCKED) {
				// ERROR - didn't lock
				return false;
			}
		}

		// Only do this when selecting...
		if ($this->querytype == BFQUERYBUILDER_SELECT) {
			$parms = (is_array($this->parms)) ? $this->parms : null;
			$this->res_count = $this->db->getOne($this->sql_count, 0, (is_array($this->parms)) ? $this->parms : null);
		}

		$res = $this->db->getOne($this->sql, 0, $parms);

		// unLock the tables
		if ($this->tablelock == TABLE_LOCKED) {
			if (! $this->dbUnlock()) { return false; }
		}

		// I don't care if it worked. The caller can deal with that.
		return $res;
	}

	function __construct($db, $type = BFQUERYBUILDER_SELECT, $tableLock = TABLE_UNLOCKED) {
		parent::__construct($db, $type, $tableLock);
		// setup oracle's quoted types array.
		$this->quotedTypes = array ("CHAR", "VARCHAR2", "VARCHAR", "STRING", "TEXT");
	}

}

/***  Duplicate this functionality
	public function addColumnConditional($field, $table, $condition, $else, $alias = null) {
		if (!array_key_exists('fields', $this->defs)) {
			$this->defs['fields'] = array();
		}
		if (!array_key_exists('tables', $this->defs) || !in_array($table, $this->defs['tables'])) {
			$this->useTable($table);
		}
		$this->defs['fields'][] =

			  'IF (' .  $condition . ',' . $this->db->quoteIdentifier($table)
			. '.' .	$this->db->quoteIdentifier($field)
			. ', \'' .	$else . '\' )'
			. ' AS ' . $this->db->quoteIdentifier((is_null($alias) ? $field : $alias));
	}
*/
// Join bf_env ON bf_projects.bf_eid = bf_env.bf_bid AND bf_env.bf_eid = 0
//		$rep->tableJoin('bf_env', array(array('bf_projects','bf_eid','=','bf_env','bf_bid'),'and',array('bf_env','bf_eid','=','0')));



?>
