<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/


/**
 * BFReport
 *
 * BFReport now works as a sort of wrapper for the BFQueryBuilder. It Needs to
 * extend the Module class in order for loading in the class as planned, etc.
 * however it also (in a manner of speaking) needs to extend the BFQueryCompiler
 * class in order to get used consistantly by the objects coded for it.
 *
 * Since PHP doesn't really like it when you try to extend two classes at once,
 * and we really only need to support the public methods of BFQueryCompiler,
 * we can use a mix of a __call() implementation and call_user_func_array() to
 * fake multiple inheritance as the outside world (or rather, the object using
 * it) cares.
 *
 * @package Console
 * @subpackage Core
 *
 **/

class BFReport extends Module {
	protected $querybuilder;
	protected $db;

	/**
	 * Yay for faked (well, totally hacked) multiple, dynamic inheritance!
	 */
	public function __call($method, $args) {
		// First, try it on the current (and parent) object
		if (method_exists($this, $method)) {
			return call_user_func_array(array($this, $method), $args);
		// Next, try it on the BFQueryBuilder instance
		} else if (method_exists($this->querybuilder, $method)) {
			return call_user_func_array(array($this->querybuilder, $method), $args);
		// Since we like consistency, panic if you can't find anything
		} else {
			trigger_error('Call to undefined method ' . get_class($this) . '::' . $method . '()', E_ERROR);
		}
	}

	public function getReportResults() {
		$res = $this->querybuilder->getResult();
		
		if (DB::isError($res)) {
			// This way, you can call BFReportResults::valid() and get "no."
			return new BFReportResults();
		}

		$info = $this->querybuilder->getInfo();
		$offset = $this->querybuilder->getLimitOffset();
		$res_count = $this->querybuilder->getCount();
		if ($offset !== false) {
			$count = $this->querybuilder->getLimitCount();
			$num_rows = ($res_count > $count && $count != 0) ? 
					($res_count > $count + $offset ? $count : $res_count - $offset) : 
					$res_count;
			return new BFReportResults($res, $info, $res_count, $num_rows, $offset, $count);
		} else {
			return new BFReportResults($res, $info, $res_count, $res_count);
		}
	}

	public function __construct($db = null) {
		$this->querybuilder = BFQueryBuilder::select($db);
	}
	
	public function getSQL() {
		return $this->querybuilder->getSQL();
	}
}


/**
 * BFFilterRequest
 *
 *  This class sets up the filter information and sends back an array with all the
 *  filters in place and then allows the report code to apply them
 *
 *  This filter system will attempt to mirror google like functionality - it must be
 *  specific to each section of the site though as the filters can be different.
 *  These searches are AND searches unless they specifiy otherwise in the filterArray
 *
 *  Examples:
 *  "project:81"
 *  "project:3.6 console"
 *  "project:73 owner:root"
 *  "project:3.6 agent result:pass owner:joe"
 *  "joe senner project:73"
 *  "joe project:agent result:pass"
 **/
class BFFilterRequest {

	private $filterRequest = array();	   // Key value pair of the user entered filter
	private $filterKeys = array();		  // Allowed filters and how to handle them

	/**
	 * Extracts the filter key value pairs from the user entered field
	 * Everything prior to the first key will be assigned to the ~ key which means ALL
	 */
	public function setFilter($sFilter) {
		preg_match_all('/([^\s]+):/i', $sFilter, $filterItems);
		$filterItems[0] = array_merge(array("~"),$filterItems[1]);
		$filterItems[1] = preg_split('/[^\s]+:/i', $sFilter, -1);
		$this->filterRequest = array_change_key_case(array_combine($filterItems[0],$filterItems[1]), CASE_LOWER);
	}

	/**
	 * Creates an array that defines what to do with each "allowed" key.
	 */
	public function setKey($sKey, $aActions, $aValueSwitch = array()) {
		$this->filterKeys[$sKey] = array('ACTION'=>$aActions, 'VALUESWITCH'=>$aValueSwitch);
	}

	/** Add / Remove Key Value pair
	 * This functionality allows you to remove a key/value pair from the filter request
	 * especially useful if you need to correct something the user has entered via code
	 */
	public function removeKeyValue($sKey, $sValue) {
		if ($this->keyHasValue($sKey, $sValue)) {
			unset($this->filterRequest[$sKey]);
			return true;
		}
		return false;
	}

	public function addKeyValue($sKey, $sValue) {
		return $this->filterRequest[strtolower($sKey)] = $sValue;
	}

	/** Does the filter have this key/value pair?
	 */
	public function keyHasValue($sKey, $sValue) {
		if (isset($this->filterRequest[$sKey]) && strtolower($this->filterRequest[$sKey]) == strtolower($sValue)) {
			return true;
		}
		return false;
	}

	/**
	 * Does the job of building the filter array - this is the main function that converts the
	 * user input to an array used by bfreport
	 * Note this function is a little on the complex side.  To that end I have documented it as
	 * best I could in the code.
	 */
	public function getFilter() {
		$filterArray = array();
		foreach($this->filterRequest as $key=>$value) {
			if (isset($key)&& isset($value) && array_key_exists($key, $this->filterKeys) && $value!="") {

				// Yank out any non-word stuff from the value to avoid sql injection - trim it too for ease of use
				$value = preg_replace('|[\s]|','%', trim($value));

				// Replace the user input value with what it actually should be ex: 'pass'='p' - defined in setKey
				$value = (array_key_exists($value, $this->filterKeys[$key]['VALUESWITCH']))?$this->filterKeys[$key]['VALUESWITCH'][$value]:$value;

				// Push the adjusted value to the end of each array element within this array but only if they are arrays
				array_walk($this->filterKeys[$key]['ACTION'], create_function('&$a,$k', 'if (is_array($a)) { array_push($a,"'.$value.'"); }return $a;'));

				// Merge it all together to be sent out
				$filterArray = array_merge($filterArray,$this->filterKeys[$key]['ACTION']);

				// That was fun
			}
		}
		return $filterArray;
	}
}

/**
 * The BFReportResults exists as an object to get
 * passed to whatever template engine needs it.
 * This keeps a large result set from ever having to
 * get passed in one huge piece from PHP to template and
 * instead provides an iterator.
 **/
$GLOBALS['controller']->require_module('BFStore');

class BFReportResults {
	protected $db;
	protected $source;
	protected $raw_info;
	protected $num_rows;
	protected $num_cols;
	protected $column_names;
	protected $current_rownum;
	protected $offset;
	protected $limit;
	protected $grand_total;

	public function rewind() {
		$this->source->row_counter = 0;
		$this->current_rownum = 0;
	}

	// For testing purposes
	public function print_r() {
		$this->current_rownum = 0;
		$foo = array();
		while(($row = $this->source->fetchRow(DB_FETCHMODE_ASSOC, $this->current_rownum++))
				&& !DB::isError($row)) {
			$foo[] = $row;
		}
		return print_r($foo,true);
	}

	public function hasResults() {
		return $this->grand_total > 0;
	}

	public function valid() {
		return $this->current_rownum < $this->num_rows;
	}

	public function next() {
		if ($this->source)  {
			$row = null;
			// Basically, if not rewound
			if ($this->current_rownum != 0) {
				$row = $this->source->fetchNext(DB_FETCHMODE_ASSOC);
			} else {
				$row = $this->source->fetchRow(DB_FETCHMODE_ASSOC, $this->current_rownum);
			}
			$this->current_rownum++;
			return DB::isError($row) ? '' : $row;
		}
		return '';
	}

	public function getCount($test=null) {
		return $this->num_rows;
	}

	public function getColumnNames() {
		if (!isset($this->column_names)) $this->parseTableInfo();
		return $this->column_names;
	}

	public function getGrandTotal() {
		return $this->grand_total;
	}

	public function getLimit() {
		return $this->limit;
	}

	public function getOffset() {
		return $this->offset;
	}

	public function getPage() {
		return ($this->limit == 0) ? 1 : ceil($this->offset / $this->limit) + 1;
	}

	public function getPageCount() {
		return ($this->grand_total == 0 || $this->limit < 1) ? 1 : ceil($this->grand_total / $this->limit);
	}

	// Is this UID a Library or Project? 
	public function linkLibProj($uid) {
	global $controller;
	$controller->addError('BFReport::linkLibProj - will be removed. us BF_ obj isntead.');
	
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_platform', 'bf_projects');
		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid', $uid)
		));
		$platform = $qb->getOne();

		$result = array(
			'BF_ID' => $uid,
			'MOD'   => (!empty($platform))?'projects':'library'
		);
		return $result;
	}

	// Returns information about the user based on the users ID
	public function getUserInfo($userid) {
		$userid = intval($userid);
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_name',   'bf_users', 'name');
		$qb->addColumn('bf_email',  'bf_users', 'email');
		$qb->where(array(
			array('=', 'bf_users', 'bf_userid', $userid)
		));
		$res = $qb->getResult();
		$userInfo = null;

		if ($userid && $userInfo = $res->fetchRow() && !DB::isError($userInfo) && is_object($userInfo)) {
			$result = array(
							'USERID'=> $userid,
							'NAME'  => $userInfo->name,
							'EMAIL' => $userInfo->email
							);
		} else {
			$result = array(
							'USERID'=> 0,
							'NAME'  => '',
							'EMAIL' => ''
							);
		}

		return $result;
	}

	// Returns a formatted date time
	public function getDateTime($timeIn, $format = null) {
		global $user, $GLOBAL_SETTINGS;

		if (empty($format)) {
			// First, see if the user has a preference.
			$format = $user->get('bf_datefmt');
			// If not, get the default
			if (empty($format)) {
				$format = $GLOBAL_SETTINGS['formatting']['date'];
			}
		}
		$timeIn = (empty($timeIn) || !is_long(intval($timeIn)))?strtotime('now'):intval($timeIn);
		//print "$format";die; // TODO Fix IE - in IE you get strange formats for some reason
		if ($ourDate = date($format,$timeIn)) {
			return $ourDate;
		} else {
			return "Invalid Date";
		}
	}

	protected function parseTableInfo() {
		$this->num_cols	 = count($this->raw_info);
		$this->column_names = array();
		if (is_array($this->raw_info)) {
			foreach ($this->raw_info as $col) $this->column_names[] = $col['name'];
		}
	}

	public function __construct($res = null, $info = null, $grand_total = 0, $num = 0, $offset = 0, $limit = 0) {
		if ($res instanceof DB_result) {
			$this->source			= $res;
			$this->num_rows			= ($num > $limit && $limit != 0) ? 
					($num > $limit + $offset ? $limit : $num - $offset) : 
					$num;
			$this->current_rownum	= 0;
			$this->raw_info			= $info;
			$this->offset			= intval($offset);
			$this->limit			= intval($limit);
			$this->grand_total		= intval($grand_total);
		} else {
			// Make an invalid result
			$this->num_rows			= 0;
			$this->current_rownum	= -1;
		}
	}
}

?>
