<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFSysConfig.php
 * 
 * This file includes two classes: BFSysConfig (extending BFDBO) and BFSysConfigManager
 * 
 * @package Console
 * @subpackage System
 */

/**
 * BFSysConfig
 * 
 * Not particularly useful without the BFSysConfigManager, as the manager caches configuration
 * settings as they get used.
 */
class BFSysConfig extends BFDBO {
	public $table =			'bf_sysconfig';
	public $primary_key =	'bf_param';
	public $type_name = 	'SysConf';
	public $descriptor_key = 'bf_title';
	
	/**
	 * Overridden for datatype check.  We'll test any submitted values to see if they are valid.
	 */
	public function set($field, $value)  {
		if ($field == 'bf_value')  {
			switch ($this->fields['bf_datatype'])  {
				case 'bool':
					if ($value != 'Y' && $value != 'N')  {
						throw new BFException(translate('invalid boolean value'));
					}
					break;
				case 'email':
					// I'll just check that the string _contains_ a valid email address and be done with it.
					if (!preg_match('/[a-zA-Z0-9._-]+\@(([a-zA-Z0-9\-]+\.)+[a-zA-Z]{2,5}|localhost)/', $value))  {
						throw new BFException(translate('invalid email value'));
					}
					break;
				case 'formatpasswd':
					if (!preg_match('/^(\d+(\.\d+(\.[uUlLnNsS]\d+)*)?)?$/', $value))  {
						throw new BFException(translate('invalid password format'));
					}
					break;
				case 'hostip':
					if (!preg_match('/^(([01]?\d\d?|2[0-4]\d|25[0-5])\.){3}([01]?\d\d?|2[0-4]\d|25[0-5])(:[0-9]+)?$/',
							$value))  {
						throw new BFException(translate('invalid ip address value'));
					}
					break;
				case 'integer':
					if (!ctype_digit($value))  {
						throw new BFException(translate('invalid integer value'));
					}
					break;
				case 'ratio':
					if (!preg_match('/^[1-9]\d*\/[1-9]\d*$/', $value))  {
						throw new BFException(translate('invalid ratio value'));
					}
					break;
				case 'real':
					if (!is_numeric($value))  {
						throw new BFException(translate('invalid numeric value'));
					}
					break;
				case 'timedays':
				case 'timemins':
				case 'timesecs':
				case 'timestamp':
					if (!ctype_digit($value) || $value < 0)  {
						throw new BFException(translate('invalid positive numeric value'));
					}
					break;
				case 'url':
					// We are NOT going to try to validate URLs.  That will only cause us pain.

				case 'formatdate':
				case 'formattime':
				case 'formattimestamp':
				case 'passwd':
				case 'string':
				default:
			}
		}
		return parent::set($field, $value);
	}
	
	/**
	 * Overridden for access check
	 */
	public function update() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('UpdateSysconfig')) {
			$success = false;
			if ($this->controller->local) {
				if ($this->fields['bf_geo_id'] == '') {
					$this->fields['bf_geo_id'] = $this->controller->geo->get('bf_geo_id');
					return $this->insert();
				}
			}
			
			if (!$this->hasRequiredFields()) {
				// Catch this and handle it sanely per-object
				throw new BFException(translate('2181'));
			}
			$qb = BFQueryBuilder::update();
	
			// No sense in sending empty queries. Just return true to keep everyone happy
			$updated = false;
			foreach ($this->tablefields as $field) {
				if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
					if (!$updated) {
						$updated = true;
					}
					// If a bfstored value, need to save that first to get the ID
					if (array_key_exists($field, $this->bfstores)) {
						$this->fields[$field] = $this->bfstore->setStorage($this->bfstores[$field]['TEXT'], $this->bfstores[$field]['UID']);
					}
					$qb->setColumn($field, $this->table, $this->fields[$field]);
				}
			}
	
			if ($updated) {
				if ($this->controller->local) {
					$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						array('=', $this->table, 'bf_geo_id', $this->fields['bf_geo_id'])
					));
				} else {
					$qb->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));
				}
				$res = $qb->getResult();
				$success = !DB::isError($res);
			} else {
				$success = true;
			}
			if ($success)  {
				$this->logChange('Updated');
				return true;
			}
			return false;				
		} else {
			throw new BFAccessException('UpdateSysconfig');
		}
	}

	/**
	 * If a local database, always tack on the geo_id
	 */
	public function insert() {
		if ($this->controller->local) {
			$this->fields['bf_geo_id'] = $this->controller->geo->get('bf_geo_id');
		}
		return parent::insert();
	}

	/**
	 * loadFields() sets up a couple of added fields from other tables,
	 * as well as a couple of automatic joins to go with.
	 */
	public function loadFields() {
		global $user, $LANG_DEFAULT;

		parent::loadFields();
		$this->fields['bf_string']   = null;

		$lang = null;
		// If not logged in, get the default language
		if (!isset($user)) {
			$lang = bf_get_language();
			if (empty($lang)) {
				// If still nothing, just use the default
				$lang = $LANG_DEFAULT;
			}
		} else {
			$lang = $user->get('bf_lang');
		}

		$this->querybuilder->addColumn('bf_message', 'bf_strings', 'bf_string');
		if ($this->controller->local) {
			$this->querybuilder->where(array(
					'(',
					array('=', 'bf_sysconfig', 'bf_geo_id', $this->controller->geo->get('bf_geo_id')),
					'OR',
					array('=', 'bf_sysconfig', 'bf_geo_id', ''),
					')'
				));
		} else if ($this->controller->master) {
			$this->querybuilder->where(array('=', 'bf_sysconfig', 'bf_geo_id', ''));
		}
		$this->querybuilder->join(array(
				'LEFT',
				'bf_strings',
				array(
					array('=', 'bf_sysconfig', 'bf_title', 'bf_strings', 'bf_key'),
					array('=', 'bf_strings', 'bf_lang', $lang),
				)
			));
		$this->querybuilder->orderBy('DESC', 'bf_sysconfig', 'bf_geo_id');
	}

	/**
	 * Overridden to prefix a ! so that the final version will be gotten from bf_strings when displayed.
	 */
	public function getObjectName()  {
		return '!' . $this->fields[$this->descriptor_key];
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		if ($this->log_changes && $this->inserted && $field == 'bf_value' &&
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			$this->controller->logAudit($this->getObjectTypeName() . 'UpdatedField', array(
					$this->getObjectName(), $before, $after));
		}
	}

}

/**
 * BFSysConfigManager
 * 
 * Just about whenever you need to CRUD a BFSysConfig, use this instead to keep queries down.
 */
class BFSysConfigManager {
	/**
	 * Associative array using params as the keys for their corresponding BFSysConfig objects
	 */
	protected $configs;
	
	/**
	 * Search through the configs by parameter name and return the value. Since a config could
	 * have a boolean value, return null instead when nothing gets located by the passed param.
	 */
	public function get($param) {
		if (array_key_exists($param, $this->configs) || $this->loadSysConfig($param)) {
			return $this->configs[$param]->get('bf_value');
		}
		
		return null;
	}
	
	public function set($param, $value) {
		if (array_key_exists($param, $this->configs) || $this->loadSysConfig($param)) {
			$this->configs[$param]->set('bf_value', $value);
			return $this->configs[$param]->save();
		}
		
		return false;
	}
	
	protected function loadSysConfig($param) {
		$tempconfig = new BFSysConfig();
		if ($tempconfig->select($tempconfig->primary_key, $param)) {
			$this->configs[$param] = $tempconfig;
			return true;
		}
		
		return false;
	}
	
	/**
	 * Sort of an alias to BFDBO::search, but imposes LIKE on the WHERE clause. At the same time
	 * that this method queries and retrieves configs, it also caches them for use later on. It
	 * only appends the wildcard to the end of the parameter because having it at the beginning
	 * just blows.
	 */
	public function getMany($param) {
		$return_configs =	array();
		$tempconfig =		new BFSysConfig();
		$param =			$param . '%';
		$configs =			$tempconfig->search($tempconfig->primary_key, $param, 'LIKE');
		
		foreach ($configs as $config) {
			$tempparam = $config->get('bf_param');
			if (!array_key_exists($tempparam, $this->configs)) {
				$this->configs[$tempparam] = $config;
			}
			$return_configs[$tempparam] = $config->get('bf_value');
		}
		
		return $return_configs;
	}
	
	public function __construct() {
		$this->configs = array();
	}
}

?>
