/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.io.IOException;
import java.io.Serializable;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Hashtable;

import org.eclipse.core.runtime.PlatformObject;

import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.exception.BuildForgeException;
import com.buildforge.buildmodel.core.util.ConnectionTest;
import com.buildforge.buildmodel.core.util.Log;

public class Console extends PlatformObject implements IConstants, IConsole, IConnectable, Serializable {
	
	private String name = null;
	private String host = null;
	private int portNumber = 80;
	private String ldapDomain = null;
	private String user = null;
	private String password = null;
	private String version = null;
	private boolean authEnabled = false;
	private boolean connected = false;
	
	// proj is the library not the nodes. 
	private transient Projects proj = new Projects();
	// projects will be populated with ProjectNode objects that are
	// the servers children in the TreeView.
	private ArrayList projects = new ArrayList();
	
	public Console() {
		super();
	}
	
	public Console (String name, String host, int portNumber, String user, String password) {
		this.name = name;
		this.host = host;
		this.portNumber = portNumber;
		this.user = user;
		this.password = password;
	}
	
	private void initializeProjectsSource() {
		if (proj == null)
			proj = new Projects();
		if (host == null || user == null || password == null || !authEnabled)
			authEnabled = false;
		else 
			authEnabled = true;
	}
	
	public String toString() {
		return name;
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#authenticate()
	 */
	public boolean authenticate() throws BuildForgeException {
		initializeProjectsSource();
		if (authEnabled) {
			proj.setHost(host);
			proj.setUserName(user);
			proj.setPassword(password);
			if (!"".equals(ldapDomain)) {
				proj.setDomain(ldapDomain);
			}
			boolean val = proj.authenticate();
			setConnected(val);
			queryConsoleForVersion();
			return val;
		}
		Log.trace("Auth isn't enabled for console [" + getName() +"]!\n", getClass().getName());
		return false;
	}
	
	
	public boolean logOff() {
		if (proj != null)
			return proj.logOff();
		return true;
	}
	
	
	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#update()
	 */
	public void update() {
		if (!authEnabled) {
			Log.trace("Auth is disabled", getClass().getName());
			setConnected(false);
			return;
		}
		int pCount = processUpdate();
		int projSize = projects.size();		
		if (pCount == -1 || projSize > 0)
			return;
	
		int attempts = 0;
		while (projSize == 0 && attempts < MAX_RELOGIN_ATTEMPTS) {
			// Due to a bug in the Server side API sometimes a session timeout isn't reported.
			// this is a hack to deal with this on old consoles that will still have this issue.
			
			Log.trace("Project Size for [" + getName() + "] is [" + projects.size() + "]\n", getClass().getName());
			setConnected(false);
			authEnabled = true;
			if (pCount > 0) {
				// we know this is the bug here... so lets be agressive and not count against our attempts.
				Log.trace("HIT SPECIAL CASE RETRY\n", getClass().getName());
				attempts++;
			} else {
				// just re-auth once this could be a normal case here.
				Log.trace("HIT NORMAL CASE RETRY\n", getClass().getName());
				attempts += 2;
			}
			try { authenticate(); } catch (BuildForgeException bfe) { }
			processUpdate();
			projSize = projects.size();
		}
		
	}

	private int processUpdate() {
		ArrayList fullProjects = (ArrayList) proj.listProjects();
		if (proj.isAuthError()) {
			Log.trace("listProjects failed to auth\n", getClass().getName());
			authEnabled = false;
			return -1;
		} else {
			Log.trace("listProjects [" + fullProjects + "]", this.getClass().toString());
			setConnected(true);
		}
		if (projects == null) {
			projects = new ArrayList();
		}
		int pCount = projects.size();
		projects.clear();
		
		for (int i = 0; i < fullProjects.size(); i++) {
			if (((Object) fullProjects.get(i)) instanceof Hashtable) {
				Hashtable element = (Hashtable) fullProjects.get(i);
				// store valid projects as children to this server.
				if ((String) element.get("bf_description") != null) {
					projects.add(new ProjectNode( Integer.parseInt((String) element.get("bf_pid")),
							(String) element.get("bf_description"), this));
				}
			}
		}
		return pCount;
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#execBuild(int)
	 */
	public int execBuild(int projectId) {
		initializeProjectsSource();
		return proj.executeQuickBuild(projectId);
	}
	
	public int execBuild(int projectId, EnvironmentGroup env) {
		initializeProjectsSource();
		return proj.executeBuild(projectId, env);
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getHost()
	 */
	public String getHost() {
		return host;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#setHost(java.lang.String)
	 */
	public void setHost(String host) {
		this.host = host;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getName()
	 */
	public String getName() {
		return name;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#setName(java.lang.String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getPassword()
	 */
	public String getPassword() {
		return password;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#setPassword(java.lang.String)
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getPortNumber()
	 */
	public int getPortNumber() {
		return portNumber;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#setPortNumber(int)
	 */
	public void setPortNumber(int portNumber) {
		this.portNumber = portNumber;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getUser()
	 */
	public String getUser() {
		return user;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#setUser(java.lang.String)
	 */
	public void setUser(String user) {
		this.user = user;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getProjects()
	 */
	public ArrayList getProjects() {
		return projects;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#setProjects(java.util.ArrayList)
	 */
	public void setProjects(ArrayList projects) {
		this.projects = projects;
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.internal.IConsole#getAuthenticationTokens()
	 */
	public String getAuthenticationTokens() {
		initializeProjectsSource();
		return this.proj.getToken();
	}

	public void invalidateAuthenticationTokens() {
		if (proj == null) {
			return;
		}
		Throwable t = new Throwable();
		StackTraceElement[] elements = t.getStackTrace();
		String callerMethod = elements[0].getMethodName();
		String callerMethodName = elements[1].getMethodName();
		String callerClassName = elements[1].getClassName();
		
		Log.trace("Caller is : [" + callerClassName + "." + callerMethodName + "()] Console is [" + this.getName() + "]\n", getClass().getName());
		this.proj.setToken(null);
	}
	
	public boolean isDisabled() {
		return (!authEnabled);
	}

	public boolean isAuthEnabled() {
		return authEnabled;
	}

	public void setAuthEnabled(boolean authEnabled) {
		this.authEnabled = authEnabled;
	}

	public String getLdapDomain() {
		return ldapDomain;
	}

	public void setLdapDomain(String ldapDomain) {
		this.ldapDomain = ldapDomain;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public void queryConsoleForVersion() {
		setVersion(proj.getConsoleVersion());
	}

	public boolean verifiedConnection() throws UnknownHostException, IOException {
		ConnectionTest connection = new ConnectionTest(this);		
		return connection.testHttpTimeout();
	}

	public boolean isConnected() {
		return connected;
	}

	public void setConnected(boolean connected) {
		if (connected == false) {
			if (this.connected == true) {
				logOff();
				this.connected = false;
			}
			invalidateAuthenticationTokens();
		}
		this.connected = connected;
	}
	
	public String getConsoleSaveData() {
		String sdata = "";
		sdata = this.getName() + consoleDelim + this.getHost() + consoleDelim + this.getPortNumber() + consoleDelim + this.getUser() + consoleDelim + this.getPassword() + consoleDelim + this.getLdapDomain() + consoleDelim + this.getVersion() + consoleLineDelim;

		return sdata;
	}

	public EnvironmentGroup getEnvironmentGroup(int id) {
		return new EnvironmentGroup(proj, id);
	}
	
}
