package com.buildforge.buildmodel.core.internal;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * This Class is stored as the value of the ReflectorStore's environment hash.  
 * This was done to allow for enumerated option sets in response to RATLC00500647
 * @author vaughans
 */
public class EnvironmentValue implements Externalizable {
		// So we can change the class and mark it as not affecting serialization compatibility
	private static final long serialVersionUID = 1L;

	private static final String DELIMITER = "|"; // Vertical Pipe for default 
	private String value;
	private String[] options;
	
	public EnvironmentValue(){ }
	
	/**
	 * Creates a new EnvironmentValue with the provided value or options enumeration string.  
	 * If the delimiter is found in the string, they will be interpreted as an option enumeration, 
	 * otherwise the provided string will be used as the actual, hard value. 
	 * @param optionsOrValue
	 */
	public EnvironmentValue(String optionsOrValue){
		if(optionsOrValue != null && optionsOrValue.indexOf(DELIMITER) >=0 ){ 
			setOptions(parseOptionsString(optionsOrValue)); 
		} else{ setValue(optionsOrValue); }
	}
	
	public String[] getOptions(){ return options; }
	public String getOptionsAsDelimitedString(){
		if(options == null || options.length == 0){ return null; }
		else{
			StringBuffer buf = new StringBuffer();
			for(int i = 0; i < options.length; i++){ buf.append(options[i]+(((i + 1) < options.length)?DELIMITER:"")); }
			return buf.toString();
		}
	}
	public String getValue(){ return value; }
	public void setOptions(String[] options){ 
		this.options = options;
		if(options != null && options.length > 0 && value == null){ setValue(options[0]); }
	}
	public void setValue(String value){ this.value = value; }
	
	public String[] parseOptionsString(String optionsString){
		if(optionsString == null || optionsString.trim().length() == 0){ return null; }
		else{
			// Using strtok for 1.4 compat
			StringTokenizer strtok = new StringTokenizer(optionsString, DELIMITER); 
			ArrayList strings = new ArrayList(strtok.countTokens());
			while(strtok.hasMoreTokens()){
				String nextElement = (String)strtok.nextElement(); 
				if(nextElement != null && nextElement.length() > 0){ strings.add(nextElement); } 
			}
			return (String[])strings.toArray(new String[strings.size()]);
		}
	}

	/**
	 * For migration purposes, this converts the old data type (plain String) into the new values (this)
	 * @param o
	 * @return
	 */
	public static EnvironmentValue getEnvironmentValue(Object o){
		if(o instanceof EnvironmentValue){ return (EnvironmentValue)o; }
		else if(o instanceof String){ return new EnvironmentValue((String)o); }
		return null;
	}
	
	public String toString(){ return value; }

	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException{
		boolean nullValue = in.readBoolean();
		if(!nullValue){ value = in.readUTF(); }
		int optionsLength = in.readInt();
		if(optionsLength > 0){
			String[] optionsArray = new String[optionsLength];
			for(int i = 0; i < optionsLength; i++){ optionsArray[i] = in.readUTF(); }
			setOptions(optionsArray);
		}
	}

	public void writeExternal(ObjectOutput out) throws IOException{
		out.writeBoolean(getValue() == null);
		if(getValue() != null){ out.writeUTF(getValue()); }
		if(options != null && options.length > 0){
			out.writeInt(options.length);
			for(int i = 0; i < options.length; i++){ out.writeUTF(options[i]); }
		} else { out.writeInt(0); }
		
	}
}
