/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;

import org.eclipse.ui.plugin.*;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.osgi.framework.BundleContext;

import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.internal.ReflectorStore;

/**
 * The main plugin class to be used in the desktop.
 */
public class ReflectorPlugin extends AbstractUIPlugin {
	private static ArrayList projectInfo = null; 
	private static Path projectInfoPath = new Path("buildforge_projects.txt"); //$NON-NLS-1$
	
	//The shared instance.
	private static ReflectorPlugin plugin;
	
	/**
	 * The constructor.
	 */
	public ReflectorPlugin() {
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		projectInfo = getProjectInfo();
	}

	/**
	 * This method pulls the serialized ReflectorStore Object from disk so we 
	 * can obtain the list of projects that are configured as Build Forge Reflector
	 * plugins.
	 * @return ArrayList
	 */
	public static ArrayList getProjectInfo() {
		Log.trace("getting project properties", ReflectorPlugin.getDefault().getClass().getName()); //$NON-NLS-1$
		
		if (projectInfo == null) {
			InputStream is = null;
			try {
				IPath statedir = ReflectorPlugin.getDefault().getStateLocation();
				File locdata = statedir.append(projectInfoPath).toFile();
				is = new FileInputStream(locdata);
				Log.trace("projectInfoFile : " + locdata.getPath(), ReflectorPlugin.getDefault().getClass().getName()); //$NON-NLS-1$
				
				ObjectInputStream objinput = new ObjectInputStream(is);
				projectInfo = (ArrayList) objinput.readObject();
			} catch (FileNotFoundException fnfe) {
				Log.trace("Project Information file was not found.  Creating Initial ArrayList.", ReflectorPlugin.getDefault().getClass().getName()); 
				projectInfo = new ArrayList();
			} catch (IOException e) {
				Log.errorLogMessage(Messages.getString("ReflectorPlugin.PROJECT_PROPERTIES_ERROR_CONS") + e.getMessage() + e.getStackTrace(),  //$NON-NLS-1$
						Log.ERROR, ReflectorPlugin.getDefault().getClass().getName(), 
						ReflectorPlugin.getDefault().getBundle().getSymbolicName());
				projectInfo = new ArrayList();
			} catch (ClassNotFoundException ce) {
				Log.errorLogMessage(Messages.getString("ReflectorPlugin.PROJECT_PROPERTIES_ERROR_CONS") + ce.getMessage() + ce.getStackTrace(),  //$NON-NLS-1$
						Log.ERROR, ReflectorPlugin.getDefault().getClass().getName(), 
						ReflectorPlugin.getDefault().getBundle().getSymbolicName());
				projectInfo = new ArrayList();
			}
		}
		
		return projectInfo;
	}
	
	
	/**
	 * This method returns a ReflectorStore object for the given project 
	 * identifier, which is the path to the project on the system currently.
	 * @param label - String
	 * @return ReflectorStore
	 * @see ReflectoreStore
	 */
	public static ReflectorStore getProjectInfoByLabel(String label) {
		if (label == null) {
			return null;
		}
		if (projectInfo == null)
			projectInfo = getProjectInfo();
		
		for (int i = 0; i < projectInfo.size(); i++) {
			if ( label.equals( ((ReflectorStore)projectInfo.get(i)).getConsoleLabel() ) )
					return (ReflectorStore)projectInfo.get(i);
		}
		return null;
	}
	
	/**
	 * This method adds the individual projects ReflectorStore to the list of projects
	 * to be stored.
	 * @param store - ReflectorStore
	 * @see ReflectorStore
	 */
	public static void addProjectInfo(ReflectorStore store) {
		String label = store.getConsoleLabel();
		
		if (projectInfo == null)
			projectInfo = getProjectInfo();
		
		for (int i = 0; i < projectInfo.size(); i++) {
			if ( label.equals( ((ReflectorStore)projectInfo.get(i)).getConsoleLabel() ) ) {
				projectInfo.set(i, store);
				return;
			}
		}
		projectInfo.add(store);	
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static ReflectorPlugin getDefault() {
		return plugin;
	}

	public void save() {
		IPath statedir = ReflectorPlugin.getDefault().getStateLocation();
		File locdata = statedir.append(projectInfoPath).toFile();
		
		Log.trace("Saving buildforge project information to " + locdata.getAbsolutePath(), ReflectorPlugin.getDefault().getClass().getName()); //$NON-NLS-1$

		try {
			OutputStream out = new FileOutputStream(locdata);
			ObjectOutputStream s = new ObjectOutputStream(out);
			s.writeObject(projectInfo);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	
	
	public static void setProjectInfo(ArrayList projectInfo) {
		ReflectorPlugin.projectInfo = projectInfo;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin("com.buildforge.buildmodel.reflector", path); //$NON-NLS-1$
	}
}
